Package: hdcuremodels
Title: High-Dimensional Cure Models
Version: 0.0.8
Date: 2026-05-17
Authors@R: 
    c(person("Han", "Fu", role = "aut"), person(c("Kellie J."), "Archer", email=
    "archer.43@osu.edu", role = c("aut","cre"), comment = c(ORCID="0000-0003-1555-5781")), 
    person("Tung Lam", "Nguyen", email = "nguyenlamtung10@gmail.com", role = "rev", comment = "Reviewed the package for ROpenSci"),
    person("Panagiotis", "Papastamoulis", email = "papapast@yahoo.gr", role = "rev", comment = "Reviewed the package for ROpenSci"))
Description: Provides functions for fitting various penalized parametric and semi-parametric mixture cure models with different penalty functions, testing for a significant cure fraction, and testing for sufficient follow-up as described in Fu et al (2022)<doi:10.1002/sim.9513> and Archer et al (2024)<doi:10.1186/s13045-024-01553-6>. False discovery rate controlled variable selection is provided using model-X knock-offs. 
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.2.0)
Imports: doParallel, flexsurv, flexsurvcure, foreach, ggplot2, ggpubr,
        glmnet, knockoff, mvnfast, parallel, plyr, methods, survival,
        withr
Config/Roxygen: list(markdown = TRUE, roclets = c("namespace", "rd",
        "srr::srr_stats_roclet"))
Config/Remotes: ropensci-review-tools/srr
Suggests: knitr, Rdsdp, rmarkdown, roxygen2, testthat (>= 3.0.0)
VignetteBuilder: knitr
LazyData: true
URL: https://github.com/ropensci/hdcuremodels
BugReports: https://github.com/ropensci/hdcuremodels/issues
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-18 00:05:49 UTC; archer.43
Author: Han Fu [aut],
  Kellie J. Archer [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1555-5781>),
  Tung Lam Nguyen [rev] (Reviewed the package for ROpenSci),
  Panagiotis Papastamoulis [rev] (Reviewed the package for ROpenSci)
Maintainer: Kellie J. Archer <archer.43@osu.edu>
Repository: CRAN
Date/Publication: 2026-05-18 07:00:15 UTC
Built: R 4.5.2; ; 2026-05-18 09:54:20 UTC; unix

Package: hdcuremodels
Title: High-Dimensional Cure Models
Version: 0.0.8
Date: 2026-05-17
Authors@R: 
    c(person("Han", "Fu", role = "aut"), person(c("Kellie J."), "Archer", email=
    "archer.43@osu.edu", role = c("aut","cre"), comment = c(ORCID="0000-0003-1555-5781")), 
    person("Tung Lam", "Nguyen", email = "nguyenlamtung10@gmail.com", role = "rev", comment = "Reviewed the package for ROpenSci"),
    person("Panagiotis", "Papastamoulis", email = "papapast@yahoo.gr", role = "rev", comment = "Reviewed the package for ROpenSci"))
Description: Provides functions for fitting various penalized parametric and semi-parametric mixture cure models with different penalty functions, testing for a significant cure fraction, and testing for sufficient follow-up as described in Fu et al (2022)<doi:10.1002/sim.9513> and Archer et al (2024)<doi:10.1186/s13045-024-01553-6>. False discovery rate controlled variable selection is provided using model-X knock-offs. 
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.2.0)
Imports: doParallel, flexsurv, flexsurvcure, foreach, ggplot2, ggpubr,
        glmnet, knockoff, mvnfast, parallel, plyr, methods, survival,
        withr
Config/Roxygen: list(markdown = TRUE, roclets = c("namespace", "rd",
        "srr::srr_stats_roclet"))
Config/Remotes: ropensci-review-tools/srr
Suggests: knitr, Rdsdp, rmarkdown, roxygen2, testthat (>= 3.0.0)
VignetteBuilder: knitr
LazyData: true
URL: https://github.com/ropensci/hdcuremodels
BugReports: https://github.com/ropensci/hdcuremodels/issues
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-18 00:05:49 UTC; archer.43
Author: Han Fu [aut],
  Kellie J. Archer [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1555-5781>),
  Tung Lam Nguyen [rev] (Reviewed the package for ROpenSci),
  Panagiotis Papastamoulis [rev] (Reviewed the package for ROpenSci)
Maintainer: Kellie J. Archer <archer.43@osu.edu>
Repository: CRAN
Date/Publication: 2026-05-18 07:00:15 UTC
Built: R 4.6.0; ; 2026-05-18 09:35:54 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(coef,mixturecure)
S3method(dim,mixturecure)
S3method(family,mixturecure)
S3method(formula,mixturecure)
S3method(logLik,mixturecure)
S3method(nobs,mixturecure)
S3method(plot,mixturecure)
S3method(predict,mixturecure)
S3method(print,mixturecure)
S3method(summary,mixturecure)
export(auc_mcm)
export(concordance_mcm)
export(cure_estimate)
export(cureem)
export(curegmifs)
export(cv_cureem)
export(cv_curegmifs)
export(generate_cure_data)
export(nonzerocure_test)
export(sufficient_fu_test)
import(doParallel)
import(flexsurv)
import(flexsurvcure)
import(foreach)
import(ggplot2)
import(glmnet)
import(graphics)
import(knockoff)
import(mvnfast)
import(parallel)
import(plyr)
import(stats)
import(survival)
importFrom(ggpubr,ggarrange)
importFrom(methods,is)
importFrom(utils,head)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# hdcuremodels

<!-- badges: start -->

[![Status at rOpenSci Software Peer
Review](https://badges.ropensci.org/692_status.svg)](https://github.com/ropensci/software-review/issues/692)
[![R-CMD-check](https://github.com/ropensci/hdcuremodels/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ropensci/hdcuremodels/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

The goal of hdcuremodels is to allow one to fit a penalized mixture cure
model (MCM) when there is a high-dimensional covariate space, such as
when high-throughput genomic data are used in modeling time-to-event
data, when some subjects will not experience the event of interest.
Conventionally, we call the subset of subjects who are immune to the
event of interest cured while all other subjects are susceptible to the
event.

## Installation

You can install the development version of hdcuremodels like so:

``` r
remotes::install_github("https://github.com/kelliejarcher/hdcuremodels")
```

## Example

After loading the `hdcuremodels` and `survival` packages, load
`amltrain` which includes 306 patients diagnosed with acute myeloid
leukemia (AML) who were cytogenetically normal at diagnosis along with
time-to-event outcomes: `cryr` is the duration of complete response (in
years), `relapse.death` is a censoring variable where 1 indicates the
patient relapsed or died and 0 indicates the patient was alive at last
follow-up. It is of interest to fit a MCM using the expression for 320
transcripts measured using RNA-sequencing as predictor variables.

``` r
library(hdcuremodels)
library(survival)
```

``` r
data(amltrain)
```

We can inspect the Kaplan-Meier survival curve to determine whether a
cure fraction seems to be present.

``` r
km_train <- survfit(Surv(cryr, relapse.death) ~ 1, data = amltrain)
```

<img src="man/figures/README-unnamed-chunk-2-1.png" width="100%" />

As can be seen from the Kaplan-Meier plot, there is a long-plateau that
does not drop down to zero. This may indicate the presence of a cured
fraction. We can test the null hypothesis that the cured fraction is
zero against the alternative hypothesis that the cured fraction is not
zero using the `nonzerocure_test` function.

``` r
nonzerocure_test(km_train)
#> Warning in Surv(object$time, object$n.event): Invalid status value, converted
#> to NA
#> $proportion_susceptible
#> [1] 0.7146919
#> 
#> $proportion_cured
#> [1] 0.2853081
#> 
#> $p_value
#> [1] 0.004
#> 
#> $time_95_percent_of_events
#> [1] 5.553847
```

Given the small p-value we reject the null hypothesis and conclude there
is a non-zero cure fraction present. We can also extract the cured
fraction as the Kaplan-Meier estimate beyond the last observed event
using the `cure_estimate` function.

``` r
cure_estimate(km_train)
#> [1] 0.2853081
```

We now fit a penalized MCM using the E-M algorithm where the penalty
parameters for incidence and latency, `lambda.inc` and `lambda.lat` were
previously determined using cross-validation.

``` r
fitem <- cureem(Surv(cryr, relapse.death) ~ .,
  data = amltrain,
  x_latency = amltrain, model = "cox",
  lambda_inc = 0.009993, lambda_lat = 0.02655
)
```

Coefficient estimates can be extracted from the fitted model using the
`coef` for any of these model criteria (“logLik”, “AIC”, “cAIC”, “mAIC”,
“BIC”, “mBIC”, “EBIC”) or by specifying the step at which the model is
desired by specifying the `model_select` parameter. For example,

``` r
coef_cAIC <- coef(fitem, model_select = "cAIC")
```

Predictions can be extracted at a given step or information criterion
(“logLik”, “AIC”, “cAIC”, “mAIC”, “BIC”, “mBIC”, “EBIC”) using the
`predict` function with `model_select` specified.

``` r
train_predict <- predict(fitem, model_select = "cAIC")
```

This returns three objects: `p_uncured` is the estimated probability of
being susceptible ($\hat{p}(\mathbf{x})$), `linear_latency` is
$\hat{\boldsymbol{\beta}}\mathbf{w}$, while `latency_risk` applies high
risk and low risk labels using zero as the cutpoint from the
`linear_latency` vector. Perhaps we want to apply the 0.5 threshold to
`p_uncured` to create Cured and Susceptible labels.

``` r
p_group <- ifelse(train_predict$p_uncured < 0.50, "Cured", "Susceptible")
```

Then we can assess how well our MCM identified patients likely to be
cured from those likely to be susceptible visually by examining the
Kaplan-Meier curves.

``` r
km_cured <- survfit(Surv(cryr, relapse.death) ~ p_group, data = amltrain)
```

<img src="man/figures/README-unnamed-chunk-5-1.png" width="100%" />

We can assess how well our MCM identified higher versus lower risk
patients among those predicted to be susceptible visually by examining
the Kaplan-Meier curves.

``` r
km_suscept <- survfit(Surv(cryr, relapse.death) ~ train_predict$latency_risk, data = amltrain, subset = (p_group == "Susceptible"))
```

<img src="man/figures/README-unnamed-chunk-6-1.png" width="100%" />

Of course, we expect our model to perform well on our training data. We
can also assess how well our fitted MCM performs using the independent
test set `amltest`. In this case we use the `predict` function with
`newdata` specified.

``` r
test_predict <- predict(fitem, newdata = amltest, model_select = "cAIC")
```

## Acknowledgements

This package was reviewed as part of rOpenSci’s software review process.

#' Extract model formula for mixturecure object
#'
#' @description Extract the model formula for `mixturecure` object
#' @param x an object from class `mixturecure`.
#' @param ... other arguments.
#'
#' @returns a formula representing the incidence and variables for the latency
#' portion of the model
#' @keywords methods
#' @method formula mixturecure
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @export
#'
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' fit <- curegmifs(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "weibull", thresh = 1e-4, maxit = 2000,
#'   epsilon = 0.01, verbose = FALSE
#' )
#' formula(fit)
formula.mixturecure <- function(x, ...) {
  if (!("mixturecure" %in% class(x))) {
    stop("Error: class of object must be mixturecure")
  }
  if (is.null(x$call))
    form <- NULL
  else
    form <- getCall(x)
  form
}

#' Extract model coefficients from a fitted mixturecure object
#'
#' @description
#' \code{coef.mixturecure} is a generic function which extracts the model
#' coefficients from a fitted \code{mixturecure} model object fit using
#' \code{curegmifs}, \code{cureem}, \code{cv_curegmifs}, or \code{cv_cureem}.
#'
#' @aliases coefficients
#'
#' @param object a \code{mixturecure} object resulting from \code{curegmifs},
#' \code{cureem}, \code{cv_curegmifs}, or \code{cv_cureem}.
#' @param model_select either a case-sensitive parameter for models fit using
#' \code{curegmifs} or \code{cureem} or any numeric step along the solution path
#' can be selected. The default is \code{model_select = "AIC"} which calculates
#' the predicted values using the coefficients from the model achieving the
#' minimum AIC. The complete list of options are:
#' \itemize{
#'     \item \code{"AIC"} for the minimum AIC (default).
#'     \item \code{"mAIC"} for the minimum modified AIC.
#'     \item \code{"cAIC"} for the minimum corrected AIC.
#'     \item \code{"BIC"}, for the minimum BIC.
#'     \item \code{"mBIC"} for the minimum modified BIC.
#'     \item \code{"EBIC"} for the minimum extended BIC.
#'     \item \code{"logLik"} for the step that maximizes the
#' log-likelihood.
#'     \item \code{n} where n is any numeric value from the
#'     solution path.
#'   }
#' This option has no effect for objects fit using \code{cv_curegmifs} or
#' \code{cv_cureem}.
#' @param ... other arguments.
#'
#' @return \item{rate}{ estimated rate parameter when fitting a Weibull or
#' exponential mixture cure model.}
#' @return \item{shape}{ estimated shape parameter when fitting a Weibull
#' mixture cure model.}
#' @return \item{b0}{ estimated intercept for the incidence portion of the
#' mixture cure model.}
#' @return \item{beta_inc}{ the vector of coefficient estimates for the
#' incidence portion of the mixture cure model.}
#' @return \item{beta_lat}{ the vector of coefficient estimates for the
#' latency portion of the mixture cure model.}
#' @return \item{p_uncured}{ a vector of probabilities from the incidence
#' portion of the fitted model representing the P(uncured).}
#'
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G5.2b} *Explicit tests should demonstrate conditions which trigger every one of those messages, and should compare the result with expected values.*
#' @srrstats {RE4.2} *Model coefficients (via `coef()` / `coefficients()`)*
#' @seealso \code{\link[hdcuremodels]{curegmifs}}, \code{\link[hdcuremodels]{cureem}},
#' \code{\link[hdcuremodels]{summary.mixturecure}}, \code{\link[hdcuremodels]{plot.mixturecure}},
#' \code{\link[hdcuremodels]{predict.mixturecure}}
#' @export
#' @keywords methods
#' @method coef mixturecure
#'
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' fit <- curegmifs(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "weibull", thresh = 1e-4, maxit = 2000,
#'   epsilon = 0.01, verbose = FALSE
#' )
#' coef(fit)
coef.mixturecure <-
  function(object, model_select = "AIC", ...) {
    if (!(class(object) %in% "mixturecure")) {
      stop("Error: class of object must be mixturecure")
    }
    if (!object$cv) {
      if (!is.numeric(model_select))
        model_select <- select_model(object, model_select)$select
      if (is.null(object$x_latency) && !is.null(object$x_incidence)) {
        if (object$model == "exponential") {
          coef <- list(
            rate = object$rate_path[model_select],
            b0 = object$b0_path[model_select],
            beta_inc = object$b_path[model_select, ])
        }
        if (object$model == "weibull") {
          coef <- list(
            rate = object$rate_path[model_select],
            shape = object$alpha_path[model_select],
            b0 = object$b0_path[model_select],
            beta_inc = object$b_path[model_select, ])
        }
        if (object$model == "cox") {
          coef <- list(
            b0 = object$b0_path[model_select],
            beta_inc = object$b_path[model_select, ])
        }
      } else if (!is.null(object$x_latency) && is.null(object$x_incidence)) {
        if (object$model == "exponential") {
          coef <- list(
            rate = object$rate_path[model_select],
            beta_lat = object$beta_path[model_select, ]
          )
        }
        if (object$model == "weibull") {
          coef <- list(
            rate = object$rate_path[model_select],
            shape = object$alpha_path[model_select],
            beta_lat = object$beta_path[model_select, ]
          )
        }
        if (object$model == "cox") {
          coef <- list(beta_lat = object$beta_path[model_select, ])
        }
      } else if (!is.null(object$x_latency) && !is.null(object$x_incidence)) {
        if (object$model == "exponential") {
          coef <- list(
            rate = object$rate_path[model_select],
            b0 = object$b0_path[model_select],
            beta_inc = object$b_path[model_select, ],
            beta_lat = object$beta_path[model_select, ]
          )
        }
        if (object$model == "weibull") {
          coef <- list(
            rate = object$rate_path[model_select],
            shape = object$alpha_path[model_select],
            b0 = object$b0_path[model_select],
            beta_inc = object$b_path[model_select, ],
            beta_lat = object$beta_path[model_select, ]
          )
        }
        if (object$model == "cox") {
          coef <- list(
            b0 = object$b0_path[model_select],
            beta_inc = object$b_path[model_select, ],
            beta_lat = object$beta_path[model_select, ]
          )
        }
      }
      } else {
      if (object$model == "weibull") {
        if (!is.null(object$x_incidence) && !is.null(object$x_latency)) {
          coef <- list(
            rate = object$rate, shape = object$alpha,
            b0 = object$b0, beta_inc = object$b,
            beta_lat = object$beta
          )
        }
        if (is.null(object$x_incidence) && !is.null(object$x_latency)) {
          coef <- list(
            rate = object$rate, shape = object$alpha,
            beta_lat = object$beta
          )
        }
        if (!is.null(object$x_incidence) && is.null(object$x_latency)) {
          coef <- list(
            rate = object$rate, shape = object$alpha,
            b0 = object$b0, beta_inc = object$b
          )
        }
      } else if (object$model == "exponential") {
        if (!is.null(object$x_incidence) && !is.null(object$x_latency)) {
          coef <- list(
            rate = object$rate, b0 = object$b0,
            beta_inc = object$b, beta_lat = object$beta
          )
        }
        if (is.null(object$x_incidence) && !is.null(object$x_latency)) {
          coef <- list(rate = object$rate, beta_lat = object$beta)
        }
        if (!is.null(object$x_incidence) && is.null(object$x_latency)) {
          coef <- list(
            rate = object$rate, b0 = object$b0,
            beta_inc = object$b
          )
        }
      } else if (object$model == "cox") {
        if (!is.null(object$x_incidence) && !is.null(object$x_latency)) {
          coef <- list(
            b0 = object$b0, beta_inc = object$b,
            beta_lat = object$beta
          )
        }
        if (is.null(object$x_incidence) && !is.null(object$x_latency)) {
          coef <- list(beta_lat = object$beta)
        }
        if (!is.null(object$x_incidence) && is.null(object$x_latency)) {
          coef <- list(b0 = object$b0, beta_inc = object$b)
        }
      }
      }
    coef
  }

#' @srrstats {G5.2b} *Explicit tests should demonstrate conditions which trigger every one of those messages, and should compare the result with expected values.*

test_that("coef function works correctly", {
  library(survival)
  withr::local_seed(1234)
  temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
  training <- temp$training
  fit <- curegmifs(Surv(Time, Censor) ~ .,
                   data = training, x_latency = training,
                   model = "weibull", thresh = 1e-4, maxit = 2000,
                   epsilon = 0.01, verbose = FALSE
  )
  output <- coef(fit)
  output$rate %>% expect_length(1)
  output$shape %>% expect_length(1)
  output$b0 %>% expect_length(1)
  output$beta_inc %>% expect_length(dim(fit$x_incidence)[2])
  output$beta_lat %>% expect_length(dim(fit$x_latency)[2])
  output$rate %>% expect_type("double")
  output$shape %>% expect_type("double")
  output$b0 %>% expect_type("double")
  output$beta_inc %>% expect_type("double")
  output$beta_lat %>% expect_type("double")
  expect_equal(round(output$rate, 6), 3.584565)
  expect_equal(round(output$shape, 6), 1.197495)
  expect_equal(round(output$b0, 6), 0.367009)
  expect_error(coef("x"))
  expect_error(coef(1))
  expect_error(coef(fit, model_select = "aic"))
  expect_error(coef(fit, model_select = "caic"))
  expect_error(coef(fit, model_select = "maic"))
  expect_error(coef(fit, model_select = "bic"))
  expect_error(coef(fit, model_select = "mbic"))
  expect_error(coef(fit, model_select = "ebic"))
  expect_setequal(names(output), c("rate", "shape", "b0", "beta_inc", "beta_lat"))

  fit.lm <- lm(Time ~ Censor, data = training)
  expect_error(coef.mixturecure(fit.lm), "Error: class of object must be mixturecure")

  expect_warning(curegmifs(Surv(Time, Censor) ~ .,
                   data = training, x_latency = training,
                   model = "exponential", thresh = 1e-4, maxit = 2000,
                   epsilon = 0.01, verbose = FALSE
  ))

  fit.cv <- cv_cureem(Surv(Time, Censor) ~ .,
                      data = training,
                      x_latency = training, fdr_control = FALSE,
                      grid_tuning = FALSE, nlambda_inc = 10, nlambda_lat = 10,
                      n_folds = 2, seed = 23, verbose = TRUE
  )
  output <- coef(fit.cv)
  output$b0 %>% expect_length(1)
  output$beta_inc %>% expect_length(dim(fit.cv$x_incidence)[2])
  output$beta_lat %>% expect_length(dim(fit.cv$x_latency)[2])
  expect_setequal(names(output), c("b0", "beta_inc", "beta_lat"))
  output$b0 %>% expect_type("double")
  output$beta_inc %>% expect_type("double")
  output$beta_lat %>% expect_type("double")
  output$beta_inc %>% expect_vector()
  output$beta_lat %>% expect_vector()

  fit.cv.fdr <- cv_cureem(Surv(Time, Censor) ~ .,
                          data = training,
                          x_latency = training, model = "weibull", penalty = "lasso",
                          fdr_control = TRUE, grid_tuning = FALSE, nlambda_inc = 10,
                          nlambda_lat = 10, n_folds = 2, seed = 23, verbose = TRUE
  )
  output <- coef(fit.cv.fdr)
  output$rate %>% expect_length(1)
  output$shape %>% expect_length(1)
  output$b0 %>% expect_length(1)
  output$beta_inc %>% expect_length(dim(fit.cv.fdr$x_incidence)[2])
  output$beta_lat %>% expect_length(dim(fit.cv.fdr$x_latency)[2])
  expect_setequal(names(output), c("rate","shape", "b0", "beta_inc", "beta_lat"))
  output$rate %>% expect_type("double")
  output$shape %>% expect_type("double")
  output$b0 %>% expect_type("double")
  output$beta_inc %>% expect_type("double")
  output$beta_lat %>% expect_type("double")
  output$beta_inc %>% expect_vector()
  output$beta_lat %>% expect_vector()

  fit.cv.fdr <- cv_cureem(Surv(Time, Censor) ~ .,
                          data = training, x_latency = training,
                          model = "exponential", penalty = "lasso",
                          fdr_control = TRUE, grid_tuning = FALSE, nlambda_inc = 10,
                          nlambda_lat = 10, n_folds = 2, seed = 23, verbose = TRUE
  )
  output <- coef(fit.cv.fdr)
  output$rate %>% expect_length(1)
  output$b0 %>% expect_length(1)
  output$beta_inc %>% expect_length(dim(fit.cv.fdr$x_incidence)[2])
  output$beta_lat %>% expect_length(dim(fit.cv.fdr$x_latency)[2])
  expect_setequal(names(output), c("rate", "b0", "beta_inc", "beta_lat"))
  output$rate %>% expect_type("double")
  output$b0 %>% expect_type("double")
  output$beta_inc %>% expect_type("double")
  output$beta_lat %>% expect_type("double")
  output$beta_inc %>% expect_vector()
  output$beta_lat %>% expect_vector()

  fit.cv.gmifs <- cv_curegmifs(Surv(Time, Censor) ~ .,
                         data = training, model = "exponential",
                         x_latency = training, fdr_control = FALSE,
                         maxit = 450, epsilon = 0.01, n_folds = 2,
                         seed = 23, verbose = TRUE
  )
  output <- coef(fit.cv.gmifs)
  output$rate %>% expect_length(1)
  output$b0 %>% expect_length(1)
  output$beta_inc %>% expect_length(dim(fit.cv.gmifs$x_incidence)[2])
  output$beta_lat %>% expect_length(dim(fit.cv.gmifs$x_latency)[2])
  expect_setequal(names(output), c("rate", "b0", "beta_inc", "beta_lat"))
  output$rate %>% expect_type("double")
  output$b0 %>% expect_type("double")
  output$beta_inc %>% expect_type("double")
  output$beta_lat %>% expect_type("double")
  output$beta_inc %>% expect_vector()
  output$beta_lat %>% expect_vector()

  fit.cv.gmifs <- cv_curegmifs(Surv(Time, Censor) ~ .,
                               data = training, model = "weibull",
                               x_latency = training, fdr_control = TRUE,
                               maxit = 450, epsilon = 0.01, n_folds = 2,
                               seed = 23, verbose = TRUE
  )
  output <- coef(fit.cv.gmifs, model_select = "cAIC")
  output$rate %>% expect_length(1)
  output$shape %>% expect_length(1)
  output$b0 %>% expect_length(1)
  output$beta_inc %>% expect_length(dim(fit.cv.gmifs$x_incidence)[2])
  output$beta_lat %>% expect_length(dim(fit.cv.gmifs$x_latency)[2])
  expect_setequal(names(output), c("rate", "shape", "b0", "beta_inc", "beta_lat"))
  output$rate %>% expect_type("double")
  output$shape %>% expect_type("double")
  output$b0 %>% expect_type("double")
  output$beta_inc %>% expect_type("double")
  output$beta_lat %>% expect_type("double")
  output$beta_inc %>% expect_vector()
  output$beta_lat %>% expect_vector()

  fit.cv <- cv_curegmifs(Surv(Time, Censor) ~ ., data = training,
                         penalty_factor_inc = rep(c(0, 1), c(1, 11)),
                         measure_inc = "auc",
                         x_latency = training, fdr_control = FALSE,
                         maxit = 450, epsilon = 0.01, n_folds = 2,
                         seed = 23, verbose = FALSE, parallel = FALSE
  )
  output <- coef(fit.cv, model_select = 375)
  output$rate %>% expect_length(1)
  output$shape %>% expect_length(1)
  output$b0 %>% expect_length(1)
  output$beta_inc %>% expect_length(dim(fit.cv.gmifs$x_incidence)[2])
  output$beta_lat %>% expect_length(dim(fit.cv.gmifs$x_latency)[2])
  expect_setequal(names(output), c("rate", "shape", "b0", "beta_inc", "beta_lat"))
  output$rate %>% expect_type("double")
  output$shape %>% expect_type("double")
  output$b0 %>% expect_type("double")
  output$beta_inc %>% expect_type("double")
  output$beta_lat %>% expect_type("double")
  output$beta_inc %>% expect_vector()
  output$beta_lat %>% expect_vector()
})

#' Summarize a fitted mixture cure object
#'
#' @description
#' \code{summary} method for a mixturecure object fit using \code{curegmifs},
#' \code{cureem}, \code{cv_curegmifs}, or \code{cv_cureem}.
#'
#' @param object a \code{mixturecure} object resulting from \code{curegmifs},
#' \code{cureem}, \code{cv_curegmifs}, or \code{cv_cureem}.
#' @param ... other arguments.
#'
#' @return prints the number of non-zero coefficients from the incidence and
#' latency portions of the fitted mixture cure model when using the minimum AIC
#' to select the final model. When fitting a model using \code{curegmifs} or
#' \code{cureem} the summary function additionally prints results associated
#' with the following model selection methods: the step and value that maximizes
#' the log-likelihood; the step and value that minimizes the AIC, modified AIC
#' (mAIC), corrected AIC (cAIC), BIC, modified BIC (mBIC), and extended BIC
#' (EBIC). This information can be used to guide the user in the selection of
#' a final model from the solution path.
#'
#' @export
#'
#' @seealso \code{\link[hdcuremodels]{curegmifs}}, \code{\link[hdcuremodels]{cureem}},
#' \code{\link[hdcuremodels]{coef.mixturecure}}, \code{\link[hdcuremodels]{plot.mixturecure}},
#' \code{\link[hdcuremodels]{predict.mixturecure}}
#' @keywords methods
#' @method summary mixturecure
#'
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {RE4.18} *Regression Software may also implement `summary` methods for model objects, and in particular should implement distinct `summary` methods for any cases in which calculation of summary statistics is computationally non-trivial (for example, for bootstrapped estimates of confidence intervals).*
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' fit <- curegmifs(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "weibull", thresh = 1e-4, maxit = 2000,
#'   epsilon = 0.01, verbose = FALSE
#' )
#' summary(fit)
summary.mixturecure <- function(object, ...) {
  if (!(class(object) %in% "mixturecure")) {
    stop("class of object must be mixturecure")
  }
  message("Mixture cure model fit using the ", object$method, " algorithm \n")
  if (object$cv == FALSE) {
    select <- select_model(object, model_select = "AIC")
    model_select_AIC <- which.min(select$AIC)
    model_select_cAIC <- which.min(select$cAIC)
    model_select_mAIC <- which.min(select$mAIC)
    model_select_BIC <- which.min(select$BIC)
    model_select_mBIC <- which.min(select$mBIC)
    model_select_EBIC <- which.min(select$EBIC)
    model_select_logLik <- which.max(select$logLik)
    message(
      "Number of non-zero incidence covariates at minimum AIC: ",
      sum(object$b_path[model_select_AIC,] != 0), "\n"
    )
    message(
      "Number of non-zero latency covariates at minimum AIC: ",
      sum(object$beta_path[model_select_AIC,] != 0), "\n"
    )
  if (object$cv == FALSE) {
    message("Optimal step for selected information criterion: ", object$method, " algorithm \n")
    message(
      "\t at step    = ", model_select_logLik, " logLik     = ",
      max(select$logLik), "\n"
    )
    message(
      "\t at step    = ", model_select_AIC, " AIC        = ",
      min(select$AIC), "\n"
    )
    message(
      "\t at step    = ", model_select_mAIC, " mAIC        = ",
      min(select$mAIC), "\n"
    )
    message(
      "\t at step    = ", model_select_cAIC, " cAIC        = ",
      min(select$cAIC), "\n"
    )
    message(
      "\t at step    = ", model_select_BIC, " BIC        = ",
      min(select$BIC), "\n"
    )
    message(
      "\t at step    = ", model_select_mBIC, " mBIC        = ",
      min(select$mBIC), "\n"
    )
    message(
      "\t at step    = ", model_select_EBIC, " EBIC        = ",
      min(select$EBIC), "\n"
    )
  }
  } else {
    message("using cross-validation \n")
    message(
      "Number of non-zero incidence covariates: ",
      sum(object$b != 0), "\n"
    )
    message(
      "Number of non-zero latency covariates: ",
      sum(object$beta != 0), "\n"
    )
  }
}

test_that("summary function works correctly", {
  library(survival)
  withr::local_seed(1234)
  temp <- generate_cure_data(n = 80, j = 100, n_true = 10, a = 1.8)
  training <- temp$training
  fit <- cureem(Surv(Time, Censor) ~ .,
                data = training, x_latency = training,
                model = "cox", penalty = "lasso", lambda_inc = 0.1,
                lambda_lat = 0.1, gamma_inc = 6, gamma_lat = 10
  )
  expect_invisible(summary(fit))
  expect_null(summary(fit))
  fit.cv <- cv_cureem(Surv(Time, Censor) ~ .,
                      data = training,
                      x_latency = training, fdr_control = FALSE,
                      grid_tuning = FALSE, nlambda_inc = 10, nlambda_lat = 10,
                      n_folds = 2, seed = 23, verbose = TRUE
  )
  expect_invisible(summary(fit.cv))
  expect_null(summary(fit.cv))

  fit.lm <- lm(Time ~ Censor, data = training)
  expect_error(summary.mixturecure(fit.lm))
})

#' Test for sufficient follow-up
#'
#' @description
#' Tests for sufficient follow-up using a Kaplan-Meier fitted object.
#'
#' @param object a \code{survfit} object.
#'
#' @return \item{p_value}{p-value from testing the null hypothesis that there
#' was not sufficient follow-up against the alternative that there was
#' sufficient follow-up}
#' @return \item{n_n}{total number of events that occurred at time >
#' pmax(0, 2*(last observed event time)-(last observed time)) and < the last
#' observed event time}
#' @return \item{N}{number of observations in the dataset}
#' @export
#'
#' @references Maller, R. A. and Zhou, X. (1996) \emph{Survival Analysis with
#' Long-Term Survivors}. John Wiley & Sons.
#'
#' @srrstats {G1.0} *Statistical Software should list at least one primary reference from published academic literature.*
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.0} *Implement assertions on lengths of inputs, particularly through asserting that inputs expected to be single- or multi-valued are indeed so.*
#' @srrstats {G2.1} *Implement assertions on types of inputs (see the initial point on nomenclature above).*
#' @srrstats {G5.2} *Appropriate error and warning behaviour of all functions should be explicitly demonstrated through tests. In particular,*
#' @srrstats {G5.2a} *Every message produced within R code by `stop()`, `warning()`, `message()`, or equivalent should be unique*
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @seealso \code{\link[survival]{survfit}}, \code{\link{cure_estimate}},
#' \code{\link{nonzerocure_test}}
#' @import survival
#' @import stats
#' @keywords htest
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' km_fit <- survfit(Surv(Time, Censor) ~ 1, data = training)
#' sufficient_fu_test(km_fit)
sufficient_fu_test <- function(object) {
  if (!(c("survfit") %in% class(object))) {
    stop("Error: object must be a survfit object")
  }
  if (length(object$strata) > 1) {
    object$strata.group <- rep(names(object$strata), object$strata)
    tn <- aggregate(object$time, by = list(object$strata.group), max)
    colnames(tn)[2] <- "tn"
    tnstar <- aggregate(object$time[object$n.event != 0],
      by = list(object$strata.group[object$n.event != 0]),
      max
    )
    colnames(tnstar)[2] <- "tnstar"
    max_surv <- merge(tn, tnstar, by = "Group.1")
    max_surv$delta <- max_surv$tn - max_surv$tnstar
    max_surv$lower <- 2 * max_surv$tnstar - max_surv$tn
    n_n <- q_n <- alpha_n <- numeric()
    for (i in seq_len(dim(max_surv)[1])) {
      events <- object$n.event[object$strata.group == names(object$strata)[i]]
      times <- object$time[object$strata.group == names(object$strata)[i]]
      n_n[i] <- sum(events[times > max_surv$lower[i] & times <=
        max_surv$tnstar[i]])
      q_n[i] <- n_n[i] / object$n[i]
      alpha_n[i] <- (1 - n_n[i] / object$n[i])^object$n[i]
    }
    output <- data.frame(alpha_n = alpha_n, n_n = n_n, q_n = q_n, n = object$n)
    rownames(output) <- names(object$strata)
  } else {
    tn <- max(object$time)
    tnstar <- max(object$time[object$n.event != 0])
    lower <- 2 * tnstar - tn
    n_n <- sum(object$n.event[object$time > lower & object$time <= tnstar])
    alpha_n <- (1 - n_n / object$n)^object$n
    q_n <- n_n / object$n
    output <- data.frame(p_value = alpha_n, n_n = n_n, N = object$n)
  }
  output
}

#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*

test_that("sufficient_fu_test works correctly", {
  library(survival)
  set.seed(1234)
  temp <- generate_cure_data(n = 200, j = 10, n_true = 10, a = 1.8)
  training <- temp$training
  training$group <- gl(2, 75)
  km_fit <- survfit(Surv(Time, Censor) ~ 1, data = training)
  output <- sufficient_fu_test(km_fit)
  expect_equal(output$n_n, 8)
  expect_equal(output$N, 150)
  output %>% expect_type("list")
  output %>% expect_length(3)
  km_fit_group <- survfit(Surv(Time, Censor) ~ group, data = training)
  output <- sufficient_fu_test(km_fit_group)
  expect_equal(output$n_n[1], 33)
  output %>% expect_type("list")
  output %>% expect_length(4)

  fit.lm <- lm(Time ~ Censor, data = training)
  expect_error(sufficient_fu_test(fit.lm), "Error: object must be a survfit object")
})

#' Fit penalized parametric mixture cure model using the GMIFS algorithm
#'
#' @description
#' Fits a penalized Weibull or exponential mixture cure model using the
#' generalized monotone incremental forward stagewise (GMIFS) algorithm (Hastie
#' et al 2007) and yields solution paths for parameters in the incidence and
#' latency portions of the model.
#'
#' @param formula an object of class "\code{formula}" (or one that can be
#' coerced to that class): a symbolic description of the model to be fitted. The
#' response must be a survival object as returned by the \code{Surv} function
#' while the variables on the right side of the formula are the covariates that
#' are included in the incidence portion of the model.
#' @param data a data.frame in which to interpret the variables named in the
#' \code{formula} or in the \code{subset} argument. Rows with missing data are
#' omitted (only \code{na.action = na.omit} is operational) therefore users may
#' want to impute missing data prior to calling this function.
#' @param subset an optional expression indicating which subset of observations
#' to be used in the fitting process, either a numeric or factor variable should
#' be used in subset, not a character variable. All observations are included by
#' default.
#' @param x_latency specifies the variables to be included in the latency
#' portion of the model and can be either a matrix of predictors, a model
#' formula with the right hand side specifying the latency variables, or the
#' same data.frame passed to the \code{data} parameter. Note that when using the
#' model formula syntax for \code{x_latency} it cannot handle
#' \code{x_latency = ~ .}.
#' @param model type of regression model to use for the latency portion of
#' mixture cure model. Can be "weibull" or "exponential"; default is "weibull".
#' @param penalty_factor_inc vector of binary indicators representing the
#' penalty to apply to each incidence coefficient: 0 implies no shrinkage and 1
#' implies shrinkage. If not supplied, 1 is applied to all incidence variables.
#' @param penalty_factor_lat vector of binary indicators representing the
#' penalty to apply to each latency coefficient: 0 implies no shrinkage and 1
#' implies shrinkage. If not supplied, 1 is applied to all latency variables.
#' @param epsilon small numeric value reflecting the incremental value used to
#' update a coefficient at a given step (default is 0.001).
#' @param thresh small numeric value. The iterative process stops when the
#' differences between successive expected penalized
#' log-likelihoods for both incidence and latency components are less than this
#' specified level of tolerance (default is 10^-5).
#' @param scale logical, if TRUE the predictors are centered and scaled.
#' @param maxit integer specifying the maximum number of steps to run in the
#' iterative algorithm (default is 10^4).
#' @param inits an optional list specifying the initial values as follows:
#' \itemize{
#' \item \code{itct} a numeric value for the unpenalized incidence
#' intercept.
#' \item \code{b_u} a numeric vector for the unpenalized incidence coefficients.
#' \item \code{beta_u} a numeric vector for unpenalized latency
#' coefficients.
#' \item \code{lambda} a numeric value for the rate parameter.
#' \item \code{alpha} a numeric value for the shape parameter
#' when \code{model = "weibull"}.
#' }
#' If not supplied or improperly supplied,
#' initialization is automatically provided by the function.
#' @param verbose logical, if TRUE running information is printed to the
#' console (default is FALSE).
#' @param suppress_warning logical, if TRUE, suppresses echoing the warning that
#' the maximum number of iterations was reached so that the algorithm may not
#' have converged. Instead, warning is returned as part of the output with this
#' message.
#' @param na.action this function requires complete data so \code{"na.omit"} is
#' invoked. Users can impute missing data as an alternative prior to model fitting.
#' @param ... additional arguments.
#'
#' @return \item{b_path}{Matrix representing the solution path of the
#' coefficients in the incidence portion of the model. Row is step and
#' column is variable.}
#' @return \item{beta_path}{Matrix representing the solution path of the
#' coefficients in the latency portion of the model. Row is step and column is
#' variable.}
#' @return \item{b0_path}{Vector representing the solution path of the intercept
#' in the incidence portion of the model.}
#' @return \item{rate_path}{Vector representing the solution path of the rate
#' parameter for the Weibull or exponential density in the latency portion of
#' the model.}
#' @return \item{logLik}{Vector representing the log-likelihood for each step
#' in the solution path.}
#' @return \item{x_incidence}{Matrix representing the design matrix of the
#' incidence predictors.}
#' @return \item{x_latency}{Matrix representing the design matrix of the latency
#' predictors.}
#' @return \item{y}{Vector representing the survival object response as returned
#' by the \code{Surv} function }
#' @return \item{model}{Character string indicating the type of regression model
#' used for the latency portion of mixture cure model ("weibull" or
#' "exponential").}
#' @return \item{scale}{Logical value indicating whether the predictors were
#' centered and scaled.}
#' @return \item{alpha_path}{Vector representing the solution path of the shape
#' parameter for the Weibull density in the latency portion of the model.}
#' @return \item{call}{the matched call.}
#' @return \item{warning}{message indicating whether the maximum number of
#' iterations was achieved which may indicate the model did not converge.}
#'
#' @references Fu, H., Nicolet, D., Mrozek, K., Stone, R. M., Eisfeld, A. K.,
#' Byrd, J. C., Archer, K. J. (2022) Controlled variable selection in Weibull
#' mixture cure models for high-dimensional data. \emph{Statistics in Medicine},
#' \bold{41}(22), 4340--4366.
#' @references Hastie, T., Taylor J., Tibshirani R., Walther G. (2007) Forward
#' stagewise regression and the monotone lasso. \emph{Electron J Stat},
#' \bold{1}:1--29.
#'
#' @srrstats {G1.0} *Statistical Software should list at least one primary reference from published academic literature.*
#' @srrstats {G1.1} *Statistical Software should document whether the algorithm(s) it implements are: The first implementation of a novel algorithm *
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G1.3} *All statistical terminology should be clarified and unambiguously defined.*
#' @srrstats {G1.5} *Software should include all code necessary to reproduce results which form the basis of performance claims made in associated publications.*
#' @srrstats {G2.0a} *Provide explicit secondary documentation of any expectations on lengths of inputs*
#' @srrstats {G2.1} *Implement assertions on types of inputs (see the initial point on nomenclature above).*
#' @srrstats {G2.3a} *Use `match.arg()` or equivalent where applicable to only permit expected values.*
#' @srrstats {G2.1a} *Provide explicit secondary documentation of expectations on data types of all vector inputs.*
#' @srrstats {G2.2} *Appropriately prohibit or restrict submission of multivariate input to parameters expected to be univariate.*
#' @srrstats {G2.3} *For univariate character input:*
#' @srrstats {G2.3b} *Either: use `tolower()` or equivalent to ensure input of character parameters is not case dependent; or explicitly document that parameters are strictly case-sensitive.*
#' @srrstats {G2.4} *Provide appropriate mechanisms to convert between different data types, potentially including:*
#' @srrstats {G2.4e} *explicit conversion from factor via `as...()` functions*
#' @srrstats {G2.10} *Software should ensure that extraction or filtering of single columns from tabular inputs should not presume any particular default behaviour, and should ensure all column-extraction operations behave consistently regardless of the class of tabular data used as input.*
#' @srrstats {G2.13} *Statistical Software should implement appropriate checks for missing data as part of initial pre-processing prior to passing data to analytic algorithms.*
#' @srrstats {G5.2} *Appropriate error and warning behaviour of all functions should be explicitly demonstrated through tests. In particular,*
#' @srrstats {G5.2a} *Every message produced within R code by `stop()`, `warning()`, `message()`, or equivalent should be unique*
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {G5.6} **Parameter recovery tests** *to test that the implementation produce expected results given data with known properties. For instance, a linear regression algorithm should return expected coefficient values for a simulated data set generated from a linear model.*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*
#' @srrstats {G5.9a} *Adding trivial noise (for example, at the scale of `.Machine$double.eps`) to data does not meaningfully change results*
#' @srrstats {RE1.0} *Regression Software should enable models to be specified via a formula interface, unless reasons for not doing so are explicitly documented.*
#' @srrstats {RE1.1} *Regression Software should document how formula interfaces are converted to matrix representations of input data.*
#' @srrstats {RE1.2} *Regression Software should document expected format (types or classes) for inputting predictor variables, including descriptions of types or classes which are not accepted.*
#' @srrstats {RE1.3} *Regression Software which passes or otherwise transforms aspects of input data onto output structures should ensure that those output structures retain all relevant aspects of input data, notably including row and column names, and potentially information from other `attributes()`.*
#' @srrstats {RE1.3a} *Where otherwise relevant information is not transferred, this should be explicitly documented.*
#' @srrstats {RE1.4} *Regression Software should document any assumptions made with regard to input data; for example distributional assumptions, or assumptions that predictor data have mean values of zero. Implications of violations of these assumptions should be both documented and tested.*
#' @srrstats {RE2.3} *Where applicable, Regression Software should enable data to be centred (for example, through converting to zero-mean equivalent values; or to z-scores) or offset (for example, to zero-intercept equivalent values) via additional parameters, with the effects of any such parameters clearly documented and tested.*
#' @srrstats {RE3.0} *Issue appropriate warnings or other diagnostic messages for models which fail to converge.*
#' @srrstats {RE3.2} *Ensure that convergence thresholds have sensible default values, demonstrated through explicit documentation.*
#' @srrstats {RE3.3} *Allow explicit setting of convergence thresholds, unless reasons against doing so are explicitly documented.*
#' @srrstats {RE4.0} *Regression Software should return some form of "model" object, generally through using or modifying existing class structures for model objects (such as `lm`, `glm`, or model objects from other packages), or creating a new class of model objects.*
#' @srrstats {RE4.4} *The specification of the model, generally as a formula (via `formula()`)*
#' @srrstats {RE4.7} *Where appropriate, convergence statistics*
#' @srrstats {RE4.8} *Response variables, and associated "metadata" where applicable.*
#' @seealso \code{\link[hdcuremodels]{cv_curegmifs}}
#' @export
#'
#' @import stats
#' @import survival
#' @importFrom methods is
#'
#' @keywords models
#' @keywords regression
#'
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#'
#' fit <- curegmifs(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "weibull", thresh = 1e-4, maxit = 2000, epsilon = 0.01,
#'   verbose = FALSE
#' )
curegmifs <- function(formula, data, subset, x_latency = NULL,
                      model = c("weibull", "exponential"),
                      penalty_factor_inc = NULL, penalty_factor_lat = NULL,
                      epsilon = 0.001, thresh = 1e-5, scale = TRUE, maxit = 1e4,
                      inits = NULL, verbose = TRUE, suppress_warning = FALSE,
                      na.action = na.omit, ...) {
  mf <- match.call(expand.dots = FALSE)
  cl <- match.call()
  m <- match(c("formula", "data", "subset", "na.action"), names(mf), 0L)
  if (m[1] == 0) stop("Error: A \"formula\" argument is required")
  mf <- mf[c(1L, m)]
  mf[[1L]] <- as.name("model.frame")
  if (missing(data)) {
    mf[["data"]] <- environment(formula)
  }
  if (missing(data)) {
    data <- environment(formula)
  }
  if (thresh < 0) {
    stop("Error: \"thresh\" must be non-negative")
  }
  if (epsilon < 0) {
    stop("Error: \"epsilon\" must be non-negative")
  }
  mf <- eval(mf, parent.frame())
  mt <- attr(mf, "terms")
  omitted <- attr(mf, "na.action")
  model <- tolower(model)
  model <- match.arg(model)
  y <- model.response(mf)
  event <- y[, 2]
  time <- y[, 1]
  x <- model.matrix(mt, mf)
  if (!is.null(x_latency)) {
    if (missing(subset)) {
      r <- rep(TRUE, dim(data)[1])
      r[omitted] <- FALSE
    } else {
      e <- substitute(subset)
      r <- eval(e, data)
      if (!is.logical(r)) {
        stop("Error: 'subset' must evaluate to logical")
      }
      r <- r & !is.na(r)
      r[omitted] <- FALSE
    }
    if ("character" %in% is(x_latency) || "numeric" %in% is(x_latency)) {
      nl <- as.list(seq_len(ncol(data)))
      names(nl) <- names(data)
      vars <- eval(substitute(x_latency), nl, parent.frame())
      x_latency <- data[r, vars, drop = FALSE]
      x_latency <- as.matrix(x_latency)
    } else if ("matrix" %in% is(x_latency) || "data.frame" %in% is(x_latency)) {
      text <- parse(text = cl)[[2]]
      survnames <- strsplit(as.character(text), ",")
      time_name <- substr(survnames[[2]][1], 6, nchar(survnames[[2]][1]))
      censor_name <- trimws(strsplit(survnames[[2]][2], ")")[[1]][1])
      x_latency <- x_latency[r, !(colnames(x_latency) %in%
                                    c(time_name, censor_name)), drop = FALSE]
      x_latency <- as.matrix(x_latency)
    } else if ("formula" %in% is(x_latency)) {
      x_latency <- model.matrix(update.formula(x_latency, new = ~ . - 1), data)
    }
  }
  x_inc <- x
  is_intercept <- grep("Intercept", dimnames(x_inc)[[2]])
  if (length(is_intercept) == 1) {
    x_inc <- x_inc[, -is_intercept, drop = FALSE]
  }
  x_lat <- x_latency
  if (nrow(x_inc) != nrow(x_lat) || nrow(x_lat) != length(time) ||
      length(time) != length(event)) {
    stop("Error: Input dimension mismatch")
  }
  if (is.data.frame(x_inc) || is.data.frame(x_lat)) {
    x_inc <- as.matrix(x_inc)
    x_lat <- as.matrix(x_lat)
  }
  if (is.na(match(model, c("weibull", "exponential")))) {
    stop("Error: Only 'weibull' or 'exponential' available for model
         parameter.")
  }
  if (is.null(penalty_factor_inc)) {
    penalty_factor_inc <- rep(1, ncol(x_inc))
  }
  if (is.null(penalty_factor_lat)) {
    penalty_factor_lat <- rep(1, ncol(x_lat))
  }
  if (any(!c(class(penalty_factor_inc), class(penalty_factor_lat)) %in% "numeric")) {
    stop("Error: Penalty factors specified in penalty_factor_inc and penalty_factor_lat
         must be numeric vectors comprised of 0 or 1")
  }
  if (any(!c(penalty_factor_inc, penalty_factor_lat) %in% c(0, 1))) {
    stop("Error: Penalty factors specified in penalty_factor_inc and penalty_factor_lat
         can only include 0 or 1")
  }
  if (!is.null(inits)) {
    inits <- inits_check(model,
                         N = length(time), penalty_factor_inc,
                         penalty_factor_lat, inits
    )
  }
  x_u <- self_scale(x_inc[, penalty_factor_inc == 0, drop = FALSE], scale)
  x_p <- self_scale(x_inc[, penalty_factor_inc == 1, drop = FALSE], scale)
  w_u <- self_scale(x_lat[, penalty_factor_lat == 0, drop = FALSE], scale)
  w_p <- self_scale(x_lat[, penalty_factor_lat == 1, drop = FALSE], scale)
  if (model == "exponential") {
    res <- exp_cure(
      x_u, x_p, w_u, w_p, time, event, epsilon, thresh, maxit,
      inits, verbose
    )
  } else if (model == "weibull") {
    res <- weibull.cure(
      x_u, x_p, w_u, w_p, time, event, epsilon, thresh, maxit,
      inits, verbose
    )
  }
  nstep <- length(res$logLikelihood)
  warning <- "No warnings"
  if (nstep == maxit) {
    warning <-
      "Warning: Maximum step of iterations achieved.
                           Algorithm may not converge."
    if (!suppress_warning)
      warning("Warning: Maximum step of iterations achieved.
                           Algorithm may not converge.")
  }
  b_path <- matrix(NA, nrow = nstep, ncol = ncol(x_inc))
  beta_path <- matrix(NA, nrow = nstep, ncol = ncol(x_lat))
  b_path[, penalty_factor_inc == 0] <- res$b_u_path
  b_path[, penalty_factor_inc == 1] <- res$b_p_path
  beta_path[, penalty_factor_lat == 0] <- res$beta_u_path
  beta_path[, penalty_factor_lat == 1] <- res$beta_p_path
  colnames(b_path) <- colnames(x_inc)
  colnames(beta_path) <- colnames(x_lat)
  output <- list(
    b_path = b_path, beta_path = beta_path,
    b0_path = res$itct_path, rate_path = res$lambda_path,
    logLik = res$logLikelihood, x_incidence = x_inc,
    x_latency = x_lat, y = y, model = model, scale = scale,
    method = "GMIFS", call = cl
  )
  if (model == "weibull") output$alpha_path <- res$alpha_path
  output$cv <- FALSE
  output$warning <- warning
  class(output) <- "mixturecure"
  return(output)
}

#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {G5.6} **Parameter recovery tests** *to test that the implementation produce expected results given data with known properties. For instance, a linear regression algorithm should return expected coefficient values for a simulated data set generated from a linear model.*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*

test_that("curegmifs works correctly", {
    # validate function output
    library(survival)
    withr::local_seed(1234)
    temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
    training <- temp$training
    fit <- curegmifs(Surv(Time, Censor) ~ .,
                   data = training, x_latency = training, scale = TRUE,
                   model = "weibull", thresh = 1e-4, maxit = 2000, epsilon = 0.01,
                   verbose = FALSE
    )
    expect_setequal(names(fit), c("b_path", "beta_path", "b0_path", "rate_path",
                               "logLik", "x_incidence", "x_latency", "y",
                               "model", "scale", "method", "call", "alpha_path",
                               "cv", "warning"))
    fit %>% expect_s3_class("mixturecure")
    fit$b0_path %>% expect_type("double")
    fit$b_path %>% expect_type("double")
    fit$beta_path %>% expect_type("double")
    fit$rate_path %>% expect_type("double")
    fit$logLik %>% expect_type("double")
    fit$x_incidence %>% expect_type("double")
    fit$x_latency %>% expect_type("double")
    fit$y %>% expect_type("double")
    fit$model %>% expect_type("character")
    fit$scale %>% expect_type("logical")
    fit$method %>% expect_type("character")
    fit$alpha_path %>% expect_type("double")
    fit$cv %>% expect_type("logical")
    expect_equal(round(fit$x_incidence[1, 1], 7), 0.1557627)
    expect_equal(round(fit$x_latency[1, 1], 7), 0.1557627)
    expect_equal(fit$model, "weibull")
    expect_equal(round(fit$alpha_path[1], 4), 0.5121)
    expect_equal(round(fit$b0_path[1], 4), 0.0597)
    expect_equal(fit$cv, FALSE)
    expect_equal(fit$method, "GMIFS")
    expect_equal(fit$scale, TRUE)
    expect_error(curegmifs(Time ~ ., data = training, x_latency = training))
    expect_error(curegmifs(Censor ~ ., data = training, x_latency = training))
    expect_equal(dim(fit$b_path)[2], dim(fit$x_incidence)[2])
    expect_equal(dim(fit$beta_path)[2], dim(fit$x_latency)[2])
    fit$b0_path %>% expect_length(dim(fit$beta_path)[1])
    fit$alpha_path %>% expect_length(length(fit$b0_path))
    fit$rate_path %>% expect_length(length(fit$b0_path))
    fit$logLik %>% expect_length(length(fit$b0_path))
    withr::local_seed(1234)
    temp <- generate_cure_data(n = 400, j = 10, n_true = 10, a = 1.8)
    training <- temp$training

    fit <- curegmifs(Surv(Time, Censor) ~ .,
                     data = training, x_latency = training, scale = TRUE,
                     model = "exponential", thresh = 1e-4, maxit = 2000, epsilon = 0.01,
                     verbose = FALSE
    )
    expect_setequal(names(fit), c("b_path", "beta_path", "b0_path", "rate_path",
                                  "logLik", "x_incidence", "x_latency", "y",
                                  "model", "scale", "method", "call", "cv", "warning" ))
    fit %>% expect_s3_class("mixturecure")
    fit$b0_path %>% expect_type("double")
    fit$b_path %>% expect_type("double")
    fit$beta_path %>% expect_type("double")
    fit$rate_path %>% expect_type("double")
    fit$logLik %>% expect_type("double")
    fit$x_incidence %>% expect_type("double")
    fit$x_latency %>% expect_type("double")
    fit$y %>% expect_type("double")
    fit$model %>% expect_type("character")
    fit$scale %>% expect_type("logical")
    fit$method %>% expect_type("character")
    fit$cv %>% expect_type("logical")
    expect_equal(round(fit$x_incidence[1, 1], 7), 0.1557627)
    expect_equal(round(fit$x_latency[1, 1], 7), 0.1557627)
    expect_equal(fit$model, "exponential")
    expect_equal(round(fit$b0_path[1], 4), 0.2396)
    expect_equal(fit$cv, FALSE)
    expect_equal(fit$method, "GMIFS")
    expect_equal(fit$scale, TRUE)
    expect_error(curegmifs(Time ~ ., data = training, x_latency = training))
    expect_error(curegmifs(Censor ~ ., data = training, x_latency = training))
    expect_equal(dim(fit$b_path)[2], dim(fit$x_incidence)[2])
    expect_equal(dim(fit$beta_path)[2], dim(fit$x_latency)[2])
    fit$b0_path %>% expect_length(dim(fit$beta_path)[1])
    fit$rate_path %>% expect_length(length(fit$b0_path))
    fit$logLik %>% expect_length(length(fit$b0_path))

    expect_error(curegmifs(Surv(Time, Censor) ~ .,
                     data = training, x_latency = training, scale = TRUE,
                     model = "exponential", thresh = -10, maxit = 2000, epsilon = 0.01,
                     verbose = FALSE))

    withr::local_seed(4)
    temp <- generate_cure_data(n = 200, j = 10, n_true = 10, a = 1.8)
    training <- temp$training
    expect_error(curegmifs(training$Time))
    training$group <- gl(2, 75)
    expect_error(curegmifs(Surv(Time, Censor) ~ .,
                           data = training, x_latency = training, thresh = -1))
    expect_error(curegmifs(Surv(Time, Censor) ~ .,
                           data = training, x_latency = training, epsilon = -1))
    expect_error(curegmifs(Surv(Time, Censor) ~ .,
                        data = training, x_latency = training,
                        subset = group))
    expect_error(curegmifs(Surv(Time, Censor) ~ .,
                        data = training, x_latency = testing))
    expect_error(curegmifs(Surv(Time, Censor) ~ .,
                        data = training, x_latency = training,
                        model = "cox"))
    expect_error(curegmifs(Surv(Time, Censor) ~ .,
                        data = training, x_latency = training,
                        penalty_factor_inc = penalty))

    withr::local_seed(17)
    temp <- generate_cure_data(n = 100, j = 15, n_true = 3, a = 1.8, rho = 0.2)
    training <- temp$training
    fit <- curegmifs(Surv(Time, Censor) ~ .,
                           data = training,
                           x_latency = training, fdr_control = FALSE,
                           maxit = 450, epsilon = 0.01, n_folds = 2,
                           seed = 23, verbose = TRUE, suppress_warning = TRUE
    )
    expect_equal(round(fit$b0_path[1], 4), 0.0962)
    expect_equal(round(fit$alpha_path[1], 7), 0.6571744)

})

test_that("plot.mixturecure function works correctly", {
  withr::local_seed(1234)
  temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
  training <- temp$training
  fit <- curegmifs(Surv(Time, Censor) ~ .,
                   data = training, x_latency = training,
                   model = "weibull", thresh = 1e-4, maxit = 2000,
                   epsilon = 0.01, verbose = FALSE
  )
  expect_invisible(plot(fit, xlab = "Step", ylab = "Log-likelihood",
                main = "trace"))
  expect_invisible(plot(fit, type = "AIC"))
  expect_invisible(plot(fit, type = "BIC"))
  expect_invisible(plot(fit, type = "logLik"))
  expect_invisible(plot(fit, type = "mAIC"))
  expect_invisible(plot(fit, type = "mBIC"))
  expect_invisible(plot(fit, type = "EBIC"))

  withr::local_seed(123)
  temp <- generate_cure_data(n = 100, j = 15, n_true = 3, a = 1.8, rho = 0.2)
  training <- temp$training
  fit.cv <- cv_curegmifs(Surv(Time, Censor) ~ .,
                         data = training,
                         x_latency = training, fdr_control = FALSE,
                         maxit = 450, epsilon = 0.01, n_folds = 2,
                         seed = 23, verbose = TRUE
  )
  expect_visible(plot(fit.cv))

  fit.cv.fdr <- cv_cureem(Surv(Time, Censor) ~ .,
                          data = training,
                          x_latency = training, model = "weibull", penalty = "lasso",
                          fdr_control = TRUE, grid_tuning = FALSE, nlambda_inc = 10,
                          nlambda_lat = 10, n_folds = 2, seed = 23, verbose = TRUE
  )
  expect_silent(plot(fit.cv.fdr))
  fit.em <- cureem(Surv(Time, Censor) ~ .,
                data = training, x_latency = training,
                model = "cox", penalty = "lasso", lambda_inc = 0.1,
                lambda_lat = 0.1, gamma_inc = 6, gamma_lat = 10
  )
  expect_silent(plot(fit.em))
})

test_that("logLik function works correctly", {
  library(survival)
  withr::local_seed(1234)
  temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
  training <- temp$training
  fit <- curegmifs(Surv(Time, Censor) ~ .,
                   data = training, x_latency = training,
                   model = "weibull", thresh = 1e-4, maxit = 2000,
                   epsilon = 0.01, verbose = FALSE
  )
  expect_equal(round(logLik(fit), 5), structure(-9.22893, df = 26))
  expect_equal(length(logLik(fit)), 1)
  expect_type(logLik(fit), "double")
})

#' @srrstats {G5.2b} *Explicit tests should demonstrate conditions which trigger every one of those messages, and should compare the result with expected values.*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*

test_that("concordance_mcm works correctly", {
  library(survival)
  withr::local_seed(234)
  temp <- generate_cure_data(n = 100, j = 10, n_true = 8, a = 2)
  training <- temp$training
  testing <- temp$testing
  fit <- curegmifs(Surv(Time, Censor) ~ .,
                   data = training, x_latency = training,
                   model = "weibull", thresh = 1e-4, maxit = 2000,
                   epsilon = 0.01, verbose = FALSE
  )
  expect_equal(round(concordance_mcm(fit, newdata = testing,
                                     model_select = "cAIC"), 7), 0.7320809)
  expect_equal(round(concordance_mcm(fit, cure_cutoff = 3,
                                     model_select = "cAIC"), 6), 0.883804)
  expect_equal(round(concordance_mcm(fit, model_select = "cAIC"), 7), 0.8830314)
  expect_equal(round(concordance_mcm(fit, model_select = "AIC"), 7), 0.8830314)
  expect_equal(round(concordance_mcm(fit, model_select = "mAIC"), 6), 0.662166)
  expect_equal(round(concordance_mcm(fit, model_select = "BIC"), 7), 0.8830314)
  expect_equal(round(concordance_mcm(fit, model_select = "mBIC"), 6), 0.662166)
  expect_equal(round(concordance_mcm(fit, model_select = "EBIC"), 7), 0.8581016)
  expect_equal(round(concordance_mcm(fit, model_select = "logLik"), 7), 0.8830314)
  expect_error(concordance_mcm(fit, model_select = "aic"))
  expect_error(concordance_mcm(fit, model_select = "bic"))
  expect_error(concordance_mcm("x"))
  expect_error(concordance_mcm(1))

  fit.lm <- lm(Time ~ Censor, data = training)
  expect_error(concordance_mcm(fit.lm), "Error: class of object must be mixturecure")

})

#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*

test_that("predict function works correctly", {
  library(survival)
  withr::local_seed(1234)
  temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
  training <- temp$training
  testing <- temp$testing
  fit <- curegmifs(Surv(Time, Censor) ~ .,
                   data = training, x_latency = training,
                   model = "weibull", thresh = 1e-4, maxit = 2000,
                   epsilon = 0.01, verbose = FALSE
  )
  predict_train <- predict(fit)
  predict_train$p_uncured %>% expect_type("double")
  predict_train$linear_latency %>% expect_type("double")
  predict_train$latency_risk %>% expect_type("character")
  expect_setequal(names(predict_train), c("p_uncured", "linear_latency",
                                       "latency_risk"))
  expect_equal(round(predict_train$p_uncured[1], 7), 0.13618)
  expect_equal(round(predict_train$linear_latency[1], 6), 1.347184)
  expect_equal(predict_train$latency_risk[1], "high risk")
  predict_test <- predict(fit, newdata = testing)
  predict_test$p_uncured %>% expect_type("double")
  predict_test$linear_latency %>% expect_type("double")
  predict_test$latency_risk %>% expect_type("character")
  fit.cv <- cv_cureem(Surv(Time, Censor) ~ .,
                      data = training,
                      x_latency = training, fdr_control = FALSE,
                      grid_tuning = FALSE, nlambda_inc = 10, nlambda_lat = 10,
                      n_folds = 2, seed = 23, verbose = TRUE
  )
  predict_train_cv <- predict(fit.cv)
  predict_train_cv$p_uncured %>% expect_type("double")
  predict_train_cv$linear_latency %>% expect_type("double")
  predict_train_cv$latency_risk %>% expect_type("character")

  fit.lm <- lm(Time ~ Censor, data = training)
  expect_error(predict.mixturecure(fit.lm), "Error: class of object must be mixturecure")
})

test_that("print works correctly", {
  library(survival)
  withr::local_seed(1234)
  temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
  training <- temp$training
  fit <- curegmifs(Surv(Time, Censor) ~ .,
                   data = training, x_latency = training,
                   model = "weibull", thresh = 1e-4, maxit = 2000,
                   epsilon = 0.01, verbose = FALSE
  )
  result <- print(fit)
  expect_true(is.list(result))
  result %>% expect_length(15)
  expect_equal(names(result), c("b_path", "beta_path", "b0_path", "rate_path",
                             "logLik", "x_incidence", "x_latency", "y",
                             "model", "scale", "method", "call",
                             "alpha_path", "cv", "warning" ))
  expect_equal(round(mean(result$alpha_path), 6), 0.978436)

  expect_warning(curegmifs(Surv(Time, Censor) ~ .,
                   data = training, x_latency = training,
                   model = "exponential", thresh = 1e-4, maxit = 2000,
                   epsilon = 0.01, verbose = FALSE
  ))

  fit.cv <- cv_cureem(Surv(Time, Censor) ~ .,
                      data = training,
                      x_latency = training, fdr_control = FALSE,
                      grid_tuning = FALSE, nlambda_inc = 10, nlambda_lat = 10,
                      n_folds = 2, seed = 23, verbose = TRUE
  )
  result <- print(fit.cv)
  expect_true(is.list(result))
  result %>% expect_length(18)
  expect_visible(result)

  fit.cv.fdr <- cv_cureem(Surv(Time, Censor) ~ .,
                          data = training,
                          x_latency = training, model = "weibull", penalty = "lasso",
                          fdr_control = TRUE, grid_tuning = FALSE, nlambda_inc = 10,
                          nlambda_lat = 10, n_folds = 2, seed = 23, verbose = TRUE
  )
  result <- print(fit.cv.fdr)
  expect_true(is.list(result))
  result %>% expect_length(17)
  expect_visible(result)

})

#' @srrstats {G2.3} *For univariate character input:*
#' @srrstats {G2.3b} *Either: use `tolower()` or equivalent to ensure input of character parameters is not case dependent; or explicitly document that parameters are strictly case-sensitive.*
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G1.4a} *All internal (non-exported) functions should also be documented in standard [`roxygen2`](https://roxygen2.r-lib.org/) format, along with a final `@noRd` tag to suppress automatic generation of `.Rd` files.*
#' @srrstats {G2.4} *Provide appropriate mechanisms to convert between different data types, potentially including:*
#' @srrstats {G2.4b} *explicit conversion to continuous via `as.numeric()`*
#' Optimization Function to Fit Cox LASSO Model Using the E-M Algorithm.
#'
#' @description
#' The cox_l1 function is the optimization algorithm for fitting a Cox
#' LASSO model using the EM algorithm and is called by cure.em.
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param mu_inc initial intercept for the incidence portion of the model.
#' @param mu_lat initial intercept for the latency portion of the model.
#' @param inits initial values used to optimization.
#' @param nIter maximum number of interations, default is 100.
#' @param tol small numeric value specifying convergence tolerance.
#'
#' @return \item{b_p_path}{Matrix representing the solution path of the
#' penalized coefficients in the incidence portion of the model. Row is step and
#' column is variable.}
#' @return \item{beta_p_path}{Matrix representing the solution path of the
#' penalized coefficients in the latency portion of the model. Row is step and
#' column is variable.}
#' @return \item{b_u_path}{Matrix representing the solution path of the
#' unpenalized coefficients in the incidence portion of the model. Row is step
#' and column is variable.}
#' @return \item{itct_path}{Vector representing the solution path of the
#' intercept for the incidence portion of the model.}
#' @return \item{beta_u_path}{Matrix representing the solution path of the
#' un penalized coefficients in the latency portion of the model. Row is step
#' and column is variable.}
#' @return \item{logLik_inc}{Vector representing the expected penalized
#' complete-data log-likelihood for the incidence portion of the model for
#' each step in the solution path.}
#' @return \item{logLik_lat}{Vector representing the expected penalized
#' complete-data log-likelihood for the latency portion of the model for each
#' step in the solution path.}
#' @noRd
cox_l1 <-
  function (X_u = NULL, X_p = NULL, W_u = NULL, W_p = NULL, time,
            delta, mu_inc, mu_lat, inits = NULL, nIter = 100, tol = 1e-04)
  {
    N <- length(time)
    J <- ncol(X_p)
    M <- ncol(W_p)
    CAP <- 10
    event_time <- time[delta == 1]
    uniq_event_time <- unique(event_time)
    n0 <- length(uniq_event_time)
    I0 <- matrix(time, ncol = 1) %*% matrix(1, 1, n0) >= matrix(1,
                                                               N, 1) %*% matrix(uniq_event_time, nrow = 1)
    d_j <- as.numeric(table(event_time)[rank(unique(event_time))])
    T_n0 <- max(event_time)
    tail_ind <- which(time >= T_n0 & delta == 0)
    step <- 1
    if (!is.null(X_p))
      b_p <- rep(0, J)
    if (!is.null(W_p))
      beta_p <- rep(0, M)
    if (is.null(inits))
      inits <- initialization_cox(X_u, W_u, time, delta)
    itct <- inits$itct
    b_u <- inits$b_u
    beta_u <- inits$beta_u
    survprob <- inits$survprob
    if (is.null(X_u))
      b_x <- rep(itct, N)
    else b_x <- itct + X_u %*% b_u
    if (is.null(X_u)) {
      pen_fac_inc <- rep(1, ncol(X_p))
    }
    else if (is.null(X_p)) {
      pen_fac_inc <- rep(0, ncol(X_u))
    }
    else {
      pen_fac_inc <- c(rep(0, ncol(X_u)), rep(1, ncol(X_p)))
    }
    if (is.null(W_u)) {
      pen_fac_lat <- rep(1, ncol(W_p))
    }
    else if (is.null(W_p)) {
      pen_fac_lat <- rep(0, ncol(W_u))
    }
    else {
      pen_fac_lat <- c(rep(0, ncol(W_u)), rep(1, ncol(W_p)))
    }
    pir <- rep(1, N)
    llp1_0 <- llp2_0 <- 0
    lik_inc <- lik_lat <- c()
    b_p_path <- beta_p_path <- b_u_path <- beta_u_path <- itct_path <- NULL
    conv1 <- conv2 <- FALSE
    repeat {
      pi_x <- 1/(1 + exp(-b_x))
      numerator <- pi_x[delta == 0] * survprob[delta == 0]
      pir[delta == 0] <- numerator/(1 - pi_x[delta == 0] + numerator)
      if (!conv1) {
        fit_inc <- glmnet::glmnet(x = cbind(X_u, X_p), y = cbind(1 -
                                                                  pir, pir), family = "binomial", penalty.factor = pen_fac_inc,
                                 lambda = mu_inc, standardize = FALSE)
        coef_inc <- coef(fit_inc)
        itct <- max(-CAP, min(CAP, coef_inc[1]))
        if (is.null(X_u)) {
          b_p <- pmax(-CAP, pmin(CAP, coef_inc[-1]))
          b_x <- itct + X_p[, b_p != 0, drop = FALSE] %*%
            b_p[b_p != 0]
        }
        else if (is.null(X_p)) {
          b_u <- pmax(-CAP, pmin(CAP, coef_inc[2:(ncol(X_u) +
                                                   1)]))
          b_x <- itct + X_u %*% b_u
        }
        else {
          b_u <- pmax(-CAP, pmin(CAP, coef_inc[2:(ncol(X_u) +
                                                   1)]))
          b_p <- pmax(-CAP, pmin(CAP, coef_inc[-(1:(ncol(X_u) +
                                                     1))]))
          b_x <- itct + X_u %*% b_u + X_p[, b_p != 0, drop = FALSE] %*%
            b_p[b_p != 0]
        }
        llp1_1 <- sum(pir * b_x - log(1 + exp(b_x))) - N *
          mu_inc * sum(abs(b_p))
      }
      lik_inc <- c(lik_inc, llp1_1)
      if (!conv2) {
        nz_pir <- which(pir > 0)
        fit_lat <- glmnet::glmnet(x = cbind(W_u, W_p)[nz_pir,
        ], y = Surv(time = time[nz_pir], event = delta[nz_pir]),
        family = "cox", offset = log(pir[nz_pir]), penalty.factor = pen_fac_lat,
        lambda = mu_lat, standardize = FALSE, cox.ties = "breslow")
        coef_lat <- coef(fit_lat)
        if (is.null(W_u)) {
          beta_p <- pmax(-CAP, pmin(CAP, as.numeric(coef_lat)))
          beta_w <- W_p[, beta_p != 0, drop = FALSE] %*%
            beta_p[beta_p != 0]
        }
        else if (is.null(W_p)) {
          beta_u <- pmax(-CAP, pmin(CAP, coef_lat[1:ncol(W_u)]))
          beta_w <- W_u %*% beta_u
        }
        else {
          beta_u <- pmax(-CAP, pmin(CAP, coef_lat[1:ncol(W_u)]))
          beta_p <- pmax(-CAP, pmin(CAP, coef_lat[-(1:ncol(W_u))]))
          beta_w <- W_u %*% beta_u + W_p[, beta_p != 0,
                                        drop = FALSE] %*% beta_p[beta_p != 0]
        }
        exp_beta_w_p <- exp(beta_w) * pir
        denom <- matrix(exp_beta_w_p, 1) %*% I0
        hazard <- rep(0, N)
        hazard[delta == 1] <- sapply(event_time, function(x) (d_j/denom)[uniq_event_time ==
                                                                          x])
        accum_hazard <- sapply(time, function(x) sum((d_j/denom)[uniq_event_time <=
                                                                  x]))
        surv_baseline <- exp(-accum_hazard)
        if (length(tail_ind) > 0) {
          wtail_alpha <- uniroot(function(a) sum(-exp_beta_w_p *
                                                  max(accum_hazard) * log(time/T_n0) * (time/T_n0)^a +
                                                  delta/a + delta * log(time/T_n0)), c(0.01,
                                                                                       10), extendInt = "yes")$root
          wtail_lambda <- (max(accum_hazard))^(1/wtail_alpha)/T_n0
          surv_baseline[tail_ind] <- exp(-(wtail_lambda *
                                            time[tail_ind])^wtail_alpha)
        }
        survprob <- surv_baseline^exp(beta_w)
        llp2_1 <- sum(log(hazard[delta == 1]) + beta_w[delta ==
                                                        1]) - sum(exp_beta_w_p * accum_hazard) - N *
          mu_lat * sum(abs(beta_p))
      }
      lik_lat <- c(lik_lat, llp2_1)
      itct_path <- c(itct_path, itct)
      if (!is.null(X_u))
        b_u_path <- rbind(b_u_path, b_u)
      if (!is.null(X_p))
        b_p_path <- rbind(b_p_path, b_p)
      if (!is.null(W_u))
        beta_u_path <- rbind(beta_u_path, beta_u)
      if (!is.null(W_p))
        beta_p_path <- rbind(beta_p_path, beta_p)
      if (!conv1 & abs(llp1_1 - llp1_0) < tol)
        conv1 <- TRUE
      if (!conv2 & abs(llp2_1 - llp2_0) < tol)
        conv2 <- TRUE
      if (step > 1 & (conv1 & conv2) | step >= nIter) {
        break
      }
      llp1_0 <- llp1_1
      llp2_0 <- llp2_1
      step <- 1 + step
    }
    output <- list(b_p_path = b_p_path, beta_p_path = beta_p_path,
                   b_u_path = b_u_path, itct_path = itct_path, beta_u_path = beta_u_path,
                   lik_inc = lik_inc, lik_lat = lik_lat)
    output
  }

#' Optimization Function for Fitting Cox MCP or SCAD Model Using the
#' E-M algorithm.
#'
#' @description
#' The cox_mcp_scad function is the optimization algorithm for fitting a Cox
#' LASSO model using the EM algorithm. This function is called by cureem.
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param penalty specifies either the MCP or SCAD penalty.
#' @param mu_inc initial intercept for the incidence portion of the model.
#' @param mu_lat initial intercept for the latency portion of the model.
#' @param gamma_inc penalty parameter for the incidence portion of the model.
#' @param gamma_lat penalty parameter for the latency portion of the model.
#' @param inits initial values used to optimization.
#' @param nIter maximum number of interations, default is 100.
#' @param tol small numeric value specifying convergence tolerance.
#'
#' @return \item{b_p_path}{Matrix representing the solution path of the
#' penalized coefficients in the incidence portion of the model. Row is step and
#' column is variable.}
#' @return \item{b_u_path}{Matrix representing the solution path of the
#' unpenalized coefficients in the incidence portion of the model. Row is step
#' and column is variable.}
#' @return \item{itct_path}{Vector representing the solution path of the
#' intercept for the incidence portion of the model.}
#' @return \item{beta_p_path}{Matrix representing the solution path of the
#' penalized coefficients in the latency portion of the model. Row is step and
#' column is variable.}
#' @return \item{beta_u_path}{Matrix representing the solution path of the
#' un penalized coefficients in the latency portion of the model. Row is step
#' and column is variable.}
#' @return \item{logLik_inc}{Vector representing the expected penalized
#' complete-data log-likelihood for the incidence portion of the model for
#' each step in the solution path.}
#' @return \item{logLik_lat}{Vector representing the expected penalized
#' complete-data log-likelihood for the latency portion of the model for each
#' step in the solution path.}
#' @noRd
cox_mcp_scad <-
  function (X_u = NULL, X_p = NULL, W_u = NULL, W_p = NULL, time,
            delta, penalty = "MCP", mu_inc, mu_lat, gamma_inc = 3, gamma_lat = 3,
            inits = NULL, nIter = 1000, tol = 1e-04)
  {
    N <- length(time)
    J <- ncol(X_p)
    M <- ncol(W_p)
    CAP <- 20
    event_time <- time[delta == 1]
    uniq_event_time <- unique(event_time)
    n0 <- length(uniq_event_time)
    I0 <- matrix(time, ncol = 1) %*% matrix(1, 1, n0) >= matrix(1,
                                                               N, 1) %*% matrix(uniq_event_time, nrow = 1)
    d_j <- as.numeric(table(event_time)[rank(uniq_event_time)])
    Z <- sapply(uniq_event_time, function(t) colSums(W_p[time ==
                                                          t & delta == 1, , drop = FALSE]))
    T_n0 <- max(event_time)
    tail_ind <- which(time >= T_n0 & delta == 0)
    step <- 1
    if (!is.null(X_p))
      b_p <- rep(0, J)
    if (!is.null(W_p))
      beta_p <- rep(0, M)
    if (is.null(inits))
      inits <- initialization_cox(X_u, W_u, time, delta)
    itct <- inits$itct
    b_u <- inits$b_u
    beta_u <- inits$beta_u
    survprob <- inits$survprob
    if (is.null(X_u))
      b_x <- rep(itct, N)
    else b_x <- itct + X_u %*% b_u
    if (is.null(W_u))
      beta_w <- rep(0, N)
    else beta_w <- W_u %*% beta_u
    pir <- rep(1, N)
    exp_beta_w_p <- exp(beta_w) * pir
    lik_inc <- lik_lat <- c()
    b_p_path <- beta_p_path <- b_u_path <- beta_u_path <- itct_path <- NULL
    llp1_0 <- llp2_0 <- 0
    conv1 <- conv2 <- FALSE
    repeat {
      pi_x <- 1/(1 + exp(pmin(CAP, -b_x)))
      numerator <- pi_x[delta == 0] * survprob[delta == 0]
      pir[delta == 0] = numerator/pmax(1e-50, 1 - pi_x[delta ==
                                                         0] + numerator)
      if (!conv1) {
        for (l in 1:J) {
          bl <- b_p[l]
          b_p[l] <- max(-CAP, min(CAP, mcp_scad_cd_upd_inc(penalty = penalty,
                                                          pir, pi_x, X_p[, l], bl, gamma_inc, mu_inc)))
          b_x <- b_x + X_p[, l, drop = FALSE] %*% (b_p[l] -
                                                    bl)
          pi_x <- 1/(1 + exp(pmin(CAP, -b_x)))
        }
        out_inc <- optim(par = c(itct, b_u), fn = mcp_scad_negloglik_inc,
                        gr = mcp_scad_gradient_inc, b_p = b_p, X_u = X_u,
                        X_p = X_p, pir = pir, CAP = CAP, method = "BFGS")
        itct <- max(-CAP, min(CAP, out_inc$par[1]))
        if (!is.null(X_u))
          b_u <- pmax(-CAP, pmin(CAP, out_inc$par[2:(ncol(X_u) +
                                                      1)]))
        pen <- ifelse(penalty == "MCP", mcp_penalty(b_p, gamma_inc,
                                                   mu_inc), scad_penalty(b_p, gamma_inc, mu_inc))
        llp1_1 <- -out_inc$value - N * pen
        if (is.null(X_u))
          b_x <- itct + X_p[, b_p != 0, drop = FALSE] %*%
          b_p[b_p != 0]
        else b_x <- itct + X_u %*% b_u + X_p[, b_p != 0, drop = FALSE] %*%
          b_p[b_p != 0]
      }
      lik_inc <- c(lik_inc, llp1_1)
      if (!conv2) {
        for (l in 1:M) {
          betal <- beta_p[l]
          beta_p[l] <- max(-CAP, min(CAP, mcp_scad_cd_upd_lat(penalty,
                                                             exp_beta_w_p, W_p[, l], betal, Z[l, ], I0,
                                                             d_j, gamma_lat, mu_lat)))
          change <- W_p[, l, drop = FALSE] %*% (beta_p[l] -
                                                 betal)
          beta_w <- beta_w + change
          exp_beta_w_p <- exp(pmin(CAP, beta_w)) * pir
          l = l + 1
        }
        if (!is.null(W_u)) {
          out_lat <- optim(par = beta_u, fn = mcp_scad_negloglik_lat,
                          gr = mcp_scad_gradient_lat, beta_p = beta_p,
                          W_u = W_u, W_p = W_p, delta = delta, pir = pir,
                          I0 = I0, d_j = d_j, CAP = CAP, method = "BFGS")
          beta_u <- pmax(-CAP, pmin(CAP, out_lat$par))
          beta_w <- W_u %*% beta_u + W_p[, beta_p != 0,
                                        drop = FALSE] %*% beta_p[beta_p != 0]
          exp_beta_w_p <- exp(pmin(CAP, beta_w)) * pir
        }
        denom <- matrix(exp_beta_w_p, 1) %*% I0
        hazard <- rep(0, N)
        hazard[delta == 1] <- sapply(event_time, function(x) (d_j/denom)[uniq_event_time ==
                                                                          x])
        accum_hazard <- sapply(time, function(x) sum((d_j/denom)[uniq_event_time <=
                                                                  x]))
        surv_baseline <- exp(-accum_hazard)
        if (length(tail_ind) > 0) {
          tmp <- try(wtail_alpha <- uniroot(function(a) sum(-exp_beta_w_p *
                                                              max(accum_hazard) * log(time/T_n0) * (time/T_n0)^a +
                                                              delta/a + delta * log(time/T_n0)), c(0.01,
                                                                                                   10), extendInt = "yes")$root, silent = TRUE)
          if (!inherits(tmp, "try-error")) {
            wtail_lambda <- (max(accum_hazard))^(1/wtail_alpha)/T_n0
            surv_baseline[tail_ind] <- exp(-(wtail_lambda *
                                              time[tail_ind])^wtail_alpha)
          }
        }
        survprob <- surv_baseline^exp(pmin(CAP, beta_w))
        pen <- ifelse(penalty == "MCP", mcp_penalty(beta_p,
                                                   gamma_lat, mu_lat), scad_penalty(beta_p, gamma_lat,
                                                                                    mu_lat))
        llp2_1 <- sum(log(hazard[delta == 1]) + beta_w[delta ==
                                                        1])
        -sum(exp_beta_w_p * accum_hazard) - N * pen
      }
      lik_lat <- c(lik_lat, llp2_1)
      itct_path <- c(itct_path, itct)
      if (!is.null(X_u))
        b_u_path <- rbind(b_u_path, b_u)
      b_p_path <- rbind(b_p_path, b_p)
      if (!is.null(W_u))
        beta_u_path <- rbind(beta_u_path, beta_u)
      beta_p_path <- rbind(beta_p_path, beta_p)
      if (!conv1 & abs(llp1_1 - llp1_0) < tol)
        conv1 <- TRUE
      if (!conv2 & abs(llp2_1 - llp2_0) < tol)
        conv2 <- TRUE
      if (step > 1 & (conv1 & conv2) | step >= nIter) {
        break
      }
      llp1_0 <- llp1_1
      llp2_0 <- llp2_1
      step <- 1 + step
    }
    output <- list(b_p_path = b_p_path, b_u_path = b_u_path,
                   itct_path = itct_path, beta_p_path = beta_p_path, beta_u_path = beta_u_path,
                   lik_inc = lik_inc, lik_lat = lik_lat)
    output
  }

#' Wrapper Function for Fitting Parametric or Semi-parametric
#' Penalized MCMs Using the E-M Algorithm.
#'
#' @description
#' The cure.em function structures the data and then fits the selected EM MCM
#' (Cox LASSO invokes cox_l1; Cox MCP and Cox SCAD invoke cox_mcp_scad, Weibull
#' LASSO invokes weib_EM, and exponential LASSO invokes exp_EM) which are  the
#' relevant optimization functions. This function is also called by cv.em.inner
#' and cv.em.nofdr when cv_cureem is used for cross-validation.
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param event event (delta = 1) and censoring (delta = 0) indicator.
#' @param penalty specifies either the MCP or SCAD penalty.
#' @param penalty.factor.inc vector with length equal to the number of penalized
#' incidence predictors where 0 indicates no penalty and 1 indicates a penalty
#' should be applied. If not specified, default is to penalize all incidence
#' predictors.
#' @param penalty.factor.lat vector with length equal to the number of penalized
#' latency predictors where 0 indicates no penalty and 1 indicates a penalty
#' should be applied. If not specified, default is to penalize all latency
#' predictors.
#' @param thresh small numeric value specifying convergence tolerance.
#' @param maxit maximum number of EM algorithm iterations.
#' @param lambda.inc LASSO penalty parameter for incidence portion of the model.
#' @param lambda.lat LASSO penalty for latency portion of the model.
#' @param gamma.inc penalty parameter for the incidence portion of the model
#' under SCAD or MCP penalty.
#' @param gamma.lat penalty parameter for the latency portion of the model
#' under SCAD or MCP penalty.
#' @param inits initial values used to optimization.
#'
#' @return \item{output}{List that includes the relevant coefficient paths and
#' additional parameter paths (alpha for Weibull, rate for Weibull and
#' exponential)}
#' @noRd
cure.em <-
  function (X_u = NULL, X_p, W_u = NULL, W_p, time, event, model = "cox",
            penalty = "lasso", penalty.factor.inc = rep(1, ncol(x.inc)),
            penalty.factor.lat = rep(1, ncol(x.lat)), thresh = 0.001,
            maxit = ifelse(penalty == "lasso", 100, 1000), lambda.inc = 0.1,
            lambda.lat = 0.1, gamma.inc = 3, gamma.lat = 3, inits = NULL)
  {
    x.inc <- cbind(X_u, X_p)
    x.lat <- cbind(W_u, W_p)
    if (is.null(dim(x.inc)) | is.null(dim(x.lat)))
      stop("Error: x.inc and x.lat should be matrices with 2 or more columns")
    if ((ncol(x.inc) <= 1) | (ncol(x.lat) <= 1))
      stop("Error: x.inc and x.lat should be matrices with 2 or more columns")
    if (nrow(x.inc) != nrow(x.lat) | nrow(x.lat) != length(time) |
        length(time) != length(event))
      stop("Error: Input dimension mismatch")
    if (is.data.frame(x.inc) | is.data.frame(x.lat)) {
      x.inc <- as.matrix(x.inc)
      x.lat <- as.matrix(x.lat)
    }
    if (!model %in% c("cox", "weibull", "exponential"))
      stop("Error: Only 'cox', 'weibull', 'exponential' available for 'model' parameter")
    if (!penalty %in% c("lasso", "MCP", "SCAD"))
      stop("Error: Only 'lasso', 'MCP', 'SCAD' available for 'penalty' parameter")
    if (any(!c(penalty.factor.inc, penalty.factor.lat) %in% c(0,
                                                              1)))
      stop("Error: Penalty factors can only contain 0 or 1")
    if (any(c(lambda.inc, lambda.lat, gamma.inc, gamma.lat) <=
            0))
      stop("Error: Penalty pamameters lambda and gamma should be positive")
    if (!is.null(inits))
      inits <- inits_check(model, N = length(time), penalty.factor.inc,
                          penalty.factor.lat, inits)
    if (model != "cox" & penalty != "lasso")
      penalty <- "lasso"
    if (model == "cox" & penalty == "lasso")
      fit <- cox_l1(X_u, X_p, W_u, W_p, time, event, lambda.inc,
                   lambda.lat, inits, maxit, thresh)
    else if (model == "cox" & penalty %in% c("MCP", "SCAD"))
      fit <- cox_mcp_scad(X_u, X_p, W_u, W_p, time, event, penalty,
                         lambda.inc, lambda.lat, gamma.inc, gamma.lat, inits,
                         maxit, thresh)
    else if (model == "weibull")
      fit <- weib_EM(X_u, X_p, W_u, W_p, time, event, lambda.inc,
                    lambda.lat, inits, maxit, thresh)
    else if (model == "exponential")
      fit <- exp_EM(X_u, X_p, W_u, W_p, time, event, lambda.inc,
                   lambda.lat, inits, maxit, thresh)
    model_select <- which.max(fit$lik_inc + fit$lik_lat)
    b <- rep(NA, ncol(x.inc))
    beta <- rep(NA, ncol(x.lat))
    b[penalty.factor.inc == 0] <- fit$b_u_path[model_select, ]
    b[penalty.factor.inc == 1] <- fit$b_p_path[model_select, ]
    beta[penalty.factor.lat == 0] <- fit$beta_u_path[model_select,
    ]
    beta[penalty.factor.lat == 1] <- fit$beta_p_path[model_select,
    ]
    output <- list(b0 = fit$itct_path[model_select], b = b, beta = beta)
    if (model %in% c("exponential", "weibull"))
      output$rate = fit$lambda_path[model_select]
    if (model == "weibull")
      output$alpha <- fit$alpha_path[model_select]
    output$logLik.inc <- fit$lik_inc[model_select]
    output$logLik.lat <- fit$lik_lat[model_select]
    return(output)
  }

#' Estimate Metrics (C-statistic and AUC) When Using Parallel Processing with
#' Cross-Validation and the EM Algorithm for Fitting a MCM.
#'
#' @description
#' When fitting the MCM using the EM algorithm, this function is called by
#' cv.em.nofdr and is used for cross-validation when parallel computing is used.
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param folds_i selected fold for the current parallel process.
#' @param k number of cross-validation folds.
#' @param model character, indicating the latency distribution, either "cox",
#' "exponential", or "weibull".
#' @param penalty character indicating the type of penalty to impose, either
#' "lasso", "MCP", or "SCAD".
#' @param thresh small numeric value specifying convergence tolerance.
#' @param nIter maximum number of interations.
#' @param grid.tuning logical, indicates whether penalty parameters for the
#' incidence and latency portions of the model are simultaneously examined.
#' @param tuning_sequence sequence of lambda penalty parameter values to examine.
#' @param gamma.inc when the penalty is "MCP" or "SCAD", the values for the
#' gamma penalty in the incidence portion of the model.
#' @param gamma.lat when the penalty is "MCP" or "SCAD", the values for the
#' gamma penalty in the latency portion of the model.
#' @param inits initial values used to optimization.
#' @param measure.inc character, indicating metric used for optimizing the model
#'  fit, either "c" for the MCM C-statistic or "auc" for the MCM AUC.
#' @param cure_cutoff numeric, value indicating at what time point MCM
#' C-statistic or MCM AUC should be calculating when optimizing.
#'
#' @return \item{AUC}{value of the MCM AUC at the indicated cure_cutoff.}
#' @return \item{Cstat}{value of the MCM C-statistic at the indicated cure_cutoff}
#' @noRd
cv.em.inner <-
  function (X_u, X_p, W_u, W_p, time, delta, folds_i, k, model = c("cox",
                                                                   "weibull", "exponential"), penalty = c("lasso", "MCP", "SCAD"),
            thresh = 1e-05, nIter = 10000, grid.tuning = FALSE, tuning_sequence = NULL,
            gamma.inc = 3, gamma.lat = 3, inits = NULL, measure.inc = c("c",
                                                                        "auc"), cure_cutoff = 5)
  {
    test_i <- which(folds_i == k)
    test_delta <- delta[test_i]
    test_time <- time[test_i]
    test_X_p <- X_p[test_i, , drop = FALSE]
    test_W_p <- W_p[test_i, , drop = FALSE]
    if (is.null(X_u)) {
      test_X_u <- NULL
      X_u_train <- NULL
      penalty.factor.inc <- rep(1, ncol(X_p))
    }
    else {
      test_X_u <- X_u[test_i, , drop = FALSE]
      X_u_train <- X_u[-test_i, , drop = FALSE]
      penalty.factor.inc <- rep(0:1, c(ncol(X_u), ncol(X_p)))
    }
    if (is.null(W_u)) {
      test_W_u <- NULL
      W_u_train <- NULL
      penalty.factor.lat <- rep(1, ncol(W_p))
    }
    else {
      test_W_u <- W_u[test_i, , drop = FALSE]
      W_u_train <- W_u[-test_i, , drop = FALSE]
      penalty.factor.lat <- rep(0:1, c(ncol(W_u), ncol(W_p)))
    }
    initial_val <- inits
    if (model == "cox")
      initial_val$survprob <- initial_val$survprob[-test_i]
    cst <- rep(NA, nrow(tuning_sequence))
    if (measure.inc == "auc")
      auc <- rep(NA, nrow(tuning_sequence))
    for (i in 1:nrow(tuning_sequence)) {
      lambda.inc <- tuning_sequence[i, 1]
      if (grid.tuning)
        lambda.lat <- tuning_sequence[i, 2]
      else lambda.lat <- lambda.inc
      train_out <- cure.em(X_u_train, X_p[-test_i, ], W_u_train,
                          W_p[-test_i, ], time[-test_i], delta[-test_i], model,
                          penalty, penalty.factor.inc, penalty.factor.lat,
                          thresh, nIter, lambda.inc, lambda.lat, gamma.inc,
                          gamma.lat, initial_val)
      if (is.null(X_u)) {
        b_u <- NULL
        b_p <- train_out$b
      }
      else {
        b_u <- train_out$b[1:ncol(X_u)]
        b_p <- train_out$b[(ncol(X_u) + 1):(length(train_out$b))]
      }
      if (is.null(W_u)) {
        beta_u <- NULL
        beta_p <- train_out$beta
      }
      else {
        beta_u <- train_out$beta[1:ncol(W_u)]
        beta_p <- train_out$beta[(ncol(W_u) + 1):(length(train_out$beta))]
      }
      cst[i] <- C.stat(cure_cutoff, b_u, train_out$b0, b_p,
                      beta_u, beta_p, test_delta, test_time, test_X_u,
                      test_X_p, test_W_u, test_W_p)
      if (measure.inc == "auc")
        auc[i] <- AUC_msi(cure_cutoff = cure_cutoff, b_u,
                         train_out$b0, b_p, test_delta, test_time, test_X_u,
                         test_X_p)
    }
    if (measure.inc == "auc") {
      res <- rbind(auc, cst)
      rownames(res) <- c("AUC", "Cstat")
    }
    else res <- cst
    return(res)
  }


#' Generate Knockoffs Within Cross-Validated MCM E-M Algorithm For False
#' Discovery Rate Control.
#'
#' @description
#' This function generates knockoffs as part of the cross-validated MCM fitting
#' procedure. Once the knockoffs are appended to the full dataset, this function
#' calls cv.em.fdr function which performs the optimization.
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param model character, indicating the latency distribution, either "cox",
#' "exponential", or "weibull".
#' @param penalty character indicating the type of penalty to impose, either
#' "lasso", "MCP", or "SCAD".
#' @param fdr numeric value between 0 and 1 indicating the FDR threshold to use
#' for variable selection.
#' @param thresh small numeric value specifying convergence tolerance.
#' @param nIter maximum number of interations.
#' @param penalty.factor.inc vector with length equal to the number of penalized
#' incidence predictors where 0 indicates no penalty and 1 indicates a penalty
#' should be applied. If not specified, default is to penalize all incidence
#' predictors.
#' @param penalty.factor.lat vector with length equal to the number of penalized
#' latency predictors where 0 indicates no penalty and 1 indicates a penalty
#' should be applied. If not specified, default is to penalize all latency
#' predictors.
#' @param grid.tuning logical, indicates whether penalty parameters for the
#' incidence and latency portions of the model are simultaneously examined.
#' @param lambda.inc.list sequence of lambda penalty parameter values for the
#' incidence portion of the model to examine.
#' @param lambda.lat.list sequence of lambda penalty parameter values for the
#' incidence portion of the model to examine.
#' @param nlambda.inc if grid.tuning is TRUE, the ten different values of the
#' lambda penalty parameter are examined for optimizing the indicence portion of
#' the model. Otherwise, 50 different values of the lambda
#' penalty parameter are examined for optimizing the indicence portion of the
#' model.
#' @param nlambda.lat if grid.tuning is TRUE, the ten different values of the
#' lambda penalty parameter are examined for optimizing the latency portion of
#' the model. Otherwise, 50 different values of the lambda
#' penalty parameter are examined for optimizing the latency portion of the
#' model.
#' @param lambda.min.ratio.inc numeric value in (0, 1) representing the smallest
#' value for the lambda penalty as a fraction of the largest lambda penalty to
#' use in the incidence portion of the model.
#' @param lambda.min.ratio.lat numeric value in (0, 1) representing the smallest
#' value for the lambda penalty as a fraction of the largest lambda penalty to
#' use in the latency portion of the model.
#' @param gamma.inc when the penalty is "MCP" or "SCAD", the values for the
#' gamma penalty in the incidence portion of the model.
#' @param gamma.lat when the penalty is "MCP" or "SCAD", the values for the
#' gamma penalty in the latency portion of the model.
#' @param inits initial values used to optimization.
#' @param n_folds number of folds to use in the cross-validation procedure.
#' @param measure.inc character, indicating metric used for optimizing the model
#'  fit, either "c" for the MCM C-statistic or "auc" for the MCM AUC.
#' @param one.se logical, if TRUE, identifies the optimal model as the model
#' within one SE of the selected and estimated measure.inc ("c" or "auc").
#' Allows selection of a more parsimonious model than that achieved when
#' maximizing "c" or "auc".
#' @param cure_cutoff numeric, value indicating at what time point MCM
#' C-statistic or MCM AUC should be calculating when optimizing.
#' @param parallel logical, if TRUE, parallel processing is used.
#' @param seed random seed to set to enhance reproducibility.
#' @param verbose logical, if TRUE displays step process to the console.
#'
#' @return \item{b0}{numeric, estimated intercept for the incidence portion of
#' the model.}
#' @return \item{b}{vector, estimated coefficients for the incidence portion of
#' the model.}
#' @return \item{beta}{vector, estimated coefficients for the latency portion of
#' the model.}
#' @return \item{rate}{numeric, estimated rate when fitting a Weibull or
#' exponential MCM.}
#' @return \item{alpha}{numeric, estimated alpha parameter when fitting a
#' Weibull MCM.}
#' @return \item{selected_b}{indicates which, if any, of the incidence
#' coefficients have an fdr less than the user-specified fdr threshold.}
#' @return \item{selected_beta}{indicates which, if any, of the latency
#' coefficients have an fdr less than the user-specified fdr threshold.}
#' @noRd
cv.em.fdr <-
  function (X_u, X_p, W_u, W_p, time, delta, model = c("weibull",
                                                       "exponential"), penalty = c("lasso", "MCP", "SCAD"), fdr = 0.2,
            thresh = 0.001, nIter = ifelse(penalty == "lasso", 100, 1000),
            penalty.factor.inc, penalty.factor.lat, grid.tuning = FALSE,
            lambda.inc.list = NULL, lambda.lat.list = NULL, nlambda.inc = ifelse(grid.tuning,
                                                                                 10, 50), nlambda.lat = ifelse(grid.tuning, 10, 50), lambda.min.ratio.inc = 0.1,
            lambda.min.ratio.lat = 0.1, gamma.inc = 3, gamma.lat = 3,
            inits = NULL, n_folds = 5, measure.inc = c("c", "auc"), one.se = FALSE,
            cure_cutoff = 5, parallel = FALSE, seed = NULL, verbose = TRUE)
  {
    if (!is.null(seed))
      withr::local_seed(seed)
    X_k <- knockoff::create.second_order(X_p, method = "asdp",
                                        shrink = T)
    W_k <- knockoff::create.second_order(W_p, method = "asdp",
                                        shrink = T)
    Xaug <- cbind(X_p, X_k)
    Waug <- cbind(W_p, W_k)
    fit <- cv.em.nofdr(X_u, Xaug, W_u, Waug, time, delta, model,
                      penalty, thresh, nIter, grid.tuning, lambda.inc.list,
                      lambda.lat.list, nlambda.inc, nlambda.lat, lambda.min.ratio.inc,
                      lambda.min.ratio.lat, gamma.inc, gamma.lat, inits, n_folds,
                      measure.inc, one.se, cure_cutoff, parallel, seed, verbose)
    if (is.null(X_u))
      b_p <- fit$b
    else {
      b_u <- fit$b[1:ncol(X_u)]
      b_p <- fit$b[(ncol(X_u) + 1):(length(fit$b))]
    }
    if (is.null(W_u))
      beta_p <- fit$beta
    else {
      beta_u <- fit$beta[1:ncol(W_u)]
      beta_p <- fit$beta[(ncol(W_u) + 1):(length(fit$beta))]
    }
    Z_b <- abs(b_p)
    Z_beta <- abs(beta_p)
    J <- ncol(X_p)
    M <- ncol(W_p)
    orig_b <- 1:J
    orig_beta <- 1:M
    W_b <- Z_b[orig_b] - Z_b[orig_b + J]
    W_beta <- Z_beta[orig_beta] - Z_beta[orig_beta + M]
    T_b <- knockoff::knockoff.threshold(W_b, fdr = fdr, offset = 1)
    T_beta <- knockoff::knockoff.threshold(W_beta, fdr = fdr,
                                          offset = 1)
    selected_b <- (1:J)[W_b > T_b]
    selected_beta <- (1:M)[W_beta > T_beta]
    if (identical(unname(selected_b), integer(0))) {
      b_p[1:J] <- 0
    }
    else {
      b_p[-selected_b] <- 0
    }
    if (identical(unname(selected_beta), integer(0))) {
      beta_p[1:M] <- 0
    }
    else {
      beta_p[-selected_beta] <- 0
    }
    if (is.null(X_u)) {
      b <- b_p[1:J]
    }
    else {
      b <- c(b_u, b_p[1:J])
    }
    if (is.null(W_u)) {
      beta <- beta_p[1:M]
    }
    else {
      beta <- c(beta_u, beta_p[1:M])
    }
    return(list(b0 = fit$b0, b = b, beta = beta, rate = fit$rate,
                alpha = fit$alpha, selected_b = selected_b, selected_beta = selected_beta))
  }

#' Cross-Validation Optimization Function for MCM Using the E-M Algorithm.
#'
#' @description
#' This function is the main optimization function for fitting a MCM using the
#' EM algorithm. This function is called by cv_cureem. It is also called by
#' cv.em.fdr as cv.em.fdr first generates knockoffs then uses
#' this function for optimization.
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param model character, indicating the latency distribution, either "cox",
#' "exponential", or "weibull".
#' @param penalty character indicating the type of penalty to impose, either
#' "lasso", "MCP", or "SCAD".
#' @param thresh small numeric value specifying convergence tolerance.
#' @param nIter maximum number of iterations.
#' @param grid.tuning logical, indicates whether penalty parameters for the
#' incidence and latency portions of the model are simultaneously examined.
#' @param lambda.inc.list sequence of lambda penalty parameter values for the
#' incidence portion of the model to examine.
#' @param lambda.lat.list sequence of lambda penalty parameter values for the
#' incidence portion of the model to examine.
#' @param nlambda.inc if grid.tuning is TRUE, the ten different values of the
#' lambda penalty parameter are examined for optimizing the indicence portion of
#' the model. Otherwise, 50 different values of the lambda
#' penalty parameter are examined for optimizing the indicence portion of the
#' model.
#' @param nlambda.lat if grid.tuning is TRUE, the ten different values of the
#' lambda penalty parameter are examined for optimizing the latency portion of
#' the model. Otherwise, 50 different values of the lambda
#' penalty parameter are examined for optimizing the latency portion of the
#' model.
#' @param lambda.min.ratio.inc numeric value in (0, 1) representing the smallest
#' value for the lambda penalty as a fraction of the largest lambda penalty to
#' use in the incidence portion of the model.
#' @param lambda.min.ratio.lat numeric value in (0, 1) representing the smallest
#' value for the lambda penalty as a fraction of the largest lambda penalty to
#' use in the latency portion of the model.
#' @param gamma.inc when the penalty is "MCP" or "SCAD", the values for the
#' gamma penalty in the incidence portion of the model.
#' @param gamma.lat when the penalty is "MCP" or "SCAD", the values for the
#' gamma penalty in the latency portion of the model.
#' @param inits initial values used to optimization.
#' @param n_folds number of folds to use in the cross-validation procedure.
#' @param measure.inc character, indicating metric used for optimizing the model
#'  fit, either "c" for the MCM C-statistic or "auc" for the MCM AUC.
#' @param one.se logical, if TRUE, identifies the optimal model as the model
#' within one SE of the selected and estimated measure.inc ("c" or "auc").
#' Allows selection of a more parsimonious model than that achieved when
#' maximizing "c" or "auc".
#' @param cure_cutoff numeric, value indicating at what time point MCM
#' C-statistic or MCM AUC should be calculating when optimizing.
#' @param parallel logical, if TRUE, parallel processing is used.
#' @param seed random seed to set to enhance reproducibility.
#' @param verbose logical, if TRUE displays step process to the console.
#'
#' @return list that includes the relevant coefficient paths and
#' additional parameter paths (alpha for Weibull, rate for Weibull and
#' exponential).
#' @noRd
cv.em.nofdr <-
  function (X_u, X_p, W_u, W_p, time, delta, model = c("cox", "weibull",
                                                       "exponential"), penalty = c("lasso", "MCP", "SCAD"), thresh = 0.001,
            nIter = ifelse(penalty == "lasso", 100, 1000), grid.tuning = FALSE,
            lambda.inc.list = NULL, lambda.lat.list = NULL, nlambda.inc = ifelse(grid.tuning,
                                                                                 10, 50), nlambda.lat = ifelse(grid.tuning, 10, 50), lambda.min.ratio.inc = 0.1,
            lambda.min.ratio.lat = 0.1, gamma.inc = 3, gamma.lat = 3,
            inits = NULL, n_folds = 5, measure.inc = c("c", "auc"), one.se = FALSE,
            cure_cutoff = 5, parallel = FALSE, seed = NULL, verbose)
  {
    if (!is.null(seed))
      withr::local_seed(seed)
    if (!grid.tuning) {
      #if ((!is.null(lambda.inc.list) & !is.null(lambda.lat.list) &
      #     !identical(lambda.inc.list, lambda.lat.list)) | (nlambda.inc !=
      #                                                      nlambda.lat) | (lambda.min.ratio.inc != lambda.min.ratio.lat))
      #  warning("Warning: Grid tuning is off. Same lambda sequence for incidence and latency was used.")
      if (length(lambda.inc.list) > length(lambda.lat.list))
        lambda.lat.list <- lambda.inc.list
      else lambda.inc.list <- lambda.lat.list
      nlambda.inc <- nlambda.lat <- max(nlambda.inc, nlambda.lat)
      lambda.min.ratio.inc <- lambda.min.ratio.lat <- min(lambda.min.ratio.inc,
                                                          lambda.min.ratio.lat)
    }
    N <- length(time)
    if (is.null(inits)) {
      if (model == "cox")
        inits <- initialization_cox(X_u, W_u, time, delta)
      else inits <- initialization_parametric(X_u, W_u, time,
                                             delta, model)
    }
    if (grid.tuning) {
      if (is.null(lambda.inc.list) | is.null(lambda.lat.list)) {
        pir <- rep(1, N)
        if (is.null(X_u))
          pi_x <- rep(mean(delta), N)
        else pi_x <- 1/(1 + exp(-inits$itct - X_u %*% inits$b_u))
        if (is.null(W_u))
          beta_w <- rep(0, N)
        else beta_w <- W_u %*% inits$beta_u
        if (model == "cox")
          survprob <- inits$survprob
        else if (model == "weibull")
          survprob <- exp(-(inits$lambda * time)^(inits$alpha) *
                           exp(beta_w))
        else if (model == "exponential")
          survprob <- exp(-inits$lambda * time * exp(beta_w))
        numerator = pi_x[delta == 0] * survprob[delta ==
                                                  0]
        pir[delta == 0] = numerator/(1 - pi_x[delta == 0] +
                                       numerator)
        if (is.null(lambda.inc.list)) {
          lambda.max.inc <- max(abs(matrix(pir - 0.5, 1,
                                          N) %*% X_p))/N
          lambda.min.inc <- lambda.max.inc * lambda.min.ratio.inc
          k1 = (0:(nlambda.inc - 1))/nlambda.inc
          lambda.inc.list <- lambda.max.inc * lambda.min.ratio.inc^k1
        }
        if (is.null(lambda.lat.list)) {
          nz_pir <- which(pir > 0)
          lambda.max.lat <- get_cox_lambda_max(x = cbind(W_u,
                                                        W_p)[nz_pir, ], time = time[nz_pir], event = delta[nz_pir],
                                              offset = log(pir[nz_pir]))
          lambda.min.lat <- lambda.max.lat * lambda.min.ratio.lat
          k2 <- (0:(nlambda.lat - 1))/nlambda.lat
          lambda.lat.list <- lambda.max.lat * lambda.min.ratio.lat^k2
        }
      }
      lambda.grid <- expand.grid(lambda.inc.list, lambda.lat.list)
    }
    else {
      if (is.null(lambda.inc.list)) {
        lambda_max <- max(colSums(abs(X_p)), colSums(abs(W_p)))/(2 *
                                                                  N)
        lambda_min <- lambda_max * lambda.min.ratio.inc
        k1 <- (0:(nlambda.inc - 1))/nlambda.inc
        lambda.list <- lambda_max * lambda.min.ratio.inc^k1
      }
      else lambda.list <- lambda.inc.list
    }
    if (grid.tuning)
      tuning_sequence <- lambda.grid
    else tuning_sequence <- matrix(lambda.list, ncol = 1)
    folds_i <- sample(rep(1:n_folds, length.out = N))
    Cstat <- matrix(NA, nrow(tuning_sequence), n_folds)
    if (measure.inc == "auc")
      AUC <- matrix(NA, nrow(tuning_sequence), n_folds)
    if (parallel) {
      ncores <- n_folds
      cl <- parallel::makeCluster(detectCores())
      doParallel::registerDoParallel(cl)
      res_list <- foreach(k = 1:n_folds) %dopar% {
        res <- cv.em.inner(X_u = X_u, X_p = X_p, W_u = W_u, W_p = W_p, time = time, delta = delta,
                          folds_i = folds_i, k = k, model = model, penalty = penalty, thresh = thresh, nIter = nIter, grid.tuning = grid.tuning,
                          tuning_sequence = tuning_sequence, gamma.inc = gamma.inc, gamma.lat = gamma.lat, inits = inits,
                          measure.inc = measure.inc, cure_cutoff = cure_cutoff)
        return(res)
      }
      parallel::stopCluster(cl)
      if (measure.inc == "auc") {
        for (k in 1:n_folds) {
          AUC[, k] <- res_list[[k]][1, ]
          Cstat[, k] <- res_list[[k]][2, ]
        }
      }
      else {
        for (k in 1:n_folds) Cstat[, k] <- res_list[[k]]
      }
    }
    else {
      for (k in 1:n_folds) {
        if (verbose)
          message("Fold ", k, " out of ", n_folds, " training...\n")
        res <- cv.em.inner(X_u = X_u, X_p = X_p, W_u = W_u, W_p = W_p, time = time, delta = delta,
                          folds_i = folds_i, k = k, model = model, penalty = penalty, thresh = thresh, nIter = nIter, grid.tuning = grid.tuning,
                          tuning_sequence = tuning_sequence, gamma.inc = gamma.inc, gamma.lat = gamma.lat, inits = inits,
                          measure.inc = measure.inc, cure_cutoff = cure_cutoff)
        if (measure.inc == "auc") {
          AUC[, k] <- res[1, ]
          Cstat[, k] <- res[2, ]
        }
        else Cstat[, k] <- res
      }
    }
    if (measure.inc == "auc") {
      auc <- rowMeans(AUC, na.rm = T)
      c_stat <- rowMeans(Cstat, na.rm = T)
      if (one.se) {
        auc_sd <- sqrt(apply(AUC, 1, var, na.rm = T) * (n_folds -
                                                         1)/n_folds/(nrow(X_p) - 1))
        c_stat_sd <- sqrt(apply(Cstat, 1, var, na.rm = T) *
                           (n_folds - 1)/n_folds/(nrow(X_p) - 1))
        opt_step_b <- which.max(auc)
        opt_step_beta <- which.max(c_stat)
        model_select_b <- which(auc >= (auc[opt_step_b] -
                                         auc_sd[opt_step_b]))[1]
        model_select_beta <- which(c_stat >= (c_stat[opt_step_beta] -
                                               c_stat_sd[opt_step_beta]))[1]
      }
      else {
        model_select_b <- which.max(auc)
        model_select_beta <- which.max(c_stat)
      }
    }
    else {
      c_stat <- rowMeans(Cstat, na.rm = T)
      if (one.se) {
        c_stat_sd <- sqrt(apply(Cstat, 1, var, na.rm = T) *
                           (n_folds - 1)/n_folds/(nrow(X_p) - 1))
        opt_step <- which.max(c_stat)
        model_select <- model_select_b <- model_select_beta <- which(c_stat >=
                                                                       (c_stat[opt_step] - c_stat_sd[opt_step]))[1]
      }
      else model_select <- model_select_b <- model_select_beta <- which.max(c_stat)
    }
    optimal_lambda_b <- tuning_sequence[model_select_b, 1]
    if (grid.tuning)
      optimal_lambda_beta <- tuning_sequence[model_select_beta,
                                            2]
    else optimal_lambda_beta <- tuning_sequence[model_select_beta,
                                               1]
    if (verbose) {
      message("Selected lambda for incidence: ", round(optimal_lambda_b,
                                                       3), "\n")
      message("Selected lambda for latency: ", round(optimal_lambda_beta,
                                                     3), "\n")
      message("Maximum C-statistic: ", max(c_stat), "\n")
      if (measure.inc == "auc")
        message("Maximum AUC: ", max(auc), "\n")
      if (verbose)
        message("Fitting a final model...\n")
    }
    if (is.null(X_u))
      penalty.factor.inc <- rep(1, ncol(X_p))
    else penalty.factor.inc <- rep(0:1, c(ncol(X_u), ncol(X_p)))
    if (is.null(W_u))
      penalty.factor.lat <- rep(1, ncol(W_p))
    else penalty.factor.lat <- rep(0:1, c(ncol(W_u), ncol(W_p)))
    output <- cure.em(X_u, X_p, W_u, W_p, time, delta, model,
                     penalty, penalty.factor.inc, penalty.factor.lat, thresh,
                     nIter, optimal_lambda_b, optimal_lambda_beta, gamma.inc,
                     gamma.lat, inits)
    output$selected_lambda_inc <- optimal_lambda_b
    output$selected_lambda_lat <- optimal_lambda_beta
    output$max_c <- max(c_stat)
    if (measure.inc == "auc")
      output$max.auc <- max(auc)
    return(output)
  }

#' Generate Knockoffs Within Cross-Validated MCM GMIFS Algorithm For False
#' Discovery Rate Control.
#'
#' @description
#' This function generates knockoffs as part of the cross-validated MCM fitting
#' procedure when using the GMIFS algorithm, so it is called by cv_curegmifs.
#' Once the knockoffs are appended to the full dataset, this function
#' calls cv.em.fdr function which performs the optimization.
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param model character, indicating the latency distribution, either
#' "exponential" or "weibull".
#' @param fdr numeric value between 0 and 1 indicating the FDR threshold to use
#' for variable selection.
#' @param thresh small numeric value specifying convergence tolerance.
#' @param nIter maximum number of iterations.
#' @param epsilon small numeric value used to update the regression coefficients.
#' @param inits initial values used to optimization.
#' @param n_folds number of folds to use in the cross-validation procedure.
#' @param measure.inc character, indicating metric used for optimizing the model
#'  fit, either "c" for the MCM C-statistic or "auc" for the MCM AUC.
#' @param one.se logical, if TRUE, identifies the optimal model as the model
#' within one SE of the selected and estimated measure.inc ("c" or "auc").
#' Allows selection of a more parsimonious model than that achieved when
#' maximizing "c" or "auc".
#' @param cure_cutoff numeric, value indicating at what time point MCM
#' C-statistic or MCM AUC should be calculating when optimizing.
#' @param parallel logical, if TRUE, parallel processing is used.
#' @param seed random seed to set to enhance reproducibility.
#' @param verbose logical, if TRUE displays step process to the console.
#'
#' @return list that includes the relevant coefficient paths and
#' additional parameter paths (alpha for Weibull, rate for Weibull and
#' exponential).
#'
#' @noRd
cv.gmifs.fdr <-
  function(X_u, X_p, W_u, W_p, time, delta, model = c("weibull", "exponential"),
           fdr = 0.2, thresh = 1e-5, nIter = 1e4, epsilon = 0.001, inits = NULL,
           n_folds = 5, measure.inc = c("c", "auc"), one.se = FALSE,
           cure_cutoff = 5, parallel = FALSE, seed = NULL, verbose = TRUE) {
    if (!is.null(seed)) withr::local_seed(seed)
    X_k <- knockoff::create.second_order(X_p, method = "asdp", shrink = T)
    W_k <- knockoff::create.second_order(W_p, method = "asdp", shrink = T)
    Xaug <- cbind(X_p, X_k)
    Waug <- cbind(W_p, W_k)
    fit <- cv.gmifs.nofdr(
      X_u, Xaug, W_u, Waug, time, delta, model, thresh, nIter, epsilon, inits,
      n_folds, measure.inc, one.se, cure_cutoff,
      parallel, seed, verbose
    )
    b_p <- fit$b_p
    b_u <- fit$b_u
    beta_p <- fit$beta_p
    beta_u <- fit$beta_u
    Z_b <- abs(fit$b_p)
    Z_beta <- abs(fit$beta_p)
    J <- ncol(X_p)
    M <- ncol(W_p)
    orig_b <- 1:J
    orig_beta <- 1:M
    W_b <- Z_b[orig_b] - Z_b[orig_b + J]
    W_beta <- Z_beta[orig_beta] - Z_beta[orig_beta + M]
    T_b <- knockoff::knockoff.threshold(W_b, fdr = fdr, offset = 1)
    T_beta <- knockoff::knockoff.threshold(W_beta, fdr = fdr, offset = 1)
    selected_b <- (1:J)[W_b > T_b]
    selected_beta <- (1:M)[W_beta > T_beta]
    # added b_u code above and this code to output b0, b, beta, rate, and alpha
    b_p <- b_p[1:J]
    beta_p <- beta_p[1:M]
    if (identical(unname(selected_b), integer(0))) {
      b_p[1:J] <- 0
    } else {
      b_p[-selected_b] <- 0
    }
    if (identical(unname(selected_beta), integer(0))) {
      beta_p[1:M] <- 0
    } else {
      beta_p[-selected_beta] <- 0
    }
    return(list(b0 = fit$b0, b_u = b_u, b_p = b_p, beta_u = beta_u,
                beta_p = beta_p, rate = fit$rate, alpha = fit$alpha,
                selected_b = selected_b, selected_beta = selected_beta))
  }

#' Estimate Metrics (C-statistic and AUC) When Using Parallel Processing with
#' Cross-Validation and the GMIFS Algorithm for Fitting a MCM.
#'
#' @description
#' Function called by cv.gmifs.nofdr for estimating AUC and C-statistics within
#' cross-validation procedure when using the GMIFS algorithm. Used with
#' foreach when parallel computing is used.
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param folds_i selected fold for the current parallel process.
#' @param k number of cross-validation folds.
#' @param model character, indicating the latency distribution, either
#' "exponential" or "weibull".
#' @param thresh small numeric value specifying convergence tolerance.
#' @param nIter maximum number of iterations.
#' @param epsilon small numeric value used to update the regression coefficients.
#' @param inits initial values used to optimization.
#' @param measure.inc character, indicating metric used for optimizing the model
#'  fit, either "c" for the MCM C-statistic or "auc" for the MCM AUC.
#' @param cure_cutoff numeric, value indicating at what time point MCM
#' C-statistic or MCM AUC should be calculating when optimizing.
#' @param verbose logical, if TRUE displays step process to the console.
#'
#' @return \item{AUC}{value of the MCM AUC at the indicated cure_cutoff.}
#' @return \item{Cstat}{value of the MCM C-statistic at the indicated
#' cure_cutoff.}
#' @noRd
cv.gmifs.inner <-
  function(X_u, X_p, W_u, W_p, time, delta, folds_i, k,
           model = c("weibull", "exponential"), thresh = 1e-5, nIter = 1e4,
           epsilon = 0.001, inits = NULL, measure.inc = c("c", "auc"),
           cure_cutoff = 5, verbose = TRUE) {
    test_i <- which(folds_i == k)
    test_delta <- delta[test_i]
    test_time <- time[test_i]
    test_X_p <- X_p[test_i, , drop = F]
    test_W_p <- W_p[test_i, , drop = F]
    if (is.null(X_u)) test_X_u <- NULL else test_X_u <- X_u[test_i, , drop = F]
    if (is.null(W_u)) test_W_u <- NULL else test_W_u <- W_u[test_i, , drop = F]
    if (is.null(X_u)) X_u_train <- NULL else X_u_train <- X_u[-test_i, , drop = F]
    if (is.null(W_u)) W_u_train <- NULL else W_u_train <- W_u[-test_i, , drop = F]
    if (model == "exponential") {
      train_out <- exp_cure(
        X_u_train, X_p[-test_i, ],
        W_u_train, W_p[-test_i, ],
        time[-test_i], delta[-test_i],
        epsilon, thresh, nIter, inits, verbose
      )
    } else if (model == "weibull") {
      train_out <- weibull.cure(
        X_u_train, X_p[-test_i, ],
        W_u_train, W_p[-test_i, ],
        time[-test_i], delta[-test_i],
        epsilon, thresh, nIter, inits, verbose
      )
    }
    cst <- sapply(
      1:length(train_out$itct_path),
      function(x) {
        if (is.null(X_u)) b_u <- NULL else b_u <- train_out$b_u_path[x, ]
        if (is.null(W_u)) beta_u <- NULL else beta_u <- train_out$beta_u_path[x, ]
        cs <- C.stat(
          cure_cutoff = cure_cutoff,
          b_u, train_out$itct_path[x], train_out$b_p_path[x, ],
          beta_u, train_out$beta_p_path[x, ],
          test_delta, test_time, test_X_u, test_X_p, test_W_u, test_W_p
        )
        return(cs)
      }
    )
    if (measure.inc == "auc") {
      auc <- sapply(
        1:length(train_out$itct_path),
        function(x) {
          if (is.null(X_u)) b_u <- NULL else b_u <- train_out$b_u_path[x, ]
          if (is.null(W_u)) beta_u <- NULL else beta_u <- train_out$beta_u_path[x, ]
          a <- AUC_msi(
            cure_cutoff = cure_cutoff, b_u, train_out$itct_path[x],
            train_out$b_p_path[x, ], test_delta, test_time, test_X_u, test_X_p
          )
          return(a)
        }
      )
      res <- rbind(c(auc, rep(NA, nIter - length(auc))), c(cst, rep(NA, nIter - length(cst))))
      rownames(res) <- c("AUC", "Cstat")
    } else {
      res <- c(cst, rep(NA, nIter - length(cst)))
    }
    return(res)
  }

#' Cross-Validation Optimization Function for MCM Using the GMIFS Algorithm.
#'
#' @description
#' Function for fitting a MCM using the GMIFS algorithm
#' Called by cv_gmifsem and cv.gmifs.fdr (latter first generates knockoffs then uses
#' this function for optimization)
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param model character, indicating the latency distribution, either
#' "exponential" or "weibull".
#' @param thresh small numeric value specifying convergence tolerance.
#' @param nIter maximum number of iterations.
#' @param epsilon small numeric value used to update the regression coefficients.
#' @param inits initial values used to optimization.
#' @param n_folds number of folds to use in the cross-validation procedure.
#' @param measure.inc character, indicating metric used for optimizing the model
#'  fit, either "c" for the MCM C-statistic or "auc" for the MCM AUC.
#' @param one.se logical, if TRUE, identifies the optimal model as the model
#' within one SE of the selected and estimated measure.inc ("c" or "auc").
#' Allows selection of a more parsimonious model than that achieved when
#' maximizing "c" or "auc".
#' @param cure_cutoff numeric, value indicating at what time point MCM
#' C-statistic or MCM AUC should be calculating when optimizing.
#' @param parallel logical, if TRUE, parallel processing is used.
#' @param seed random seed to set to enhance reproducibility.
#' @param verbose logical, if TRUE displays step process to the console.
#'
#' @return \item{model.select.inc}{which model in the incidence path is optimal.}
#' @return \item{model.select.lat}{which model in the latency path is optimal.}
#' @return \item{b0}{estimated intercept for the incidence portion of the model.}
#' @return \item{b_u}{vector of estimated unpenalized coefficients in the
#' incidence portion of the model.}
#' @return \item{b_p}{vector of estimated penalized coefficients in the
#' incidence portion of the model.}
#' @return \item{beta_u}{vector of estimated unpenalized coefficients in the
#' latency portion of the model.}
#' @return \item{beta_p}{vector of estimated penalized coefficients in the
#' latency portion of the model.}
#' @return \item{alpha}{estimated alpha parameter if fitting a Weibull MCM.}
#' @return \item{rate}{estimated rate parameter if fitting a Weibull or
#' exponential MCM.}
#' @return \item{logLik}{log-Likelihood for the selected model.}
#' @return \item{max.c}{C-statistic for the selected model.}
#' @return \item{max.auc}{AUC for the selected model.}
#' @noRd
cv.gmifs.nofdr <-
  function(X_u, X_p, W_u, W_p, time, delta, model = c("weibull", "exponential"),
           thresh = 1e-5, nIter = 1e4, epsilon = 0.001, inits = NULL,
           n_folds = 5, measure.inc = c("c", "auc"), one.se = FALSE,
           cure_cutoff = 5, parallel = FALSE, seed = NULL, verbose) {
    if (!is.null(seed)) withr::local_seed(seed)
    folds_i <- sample(rep(1:n_folds, length.out = length(time)))
    Cstat <- matrix(NA, nIter, n_folds)
    if (measure.inc == "auc") AUC <- matrix(NA, nIter, n_folds)
    if (parallel) { # parallel computing
      ncores <- n_folds
      # registerDoMC(ncores) # for doMC
      cl <- parallel::makeCluster(detectCores())
      doParallel::registerDoParallel(cl)
      res_list <- foreach(k = 1:n_folds) %dopar% {
        res <- cv.gmifs.inner(
          X_u, X_p, W_u, W_p, time, delta, folds_i, k,
          model, thresh, nIter, epsilon, inits, measure.inc, cure_cutoff, verbose
        )
        return(res)
      }
      parallel::stopCluster(cl)
      if (measure.inc == "auc") {
        for (k in 1:n_folds) {
          AUC[, k] <- res_list[[k]][1, ]
          Cstat[, k] <- res_list[[k]][2, ]
        }
      } else {
        for (k in 1:n_folds) Cstat[, k] <- res_list[[k]]
      }
    } else { # no parallel computing
      for (k in 1:n_folds) {
        if (verbose) message("Fold ", k, " out of ", n_folds, " training...\n")
        res <- cv.gmifs.inner(
          X_u, X_p, W_u, W_p, time, delta, folds_i, k,
          model, thresh, nIter, epsilon, inits, measure.inc, cure_cutoff, verbose
        )
        if (measure.inc == "auc") {
          AUC[, k] <- res[1, ]
          Cstat[, k] <- res[2, ]
        } else {
          Cstat[, k] <- res
        }
      }
    }
    if (measure.inc == "auc") { # use AUC to tune incidence and C-statistic to tune latency
      auc <- rowMeans(AUC, na.rm = T)
      c_stat <- rowMeans(Cstat, na.rm = T)
      if (one.se) {
        auc_sd <- sqrt(apply(AUC, 1, var, na.rm = T) * (n_folds - 1) / n_folds / (dim(X_p)[1] - 1))
        c_stat_sd <- sqrt(apply(Cstat, 1, var, na.rm = T) * (n_folds - 1) / n_folds / (dim(X_p)[1] - 1))
        opt_step_b <- which.max(auc)
        opt_step_beta <- which.max(c_stat)
        model_select_b <- which(auc >= (auc[opt_step_b] - auc_sd[opt_step_b]))[1]
        model_select_beta <- which(c_stat >= (c_stat[opt_step_beta] - c_stat_sd[opt_step_beta]))[1]
      } else {
        model_select_b <- which.max(auc)
        model_select_beta <- which.max(c_stat)
      }
      if (verbose) {
        message("Selected step for incidence: ", model_select_b, "\n")
        message("Selected step for latency: ", model_select_beta, "\n")
        message("Maximum AUC: ", max(auc, na.rm = T), "\n")
        message("Maximum C-statistic: ", max(c_stat, na.rm = T), "\n")
        message("Fitting a final model...\n")
      }
      if (model == "exponential") {
        fit <- exp_cure(X_u, X_p, W_u, W_p, time, delta, epsilon, thresh,
          nIter = max(model_select_b, model_select_beta), inits, verbose
        )
      } else if (model == "weibull") {
        fit <- weibull.cure(X_u, X_p, W_u, W_p, time, delta, epsilon, thresh,
          nIter = max(model_select_b, model_select_beta), inits, verbose
        )
      }
    } else { # use C-statistic to tune both components
      c_stat <- rowMeans(Cstat, na.rm = T)
      if (one.se) {
        c_stat_sd <- sqrt(apply(Cstat, 1, var, na.rm = T) * (n_folds - 1) / n_folds / (dim(X_p)[1] - 1))
        opt_step <- which.max(c_stat)
        model_select <- model_select_b <- model_select_beta <-
          which(c_stat >= (c_stat[opt_step] - c_stat_sd[opt_step]))[1]
      } else {
        model_select <- model_select_b <- model_select_beta <- which.max(c_stat)
      }
      if (verbose) {
        message("Selected step: ", model_select, "\n")
        message("Maximum C-statistic: ", max(c_stat, na.rm = T), "\n")
        message("Fitting a final model...\n")
      }
      if (model == "exponential") {
        fit <- exp_cure(X_u, X_p, W_u, W_p, time, delta, epsilon, thresh,
          nIter = model_select, inits,
          verbose
        )
      } else if (model == "weibull") {
        fit <- weibull.cure(X_u, X_p, W_u, W_p, time, delta, epsilon, thresh,
          nIter = model_select,
          inits, verbose
        )
      }
    }
    nstep <- length(fit$logLikelihood)
    b_p <- fit$b_p_path[nstep, ]
    b_u <- fit$b_u_path[nstep, ]
    b0 <- fit$itct_path[nstep]
    beta_p <- fit$beta_p_path[nstep, ]
    beta_u <- fit$beta_u_path[nstep, ]
    rate <- fit$lambda_path[nstep]
    if (model == "weibull") {
      alpha <- fit$alpha_path[nstep]
    } else {
      alpha <- 1
    }
    output <- list(
      model.select.inc = model_select_b, model.select.lat = model_select_beta,
      b0 = b0, b_u = b_u, b_p = b_p, beta_u = beta_u, beta_p = beta_p,
      alpha = alpha, rate = rate,
      logLik = fit$logLikelihood[nstep], max.c = max(c_stat, na.rm = T)
    )
    if (measure.inc == "auc") output$max.auc <- max(auc, na.rm = T)
    return(output)
  }

#' Optimization Function to Fit a Penalized Exponential MCM Using the
#' GMIFS Algorithm.
#'
#' @description
#' The exp_cure function is the optimization algorithm for fitting a
#' penalized exponential MCM using the GMIFS algorithm.
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param epsilon small numeric value used to update the regression coefficients.
#' @param tol small numeric value specifying convergence tolerance.
#' @param nIter maximum number of interations.
#' @param inits initial values used to optimization.
#' @param verbose logical, if TRUE displays step process to the console.
#'
#' @return \item{b_p_path}{Matrix representing the solution path of the
#' penalized coefficients in the incidence portion of the model. Row is step and
#' column is variable.}
#' @return \item{beta_p_path}{Matrix representing the solution path of the
#' penalized coefficients in the latency portion of the model. Row is step and
#' column is variable.}
#' @return \item{b_u_path}{Matrix representing the solution path of the
#' unpenalized coefficients in the incidence portion of the model. Row is step
#' and column is variable.}
#' @return \item{itct_path}{Vector representing the solution path of the
#' intercept for the incidence portion of the model.}
#' @return \item{beta_u_path}{Matrix representing the solution path of the
#' un penalized coefficients in the latency portion of the model. Row is step
#' and column is variable.}
#' @return \item{lambda_path}{Vector representing the solution path for the alpha
#' parameter in the Weibull MCM.}
#' @return \item{logLikelihood}{Vector representing the expected penalized
#' complete-data log-likelihood for each step in the solution path.}
#' @noRd
exp_cure <-
function(X_u = NULL, X_p, W_u = NULL, W_p, time, delta, epsilon = 0.001,
         tol = 1e-05, nIter = 1e4, inits = NULL, verbose) {
  # X_u: N by nonp, non-penalized covariate matrix associated with incidence
  # X_p: N by J, penalized covariate matrix associated with incidence
  # W_u: N by nonp, non-penalized covariate matrix associated with latency
  # W_p:  N by M, penalized covariate matrix associated with incidence
  # time: vector of length N, observed survival time
  # delta: vector of length N, censoring status (not censored = 0)
  # epsilon: incremental size
  # tol: difference between log-likelihood
  N <- length(time)
  J <- ncol(X_p) # number of penalized incidence covariates
  M <- ncol(W_p) # number of penalized latency covariates
  X_p <- cbind(X_p, -X_p) # N by 2J
  W_p <- cbind(W_p, -W_p) # N by 2M

  ######## initialization ########
  step <- 1
  b_p <- rep(0, 2 * J) # penalized incidence
  beta_p <- rep(0, 2 * M) # penalized latency
  if (is.null(inits)) inits <- initialization_parametric(X_u, W_u, time, delta,
                                                         model = "weibull")
  itct <- inits$itct
  b_u <- inits$b_u
  beta_u <- inits$beta_u
  lambda <- inits$lambda
  log_lambda <- log(max(lambda, 1e-15))
  LL0 <- 0

  b_p_path <- matrix(0, nrow = nIter, ncol = 2 * J)
  beta_p_path <- matrix(0, nrow = nIter, ncol = 2 * M)
  lambda_path <- itct_path <- logLikelihood <- rep(0, nIter)
  b_u_path <- matrix(0, nrow = nIter, ncol = length(b_u))
  beta_u_path <- matrix(0, nrow = nIter, ncol = length(beta_u))

  ######## loop ########

  repeat{
    #### update penalized parameters
    upd <- exp_update(lambda, b_p, beta_p, b_u, itct, beta_u, X_u, X_p, W_u,
                      W_p, time, delta, epsilon)
    b_p <- upd$b_p
    beta_p <- upd$beta_p
    b_p_path[step,] <- b_p
    beta_p_path[step,] <- beta_p

    #### update other parameters
    out <- optim(
      par = c(log_lambda, b_u, itct, beta_u), fn = exp_negloglik,
      gr = exp_gradient, b_p = b_p, beta_p = beta_p, X_u = X_u, X_p = X_p,
      W_u = W_u, W_p = W_p, time = time, delta = delta, method = "BFGS"
    )
    log_lambda <- out$par[1]
    lambda <- exp(log_lambda)
    if (!is.null(X_u)) {
      b_u <- out$par[2:(ncol(X_u) + 1)] # unpenalized incidence
      itct <- out$par[ncol(X_u) + 2]
      if (!is.null(W_u)) beta_u <- out$par[(ncol(X_u) + 3):(ncol(X_u) + ncol(W_u) + 2)] # unpenalized latency
    } else {
      itct <- out$par[2]
      if (!is.null(W_u)) beta_u <- out$par[3:(ncol(W_u) + 2)] # unpenalized latency
    }

    lambda_path[step] <- lambda
    if (!is.null(X_u)) b_u_path[step,] <- b_u
    itct_path[step] <- itct
    if (!is.null(W_u)) beta_u_path[step,] <- beta_u

    LL1 <- -out$value
    logLikelihood[step] <- LL1

    if (verbose & step %% 1000 == 0) message("step = ", step, "\n")
    if (step > 1 && (abs(LL1 - LL0) < tol | step >= nIter)) { #
      break
    }
    LL0 <- LL1
    step <- 1 + step
  }
  ######## output ########
  b_p_path <- b_p_path[, 1:J] - b_p_path[, (J + 1):(2 * J)]
  beta_p_path <- beta_p_path[, 1:M] - beta_p_path[, (M + 1):(2 * M)]
  if (step < nIter) {
    b_p_path <- b_p_path[1:step,]
    beta_p_path <- beta_p_path[1:step,]
    lambda_path <- lambda_path[1:step]
    b_u_path <- b_u_path[1:step,]
    itct_path <- itct_path[1:step]
    beta_u_path <- beta_u_path[1:step,]
    logLikelihood <- logLikelihood[1:step]
  }
  output <- list(
    b_p_path = b_p_path, beta_p_path = beta_p_path,
    lambda_path = lambda_path, b_u_path = b_u_path, itct_path = itct_path,
    beta_u_path = beta_u_path, logLikelihood = logLikelihood
  )
  output
}

#' Optimization Function to Fit a Penalized Exponential MCM Using the
#' E-M Algorithm.
#'
#' @description
#' The exp_EM function is the optimization algorithm for fitting a
#' penalized exponential MCM called by cureem (which calls internal cure.em
#' function).
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param mu_inc initial intercept for the incidence portion of the model.
#' @param mu_lat initial intercept for the latency portion of the model.
#' @param inits initial values used to optimization.
#' @param nIter maximum number of iterations.
#' @param tol small numeric value specifying convergence tolerance.
#'
#' @return \item{b_p_path}{Matrix representing the solution path of the
#' penalized coefficients in the incidence portion of the model. Row is step and
#' column is variable.}
#' @return \item{beta_p_path}{Matrix representing the solution path of the
#' penalized coefficients in the latency portion of the model. Row is step and
#' column is variable.}
#' @return \item{b_u_path}{Matrix representing the solution path of the
#' unpenalized coefficients in the incidence portion of the model. Row is step
#' and column is variable.}
#' @return \item{itct_path}{Vector representing the solution path of the
#' intercept for the incidence portion of the model.}
#' @return \item{beta_u_path}{Matrix representing the solution path of the
#' un penalized coefficients in the latency portion of the model. Row is step
#' and column is variable.}
#' @return \item{lambda_path}{Vector representing the solution path for the
#' lambda parameter in the exponential MCM.}
#' @return \item{lik_inc}{Vector representing the expected penalized
#' complete-data log-likelihood for each step in the solution path for the
#' incidence portion of the model.}
#' @return \item{lik_lat}{Vector representing the expected penalized
#' complete-data log-likelihood for each step in the solution path for the
#' latency portion of the model.}
#' @noRd
exp_EM <-
  function (X_u = NULL, X_p, W_u = NULL, W_p, time, delta, mu_inc,
            mu_lat, inits = NULL, nIter = 100, tol = 1e-04)
  {
    N <- length(time)
    J <- ncol(X_p)
    M <- ncol(W_p)
    step <- 1
    b_p <- rep(0, J)
    beta_p <- rep(0, M)
    b_p_ext <- rep(0, 2 * J)
    beta_p_ext <- rep(0, 2 * M)
    if (is.null(inits))
      inits <- initialization_parametric(X_u, W_u, time, delta,
                                        model = "weibull")
    itct <- inits$itct
    b_u <- inits$b_u
    beta_u <- inits$beta_u
    lambda <- inits$lambda
    log_lambda <- log(max(lambda, 1e-15))
    pir <- rep(1, N)
    llp1_0 <- llp2_0 <- 0
    lik_inc <- lik_lat <- c()
    b_p_path <- beta_p_path <- b_u_path <- beta_u_path <- itct_path <- lambda_path <- NULL
    conv1 <- conv2 <- FALSE
    repeat {
      if (is.null(X_u))
        b_x <- itct + X_p[, b_p != 0, drop = FALSE] %*% b_p[b_p !=
                                                             0]
      else b_x <- itct + X_u %*% b_u + X_p[, b_p != 0, drop = FALSE] %*%
          b_p[b_p != 0]
      if (is.null(W_u))
        beta_w <- W_p[, beta_p != 0, drop = FALSE] %*% beta_p[beta_p !=
                                                               0]
      else beta_w <- W_u %*% beta_u + W_p[, beta_p != 0, drop = FALSE] %*%
          beta_p[beta_p != 0]
      pir[delta == 0] <- 1/(1 + exp(-b_x[delta == 0] + lambda *
                                     time[delta == 0] * exp(beta_w[delta == 0])))
      if (!conv1) {
        b_p_ext <- optim(par = b_p_ext, fn = l1_negloglik_inc_b,
                        gr = l1_grad_b, itct = itct, b_u = b_u, X_u = X_u,
                        X_p = X_p, pir = pir, mu = mu_inc, method = "L-BFGS-B",
                        lower = rep(0, 2 * J))$par
        b_p <- b_p_ext[1:J] - b_p_ext[(J + 1):(2 * J)]
      }
      if (!conv2) {
        beta_p_ext <- optim(par = beta_p_ext, fn = exp_negloglik_lat_beta,
                           gr = exp_grad_beta, lambda = lambda, beta_u = beta_u,
                           W_u = W_u, W_p = W_p, time = time, delta = delta,
                           pir = pir, mu = mu_lat, method = "L-BFGS-B",
                           lower = rep(0, 2 * M))$par
        beta_p <- beta_p_ext[1:M] - beta_p_ext[(M + 1):(2 *
                                                         M)]
      }
      if (!conv1) {
        out_inc <- optim(par = c(itct, b_u), fn = l1_negloglik_inc,
                        gr = l1_gradient_inc, b_p = b_p, X_u = X_u, X_p = X_p,
                        pir = pir, mu = mu_inc, method = "BFGS")
        itct <- out_inc$par[1]
        if (!is.null(X_u))
          b_u <- out_inc$par[2:(ncol(X_u) + 1)]
        llp1_1 <- -out_inc$value
      }
      lik_inc <- c(lik_inc, llp1_1)
      if (!conv2) {
        out_lat <- optim(par = c(log_lambda, beta_u), fn = exp_negloglik_lat,
                        gr = exp_gradient_lat, beta_p = beta_p, W_u = W_u,
                        W_p = W_p, time = time, delta = delta, pir = pir,
                        mu = mu_lat, method = "BFGS")
        log_lambda <- out_lat$par[1]
        lambda <- exp(log_lambda)
        if (!is.null(W_u))
          beta_u <- out_lat$par[2:(ncol(W_u) + 1)]
        llp2_1 <- -out_lat$value
      }
      lik_lat <- c(lik_lat, llp2_1)
      itct_path <- c(itct_path, itct)
      if (!is.null(X_u))
        b_u_path <- rbind(b_u_path, b_u)
      b_p_path <- rbind(b_p_path, b_p)
      if (!is.null(W_u))
        beta_u_path <- rbind(beta_u_path, beta_u)
      beta_p_path <- rbind(beta_p_path, beta_p)
      lambda_path <- c(lambda_path, lambda)
      if (!conv1 & abs(llp1_1 - llp1_0) < tol)
        conv1 <- TRUE
      if (!conv2 & abs(llp2_1 - llp2_0) < tol)
        conv2 <- TRUE
      if (step > 1 & (conv1 & conv2) | step >= nIter) {
        break
      }
      llp1_0 <- llp1_1
      llp2_0 <- llp2_1
      step <- 1 + step
    }
    output <- list(b_p_path = b_p_path, b_u_path = b_u_path,
                   itct_path = itct_path, beta_p_path = beta_p_path, beta_u_path = beta_u_path,
                   lambda_path = lambda_path, lik_inc = lik_inc, lik_lat = lik_lat)
    output
  }


#' Gradient for Penalized Predictors in Latency Portion of Exponential MCM Using
#' the E-M Algorithm.
#'
#' @description
#' Gradient for optim when updating penalized predictors in the latency portion
#' when using E-M algorithm to fit an exponential MCM (called by exp_EM).
#'
#' @param theta numeric vector for the penalized predictors in the
#' exponential latency distribution.
#' @param lambda estimated lambda parameter for exponential latency distribution.
#' @param beta_u vector of estimated unpenalized latency coefficients.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param mu initial intercept for the latency portion of the model.
#'
#' @return value of the gradient.
#' @noRd
exp_grad_beta <-
  function(theta, lambda, beta_u, W_u, W_p, time, delta, pir, mu) {
    N <- dim(W_p)[1]
    M <- ncol(W_p)
    beta_p_ext <- theta
    beta_p <- beta_p_ext[1:M] - beta_p_ext[(M + 1):(2 * M)]
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(W_u)) {
      betaw <- W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    } else {
      betaw <- W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    }
    temp1 <- pir * lambda * time * exp(betaw)
    grad <- matrix(delta - temp1, 1) %*% W_p
    return(c(-grad + N * mu, grad + N * mu))
  }

#' Gradient for Unpenalized Predictors in Latency Portion of Exponential MCM
#' Using the E-M Algorithm.
#'
#' @description
#' Gradient for optim when updating unpenalized predictors in the latency
#' portion when using E-M algorithm for an exponential MCM (called by exp_EM).
#'
#' @param theta numeric vector for the unpenalized predictors in the
#' exponential latency distribution.
#' @param beta_p vector of estimated penalized latency coefficients.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param mu initial intercept for the latency portion of the model.
#'
#' @return value of the gradient.
#' @noRd
exp_gradient_lat <-
  function(theta, beta_p, W_u, W_p, time, delta, pir, mu) { # latency
    log_lambda <- theta[1]
    lambda <- exp(log_lambda)
    if (!is.null(W_u)) beta_u <- theta[2:(ncol(W_u) + 1)]
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(W_u)) {
      betaw <- W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    } else {
      betaw <- W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    }
    temp1 <- pir * lambda * time * exp(betaw)
    temp2 <- delta - temp1
    grad2 <- sum(temp2)
    if (!is.null(W_u)) grad3 <- matrix(temp2, 1) %*% W_u else grad3 <- NULL
    return(-c(grad2, grad3))
  }

#' Gradient for Unpenalized Predictors in Latency Portion of Exponential MCM
#' Using the GMIFS Algorithm.
#'
#' @description
#' Gradient for optim when updating unpenalized predictors in an
#' exponential MCM when fit using GMIFS (called by exp_cure).
#'
#' @param theta numeric vector for the unpenalized predictors in the
#' exponential latency distribution.
#' @param b_p vector of estimated penalized incidence coefficients.
#' @param beta_p vector of estimated penalized latency coefficients.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#'
#' @return value of the gradient.
#' @noRd
exp_gradient <-
  function(theta, b_p, beta_p, X_u, X_p, W_u, W_p, time, delta) {
    N <- length(time)
    lambda <- exp(theta[1])
    if (!is.null(X_u)) {
      b_u <- theta[2:(ncol(X_u) + 1)] # unpenalized incidence
      itct <- theta[ncol(X_u) + 2]
      if (!is.null(W_u)) beta_u <- theta[(ncol(X_u) + 3):(ncol(X_u) + ncol(W_u) + 2)] # unpenalized latency
    } else {
      itct <- theta[2]
      if (!is.null(W_u)) beta_u <- theta[3:(ncol(W_u) + 2)] # unpenalized latency
    }
    b_nonzero <- which(b_p != 0)
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(X_u)) {
      C_b <- exp(itct + X_u %*% b_u + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero])
    } else {
      C_b <- exp(itct + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero])
    }
    if (!is.null(W_u)) {
      C_beta <- exp(W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero])
    } else {
      C_beta <- exp(W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero])
    }
    C_lbeta <- exp(-lambda * time * C_beta)
    temp1 <- log(pmax(lambda * time, rep(1e-100, N)))
    temp2 <- lambda * time * C_beta
    temp3 <- 1 / (1 + C_b * C_lbeta)
    temp4 <- (1 - delta) * C_b * C_lbeta * temp2 * temp3
    temp5 <- delta * (1 - temp2)
    temp6 <- 1 / (1 + C_b)
    temp7 <- temp6 * (delta - (1 - delta) * C_b * (1 - C_lbeta) * temp3)
    grad2 <- sum(temp5 - temp4)
    if (!is.null(X_u)) grad3 <- matrix(temp7, 1) %*% X_u else grad3 <- NULL
    grad4 <- sum(temp7) # intercept
    if (!is.null(W_u)) grad5 <- matrix(temp5 - temp4, 1) %*% W_u else grad5 <- NULL
    return(-c(grad2, grad3, grad4, grad5))
  }

#' Gradient for Penalized Predictors in Latency Portion of Exponential MCM
#' Using the GMIFS Algorithm.
#'
#' @description
#' Function for optim when updating penalized predictors in the latency portion
#' when using EM algorithm for an exponential MCM (called by exp_EM).
#'
#' @param theta numeric vector for the penalized predictors in the
#' exponential latency distribution.
#' @param lambda estimated lambda parameter for exponential latency distribution.
#' @param beta_u vector of estimated unpenalized latency coefficients.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param mu initial intercept for the latency portion of the model.
#'
#' @return value of the negative log-likelihood.
#' @noRd
exp_negloglik_lat_beta <-
  function(theta, lambda, beta_u, W_u, W_p, time, delta, pir, mu) { # latency
    N <- dim(W_p)[1]
    M <- ncol(W_p)
    beta_p_ext <- theta
    beta_p <- beta_p_ext[1:M] - beta_p_ext[(M + 1):(2 * M)]
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(W_u)) {
      betaw <- W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    } else {
      betaw <- W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    }
    ll1 <- delta * (log(lambda) + betaw)
    ll2 <- -pir * lambda * time * exp(betaw)
    return(-sum(ll1 + ll2) + N * mu * sum(beta_p_ext))
  }

#


#' Negative Log-likelihood for Unpenalized Predictors in Latency Portion of
#' Exponential MCM Using the E-M Algorithm.
#'
#' @description
#' Function for calculating negative log-likelihood passed to optim when
#' updating unpenalized predictors in the latency portion of the model
#' using E-M algorithm for an exponential MCM (called by exp_EM).
#'
#' @param theta numeric vector for the unpenalized predictors in the
#' exponential latency distribution.
#' @param beta_p vector of estimated penalized latency coefficients.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param mu initial intercept for the latency portion of the model.
#'
#' @return value of the negative log-likelihood.
#' @noRd
exp_negloglik_lat <-
  function(theta, beta_p, W_u, W_p, time, delta, pir, mu) { # latency
    N <- dim(W_p)[1]
    log_lambda <- theta[1]
    lambda <- exp(log_lambda)
    if (!is.null(W_u)) beta_u <- theta[2:(ncol(W_u) + 1)]
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(W_u)) {
      betaw <- W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    } else {
      betaw <- W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    }
    ll1 <- delta * (log_lambda + betaw)
    ll2 <- -pir * lambda * time * exp(betaw)
    return(-sum(ll1 + ll2) + N * mu * sum(abs(beta_p)))
  }


#' Negative Log-likelihood for Unpenalized Predictors in Latency Portion of
#' Exponential MCM Using the GMIFS Algorithm.
#'
#' @description
#' Function for calculating negative log-likelihood passed to optim when
#' updating unpenalized predictors in the latency portion of the model
#' using GMIFS algorithm for an exponential MCM (called by exp_cure).
#'
#' @param theta numeric vector for the unpenalized predictors in the
#' exponential latency distribution.
#' @param b_p vector of estimated penalized incidence coefficients.
#' @param beta_p vector of estimated penalized latency coefficients.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#'
#' @return value of the negative log-likelihood.
#' @noRd
exp_negloglik <-
  function(theta, b_p, beta_p, X_u, X_p, W_u, W_p, time, delta) {
    N <- length(time)
    lambda <- exp(theta[1])
    if (!is.null(X_u)) {
      b_u <- theta[2:(ncol(X_u) + 1)] # unpenalized incidence
      itct <- theta[ncol(X_u) + 2]
      if (!is.null(W_u)) beta_u <- theta[(ncol(X_u) + 3):(ncol(X_u) + ncol(W_u) + 2)] # unpenalized latency
    } else {
      itct <- theta[2]
      if (!is.null(W_u)) beta_u <- theta[3:(ncol(W_u) + 2)] # unpenalized latency
    }
    b_nonzero <- which(b_p != 0)
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(X_u)) {
      logC_b <- itct + X_u %*% b_u + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    } else {
      logC_b <- itct + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    }
    if (!is.null(W_u)) {
      logC_beta <- W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    } else {
      logC_beta <- W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    }
    temp1 <- 1 / (1 + exp(-logC_b)) # exp(logC_b) / (1+exp(logC_b)) #Mar17
    logC_lbeta <- -lambda * time * exp(logC_beta)
    ll1 <- delta * (log(temp1) + theta[1] + logC_beta + logC_lbeta)
    ll2 <- (1 - delta) * log(pmax(1 - temp1 * (1 - exp(logC_lbeta)), rep(1e-100, N)))
    return(-sum(ll1 + ll2))
  }


#' Identify GMIFS Coefficient Update for Exponential MCM.
#'
#' @description
#' Function for identifying which coefficient in the incidence and latency
#' portions of the exponential MCM to update when using GMIFS, called by
#' exp_cure.
#'
#' @param lambda estimated lambda parameter for exponential latency distribution.
#' @param b_p vector of estimated penalized incidence coefficients.
#' @param beta_p vector of estimated penalized latency coefficients.
#' @param b_u vector of estimated unpenalized incidence coefficients.
#' @param itct estimated intercept for incidence portion of the model.
#' @param beta_u vector of estimated unpenalized latency coefficients.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param epsilon small numeric value used to update the regression coefficients.
#'
#' @return \item{b_p}{penalized incidence coefficient to update.}
#' @return \item{beta_p}{penalized latency coefficient to update.}
#' @noRd
exp_update <-
  function(lambda, b_p, beta_p, b_u, itct, beta_u, X_u, X_p, W_u, W_p, time,
           delta, epsilon) {
    b_nonzero <- which(b_p != 0)
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(X_u)) {
      C_b <- exp(itct + X_u %*% b_u + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero])
    } else {
      C_b <- exp(itct + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero])
    }
    if (!is.null(W_u)) {
      C_beta <- exp(W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero])
    } else {
      C_beta <- exp(W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero])
    }
    C_lbeta <- exp(-lambda * time * C_beta)
    temp2 <- lambda * time * C_beta
    temp3 <- 1 / (1 + C_b * C_lbeta)
    temp4 <- (1 - delta) * C_b * C_lbeta * temp2 * temp3
    temp5 <- delta * (1 - temp2)
    temp6 <- 1 / (1 + C_b)
    ### update b_p
    grad_b <- matrix(temp6 * (delta - (1 - delta) * C_b * (1 - C_lbeta) * temp3), 1) %*% X_p # length J
    j_b <- which.max(grad_b)
    b_p[j_b] <- b_p[j_b] + epsilon
    ### update beta_p
    grad_beta <- matrix(temp5 - temp4, 1) %*% W_p # length M
    j_beta <- which.max(grad_beta)
    beta_p[j_beta] <- beta_p[j_beta] + epsilon

    return(list(b_p = b_p, beta_p = beta_p))
  }

#' Function for Initializing Unpenalized Parameters for Cox MCM when Using E-M
#' Algorithm.
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#'
#' @return \item{itct}{estimated intercept for the incidence portion of the
#' model.}
#' @return \item{b_u}{estimated unpenalized coefficients for the incidence
#' portion of the model.}
#' @return \item{beta_u}{estimated unpenalized coefficients for the latency
#' portion of the model.}
#' @return \item{survprob}{estimated probabilities of survival.}
#' @noRd
initialization_cox <-
  function (X_u = NULL, W_u = NULL, time, delta)
  {
    data_init = as.data.frame(cbind(time, delta, X_u, W_u))
    if (is.null(X_u) & is.null(W_u)) {
      fit <- glm(delta ~ 1, family = "binomial")
      surv <- survival::survfit(Surv(time, delta) ~ 1)
      survprob <- sapply(time, function(x) summary(surv, times = x,
                                                  extend = T)$surv)
      inits <- list(itct = coef(fit), b_u = NULL, beta_u = NULL,
                   survprob = survprob)
    }
    else if (is.null(X_u)) {
      colnames(data_init) <- c("time", "delta", paste0("lat",
                                                       1:ncol(W_u)))
      formula_lat <- as.formula(paste("Surv(time, delta) ~",
                                     paste(paste0("lat", 1:ncol(W_u)), collapse = " + ")))
      fit1 <- survival::coxph(formula_lat, data = data_init)
      surv <- survival::survfit(fit1)
      survprob <- sapply(time, function(x) summary(surv, times = x,
                                                  extend = T)$surv)
      fit2 <- glm(delta ~ 1, family = "binomial")
      inits <- list(itct = coef(fit2), b_u = NULL, beta_u = coef(fit1),
                   survprob = survprob)
    }
    else if (is.null(W_u)) {
      colnames(data_init) <- c("time", "delta", paste0("inc",
                                                       1:ncol(X_u)))
      formula_inc <- as.formula(paste("delta ~", paste(paste0("inc",
                                                             1:ncol(X_u)), collapse = " + ")))
      fit <- glm(formula_inc, family = "binomial", data = data_init)
      surv <- survival::survfit(Surv(time, delta) ~ 1)
      survprob <- sapply(time, function(x) summary(surv, times = x,
                                                  extend = T)$surv)
      itct <- coef(fit)[1]
      b_u <- coef(fit)[-1]
      inits <- list(itct = itct, b_u = b_u, beta_u = NULL, survprob = survprob)
    }
    else {
      colnames(data_init) <- c("time", "delta", paste0("inc",
                                                       1:ncol(X_u)), paste0("lat", 1:ncol(W_u)))
      formula_lat <- as.formula(paste("Surv(time, delta) ~",
                                     paste(paste0("lat", 1:ncol(W_u)), collapse = " + ")))
      formula_inc <- as.formula(paste("delta ~", paste(paste0("inc",
                                                             1:ncol(X_u)), collapse = " + ")))
      fit1 <- glm(formula_inc, family = "binomial", data = data_init)
      itct <- coef(fit1)[1]
      b_u <- coef(fit1)[-1]
      fit2 <- survival::coxph(formula_lat, data = data_init)
      beta_u <- coef(fit2)
      surv <- survival::survfit(fit2)
      survprob <- sapply(time, function(x) summary(surv, times = x,
                                                  extend = T)$surv)
      inits <- list(itct = itct, b_u = b_u, beta_u = beta_u,
                   survprob = survprob)
    }
    return(inits)
  }


#' Function for Calulating Maximum Penalty Parameter, lambda, for Cox MCM.
#'
#' @param x matrix of predictors.
#' @param time time-to-event of interest.
#' @param event event (delta = 1) and censoring (delta = 0) indicator.
#' @param offset offset.
#'
#' @return value of maximum lambda.
#' @noRd
get_cox_lambda_max <- function(x, time, event, offset = rep(0, dim(x)[1])) {
  N <- dim(x)[1]
  beta_w <- offset
  beta_w <- beta_w - mean(beta_w)

  event_time <- time[event == 1]
  uniq_event_time <- unique(event_time)
  n0 <- length(uniq_event_time)
  I0 <- matrix(time, ncol = 1) %*% matrix(1, 1, n0) >=
    matrix(1, N, 1) %*% matrix(uniq_event_time, nrow = 1) # N*n0
  d_j <- as.numeric(table(event_time)[rank(uniq_event_time)])

  exp_beta_w <- exp(beta_w)
  sum0 <- pmax(1e-50, matrix(exp_beta_w, 1) %*% I0) # 1*n0
  sum1 <- t(x) %*% diag(as.numeric(exp_beta_w)) %*% I0 # ncol(x) * n0
  null_grad <- colSums(x[event == 1, , drop = FALSE]) - rowSums(sum1 %*% diag(as.numeric(d_j / sum0))) # ncol(x) * 1
  g <- abs(null_grad)
  lambda_max <- max(g) / N
  return(lambda_max)
}


#' Function for Initializing Unpenalized Parameters for Weibull and
#' Exponential MCM when Using E-M Algorithm.
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param model character, indicating the latency distribution, either
#' "exponential" or "weibull".
#'
#' @return \item{itct}{estimated intercept for the incidence portion of the
#' model.}
#' @return \item{b_u}{estimated unpenalized coefficients for the incidence
#' portion of the model.}
#' @return \item{beta_u}{estimated unpenalized coefficients for the latency
#' portion of the model.}
#' @return \item{lambda}{estimated lambda parameter for Weibull and exponential
#' latency distribution.}
#' @return \item{alpha}{if model is "weibull", estimated alpha parameter.}
#' @noRd
initialization_parametric <-
  function (X_u = NULL, W_u = NULL, time, delta, model = "weibull")
  {
    if (!is.null(X_u)) {
      fit1 <- glm(as.formula(paste("delta ~", paste(colnames(as.data.frame(X_u)),
                                                   collapse = " + "))), data = as.data.frame(X_u), family = binomial(link = "logit"))
      b_u <- fit1$coefficients[-1]
      itct <- fit1$coefficients[1]
    }
    else {
      itct <- log(mean(delta)/(1 - mean(delta)))
      b_u <- NULL
    }
    if (!is.null(W_u)) {
      fit2 <- survival::coxph(as.formula(paste("Surv(time, delta) ~",
                                              paste(colnames(as.data.frame(W_u)), collapse = " + "))),
                             data = as.data.frame(W_u), ties = "breslow")
      beta_u <- coef(fit2)
    }
    else beta_u <- NULL
    if (model == "weibull") {
      alpha <- uniroot(function(a) log(gamma(1 + 2/a)) - 2 *
                        log(gamma(1 + 1/a)) - log(var(time) + (mean(time))^2) +
                        2 * log(mean(time)), c(0.01, 10))$root
      lambda <- gamma(1 + 1/alpha)/mean(time)
    }
    else if (model == "exponential") {
      lambda <- 1/mean(time)
    }
    inits <- list(itct = itct, b_u = b_u, beta_u = beta_u, lambda = lambda)
    if (model == "weibull")
      inits$alpha <- alpha
    return(inits)
  }


#' Check Initial Values.
#'
#' @param model character, indicating the latency distribution, either "cox",
#' "exponential", or "weibull".
#' @param N number of observations.
#' @param penalty.factor.inc vector with length equal to the number of penalized
#' incidence predictors where 0 indicates no penalty and 1 indicates a penalty
#' should be applied. If not specified, default is to penalize all incidence
#' predictors.
#' @param penalty.factor.lat vector with length equal to the number of penalized
#' latency predictors where 0 indicates no penalty and 1 indicates a penalty
#' should be applied. If not specified, default is to penalize all latency
#' predictors.
#' @param inits estimated initial values used to optimization.
#'
#' @return if no warning, returns inital values.
#' @noRd
inits_check <-
  function (model, N, penalty.factor.inc, penalty.factor.lat, inits) {
    if (is.null(inits$itct)) {
      warning("Warning: Initial value of intercept is missing. Initial values were generated by the program.")
      return(NULL)
    }
    if (model == "cox") {
      if (is.null(inits$survprob)) {
        warning("Warning: Initial value of survprob is missing. Initial values were generated by the program.")
        return(NULL)
      }
      else if (length(inits$survprob) != N) {
        warning("Warning: Initial value of survprob has incorrect dimension. Initial values were generated by the program.")
        return(NULL)
      }
    }
    if (model %in% c("weibull", "exponential")) {
      if (is.null(inits$lambda)) {
        warning("Warning: Initial value of lambda is missing. Initial values were generated by the program.")
        return(NULL)
      }
      else if (inits$lambda <= 0) {
        warning("Warning: Initial value of lambda should be positive. Initial values were generated by the program.")
        return(NULL)
      }
    }
    if (model == "weibull") {
      if (is.null(inits$alpha)) {
        warning("Warning: Initial value of alpha is missing. Initial values were generated by the program.")
        return(NULL)
      }
      else if (inits$alpha <= 0) {
        warning("Warning: Initial value of alpha should be positive. Initial values were generated by the program.")
        return(NULL)
      }
    }
    if (any(penalty.factor.inc == 0))
      if (is.null(inits$b_u)) {
        warning("Warning: Initial value of b_u is missing. Initial values were generated by the program.")
        return(NULL)
      }
    else if (length(inits$b_u) != sum(penalty.factor.inc ==
                                      0)) {
      warning("Warning: Initial value of b_u has incorrect dimension. Initial values were generated by the program.")
      return(NULL)
    }
    if (any(penalty.factor.lat == 0))
      if (is.null(inits$beta_u)) {
        warning("Warning: Initial value of beta_u is missing. Initial values were generated by the program.")
        return(NULL)
      }
    else if (length(inits$beta_u) != sum(penalty.factor.lat ==
                                         0)) {
      warning("Warning: Initial value of beta_u has incorrect dimension. Initial values were generated by the program.")
      return(NULL)
    }
    return(inits)
  }

#' Gradient for Penalized Predictors in the Incidence Portion
#' of Parametric MCM Using the E-M Algorithm.
#'
#' @description
#' This function is passed to optim when updating penalized predictors in the
#' incidence portion of the model when using E-M algorithm for fitting either an
#' exponential or Weibull MCM and estimates the gradient. Called
#' by exp_EM and weib_EM.
#'
#' @param theta vector of penalized incidence coefficients.
#' @param itct estimated intercept for the incidence portion of the model.
#' @param b_u vector of estimated unpenalized incidence coefficients.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param mu lasso penalty parameter for incidence portion of the model.
#'
#' @return value of the gradient.
#' @noRd
l1_grad_b <-
  function(theta, itct, b_u, X_u, X_p, pir, mu) {
    N <- dim(X_p)[1]
    J <- ncol(X_p)
    b_p_ext <- theta
    b_p <- b_p_ext[1:J] - b_p_ext[(J + 1):(2 * J)]
    b_nonzero <- which(b_p != 0)
    if (!is.null(X_u)) {
      bx <- itct + X_u %*% b_u + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    } else {
      bx <- itct + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    }
    pix <- 1 / (1 + exp(-bx))
    grad <- matrix(pir - pix, 1) %*% X_p
    return(c(-grad + N * mu, grad + N * mu))
  }


#' Gradient for Unpenalized Predictors in the Incidence Portion
#' of Parametric MCM Using the E-M Algorithm.
#'
#' @description
#' This function is passed to optim when updating unpenalized predictors in the
#' incidence portion of the model when using E-M algorithm for fitting either an
#' exponential or Weibull MCM and estimates the gradient. Called
#' by exp_EM and weib_EM.
#'
#' @param theta vector of unpenalized incidence coefficients.
#' @param b_p vector of estimated penalized incidence coefficients.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param mu lasso penalty parameter for incidence portion of the model.
#'
#' @return value of the gradient.
#' @noRd
l1_gradient_inc <-
  function(theta, b_p, X_u, X_p, pir, mu) # incidence
  {
    itct <- theta[1]
    if (!is.null(X_u)) b_u <- theta[2:(ncol(X_u) + 1)]
    b_nonzero <- which(b_p != 0)
    if (!is.null(X_u)) {
      bx <- itct + X_u %*% b_u + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    } else {
      bx <- itct + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    }
    pix <- 1 / (1 + exp(-bx))
    grad1 <- sum(pir - pix)
    if (!is.null(X_u)) grad2 <- matrix(pir - pix, 1) %*% X_u else grad2 <- NULL
    return(-c(grad1, grad2))
  }


#' Negative Log-likelihood for Penalized Predictors in the Incidence Portion
#' of Parametric MCM Using the E-M Algorithm.
#'
#' @description
#' This function is passed to optim when updating penalized predictors in the
#' incidence portion of the model when using E-M algorithm for fitting either an
#' exponential or Weibull MCM and estimates the negative log-likelihood. Called
#' by exp_EM and weib_EM.
#'
#' @param theta vector of unpenalized incidence coefficients.
#' @param itct estimated intercept for the incidence portion of the model.
#' @param b_u vector of estimated unpenalized incidence coefficients.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param mu lasso penalty parameter for incidence portion of the model.
#'
#' @return value of negative log-likelihood.
#' @noRd
l1_negloglik_inc_b <-
  function(theta, itct, b_u, X_u, X_p, pir, mu) # incidence
  {
    N <- dim(X_p)[1]
    J <- ncol(X_p)
    b_p_ext <- theta
    b_p <- b_p_ext[1:J] - b_p_ext[(J + 1):(2 * J)]
    b_nonzero <- which(b_p != 0)
    if (!is.null(X_u)) {
      bx <- itct + X_u %*% b_u + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    } else {
      bx <- itct + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    }
    ll <- sum(pir * bx - log(1 + exp(bx))) - N * mu * sum(b_p_ext)
    return(-ll)
  }


#' Negative Log-likelihood for Unpenalized Predictors in the Incidence Portion
#' of Parametric MCM Using the E-M Algorithm.
#'
#' @description
#' This function is passed to optim when updating unpenalized predictors in the
#' incidence portion of the model when using E-M algorithm for fitting either an
#' exponential or Weibull MCM and estimates the negative log-likelihood. Called
#' by exp_EM and weib_EM.
#'
#' @param theta vector of unpenalized incidence coefficients.
#' @param b_p vector of estimated penalized incidence coefficients.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param mu lasso penalty parameter for incidence portion of the model.
#'
#' @return value of negative log-likelihood.
#' @noRd
l1_negloglik_inc <-
  function(theta, b_p, X_u, X_p, pir, mu) # incidence
  {
    itct <- theta[1]
    if (!is.null(X_u)) b_u <- theta[2:(ncol(X_u) + 1)]
    N <- dim(X_p)[1]
    b_nonzero <- which(b_p != 0)
    if (!is.null(X_u)) {
      bx <- itct + X_u %*% b_u + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    } else {
      bx <- itct + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    }
    ll <- sum(pir * bx - log(1 + exp(bx))) - N * mu * sum(abs(b_p))
    return(-ll)
  }

#' Identify Model From Solution Path Optimizes the Information Criterion.
#'
#' @description
#' This function is called by coef, summary, predict, plot, auc_mcm,
#' concordance_mcm to identify the step at which the user decides is the final
#' model. This function then extracts the optimal step and values for
#' information criteria at that step.
#'
#' @param object an object of class "mixturecure".
#' @param model_select a case-sensitive parameter or any (numeric) step along
#' the solution path can be selected. The default is model_select = "AIC" which
#' returns which model along the solution path attained the lowest AIC. Other
#' options are model_select = "mAIC" for the modified AIC, model_select = "cAIC"
#' for the corrected AIC, model_select = "BIC", model_select = "mBIC" for the
#' modified BIC, model_select = "EBIC" for the extended BIC,
#' model_select = "logLik" for the step that maximizes the log-likelihood, or
#' any numeric value from the solution path.
#' @return \item{select} step at which use-defined model_select attains optimum.
#' @return \item{AIC} AIC path.
#' @return \item{BIC} BIC path.
#' @return \item{mAIC} modified AIC path.
#' @return \item{mBIC} modified BIC path.
#' @return \item{EBIC} extended BIC path.
#' @return \item{cAIC} corrected AIC path.
#' @return \item{logLik} log-likelihood path.
#' @noRd
select_model <- function(object, model_select) {
  if (!object$cv) {
  if (is.character(model_select)) {
    model_select <- c(
      "AIC", "BIC", "logLik", "cAIC", "mAIC", "mBIC",
      "EBIC"
    )[pmatch(
      model_select,
      c(
        "AIC", "BIC", "logLik", "cAIC", "mAIC", "mBIC",
        "EBIC"
      )
    )]
    if (any(!model_select %in%
            c("AIC", "BIC", "logLik", "cAIC", "mAIC", "mBIC", "EBIC"))) {
      stop("Error: model_select must be either 'AIC', 'BIC', 'logLik', 'cAIC',
               'mAIC', 'mBIC', or 'EBIC' ")
    }
    if (!is.null(object$x_incidence)) {
      vars_inc <- apply(object$b_path, 1, function(x) sum(x != 0))
    } else {
      vars_inc <- 0
    }
    if (!is.null(object$x_latency)) {
      vars_lat <- apply(object$beta_path, 1, function(x) sum(x != 0))
    } else {
      vars_lat <- 0
    }
  }
  } else {
    if (!is.null(object$x_latency)) {
      vars_lat <- sum(object$beta != 0)
    } else {
      vars_lat <- 0
    }
    if (!is.null(object$x_incidence)) {
      vars_inc <- sum(object$b != 0)
    } else {
      vars_inc <- 0
    }
  }
  if (object$model == "weibull") {
    df <- vars_inc + vars_lat + 3
  } else if (object$model == "exponential") {
    df <- vars_inc + vars_lat + 2
  } else if (object$model == "cox") {
    df <- vars_inc + vars_lat + 1
  }
  logLik <- object$logLik
  p <- dim(object$x_incidence)[2] + dim(object$x_latency)[2]
  AIC <- 2 * df - 2 * logLik
  cAIC <- AIC + (2 * df * (df + 1)) / (length(object$y) - df - 1)
  mAIC <- (2 + 2 * log(p / .5)) * df - 2 * logLik
  BIC <- df * (log(length(object$y))) - 2 * logLik
  mBIC <- df * (log(length(object$y)) + 2 * log(p / 4)) - 2 * logLik
  EBIC <- log(length(object$y)) * df + 2 * (1 - .5) * log(choose(p, df)) -
    2 * logLik
  if (object$model != "cox") {
    if (object$mode == "weibull") {
      if (!exists("alpha_path", object)) {
        object$alpha_path <- object$alpha
      }
    }
    if (!exists("rate_path", object)) {
      object$rate_path <- object$rate
    }
  }
  if (!object$cv) {
    if (model_select == "AIC") {
      select <- which.min(AIC)
    } else if (model_select == "BIC") {
      select <- which.min(BIC)
    } else if (model_select == "mAIC") {
      select <- which.min(mAIC)
    } else if (model_select == "mBIC") {
      select <- which.min(mBIC)
    } else if (model_select == "EBIC") {
      select <- which.min(EBIC)
    } else if (model_select == "cAIC") {
      select <- which.min(cAIC)
    } else if (model_select == "logLik") {
      select <- which.max(logLik)
    }
  }
  list(select = select, AIC = AIC, BIC = BIC, mAIC = mAIC, mBIC = mBIC,
       EBIC = EBIC, cAIC = cAIC, logLik = logLik)
}


#' Non-Parametric Time-to-Event Generator.
#'
#' @description
#' Function called by generate_cure_data when the model is to generate
#' nonparametric times.
#'
#' @param N number of observations
#' @param beta_w linear predictor for the latency portion of the model.
#' @param maxT maximum observable time-to-event.
#' @param knots number of knots.
#'
#' @return time-to-event.
#' @noRd
nonparametric_time_generator <-
  function(N, beta_w, maxT = 20, knots = 8) {
    time <- 0:maxT
    k <- c(0, sort(sample(time[2:maxT], size = knots, replace = FALSE)), maxT)
    heights <- c(0, sort(1 - pmin(rexp(knots, rate = 10), 1 - 1e-15)), 1)
    # heights <- c(0, sort(runif(knots)), 1)
    tk <- merge(data.frame(time), data.frame(time = k, heights),
      by = "time", all = FALSE
    )
    MonotonicSpline <- stats::splinefun(
      x = tk$time, y = tk$heights,
      method = "hyman"
    )
    t <- rep(NA, N)
    for (i in 1:N) {
      u <- runif(1)
      t[i] <- uniroot(function(a) (1 - MonotonicSpline(a))^exp(beta_w[i]) - u, c(0, maxT))$root
    }
    return(t)
  }


#' MCP Penalty.
#'
#' @description
#' Called by cox_mcp_scad when using EM algorithm with MCP option.
#'
#' @param b_p vector of penalized coefficients.
#' @param gamma MCP penalty is parameterized by gamma > 1 and lambda >= 0.
#' @param lambda MCP penalty is parameterized by gamma > 1and lambda >= 0.
#'
#' @return MCP.
#' @noRd
mcp_penalty <-
  function(b_p, gamma, lambda) {
    idx <- which(b_p != 0 & abs(b_p) <= gamma * lambda)
    s1 <- lambda * sum(abs(b_p[idx])) - sum((b_p[idx])^2) / (2 * gamma)
    s2 <- gamma * lambda^2 / 2 * sum(abs(b_p) > gamma * lambda)
    return(s1 + s2)
  }


#' Identify MCP and SCAD Incidence Coefficient Update for Cox MCM Using E-M
#' Algorithm.
#'
#' @description
#' Function for identifying which coefficient in the incidence
#' portion of the Cox MCM to update when using the E-M algorithm with either
#' the MCP or SCAD penalty.
#'
#' @param penalty character indicating "MCP" or "SCAD" penalty.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param pi_x vector of estimated probabilities of being susceptible to the
#' event of interest.
#' @param x_l vector for lth penalized covariate in the incidence portion of the
#' model.
#' @param bl coefficient estimate for the lth penalized covariate in the
#' incidence portion of the model.
#' @param gamma gamma penalty parameter for the incidence portion of the model.
#' @param lambda lambda penalty parameter for the incidence portion of the model.
#'
#' @return which coefficient to update.
#' @noRd
mcp_scad_cd_upd_inc <-
  function(penalty, pir, pi_x, x_l, bl, gamma, lambda) {
    d1 <- mean((pir - pi_x) * x_l)
    vl <- mean(pi_x * (1 - pi_x) * x_l^2)
    zl <- d1 + vl * bl
    if (penalty == "MCP") {
      if (abs(bl) <= gamma * lambda) {
        return(soft(zl, lambda) / (vl - 1 / gamma))
      } else {
        return(ifelse(vl == 0, 0, zl / vl))
      }
    } else if (penalty == "SCAD") {
      if (abs(bl) <= lambda) {
        return(ifelse(vl == 0, 0, soft(zl, lambda) / vl))
      } else if (abs(bl) <= gamma * lambda) {
        return(soft(zl, gamma * lambda / (gamma - 1)) / (vl - 1 / (gamma - 1)))
      } else {
        return(ifelse(vl == 0, 0, zl / vl))
      }
    }
  }

#' Identify MCP and SCAD Latency Coefficient Update for Cox MCM Using E-M
#' Algorithm.
#'
#' @description
#' Function for identifying which coefficient in the  latency
#' portion of the Cox MCM to update when using the E-M algorithm with either
#' the MCP or SCAD penalty.
#'
#' @param penalty character indicating "MCP" or "SCAD" penalty.
#' @param exp_beta_w_p exponentiated coefficient for latency portion of the model.
#' @param w_l vector for lth penalized covariate in the latency portion of the
#' model.
#' @param betal coefficient estimate for the lth penalized covariate in the
#' latency portion of the model.
#' @param Zl sum of for the lth penalized coefficient at each unique event time.
#' @param I0 logical, matrix with number of rows corresponding to observations
#' and number of columns corresponding to unique event times.
#' @param d_j vector representing number of events at each unique event time.
#' @param gamma gamma penalty parameter for the latency portion of the model.
#' @param lambda lambda penalty parameter for the latency portion of the model.
#'
#' @return which coefficients to update.
#' @noRd
mcp_scad_cd_upd_lat <-
  function(penalty, exp_beta_w_p, w_l, betal, Zl, I0, d_j, gamma, lambda) {
    N <- length(exp_beta_w_p)
    sum0 <- pmax(1e-50, matrix(exp_beta_w_p, 1) %*% I0) # 1*n0
    sum1 <- matrix(exp_beta_w_p * w_l, 1) %*% I0 # 1*n0
    sum2 <- matrix(exp_beta_w_p * w_l^2, 1) %*% I0 # 1*n0
    d1 <- sum(Zl - d_j * sum1 / sum0) / N
    vl <- -sum(d_j * (sum1^2 / sum0^2 - sum2 / sum0)) / N
    zl <- d1 + vl * betal
    if (penalty == "MCP") {
      if (abs(betal) <= gamma * lambda) {
        return(soft(zl, lambda) / (vl - 1 / gamma))
      } else {
        return(ifelse(vl == 0, 0, zl / vl))
      }
    } else if (penalty == "SCAD") {
      if (abs(betal) <= lambda) {
        return(ifelse(vl == 0, 0, soft(zl, lambda) / vl))
      } else if (abs(betal) <= gamma * lambda) {
        return(soft(zl, gamma * lambda / (gamma - 1)) / (vl - 1 / (gamma - 1)))
      } else {
        return(ifelse(vl == 0, 0, zl / vl))
      }
    }
  }


#' MCP and SCAD Gradient for Cox Incidence for the E-M Algorithm.
#'
#' @description
#' When fitting a Cox model using either the MCP or SCAD penalty, this
#' gradient function is used by optim in the M-step of
#' cox_mcp_scad in M-step of the E-M algorithm for the incidence portion of the
#' model.
#'
#' @param theta value for the gradient.
#' @param b_p vector of estimated penalized incidence coefficients.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param CAP absolute value of the maximum intercept estimate (to avoid
#' numerical instability), default is 20.
#'
#' @return value of the gradient.
#' @noRd
mcp_scad_gradient_inc <-
  function(theta, b_p, X_u, X_p, pir, CAP) # incidence
  {
    itct <- theta[1]
    if (!is.null(X_u)) b_u <- theta[2:(ncol(X_u) + 1)]
    b_nonzero <- which(b_p != 0)
    if (is.null(X_u)) {
      bx <- itct + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    } else {
      bx <- itct + X_u %*% b_u + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    }
    pix <- 1 / (1 + exp(pmin(CAP, -bx)))
    grad1 <- sum(pir - pix)
    if (!is.null(X_u)) grad2 <- matrix(pir - pix, 1) %*% X_u else grad2 <- NULL
    return(-c(grad1, grad2))
  }


#' MCP and SCAD Gradient for Cox Latency for the E-M Algorithm.
#'
#' @description
#' When fitting a Cox model using either the MCP or SCAD penalty, this
#' gradient function is used by optim in the M-step of
#' cox_mcp_scad in M-step of the E-M algorithm for the latency portion of the
#' model.
#'
#' @param theta value for the gradient.
#' @param beta_p vector of estimated penalized latency coefficients.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param I0 logical, matrix with number of rows corresponding to observations
#' and number of columns corresponding to unique event times.
#' @param d_j vector representing number of events at each unique event time.
#' @param CAP absolute value of the maximum intercept estimate (to avoid
#' numerical instability), default is 20.
#'
#' @return value of the gradient.
#' @noRd
mcp_scad_gradient_lat <-
  function(theta, beta_p, W_u, W_p, delta, pir, I0, d_j, CAP) { # latency
    beta_u <- theta
    N <- dim(W_p)[1]
    beta_nonzero <- which(beta_p != 0)
    beta_w <- W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    exp_beta_w_p <- exp(pmin(CAP, beta_w)) * pir
    sum0 <- pmax(1e-50, matrix(exp_beta_w_p, 1) %*% I0) # 1*n0
    sum1 <- t(W_u) %*% diag(as.numeric(exp_beta_w_p)) %*% I0 # ncol(W_u) * n0
    du <- colSums(W_u[delta == 1, , drop = FALSE]) - rowSums(sum1 %*% diag(as.numeric(d_j / sum0))) # ncol(W_u) * 1
    return(-du)
  }


#' MCP and SCAD Negative Log-Likelihood Cox Incidence for the E-M Algorithm.
#'
#' @description
#' When fitting a Cox model using either the MCP or SCAD penalty, this
#' negative log-likelihood function is used by optim in the M-step of
#' cox_mcp_scad in M-step of the E-M algorithm for the incidence portion of the
#' model.
#'
#' @param theta negative log-likelihood.
#' @param b_p vector of estimated penalized incidence coefficients.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param CAP absolute value of the maximum intercept estimate (to avoid
#' numerical instability), default is 20.
#'
#' @return value of the negative log-likelihood.
#' @noRd
mcp_scad_negloglik_inc <-
  function(theta, b_p, X_u, X_p, pir, CAP) # incidence
  {
    itct <- theta[1]
    if (!is.null(X_u)) b_u <- theta[2:(ncol(X_u) + 1)]
    N <- dim(X_p)[1]
    b_nonzero <- which(b_p != 0)
    if (is.null(X_u)) {
      bx <- itct + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    } else {
      bx <- itct + X_u %*% b_u + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    }
    ll <- sum(pir * bx - log(1 + exp(pmin(CAP, bx))))
    return(-ll)
  }

#' MCP and SCAD Negative Log-Likelihood Cox Latency for the E-M Algorithm.
#'
#' @description
#' When fitting a Cox model using either the MCP or SCAD penalty, this
#' negative log-likelihood function is used by optim in the M-step of
#' cox_mcp_scad in M-step of the E-M algorithm for the latency portion of the
#' model.
#'
#' @param theta negative log-likelihood.
#' @param beta_p vector of estimated penalized latency coefficients.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param I0 logical, matrix with number of rows corresponding to observations
#' and number of columns corresponding to unique event times.
#' @param d_j vector representing number of events at each unique event time.
#' @param CAP absolute value of the maximum intercept estimate (to avoid
#' numerical instability), default is 20.
#'
#' @return value of the negative log-likelihood.
#' @noRd
mcp_scad_negloglik_lat <-
  function(theta, beta_p, W_u, W_p, delta, pir, I0, d_j, CAP) { # latency
    beta_u <- theta
    N <- dim(W_p)[1]
    beta_nonzero <- which(beta_p != 0)
    beta_w <- W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    exp_beta_w_p <- exp(pmin(CAP, beta_w)) * pir
    sum0 <- pmax(1e-50, matrix(exp_beta_w_p, 1) %*% I0) # 1*n0
    ll <- sum(beta_w[delta == 1]) - sum(d_j * log(sum0))
    return(-ll)
  }

#' SCAD Penalty.
#'
#' @description
#' Called by cox_mcp_scad when using EM algorithm with SCAD option.
#'
#' @param b_p vector of penalized coefficients.
#' @param gamma SCAD penalty is parameterized by gamma > 1 and lambda >= 0.
#' @param lambda SCAD penalty is parameterized by gamma > 1and lambda >= 0.
#'
#' @return SCAD.
#' @noRd
scad_penalty <-
  function(b_p, gamma, lambda) {
    idx1 <- which(b_p != 0 & abs(b_p) <= lambda)
    idx2 <- which(abs(b_p) > lambda & abs(b_p) <= gamma * lambda)
    s1 <- lambda * sum(abs(b_p[idx1]))
    s2 <- (gamma * lambda * sum(abs(b_p[idx2])) - 0.5 * (sum((b_p[idx2])^2 + lambda^2))) / (gamma - 1)
    s3 <- lambda^2 * (gamma^2 - 1) / (2 * (gamma - 1)) * sum(abs(b_p) > gamma * lambda)
    return(s1 + s2 + s3)
  }


#' Center and Scale a Matrix.
#'
#' @description
#' If scale = TRUE, this function centers and scales all covariates in matrix X.
#' It differs from R's scale function in that a vector of zeros is returned
#' if the SD for a vectors is 1.
#'
#' @param X data matrix.
#' @param scale logical, if TRUE, columns are to be centered and scaled.
#'
#' @return data matrix.
#' @noRd
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
self_scale <-
  function(X, scale) {
    if (is.null(X)) {
      return(NULL)
    }
    if (dim(X)[2] == 0) {
      return(NULL)
    }
    if (scale) {
      n <- dim(X)[1]
      mean.x <- colMeans(X)
      sd.x <- sqrt(colMeans(X^2) - mean.x^2)
      Xs <-  scale(X)
      Xs[,sd.x==0] <- rep(0, n)
    } else {
      Xs <- X
    }
    return(Xs)
  }


#' Softmax.
#'
#' @description
#' Called by mcp_scad_cd_upd_inc and mcp_scad_cd_upd_lat
#'
#' @param z temp.
#' @param gamma temp.
#'
#' @return softmax.
#' @noRd
soft <-
  function(z, gamma) {
    if (gamma >= abs(z)) {
      return(0)
    } else {
      return(ifelse(z > 0, z - gamma, z + gamma))
    }
  }


#' Internal optimization function to fit Weibull LASSO model using the E-M
#' algorithm
#'
#' @description
#' The weib_EM function is the optimization algorithm for fitting a Weibull
#' LASSO model called by cureem for fitting the model using the EM algorithm
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param mu_inc initial intercept for the incidence portion of the model.
#' @param mu_lat initial intercept for the latency portion of the model.
#' @param inits initial values used to optimization.
#' @param nIter maximum number of interations, default is 100.
#' @param tol small numeric value specifying convergence tolerance.
#'
#' @return \item{b_p_path}{Matrix representing the solution path of the
#' penalized coefficients in the incidence portion of the model. Row is step and
#' column is variable.}
#' @return \item{beta_p_path}{Matrix representing the solution path of the
#' penalized coefficients in the latency portion of the model. Row is step and
#' column is variable.}
#' @return \item{b_u_path}{Matrix representing the solution path of the
#' unpenalized coefficients in the incidence portion of the model. Row is step
#' and column is variable.}
#' @return \item{itct_path}{Vector representing the solution path of the
#' intercept for the incidence portion of the model.}
#' @return \item{beta_u_path}{Matrix representing the solution path of the
#' un penalized coefficients in the latency portion of the model. Row is step
#' and column is variable.}
#' @return \item{alpha_path}{Vector representing the solution path for the alpha
#' parameter in the Weibull MCM.}
#' @return \item{lambda_path}{Vector representing the solution path for the
#' lambda parameter in the Weibull MCM.}
#' @return \item{lik_inc}{Vector representing the expected penalized
#' complete-data log-likelihood for the incidence portion of the model for
#' each step in the solution path.}
#' @return \item{lik_lat}{Vector representing the expected penalized
#' complete-data log-likelihood for the latency portion of the model for each
#' step in the solution path.}
#' @noRd
weib_EM <-
  function (X_u = NULL, X_p, W_u = NULL, W_p, time, delta, mu_inc,
            mu_lat, inits = NULL, nIter = 100, tol = 1e-04)
  {
    N <- length(time)
    J <- ncol(X_p)
    M <- ncol(W_p)
    step <- 1
    b_p <- rep(0, J)
    beta_p <- rep(0, M)
    b_p_ext <- rep(0, 2 * J)
    beta_p_ext <- rep(0, 2 * M)
    if (is.null(inits))
      inits <- initialization_parametric(X_u, W_u, time, delta,
                                        model = "weibull")
    itct <- inits$itct
    b_u <- inits$b_u
    beta_u <- inits$beta_u
    lambda <- inits$lambda
    alpha <- inits$alpha
    log_alpha <- log(max(alpha, 1e-15))
    log_lambda <- log(max(lambda, 1e-15))
    pir <- rep(1, N)
    llp1_0 <- llp2_0 <- 0
    lik_inc <- lik_lat <- c()
    b_p_path <- beta_p_path <- b_u_path <- beta_u_path <- itct_path <- alpha_path <- lambda_path <- NULL
    conv1 <- conv2 <- FALSE
    repeat {
      if (is.null(X_u))
        b_x <- itct + X_p[, b_p != 0, drop = FALSE] %*% b_p[b_p !=
                                                             0]
      else b_x <- itct + X_u %*% b_u + X_p[, b_p != 0, drop = FALSE] %*%
          b_p[b_p != 0]
      if (is.null(W_u))
        beta_w <- W_p[, beta_p != 0, drop = FALSE] %*% beta_p[beta_p !=
                                                               0]
      else beta_w <- W_u %*% beta_u + W_p[, beta_p != 0, drop = FALSE] %*%
          beta_p[beta_p != 0]
      pir[delta == 0] <- 1/(1 + exp(-b_x[delta == 0] + (lambda *
                                                         time[delta == 0])^alpha * exp(beta_w[delta == 0])))
      if (!conv1) {
        b_p_ext <- optim(par = b_p_ext, fn = l1_negloglik_inc_b,
                        gr = l1_grad_b, itct = itct, b_u = b_u, X_u = X_u,
                        X_p = X_p, pir = pir, mu = mu_inc, method = "L-BFGS-B",
                        lower = rep(0, 2 * J))$par
        b_p <- b_p_ext[1:J] - b_p_ext[(J + 1):(2 * J)]
      }
      if (!conv2) {
        beta_p_ext <- optim(par = beta_p_ext, fn = weib_negloglik_lat_beta,
                           gr = weib_grad_beta, alpha = alpha, lambda = lambda,
                           beta_u = beta_u, W_u = W_u, W_p = W_p, time = time,
                           delta = delta, pir = pir, mu = mu_lat, method = "L-BFGS-B",
                           lower = rep(0, 2 * M))$par
        beta_p <- beta_p_ext[1:M] - beta_p_ext[(M + 1):(2 *
                                                         M)]
      }
      if (!conv1) {
        out_inc <- optim(par = c(itct, b_u), fn = l1_negloglik_inc,
                        gr = l1_gradient_inc, b_p = b_p, X_u = X_u, X_p = X_p,
                        pir = pir, mu = mu_inc, method = "BFGS")
        itct <- out_inc$par[1]
        if (!is.null(X_u))
          b_u <- out_inc$par[2:(ncol(X_u) + 1)]
        llp1_1 <- -out_inc$value
      }
      lik_inc <- c(lik_inc, llp1_1)
      if (!conv2) {
        out_lat <- optim(par = c(log_alpha, log_lambda, beta_u),
                        fn = weib_negloglik_lat, gr = weib_gradient_lat,
                        beta_p = beta_p, W_u = W_u, W_p = W_p, time = time,
                        delta = delta, pir = pir, mu = mu_lat, method = "BFGS")
        log_alpha <- out_lat$par[1]
        alpha <- exp(log_alpha)
        log_lambda <- out_lat$par[2]
        lambda <- exp(log_lambda)
        if (!is.null(W_u))
          beta_u <- out_lat$par[3:(ncol(W_u) + 2)]
        llp2_1 <- -out_lat$value
      }
      lik_lat <- c(lik_lat, llp2_1)
      itct_path <- c(itct_path, itct)
      if (!is.null(X_u))
        b_u_path <- rbind(b_u_path, b_u)
      b_p_path <- rbind(b_p_path, b_p)
      if (!is.null(W_u))
        beta_u_path <- rbind(beta_u_path, beta_u)
      beta_p_path <- rbind(beta_p_path, beta_p)
      alpha_path <- c(alpha_path, alpha)
      lambda_path <- c(lambda_path, lambda)
      if (!conv1 & abs(llp1_1 - llp1_0) < tol)
        conv1 <- TRUE
      if (!conv2 & abs(llp2_1 - llp2_0) < tol)
        conv2 <- TRUE
      if (step > 1 & (conv1 & conv2) | step >= nIter) {
        break
      }
      llp1_0 <- llp1_1
      llp2_0 <- llp2_1
      step <- 1 + step
    }
    output <- list(b_p_path = b_p_path, b_u_path = b_u_path,
                   itct_path = itct_path, beta_p_path = beta_p_path, beta_u_path = beta_u_path,
                   alpha_path = alpha_path, lambda_path = lambda_path, lik_inc = lik_inc,
                   lik_lat = lik_lat)
    output
  }

#' Gradient for Penalized Predictors in Weibull MCM Fit Using EM Algorithm.
#'
#' @description
#' Gradient for optim when updating penalized predictors in the latency portion
#' of a Weibull MCM when fit using EM (called by weib_EM).
#'
#' @param theta numeric vector for the penalized predictors in the
#' Weibull latency distribution.
#' @param alpha estimated alpha parameter for Weibull latency distribution.
#' @param lambda estimated lambda parameter for Weibull latency distribution.
#' @param beta_u vector of estimated unpenalized latency coefficients.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param mu initial intercept for the latency portion of the model.
#'
#' @return value of the gradient.
#' @noRd
weib_grad_beta <-
  function(theta, alpha, lambda, beta_u, W_u, W_p, time, delta, pir, mu) {
    N <- dim(W_p)[1]
    M <- ncol(W_p)
    beta_p_ext <- theta
    beta_p <- beta_p_ext[1:M] - beta_p_ext[(M + 1):(2 * M)]
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(W_u)) {
      betaw <- W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    } else {
      betaw <- W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    }
    temp1 <- pir * (lambda * time)^alpha * exp(betaw)
    grad <- matrix(delta - temp1, 1) %*% W_p
    return(c(-grad + N * mu, grad + N * mu))
  }

#' Gradient for Unpenalized Predictors in Weibull MCM Fit Using EM Algorithm.
#'
#' @description
#' Gradient for optim when updating unpenalized predictors in the latency portion
#' of a Weibull MCM when fit using EM (called by weib_EM)
#'
#' @param theta numeric vector for the unpenalized predictors in the
#' Weibull latency distribution.
#' @param beta_p vector of estimated penalized latency coefficients.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param mu initial intercept for the latency portion of the model.
#'
#' @return value of the gradient.
#' @noRd
weib_gradient_lat <-
  function(theta, beta_p, W_u, W_p, time, delta, pir, mu) { # latency
    log_alpha <- theta[1]
    log_lambda <- theta[2]
    alpha <- exp(log_alpha)
    lambda <- exp(log_lambda)
    if (!is.null(W_u)) beta_u <- theta[3:(ncol(W_u) + 2)]
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(W_u)) {
      betaw <- W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    } else {
      betaw <- W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    }
    temp1 <- pir * (lambda * time)^alpha * exp(betaw)
    temp2 <- delta - temp1
    grad1 <- sum(delta + log(pmax(lambda * time, 1e-100)) * temp2 * alpha)
    grad2 <- sum(alpha * temp2)
    if (!is.null(W_u)) grad3 <- matrix(temp2, 1) %*% W_u else grad3 <- NULL
    return(-c(grad1, grad2, grad3))
  }


#' Identify GMIFS Coefficient Update for Weibull MCM.
#'
#' @description
#' Function for identifying which coefficient in the incidence and latency
#' portions of the Weibull MCM to update when using GMIFS, called by weibull.cure
#'
#' @param alpha estimated alpha parameter for Weibull latency distribution.
#' @param lambda estimated lambda parameter for Weibull latency distribution.
#' @param b_p vector of estimated penalized incidence coefficients.
#' @param beta_p vector of estimated penalized latency coefficients.
#' @param b_u vector of estimated unpenalized incidence coefficients.
#' @param itct estimated intercept for incidence portion of the model.
#' @param beta_u vector of estimated unpenalized latency coefficients.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param epsilon small numeric value used to update the regression coefficients.
#'
#' @return \item{b_p}{penalized incidence coefficient to update.}
#' @return \item{beta_p}{penalized latency coefficient to update.}
#' @noRd
weib.cure.update <-
  function(alpha, lambda, b_p, beta_p, b_u, itct, beta_u, X_u, X_p, W_u, W_p, time, delta, epsilon) {
    b_nonzero <- which(b_p != 0)
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(X_u)) {
      C_b <- exp(itct + X_u %*% b_u + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero])
    } else {
      C_b <- exp(itct + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero])
    }
    if (!is.null(W_u)) {
      C_beta <- exp(W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero])
    } else {
      C_beta <- exp(W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero])
    }
    C_lbeta <- exp(-(lambda * time)^alpha * C_beta)
    temp2 <- (lambda * time)^alpha * C_beta
    temp3 <- 1 / (1 + C_b * C_lbeta)
    temp4 <- (1 - delta) * C_b * C_lbeta * temp2 * temp3
    temp5 <- delta * (1 - temp2)
    temp6 <- 1 / (1 + C_b)
    ### update b_p
    grad_b <- matrix(temp6 * (delta - (1 - delta) * C_b * (1 - C_lbeta) * temp3), 1) %*% X_p # length J
    j_b <- which.max(grad_b)
    b_p[j_b] <- b_p[j_b] + epsilon
    ### update beta_p
    grad_beta <- matrix(temp5 - temp4, 1) %*% W_p # length M
    j_beta <- which.max(grad_beta)
    beta_p[j_beta] <- beta_p[j_beta] + epsilon
    return(list(b_p = b_p, beta_p = beta_p))
  }


#' Optimization Function for Weibull MCM Using GMIFS.
#'
#' @description
#' This function is Called by curegmifs, cv.gmifs.inner, and cv.gmifs.nofdr as
#' this function is the optimiziation algorithm for fitting a Weibull mixture
#' cure model using the GMIFS algorithm.
#'
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param epsilon small numeric value used to update the regression coefficients.
#' @param tol small numeric value specifying convergence tolerance.
#' @param nIter maximum number of interations.
#' @param inits initial values used to optimization.
#' @param verbose logical, if TRUE displays step process to the console.
#'
#' @return \item{b_p_path}{Matrix representing the solution path of the
#' penalized coefficients in the incidence portion of the model. Row is step and
#' column is variable.}
#' @return \item{beta_p_path}{Matrix representing the solution path of the
#' penalized coefficients in the latency portion of the model. Row is step and
#' column is variable.}
#' @return \item{alpha_path}{Vector representing the solution path for the alpha
#' parameter in the Weibull MCM.}
#' @return \item{lambda_path}{Vector representing the solution path for the alpha
#' parameter in the Weibull MCM.}
#' @return \item{b_u_path}{Matrix representing the solution path of the
#' unpenalized coefficients in the incidence portion of the model. Row is step
#' and column is variable.}
#' @return \item{itct_path}{Vector representing the solution path of the
#' intercept for the incidence portion of the model.}
#' @return \item{beta_u_path}{Matrix representing the solution path of the
#' unpenalized coefficients in the latency portion of the model. Row is step and
#' column is variable.}
#' @return \item{logLikelihood}{Vector representing the expected penalized
#' complete-data log-likelihood for each step in the solution path.}
#'
#' @noRd
weibull.cure <-
  function(X_u = NULL, X_p, W_u = NULL, W_p, time, delta, epsilon = 0.001,
           tol = 1e-05, nIter = 1e4, inits = NULL, verbose) {
    # X_u: N by nonp, non-penalized covariate matrix associated with incidence
    # X_p: N by J, penalized covariate matrix associated with incidence
    # W_u: N by nonp, non-penalized covariate matrix associated with latency
    # W_p:  N by M, penalized covariate matrix associated with incidence
    # time: vector of length N, observed survival time
    # delta: vector of length N, censoring status (not censored = 0)
    # epsilon: incremental size
    # tol: difference between log-likelihood

    N <- length(time)
    J <- ncol(X_p) # number of penalized incidence covariates
    M <- ncol(W_p) # number of penalized latency covariates
    X_p <- cbind(X_p, -X_p) # N by 2J
    W_p <- cbind(W_p, -W_p) # N by 2M
    ######## initialization ########
    step <- 1
    b_p <- rep(0, 2 * J) # penalized incidence
    beta_p <- rep(0, 2 * M) # penalized latency
    if (is.null(inits)) inits <- initialization_parametric(X_u, W_u, time,
                                                     delta, model = "weibull")
    itct <- inits$itct
    b_u <- inits$b_u
    beta_u <- inits$beta_u
    lambda <- inits$lambda
    alpha <- inits$alpha
    log_alpha <- log(max(alpha, 1e-15))
    log_lambda <- log(max(lambda, 1e-15))
    LL0 <- 0

    alpha_path <- lambda_path <- itct_path <- logLikelihood <- rep(0, nIter)
    b_p_path <- matrix(0, nrow = nIter, ncol = 2 * J)
    beta_p_path <- matrix(0, nrow = nIter, ncol = 2 * M)
    b_u_path <- matrix(0, nrow = nIter, ncol = length(b_u))
    beta_u_path <-  matrix(0, nrow = nIter, ncol = length(beta_u))

    ######## loop ########

    repeat{
      #### update penalized parameters
      upd <- weib.cure.update(alpha, lambda, b_p, beta_p, b_u, itct, beta_u, X_u, X_p, W_u, W_p, time, delta, epsilon)
      b_p <- upd$b_p
      beta_p <- upd$beta_p
      b_p_path[step,] <- b_p
      beta_p_path[step,] <- beta_p

      #### update other parameters
      out <- optim(
        par = c(log_alpha, log_lambda, b_u, itct, beta_u), fn = weib.cure.negloglik, gr = weib.cure.gradient,
        b_p = b_p, beta_p = beta_p, X_u = X_u, X_p = X_p, W_u = W_u, W_p = W_p,
        time = time, delta = delta, method = "BFGS"
      )
      log_alpha <- out$par[1]
      alpha <- exp(log_alpha)
      log_lambda <- out$par[2]
      lambda <- exp(log_lambda)
      if (!is.null(X_u)) {
        b_u <- out$par[3:(ncol(X_u) + 2)] # unpenalized incidence
        itct <- out$par[ncol(X_u) + 3]
        if (!is.null(W_u)) beta_u <- out$par[(ncol(X_u) + 4):(ncol(X_u) + ncol(W_u) + 3)] # unpenalized latency
      } else {
        itct <- out$par[3]
        if (!is.null(W_u)) beta_u <- out$par[4:(ncol(W_u) + 3)] # unpenalized latency
      }

      alpha_path[step] <- alpha
      lambda_path[step] <- lambda
      if (!is.null(X_u)) b_u_path[step,] <- b_u
      itct_path[step] <- itct
      if (!is.null(W_u)) beta_u_path[step,] <- beta_u

      LL1 <- -out$value
      logLikelihood[step] <- LL1

      if (verbose & step %% 1000 == 0) message("step = ", step, "\n")
      if (step > 1 && (abs(LL1 - LL0) < tol | step >= nIter)) { #
        break
      }
      LL0 <- LL1
      step <- 1 + step
    }

    ######## output ########
    b_p_path <- b_p_path[, 1:J] - b_p_path[, (J + 1):(2 * J)]
    beta_p_path <- beta_p_path[, 1:M] - beta_p_path[, (M + 1):(2 * M)]
    # Remove rows of matrices and elements of vectors that are 0 because
    # algorithm converged before nIter
    if (step < nIter) {
      b_p_path <- b_p_path[1:step,]
      beta_p_path <- beta_p_path[1:step,]
      alpha_path <- alpha_path[1:step]
      lambda_path <- lambda_path[1:step]
      b_u_path <- b_u_path[1:step,]
      itct_path <- itct_path[1:step]
      beta_u_path <- beta_u_path[1:step,]
      logLikelihood <- logLikelihood[1:step]
    }
    output <- list(
      b_p_path = b_p_path, beta_p_path = beta_p_path, alpha_path = alpha_path,
      lambda_path = lambda_path, b_u_path = b_u_path, itct_path = itct_path,
      beta_u_path = beta_u_path, logLikelihood = logLikelihood
    )
    output
  }

#' Gradient for Updating Unpenalized Predictors in Weibull MCM Using GMIFS.
#'
#' @description
#' Gradient for optim when updating unpenalized predictors in an
#' Weibull MCM when fit using GMIFS (called by weibull.cure)
#'
#' @param theta vector of unpenalized parameters to estimate.
#' @param b_p vector of estimated penalized incidence coefficients.
#' @param beta_p vector of estimated penalized latency coefficients.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#'
#' @return negative gradient.
#' @noRd
weib.cure.gradient <-
  function(theta, b_p, beta_p, X_u, X_p, W_u, W_p, time, delta) {
    N <- length(time)
    alpha <- exp(theta[1])
    lambda <- exp(theta[2])
    if (!is.null(X_u)) {
      b_u <- theta[3:(ncol(X_u) + 2)] # unpenalized incidence
      itct <- theta[ncol(X_u) + 3]
      if (!is.null(W_u)) beta_u <- theta[(ncol(X_u) + 4):(ncol(X_u) + ncol(W_u) + 3)] # unpenalized latency
    } else {
      itct <- theta[3]
      if (!is.null(W_u)) beta_u <- theta[4:(ncol(W_u) + 3)] # unpenalized latency
    }
    b_nonzero <- which(b_p != 0)
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(X_u)) {
      C_b <- exp(itct + X_u %*% b_u + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero])
    } else {
      C_b <- exp(itct + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero])
    }
    if (!is.null(W_u)) {
      C_beta <- exp(W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero])
    } else {
      C_beta <- exp(W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero])
    }
    C_lbeta <- exp(-(lambda * time)^alpha * C_beta)
    temp1 <- log(pmax(lambda * time, rep(1e-100, N)))
    temp2 <- (lambda * time)^alpha * C_beta
    temp3 <- 1 / (1 + C_b * C_lbeta)
    temp4 <- (1 - delta) * C_b * C_lbeta * temp2 * temp3
    temp5 <- delta * (1 - temp2)
    temp6 <- 1 / (1 + C_b)
    temp7 <- temp6 * (delta - (1 - delta) * C_b * (1 - C_lbeta) * temp3)
    grad1 <- sum(delta * (1 / alpha + temp1 - temp2 * temp1) - temp4 * temp1) * alpha
    grad2 <- alpha * sum(temp5 - temp4)
    if (!is.null(X_u)) grad3 <- matrix(temp7, 1) %*% X_u else grad3 <- NULL
    grad4 <- sum(temp7) # intercept
    if (!is.null(W_u)) grad5 <- matrix(temp5 - temp4, 1) %*% W_u else grad5 <- NULL
    return(-c(grad1, grad2, grad3, grad4, grad5))
  }


#' Negative Log-Likelihood for the Latency Portion of a Weibull Mixture Cure
#' Model for GMIFS Algorithm to Update Unpenalized Predictors.
#'
#' @description
#' Function for optim when updating unpenalized predictors in an
#' Weibull mixture cure model when fit using GMIFS (called by weibull.cure).
#'
#' @param theta vector of unpenalized parameters to estimate.
#' @param b_p vector of estimated penalized incidence coefficients.
#' @param beta_p vector of estimated penalized latency coefficients.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#'
#' @return returns the negative log-likelihood value.
#' @noRd
weib.cure.negloglik <-
  function(theta, b_p, beta_p, X_u, X_p, W_u, W_p, time, delta) {
    N <- length(time)
    alpha <- exp(theta[1])
    lambda <- exp(theta[2])
    if (!is.null(X_u)) {
      b_u <- theta[3:(ncol(X_u) + 2)] # unpenalized incidence
      itct <- theta[ncol(X_u) + 3]
      if (!is.null(W_u)) beta_u <- theta[(ncol(X_u) + 4):(ncol(X_u) + ncol(W_u) + 3)] # unpenalized latency
    } else {
      itct <- theta[3]
      if (!is.null(W_u)) beta_u <- theta[4:(ncol(W_u) + 3)] # unpenalized latency
    }
    b_nonzero <- which(b_p != 0)
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(X_u)) {
      logC_b <- itct + X_u %*% b_u + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    } else {
      logC_b <- itct + X_p[, b_nonzero, drop = FALSE] %*% b_p[b_nonzero]
    }
    if (!is.null(W_u)) {
      logC_beta <- W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    } else {
      logC_beta <- W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    }
    temp1 <- 1 / (1 + exp(-logC_b)) # exp(logC_b) / (1+exp(logC_b)) #Mar17
    logC_lbeta <- -(lambda * time)^alpha * exp(logC_beta)
    ll1 <- delta * (log(temp1) + theta[2] + theta[1] +
      (alpha - 1) * log(pmax(lambda * time, rep(1e-100, N))) +
      logC_beta + logC_lbeta)
    ll2 <- (1 - delta) * log(pmax(1 - temp1 * (1 - exp(logC_lbeta)), rep(1e-100, N)))
    return(-sum(ll1 + ll2))
  }


#' Negative Log-Likelihood for the Latency Portion of a Weibull Mixture Cure
#' Model for EM Algorithm to Update Penalized Predictors.
#'
#' @description
#' Function for optim when updating penalized predictors in the latency portion
#' of a Weibull MCM when fit using EM (called by weib_EM)
#'
#' @param theta numeric vector for the penalized predictors in the
#' Weibull latency distribution.
#' @param beta_p vector of estimated latency coefficients.
#' @param alpha estimated alpha parameter for Weibull latency distribution.
#' @param lambda estimated lambda parameter for Weibull latency distribution.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param mu initial intercept for the latency portion of the model.
#'
#' @return returns the negative log-likelihood value.
#' @noRd
weib_negloglik_lat_beta <-
  function(theta, alpha, lambda, beta_u, W_u, W_p, time, delta, pir, mu) { # latency
    N <- dim(W_p)[1]
    M <- ncol(W_p)
    beta_p_ext <- theta
    beta_p <- beta_p_ext[1:M] - beta_p_ext[(M + 1):(2 * M)]
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(W_u)) {
      betaw <- W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    } else {
      betaw <- W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    }
    ll1 <- delta * (log(alpha) + log(lambda) + (alpha - 1) * log(pmax(lambda * time, 1e-100)) + betaw)
    ll2 <- -pir * (lambda * time)^alpha * exp(betaw)
    return(-sum(ll1 + ll2) + N * mu * sum(beta_p_ext))
  }

#' Negative Log-Likelihood for the Latency Portion of a Weibull Mixture Cure
#' Model for EM Algorithm to Update Unpenalized Predictors.
#'
#' @description
#' Function for optim when updating unpenalized predictors in the latency
#' portion of a Weibull MCM when fit using EM (called by weib_EM).
#'
#' @param theta numeric vector for alpha, lambda, and any unpenalized
#' coefficients in the Weibull latency distribution.
#' @param beta_p vector of estimated latency coefficients.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#' @param time time-to-event of interest.
#' @param delta event (delta = 1) and censoring (delta = 0) indicator.
#' @param pir vector of estimates for the conditional distribution of the event
#' at the current parameter estimates and observed datafor the latency
#' distribution E-step.
#' @param mu initial intercept for the latency portion of the model.
#'
#' @return returns the negative log-likelihood value.
#' @noRd
weib_negloglik_lat <-
  function(theta, beta_p, W_u, W_p, time, delta, pir, mu) { # latency
    N <- dim(W_p)[1]
    log_alpha <- theta[1]
    log_lambda <- theta[2]
    alpha <- exp(log_alpha)
    lambda <- exp(log_lambda)
    if (!is.null(W_u)) beta_u <- theta[3:(ncol(W_u) + 2)]
    beta_nonzero <- which(beta_p != 0)
    if (!is.null(W_u)) {
      betaw <- W_u %*% beta_u + W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    } else {
      betaw <- W_p[, beta_nonzero, drop = FALSE] %*% beta_p[beta_nonzero]
    }
    ll1 <- delta * (log_alpha + log_lambda + (alpha - 1) * log(pmax(lambda * time, 1e-100)) + betaw)
    ll2 <- -pir * (lambda * time)^alpha * exp(betaw)
    return(-sum(ll1 + ll2) + N * mu * sum(abs(beta_p)))
  }

#' Simulate Data under a Mixture Cure Model.
#'
#' @description Called by nonzerocure_test to simulate survival data under the
#' null model. If b = NULL, censoring times are generated from an exponential
#' distribution. If b is not null, censoring times are generated from a
#' uniform distribution with 0 as the lower limit and b as the upper limit.
#'
#' @param n integer, number of observations to generate.
#' @param mu rate for generating time-to-event from an exponential is 1/mean.
#' @param censor_mu rate for generating censoring time from an exponential is
#' 1/censor_mu.
#' @param b if non-null, censoring time is generated from a uniform distribution
#' on the interval from 0 to b.
#' @param reps number of times to simulate data.
#' @param seed random seed to set to enhance reproducibility.
#'
#' @return estimated susceptible proportion.
#'
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @noRd
sim_cure <-
  function(n, mu = 1, censor_mu = 3, b = NULL, reps = 10000, seed = NULL) {
    susceptible_prop <- numeric(length = reps)
    if (!is.null(seed)) {
      withr::local_seed(seed)
    }
    for (i in 1:reps) {
      surv <- rexp(n, 1 / mu)
      if (is.null(b)) {
        u <- rexp(n, 1 / censor_mu) ## rate is 1/mu
      } else {
        u <- runif(n, 0, b)
      }
      time <- ifelse(surv <= u, surv, u)
      censor <- ifelse(surv <= u, 1, 0)
      fit <- survival::survfit(survival::Surv(time, censor) ~ 1)
      susceptible_prop[i] <- 1 - min(fit$surv)
    }
    susceptible_prop
  }

#' Internal Function for Calculating the Area Under the Incidence Curve.
#'
#' @description Called by cv.em.inner and cv.gmifs.inner when the AUC is used
#' to optimized the mixture cure model.
#'
#' @param cure_cutoff time at which the C-statistic should be estimated.
#' @param b_u_hat vector of estimated unpenalized incidence coefficients.
#' @param itct_hat estimated intercept for incidence portion of the model.
#' @param b_p_hat vector of estimated penalized incidence coefficients.
#' @param testing_delta vector of censoring indicator for dataset of interest.
#' @param testing_time vector of time-to-event for dataset of interest.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#'
#' @return AUC under the incidence curve at the specified
#' cure_cutoff timepoint.
#' @noRd
AUC_msi <- function(cure_cutoff = 5, b_u_hat = NULL, itct_hat, b_p_hat, testing_delta, testing_time,
                    X_u = NULL, X_p) {
  testing_n <- length(testing_time)
  v <- rep(0, testing_n)
  y <- rep(999, testing_n)
  y[testing_time > cure_cutoff] <- 0
  y[testing_time <= cure_cutoff & testing_delta == 1] <- 1
  v[y < 2] <- 1
  if (all(b_p_hat == 0)) {
    if (is.null(X_u)) xb <- rep(itct_hat, testing_n) else xb <- itct_hat + X_u %*% b_u_hat
  } else {
    if (is.null(X_u)) {
      xb <- itct_hat + X_p[, b_p_hat != 0, drop = FALSE] %*% b_p_hat[b_p_hat != 0]
    } else {
      xb <- itct_hat + X_u %*% b_u_hat + X_p[, b_p_hat != 0, drop = FALSE] %*% b_p_hat[b_p_hat != 0]
    }
  }
  p_hat <- 1 / (1 + exp(-xb))
  temp <- v * y + (1 - v) * p_hat
  temp1 <- temp[order(p_hat, decreasing = T)]
  temp_f <- v * (1 - y) + (1 - v) * (1 - p_hat)
  temp1f <- temp_f[order(p_hat, decreasing = T)]
  TPR <- c(0, cumsum(temp1) / cumsum(temp1)[testing_n])
  FPR <- c(0, cumsum(temp1f) / cumsum(temp1f)[testing_n])
  height <- (TPR[-1] + TPR[-length(TPR)]) / 2
  width <- diff(FPR)
  auc_msi <- sum(height * width)
  return(auc_msi)
}

#' Calculate C-statistic for Mixture Cure Model.

#' @description Called by cv.em.inner and cv.gmifs.inner when
#' the concordance statistic is used in optimization
#'
#' @param cure_cutoff time at which the C-statistic should be estimated.
#' @param b_u_hat vector of estimated unpenalized incidence coefficients.
#' @param itct_hat estimated intercept for incidence portion of the model.
#' @param b_p_hat vector of estimated penalized incidence coefficients.
#' @param beta_u_hat vector of estimated unpenalized latency coefficients.
#' @param beta_p_hat vector of estimated penalized latency coefficients.
#' @param testing_delta vector of censoring indicator for dataset of interest.
#' @param testing_time vector of time-to-event for dataset of interest.
#' @param X_u unpenalized predictors in the incidence portion of the model.
#' @param X_p penalized predictors in the incidence portion of the model.
#' @param W_u unpenalized predictors in the latency portion of the model.
#' @param W_p penalized predictors in the latency portion of the model.
#'
#' @return mixture cure model c-statistic at the specified
#' cure_cutoff timepoint.
#' @noRd
C.stat <-
  function(cure_cutoff = 5, b_u_hat = NULL, itct_hat, b_p_hat,
           beta_u_hat = NULL, beta_p_hat, testing_delta, testing_time,
           X_u = NULL, X_p, W_u = NULL, W_p) {
    C_csw_num <- 0
    C_csw_denom <- 0
    testing_n <- length(testing_time)
    v <- rep(0, testing_n)
    y <- rep(999, testing_n)
    y[testing_time > cure_cutoff] <- 0
    y[testing_time <= cure_cutoff & testing_delta == 1] <- 1
    v[y < 2] <- 1
    if (all(b_p_hat == 0)) {
      if (is.null(X_u)) {
        p_hat <- 1 / (1 + exp(-itct_hat))
      } else {
        p_hat <- 1 / (1 + exp(-itct_hat - X_u %*% b_u_hat))
      }
    } else {
      if (is.null(X_u)) {
        p_hat <- 1 / (1 + exp(-itct_hat - X_p[, b_p_hat != 0,
          drop = FALSE
        ] %*% b_p_hat[b_p_hat != 0]))
      } else {
        p_hat <- 1 / (1 + exp(-itct_hat - X_u %*% b_u_hat -
          X_p[, b_p_hat != 0, drop = FALSE] %*% b_p_hat[b_p_hat !=
            0]))
      }
    }
    temp <- v * y + (1 - v) * as.vector(p_hat)
    if (all(beta_p_hat == 0)) {
      if (is.null(W_u)) {
        W_beta <- rep(0, testing_n)
      } else {
        W_beta <- W_u %*% beta_u_hat
      }
    } else {
      if (is.null(W_u)) {
        W_beta <- W_p[, beta_p_hat != 0, drop = FALSE] %*%
          beta_p_hat[beta_p_hat != 0]
      } else {
        W_beta <- W_u %*% beta_u_hat + W_p[, beta_p_hat !=
          0, drop = FALSE] %*% beta_p_hat[beta_p_hat != 0]
      }
    }
    for (i in 1:testing_n) {
      for (j in 1:testing_n) {
        if (j == i | !testing_delta[i] | testing_time[i] > testing_time[j]) {
          next
        }
        I_ij <- testing_time[i] < testing_time[j] | (testing_time[i] ==
          testing_time[j] & !testing_delta[j])
        if (!I_ij) {
          next
        }
        if (W_beta[i] > W_beta[j]) {
          C_csw_num <- C_csw_num + temp[j]
        }
        C_csw_denom <- C_csw_denom + temp[j]
      }
    }
    return(C_csw_num / C_csw_denom)
  }


#' Extract Variables from the Right-hand Side of a Model Formula.
#'
#' @param model a model formula.
#'
#' @return names of variables on the right-hand side of a
#' model formula.
#'
#' @noRd
extract_rhs_values <- function(model) {
  # Extract the model formula
  model_formula <- formula(model)
  # Get the right-hand side (RHS) variables from the formula
  rhs_vars <- attr(terms(model_formula), "term.labels")
  # Extract the model call
  model_call <- model$call
  # Check if the model call contains the data argument
  if (!"data" %in% names(model_call)) {
    stop("Error: Model call does not reference a dataset.")
  }
  # Evaluate the data argument from the model call in the model's environment
  data <- eval(model_call$data, environment(model$terms))
  if (is.null(data)) {
    stop("Error: Could not evaluate dataset from model call.")
  }
  # Extract the RHS variable values from the dataset
  rhs_values <- data[rhs_vars]
  return(rhs_values)
}


identify_missing <- function(formula, data) {
  mf <- model.frame(formula = formula, data = data, na.action = na.omit)
  omitted <- attr(mf, "na.action")
  as.numeric(omitted)
}

#' Number of parameters in fitted mixture cure model
#'
#' @description
#' This function returns the number of parameters in a user-specified model
#' criterion or step for a \code{curegmifs}, \code{cureem},
#' \code{cv_curegmifs} or \code{cv_cureem} fitted object.
#'
#' @param object a \code{mixturecure} object resulting from \code{curegmifs},
#' \code{cureem}, \code{cv_curegmifs}, \code{cv_cureem}.
#' @param model_select either a case-sensitive parameter for models fit using
#' \code{curegmifs} or \code{cureem} or any numeric step along the solution path
#' can be selected. The default is \code{model_select = "AIC"} which calculates
#' the predicted values using the coefficients from the model achieving the
#' minimum AIC. The complete list of options are:
#' \itemize{
#'     \item \code{"AIC"} for the minimum AIC (default).
#'     \item \code{"mAIC"} for the minimum modified AIC.
#'     \item \code{"cAIC"} for the minimum corrected AIC.
#'     \item \code{"BIC"}, for the minimum BIC.
#'     \item \code{"mBIC"} for the minimum modified BIC.
#'     \item \code{"EBIC"} for the minimum extended BIC.
#'     \item \code{"logLik"} for the step that maximizes the
#' log-likelihood.
#'     \item \code{n} where n is any numeric value from the
#'     solution path.
#'   }
#' This option has no effect for objects fit using \code{cv_curegmifs} or
#' \code{cv_cureem}.
#'
#' @return number of paramaters of the fitted mixture cure model using the
#' specified criteria.
#'
#'
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
npar_mixturecure <- function (object, model_select = "AIC")
{
  coef <- coef(object, model_select = model_select)
  n <- length(coef$rate) + length(coef$shape) + length(coef$b0) +
    sum(coef$beta_inc != 0) + sum(coef$beta_lat != 0)
  return(n)
}

test_that("multiplication works", {
  output <- hdcuremodels:::sim_cure(100, mu = 1, censor_mu = 3, reps = 1000,
                                    seed = 13)
  expect_equal(round(output[2], 6), 0.983344)
})

#' Predicted probabilities for susceptibles, linear predictor for latency, and
#' risk class for latency for mixture cure fit
#'
#' @description
#' This function returns a list that includes the predicted probabilities for
#' susceptibles as well as the linear predictor for the latency distribution
#' and a dichotomous risk for latency for a \code{curegmifs}, \code{cureem},
#' \code{cv_curegmifs} or \code{cv_cureem} fitted object.
#'
#' @param object a \code{mixturecure} object resulting from \code{curegmifs},
#' \code{cureem}, \code{cv_curegmifs}, or \code{cv_cureem}.
#' @param newdata an optional data.frame that minimally includes the incidence
#' and/or latency variables to use for predicting the response. If omitted, the
#' training data are used.
#' @param model_select either a case-sensitive parameter for models fit using
#' \code{curegmifs} or \code{cureem} or any numeric step along the solution path
#' can be selected. The default is \code{model_select = "AIC"} which calculates
#' the predicted values using the coefficients from the model achieving the
#' minimum AIC. The complete list of options are:
#' \itemize{
#'     \item \code{"AIC"} for the minimum AIC (default).
#'     \item \code{"mAIC"} for the minimum modified AIC.
#'     \item \code{"cAIC"} for the minimum corrected AIC.
#'     \item \code{"BIC"}, for the minimum BIC.
#'     \item \code{"mBIC"} for the minimum modified BIC.
#'     \item \code{"EBIC"} for the minimum extended BIC.
#'     \item \code{"logLik"} for the step that maximizes the
#' log-likelihood.
#'     \item \code{n} where n is any numeric value from the
#'     solution path.
#'   }
#' This option has no effect for objects fit using \code{cv_curegmifs} or
#' \code{cv_cureem}.
#' @param ... other arguments
#'
#' @return \item{p_uncured}{ a vector of probabilities from the incidence
#' portion of the fitted model representing the P(uncured).}
#' @return \item{linear_latency}{ a vector for the linear predictor from the
#' latency portion of the model.}
#' @return \item{latency_risk}{ a dichotomous class representing low (below the
#' median) versus high risk for the latency portion of the model.}
#'
#' @seealso \code{\link[hdcuremodels]{curegmifs}}, \code{\link[hdcuremodels]{cureem}},
#' \code{\link[hdcuremodels]{coef.mixturecure}}, \code{\link[hdcuremodels]{summary.mixturecure}},
#' \code{\link[hdcuremodels]{plot.mixturecure}}
#' @keywords methods
#' @method predict mixturecure
#'
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {RE4.9} *Modelled values of response variables.*
#' @export
#'
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' fit <- curegmifs(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "weibull", thresh = 1e-4, maxit = 2000,
#'   epsilon = 0.01, verbose = FALSE
#' )
#' predict_train <- predict(fit)
#' names(predict_train)
#' testing <- temp$testing
#' predict_test <- predict(fit, newdata = testing)
predict.mixturecure <- function(object, newdata, model_select = "AIC", ...) {
  if (!("mixturecure" %in% class(object))) {
    stop("Error: class of object must be mixturecure")
  }
  no_data <- (missing(newdata) || is.null(newdata))
  if (no_data) {
    x_inc <- object$x_incidence
    x_lat <- object$x_latency
  } else {
    x_inc <- as.matrix(model.frame(as.formula(paste(
      " ~ ",
      paste(colnames(object$x_incidence), collapse = "+")
    )), newdata))
    x_lat <- as.matrix(model.frame(as.formula(paste(
      " ~ ",
      paste(colnames(object$x_latency), collapse = "+")
    )), newdata))
  }
  # Scale x_inc if scale is TRUE; match to train if test data used
  if (!is.null(x_inc) && identical(x_inc, object$x_incidence)) {
    if (object$scale) {
      x_inc <- self_scale(x_inc, object$scale)
    }
  } else if (!is.null(x_inc) && object$scale) {
    newx <- rbind(object$x_incidence, x_inc)
    newx <- self_scale(newx, object$scale)
    x_inc <- as.matrix(newx[-(seq_len(dim(object$x_incidence)[1])), ,
                          drop = FALSE
    ])
  }
  # Scale x_lat if scale is TRUE; match to train if test data used
  if (!is.null(x_lat) && identical(x_lat, object$x_latency)) {
    if (object$scale) {
      x_lat <- self_scale(x_lat, object$scale)
    }
  } else if (!is.null(x_lat) && object$scale) {
    newx <- rbind(object$x_latency, x_lat)
    newx <- self_scale(newx, object$scale)
    x_lat <- as.matrix(newx[-(seq_len(dim(object$x_latency)[1])), , drop = FALSE])
  }
  if (!object$cv) {
    if (!is.numeric(model_select))
      model_select <- select_model(object, model_select)$select
    if (is.null(x_inc)) {
      p_hat <- 1 / (1 + exp(-object$b0_path[model_select]))
    } else {
      p_hat <- as.numeric(1 / (1 + exp(-object$b0_path[model_select] -
        x_inc %*%
        t(object$b_path[model_select, ,
          drop = FALSE]))))
    }
    if (is.null(x_lat)) {
      w_beta <- rep(0, dim(x_inc)[1])
    } else {
      w_beta <- as.numeric(x_lat %*% t(object$beta_path[model_select, ,
        drop = FALSE]))
      latency <- ifelse(w_beta < 0, "low risk", "high risk")
    }
    output <- list(p_uncured = p_hat, linear_latency = w_beta,
      latency_risk = latency)
    } else {
      if (is.null(x_inc)) {
        p_hat <- 1 / (1 + exp(-object$b0))
      } else {
        p_hat <- 1 / (1 + exp(-object$b0 - x_inc %*% matrix(object$b,
        ncol = 1)))
      }
      if (is.null(x_lat)) {
        w_beta <- rep(0, dim(x_inc)[1])
      } else {
        w_beta <- as.numeric(x_lat %*% matrix(object$beta,
        ncol = 1))
        latency <- ifelse(w_beta < 0, "low risk", "high risk")
      }
      output <- list(
      p_uncured = as.numeric(p_hat), linear_latency = w_beta,
      latency_risk = latency)
    }
  output
}

#' Fit penalized mixture cure model using the E-M algorithm with
#' cross-validation for parameter tuning
#'
#' @description
#' Fits penalized parametric and semi-parametric mixture cure models (MCM)
#' using the E-M algorithm with with k-fold cross-validation for parameter
#' tuning. The lasso (L1), MCP and SCAD penalty are supported for the Cox MCM
#' while only lasso is currently supported for parametric MCMs. When FDR
#' controlled variable selection is used, the model-X knockoffs method is
#' applied and indices of selected variables are returned.
#'
#' @param formula an object of class "\code{formula}" (or one that can be
#' coerced to that class): a symbolic description of the model to be fitted.
#' The response must be a survival object as returned by the \code{Surv}
#' function while the variables on the right side of the formula are the
#' covariates that are included in the incidence portion of the model.
#' @param data a data.frame in which to interpret the variables named in
#' the \code{formula} or in the \code{subset} argument. Rows with missing data are
#' omitted (only \code{na.action = na.omit} is operational) therefore users may
#' want to impute missing data prior to calling this function.
#' @param subset an optional expression indicating which subset of observations
#' to be used in the fitting process, either a numeric or factor variable
#' should be used in subset, not a character variable. All observations are
#' included by default.
#' @param x_latency specifies the variables to be included in the latency
#' portion of the model and can be either a matrix of predictors, a model
#' formula with the right hand side specifying the latency variables, or the
#' same data.frame passed to the \code{data} parameter. Note that when using
#' the model formula syntax for \code{x_latency} it cannot handle
#' \code{x_latency = ~ .}.
#' @param model type of regression model to use for the latency portion of
#' mixture cure model. Can be "cox", "weibull", or "exponential" (default is
#' "cox").
#' @param penalty type of penalty function. Can be "lasso", "MCP", or "SCAD"
#' (default is "lasso").
#' @param penalty_factor_inc vector of binary indicators representing the
#' penalty to apply to each incidence coefficient: 0 implies no shrinkage and
#' 1 implies shrinkage. If not supplied, 1 is applied to all incidence
#' variables.
#' @param penalty_factor_lat vector of binary indicators representing the
#' penalty to apply to each latency coefficient: 0 implies no shrinkage and 1
#' implies shrinkage. If not supplied, 1 is applied to all latency variables.
#' @param fdr_control logical, if TRUE, model-X knockoffs are used for
#' FDR-controlled variable selection and indices of selected variables are
#' returned (default is FALSE).
#' @param fdr numeric value in (0, 1) range specifying the target FDR level to
#' use for variable selection when \code{fdr_control = TRUE} (default is 0.2).
#' @param grid_tuning logical, if TRUE a 2-D grid tuning approach is used to
#' select the optimal pair of \eqn{\lambda_b} and \eqn{\lambda_{\beta}} penalty
#' parameters for the incidence and latency portions of the model, respectively.
#' Otherwise the \eqn{\lambda_b} and \eqn{\lambda_{\beta}} are selected from a
#' 1-D sequence and are equal to one another (default is FALSE).
#' @param thresh small numeric value. The iterative process stops when the
#' differences between successive expected penalized complete-data
#' log-likelihoods for both incidence and latency components are less than this
#' specified level of tolerance (default is 10^-3).
#' @param scale logical, if TRUE the predictors are centered and scaled.
#' @param maxit maximum number of passes over the data for each lambda. If not
#' specified, 100 is applied when \code{penalty = "lasso"} and 1000 is applied
#' when \code{penalty = "MCP"} or \code{penalty = "SCAD"}.
#' @param inits an optional list specifiying the initial values to be used for
#' model fitting as follows:
#' \itemize{
#' \item \code{itct} the incidence intercept.
#' \item \code{b_u} a numeric vector for the unpenalized.
#' incidence coefficients for the incidence portion of the model.
#' \item \code{beta_u} a numeric vector for unpenalized
#' latency coefficients in the incidence portion of the model.
#' \item \code{lambda} a numeric value for the rate parameter when fitting
#' either a Weibull or exponential MCM using \code{model = "weibull"} or
#' \code{model = "exponential"}.
#' \item \code{alpha} a numeric value for the shape parameter when fitting a
#' Weibull MCM using \code{model = "weibull"}.
#' \item \code{survprob} a numeric vector for the
#' latency survival probabilities \eqn{S_u(t_i|w_i)} for i=1,...,N when fitting
#' a Cox MCM \code{model = "cox"}.
#' }
#' Penalized coefficients are initialized to zero. If \code{inits} is not specified or improperly specified, initialization is
#' automatically provided by the function.
#' @param lambda_inc_list a numeric vector used to search for the optimal
#' \eqn{\lambda_b} tuning parameter. If not supplied, the function computes a
#' \eqn{\lambda_b} sequence based on \code{nlambda_inc} and
#' \code{lambda_min_ratio_inc}. If \code{grid_tuning = FALSE}, the same sequence
#' should be used for both \eqn{\lambda_b} and \eqn{\lambda_{\beta}}.
#' @param lambda_lat_list a numeric vector used to search for the optimal
#' \eqn{\lambda_{\beta}} tuning parameter. If not supplied, the function
#' computes a \eqn{\lambda_{\beta}} sequence based on \code{nlambda_lat} and
#' \code{lambda_min_ratio_lat}. If \code{grid_tuning = FALSE}, the same sequence
#' should be used for both \eqn{\lambda_b} and \eqn{\lambda_{\beta}}.
#' @param nlambda_inc an integer specifying the number of values to search for
#' the optimal \eqn{\lambda_b} tuning parameter; default is 10 if
#' \code{grid_tuning = TRUE} and 50 otherwise.
#' @param nlambda_lat an integer specifying the number of values to search
#' for the optimal \eqn{\lambda_{\beta}} tuning parameter; default is 10 if
#' \code{grid_tuning = TRUE} and 50 otherwise.
#' @param gamma_inc numeric value for the penalization parameter \eqn{\gamma}
#' for variables in the incidence portion of the model when
#' \code{penalty = "MCP"} or \code{penalty = "SCAD"} (default is 3).
#' @param gamma_lat numeric value for the penalization parameter \eqn{\gamma}
#' for variables in the latency portion of the model when \code{penalty = "MCP"}
#' or \code{penalty = "SCAD"} (default is 3).
#' @param lambda_min_ratio_inc numeric value in (0,1) representing the smallest
#' value for \eqn{\lambda_b} as a fraction of \code{lambda.max_inc}, the
#' data-derived entry value at which essentially all penalized variables in the
#' incidence portion of the model have a coefficient estimate of 0 (default is
#' 0.1).
#' @param lambda_min_ratio_lat numeric value in (0.1) representing the smallest
#' value for \eqn{\lambda_{\beta}} as a fraction of \code{lambda.max_lat}, the
#' data-derived entry value at essentially all penalized variables in the
#' latency portion of the model have a coefficient estimate of 0 (default is
#' 0.1).
#' @param n_folds an integer specifying the number of folds for the k-fold
#' cross-valiation procedure (default is 5).
#' @param measure_inc character string specifying the evaluation criterion used
#' in selecting the optimal \eqn{\lambda_b} which can be either
#' \itemize{
#' \item \code{"c"} specifying to use the C-statistic for cure status
#' weighting (CSW) method proposed by Asano and Hirakawa (2017) to
#' select both \eqn{\lambda_b} and \eqn{\lambda_{\beta}}
#' \item \code{"auc"} specifying to use the AUC for cure prediction using the
#' mean score imputation (MSI) method proposed by Asano et al. (2014) to select
#' \eqn{\lambda_b} while the C-statistic with CSW is used for
#' \eqn{\lambda_{\beta}}.
#' }
#' @param one_se logical, if TRUE then the one standard error rule is applied
#' for selecting the optimal parameters. The one standard error rule selects the
#' most parsimonious model having evaluation criterion no more than one standard
#' error worse than that of the best evaluation criterion (default is FALSE).
#' @param cure_cutoff numeric value representing the cutoff time value that
#' represents subjects not experiencing the event by this time are cured. This
#' value is used to produce a proxy for the unobserved cure status when
#' calculating C-statistic and AUC (default is 5 representing 5 years). Users
#' should be careful to note the time scale of their data and adjust this
#' according to the time scale and clinical application.
#' @param parallel logical. If TRUE, parallel processing is performed for K-fold
#' CV using \code{foreach} and the \pkg{doParallel} package is required.
#' @param seed optional integer representing the random seed. Setting the random
#' seed fosters reproducibility of the results.
#' @param verbose logical, if TRUE running information is printed to the console
#' (default is FALSE).
#' @param na.action this function requires complete data so \code{"na.omit"} is
#' invoked. Users can impute missing data as an alternative prior to model fitting.
#' @param ... additional arguments.
#'
#' @return \item{b0}{Estimated intercept for the incidence portion of the
#' model.}
#' @return \item{b}{Estimated coefficients for the incidence portion of the
#' model.}
#' @return \item{beta}{Estimated coefficients for the latency portion of the
#' model.}
#' @return \item{alpha}{Estimated shape parameter if the Weibull model is fit.}
#' @return \item{rate}{Estimated rate parameter if the Weibull or exponential
#' model is fit.}
#' @return \item{logLik_inc}{Expected penalized complete-data log-likelihood for
#' the incidence portion of the model.}
#' @return \item{logLik_lat}{Expected penalized complete-data log-likelihood for
#' the latency portion of the model.}
#' @return \item{selected_lambda_inc}{Value of \eqn{\lambda_b} selected using
#' cross-validation. NULL when fdr_control is TRUE.}
#' @return \item{selected_lambda_lat}{Value of \eqn{\lambda_{\beta}} selected
#' using cross-validation. NULL when fdr_control is TRUE.}
#' @return \item{max_c}{Maximum C-statistic achieved.}
#' @return \item{max_auc}{Maximum AUC for cure prediction achieved; only output
#' when \code{measure_inc="auc"}.}
#' @return \item{selected_index_inc }{Indices of selected variables for the
#' incidence portion of the model when \code{fdr_control=TRUE}. If no variables
#' are selected, \code{int(0)} will be returned.}
#' @return \item{selected_index_lat }{Indices of selected variables for the
#' latency portion of the model when \code{fdr_control=TRUE}. If no variables
#' are selected, \code{int(0)} will be returned.}
#' @return \item{call}{the matched call.}

#' @export
#'
#' @import knockoff
#' @import doParallel
#' @import parallel
#' @import stats
#' @import survival
#' @import glmnet
#' @importFrom methods is
#'
#' @references Archer, K. J., Fu, H., Mrozek, K., Nicolet, D., Mims, A. S.,
#' Uy, G. L., Stock, W., Byrd, J. C., Hiddemann, W., Braess, J.,
#' Spiekermann, K., Metzeler, K. H., Herold, T., Eisfeld, A.-K. (2024)
#' Identifying long-term survivors and those at higher or lower risk of relapse
#' among patients with cytogenetically normal acute myeloid leukemia using a
#' high-dimensional mixture cure model. \emph{Journal of Hematology & Oncology},
#' \bold{17}:28.
#'
#' @srrstats {G1.0} *Statistical Software should list at least one primary reference from published academic literature.*
#' @srrstats {G1.1} *Statistical Software should document whether the algorithm(s) it implements are: An improvement on other implementations of similar algorithms in **R**.
#' @srrstats {G2.1} *Implement assertions on types of inputs (see the initial point on nomenclature above).*
#' @srrstats {G2.3a} *Use `match.arg()` or equivalent where applicable to only permit expected values.*
#' @srrstats {G2.1a} *Provide explicit secondary documentation of expectations on data types of all vector inputs.*
#' @srrstats {G2.2} *Appropriately prohibit or restrict submission of multivariate input to parameters expected to be univariate.*
#' @srrstats {G2.3} *For univariate character input:*
#' @srrstats {G2.3b} *Either: use `tolower()` or equivalent to ensure input of character parameters is not case dependent; or explicitly document that parameters are strictly case-sensitive.*
#' @srrstats {G2.4} *Provide appropriate mechanisms to convert between different data types, potentially including:*
#' @srrstats {G2.4e} *explicit conversion from factor via `as...()` functions*
#' @srrstats {G2.10} *Software should ensure that extraction or filtering of single columns from tabular inputs should not presume any particular default behaviour, and should ensure all column-extraction operations behave consistently regardless of the class of tabular data used as input.*
#' @srrstats {G2.13} *Statistical Software should implement appropriate checks for missing data as part of initial pre-processing prior to passing data to analytic algorithms.*
#' @srrstats {G5.2} *Appropriate error and warning behaviour of all functions should be explicitly demonstrated through tests. In particular,*
#' @srrstats {G5.2a} *Every message produced within R code by `stop()`, `warning()`, `message()`, or equivalent should be unique*
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*
#' @srrstats {G5.6} **Parameter recovery tests** *to test that the implementation produce expected results given data with known properties. For instance, a linear regression algorithm should return expected coefficient values for a simulated data set generated from a linear model.*
#' @srrstats {G5.9} **Noise susceptibility tests** *Packages should test for expected stochastic behaviour, such as through the following conditions:*
#' @srrstats {G5.9a} *Adding trivial noise (for example, at the scale of `.Machine$double.eps`) to data does not meaningfully change results*
#' @srrstats {G5.9b} *Running under different random seeds or initial conditions does not meaningfully change results*
#' @srrstats {RE1.0} *Regression Software should enable models to be specified via a formula interface, unless reasons for not doing so are explicitly documented.*
#' @srrstats {RE1.1} *Regression Software should document how formula interfaces are converted to matrix representations of input data.*
#' @srrstats {RE1.2} *Regression Software should document expected format (types or classes) for inputting predictor variables, including descriptions of types or classes which are not accepted.*
#' @srrstats {RE1.3} *Regression Software which passes or otherwise transforms aspects of input data onto output structures should ensure that those output structures retain all relevant aspects of input data, notably including row and column names, and potentially information from other `attributes()`.*
#' @srrstats {RE1.3a} *Where otherwise relevant information is not transferred, this should be explicitly documented.*
#' @srrstats {RE1.4} *Regression Software should document any assumptions made with regard to input data; for example distributional assumptions, or assumptions that predictor data have mean values of zero. Implications of violations of these assumptions should be both documented and tested.*
#' @srrstats {RE2.3} *Where applicable, Regression Software should enable data to be centred (for example, through converting to zero-mean equivalent values; or to z-scores) or offset (for example, to zero-intercept equivalent values) via additional parameters, with the effects of any such parameters clearly documented and tested.*
#' @srrstats {RE4.0} *Regression Software should return some form of "model" object, generally through using or modifying existing class structures for model objects (such as `lm`, `glm`, or model objects from other packages), or creating a new class of model objects.*
#' @srrstats {RE4.4} *The specification of the model, generally as a formula (via `formula()`)*
#' @srrstats {RE4.8} *Response variables, and associated "metadata" where applicable.*
#' @seealso \code{\link{cureem}}
#'
#' @keywords models
#' @keywords regression
#'
#' @examples
#' \dontrun{
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 200, j = 25, n_true = 5, a = 1.8, rho = 0.2)
#' training <- temp$training
#' # Fit a penalized Cox MCM selecting parameters using 2-fold CV
#' fit.cv <- cv_cureem(Surv(Time, Censor) ~ .,
#'   data = training,
#'   x_latency = training, fdr_control = FALSE,
#'   grid_tuning = FALSE, nlambda_inc = 10, nlambda_lat = 10,
#'   n_folds = 2, seed = 23, verbose = FALSE
#' )
#' fit.cv.fdr <- cv_cureem(Surv(Time, Censor) ~ .,
#'    data = training,
#'    x_latency = training, model = "weibull", penalty = "lasso",
#'    fdr_control = TRUE, grid_tuning = FALSE, nlambda_inc = 10,
#'    nlambda_lat = 10, n_folds = 2, seed = 23, verbose = TRUE
#'    )
#' }
cv_cureem <- function(formula, data, subset, x_latency = NULL,
                      model = c("cox", "weibull", "exponential"),
                      penalty = c("lasso", "MCP", "SCAD"),
                      penalty_factor_inc = NULL, penalty_factor_lat = NULL,
                      fdr_control = FALSE, fdr = 0.2, grid_tuning = FALSE,
                      thresh = 1e-03, scale = TRUE, maxit = NULL, inits = NULL,
                      lambda_inc_list = NULL, lambda_lat_list = NULL,
                      nlambda_inc = NULL, nlambda_lat = NULL, gamma_inc = 3,
                      gamma_lat = 3,
                      lambda_min_ratio_inc = 0.1, lambda_min_ratio_lat = 0.1,
                      n_folds = 5, measure_inc = c("c", "auc"), one_se = FALSE,
                      cure_cutoff = 5, parallel = FALSE, seed = NULL,
                      verbose = TRUE, na.action = na.omit, ...) {
  mf <- match.call(expand.dots = FALSE)
  cl <- match.call()
  m <- match(c("formula", "data", "subset", "na.action"), names(mf), 0L)
  if (m[1] == 0) stop("Error: A \"formula\" argument is required")
  mf <- mf[c(1L, m)]
  mf[[1L]] <- as.name("model.frame")
  if (missing(data)) {
    mf[["data"]] <- environment(formula)
  }
  if (missing(data)) {
    data <- environment(formula)
  }
  mf <- eval(mf, parent.frame())
  mt <- attr(mf, "terms")
  omitted <- attr(mf, "na.action")
  model <- match.arg(model)
  penalty <- match.arg(penalty)
  measure_inc <- match.arg(measure_inc)
  y <- model.response(mf)
  event <- y[, 2]
  time <- y[, 1]
  x <- model.matrix(mt, mf)
  if (!is.null(x_latency)) {
    if (missing(subset)) {
      r <- rep(TRUE, dim(data)[1])
      r[omitted] <- FALSE
    } else {
      e <- substitute(subset)
      r <- eval(e, data)
      if (!is.logical(r)) {
        stop("Error: 'subset' must evaluate to logical")
      }
      r <- r & !is.na(r)
      r[omitted] <- FALSE
    }
    if ("character" %in% is(x_latency) || "numeric" %in% is(x_latency)) {
      nl <- as.list(seq_len(ncol(data)))
      names(nl) <- names(data)
      vars <- eval(substitute(x_latency), nl, parent.frame())
      x_latency <- data[r, vars, drop = FALSE]
      x_latency <- as.matrix(x_latency)
    } else if ("matrix" %in% is(x_latency) || "data.frame" %in% is(x_latency)) {
      text <- parse(text = cl)[[2]]
      survnames <- strsplit(as.character(text), ",")
      time_name <- substr(survnames[[2]][1], 6, nchar(survnames[[2]][1]))
      censor_name <- trimws(strsplit(survnames[[2]][2], ")")[[1]][1])
      x_latency <- x_latency[r, !(colnames(x_latency) %in%
        c(time_name, censor_name)), drop = FALSE]
      x_latency <- as.matrix(x_latency)
    } else if ("formula" %in% is(x_latency)) {
      x_latency <- model.matrix(update.formula(x_latency,
        new = ~ . - 1,
        data = data
      ), data)
    }
  }
  x_inc <- x
  is_intercept <- grep("Intercept", dimnames(x_inc)[[2]])
  if (length(is_intercept) == 1) {
    x_inc <- x_inc[, -is_intercept, drop = FALSE]
  }
  x_lat <- x_latency
  if (is.null(maxit)) maxit <- ifelse(penalty == "lasso", 100, 1000)
  if (grid_tuning) {
    if (is.null(nlambda_inc)) nlambda_inc <- 10
    if (is.null(nlambda_lat)) nlambda_lat <- 10
  } else {
    if (is.null(nlambda_inc)) nlambda_inc <- 50
    if (is.null(nlambda_inc)) nlambda_lat <- 50
  }
  if (nrow(x_inc) != nrow(x_lat) || nrow(x_lat) != length(time) ||
    length(time) != length(event)) {
    stop("Error: Input dimension mismatch")
  }
  if (is.null(penalty_factor_inc)) {
    penalty_factor_inc <- rep(1, ncol(x_inc))
  }
  if (is.null(penalty_factor_lat)) {
    penalty_factor_lat <- rep(1, ncol(x_lat))
  }
  if (any(!c(class(penalty_factor_inc), class(penalty_factor_lat)) %in% "numeric")) {
    stop("Error: Penalty factors specified in penalty_factor_inc and penalty_factor_inc
         must be numeric vectors comprised of 0 or 1")
  }
  if (any(!c(penalty_factor_inc, penalty_factor_inc) %in% c(0, 1))) {
    stop("Error: Penalty factors specified in penalty_factor_inc and penalty_factor_inc
         can only include 0 or 1")
  }
  if (any(c(lambda_inc_list, lambda_lat_list, gamma_inc, gamma_lat) <= 0)) {
    stop("Error: Penalty pamameters lambda and gamma should be positive")
  }
  if (fdr > 1 || fdr < 0) {
    stop("Error: FDR should be between 0 and 1")
  }
  if (lambda_min_ratio_inc > 1 || lambda_min_ratio_inc < 0 ||
    lambda_min_ratio_lat > 1 || lambda_min_ratio_lat < 0) {
    stop("Error: lambda_min_ratio_inc and lambda_min_ratio_lat should be between
         0 and 1")
  }
  if (any(c(lambda_inc_list, lambda_lat_list, gamma_inc, gamma_lat) <= 0)) {
    stop("Error: Penalty pamameters lambda and gamma should be positive")
  }
  if (!is.null(inits)) {
    inits <- inits_check(model,
      N = length(time), penalty_factor_inc,
      penalty_factor_lat, inits
    )
  }
  if (model != "cox" && penalty != "lasso") {
    warning("Warning: MCP/SCAD penalized parametric models are not currently supported.
            An L1 penalized model was fitted instead.")
    penalty <- "lasso"
  }
  x_u <- self_scale(x_inc[, penalty_factor_inc == 0, drop = FALSE], scale)
  x_p <- self_scale(x_inc[, penalty_factor_inc == 1, drop = FALSE], scale)
  w_u <- self_scale(x_lat[, penalty_factor_lat == 0, drop = FALSE], scale)
  w_p <- self_scale(x_lat[, penalty_factor_lat == 1, drop = FALSE], scale)
  if (fdr_control) {
    res <- cv.em.fdr(x_u, x_p, w_u, w_p, time, event, model, penalty, fdr,
      thresh,
      nIter = maxit, penalty_factor_inc,
      penalty_factor_lat, grid_tuning, lambda_inc_list,
      lambda_lat_list, nlambda_inc, nlambda_lat,
      lambda_min_ratio_inc, lambda_min_ratio_lat,
      gamma_inc, gamma_lat, inits, n_folds, measure_inc, one_se,
      cure_cutoff, parallel, seed, verbose
    )
    if (!is.null(x_u)) {
      b <- rep(NA, ncol(x_inc))
      b[penalty_factor_inc == 0] <- res$b[1:ncol(x_u)]
      b[penalty_factor_inc == 1] <- res$b[(ncol(x_u) + 1):(length(res$b))]
    } else {
      b <- res$b
    }
    if (!is.null(w_u)) {
      beta <- rep(NA, ncol(x_lat))
      beta[penalty_factor_lat == 0] <- res$beta[1:ncol(w_u)]
      beta[penalty_factor_lat == 1] <- res$beta[(ncol(w_u) + 1):
      (length(res$beta))]
    } else {
      beta <- res$beta
    }
    names(b) <- colnames(x_inc)
    names(beta) <- colnames(x_latency)
    output <- list(
      b0 = res$b0, b = b, beta = beta, rate = res$rate,
      alpha = res$alpha,
      selected_index_inc =
        (seq_len(ncol(x_inc)))[penalty_factor_inc == 1][
          res$selected_b
        ],
      selected_index_lat =
        (seq_len(ncol(x_lat)))[penalty_factor_lat == 1][
          res$selected_beta
        ]
    )
    if (!is.null(colnames(x_inc))) {
      names(output$selected_index_inc) <-
        colnames(x_inc)[output$selected_index_inc]
    }
    if (!is.null(colnames(x_lat))) {
      names(output$selected_index_lat) <-
        colnames(x_lat)[output$selected_index_lat]
    }
  } else {
    res <- cv.em.nofdr(x_u, x_p, w_u, w_p, time, event, model, penalty,
      thresh,
      nIter = maxit, grid_tuning, lambda_inc_list,
      lambda_lat_list, nlambda_inc, nlambda_lat,
      lambda_min_ratio_inc, lambda_min_ratio_lat, gamma_inc,
      gamma_lat, inits, n_folds, measure_inc, one_se,
      cure_cutoff, parallel, seed, verbose
    )
    if (!is.null(x_u)) {
      b <- rep(NA, ncol(x_inc))
      b[penalty_factor_inc == 0] <- res$b[1:ncol(x_u)]
      b[penalty_factor_inc == 1] <- res$b[(ncol(x_u) + 1):(length(res$b))]
    } else {
      b <- res$b
    }
    if (!is.null(w_u)) {
      beta <- rep(NA, ncol(x_lat))
      beta[penalty_factor_lat == 0] <- res$beta[1:ncol(w_u)]
      beta[penalty_factor_lat == 1] <-
        res$beta[(ncol(w_u) + 1):(length(res$beta))]
    } else {
      beta <- res$beta
    }
    names(b) <- colnames(x_inc)
    names(beta) <- colnames(x_latency)
    output <- res
    output$b <- b
    output$beta <- beta
  }
  output$method <- "EM"
  output$model <- model
  output$penalty <- penalty
  output$cv <- TRUE
  output$y <- y
  output$x_incidence <- x_inc
  output$x_latency <- x_latency
  output$scale <- scale
  output$call <- cl
  output$fdr_control <- fdr_control
  class(output) <- "mixturecure"
  return(output)
}

#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*
#' @srrstats {G5.6} **Parameter recovery tests** *to test that the implementation produce expected results given data with known properties. For instance, a linear regression algorithm should return expected coefficient values for a simulated data set generated from a linear model.*
#' @srrstats {G5.9} **Noise susceptibility tests** *Packages should test for expected stochastic behaviour, such as through the following conditions:*
#' @srrstats {G5.9a} *Adding trivial noise (for example, at the scale of `.Machine$double.eps`) to data does not meaningfully change results*

test_that("cv_cureem function works correctly", {
  library(survival)
  withr::local_seed(1234)
  temp <- generate_cure_data(n = 200, j = 25, n_true = 5, a = 1.8)
  training <- temp$training
  fit.cv <- cv_cureem(Surv(Time, Censor) ~ .,
                      data = training,
                      x_latency = training, fdr_control = FALSE,
                      grid_tuning = FALSE, nlambda_inc = 10, nlambda_lat = 10,
                      n_folds = 2, seed = 23, verbose = FALSE
  )
  fit.cv %>% expect_s3_class("mixturecure")
  expect_setequal(names(fit.cv), c("b0", "b", "beta", "logLik.inc", "logLik.lat",
                                   "selected_lambda_inc", "selected_lambda_lat",
                                   "max_c", "method", "model", "penalty", "cv",
                                   "y", "x_incidence", "x_latency",
                                   "scale", "call", "fdr_control"))
  fit.cv$b0 %>% expect_type("double")
  fit.cv$b %>% expect_type("double")
  fit.cv$beta %>% expect_type("double")
  fit.cv$logLik.inc %>% expect_type("double")
  fit.cv$logLik.lat %>% expect_type("double")
  fit.cv$selected_lambda_inc %>% expect_type("double")
  fit.cv$selected_lambda_lat %>% expect_type("double")
  fit.cv$max_c %>% expect_type("double")
  fit.cv$x_incidence %>% expect_type("double")
  fit.cv$x_latency %>% expect_type("double")
  fit.cv$y %>% expect_type("double")
  fit.cv$model %>% expect_type("character")
  fit.cv$scale %>% expect_type("logical")
  fit.cv$method %>% expect_type("character")
  fit.cv$penalty %>% expect_type("character")
  fit.cv$cv %>% expect_type("logical")
  fit.cv$fdr_control %>% expect_type("logical")
  fit.cv$call %>% expect_type("language")
  fit.cv$b %>% expect_length(dim(fit.cv$x_incidence)[2])
  fit.cv$beta %>% expect_length(dim(fit.cv$x_latency)[2])
  fit.cv$b0 %>% expect_length(1)

  fit.cv.exp <- cv_cureem(Surv(Time, Censor) ~ .,
                      data = training, model = "exponential",
                      x_latency = training, fdr_control = FALSE,
                      grid_tuning = FALSE, nlambda_inc = 10, nlambda_lat = 10,
                      n_folds = 2, seed = 23, verbose = FALSE
  )
  fit.cv.exp %>% expect_s3_class("mixturecure")
  expect_setequal(names(fit.cv.exp), c("b0", "b", "beta", "rate", "logLik.inc",
                                       "logLik.lat",
                                   "selected_lambda_inc", "selected_lambda_lat",
                                   "max_c", "method", "model", "penalty", "cv",
                                   "y", "x_incidence", "x_latency",
                                   "scale", "call", "fdr_control"))
  fit.cv.exp$b0 %>% expect_type("double")
  fit.cv.exp$b %>% expect_type("double")
  fit.cv.exp$beta %>% expect_type("double")
  fit.cv.exp$rate %>% expect_type("double")
  fit.cv.exp$logLik.inc %>% expect_type("double")
  fit.cv.exp$logLik.lat %>% expect_type("double")
  fit.cv.exp$selected_lambda_inc %>% expect_type("double")
  fit.cv.exp$selected_lambda_lat %>% expect_type("double")
  fit.cv.exp$max_c %>% expect_type("double")
  fit.cv.exp$x_incidence %>% expect_type("double")
  fit.cv.exp$x_latency %>% expect_type("double")
  fit.cv.exp$y %>% expect_type("double")
  fit.cv.exp$model %>% expect_type("character")
  fit.cv.exp$scale %>% expect_type("logical")
  fit.cv.exp$method %>% expect_type("character")
  fit.cv.exp$penalty %>% expect_type("character")
  fit.cv.exp$cv %>% expect_type("logical")
  fit.cv.exp$fdr_control %>% expect_type("logical")
  fit.cv.exp$call %>% expect_type("language")
  fit.cv.exp$b %>% expect_length(dim(fit.cv$x_incidence)[2])
  fit.cv.exp$beta %>% expect_length(dim(fit.cv$x_latency)[2])
  fit.cv.exp$b0 %>% expect_length(1)
  fit.cv.exp$rate %>% expect_length(1)

  fit.cv.wei <- cv_cureem(Surv(Time, Censor) ~ .,
                          data = training, model = "weibull", measure_inc = "auc",
                          x_latency = training, fdr_control = FALSE,
                          grid_tuning = FALSE, nlambda_inc = 10, nlambda_lat = 10,
                          n_folds = 2, seed = 23, verbose = FALSE
  )
  fit.cv.wei %>% expect_s3_class("mixturecure")
  expect_setequal(names(fit.cv.wei), c("b0", "b", "beta", "rate", "alpha",
                                       "logLik.inc", "logLik.lat",
                                       "selected_lambda_inc", "selected_lambda_lat",
                                       "max_c", "max.auc", "method", "model", "penalty",
                                       "cv", "y", "x_incidence", "x_latency",
                                       "scale", "call", "fdr_control"))
  fit.cv.wei$b0 %>% expect_type("double")
  fit.cv.wei$b %>% expect_type("double")
  fit.cv.wei$beta %>% expect_type("double")
  fit.cv.wei$rate %>% expect_type("double")
  fit.cv.wei$alpha %>% expect_type("double")
  fit.cv.wei$logLik.inc %>% expect_type("double")
  fit.cv.wei$logLik.lat %>% expect_type("double")
  fit.cv.wei$selected_lambda_inc %>% expect_type("double")
  fit.cv.wei$selected_lambda_lat %>% expect_type("double")
  fit.cv.wei$max_c %>% expect_type("double")
  fit.cv.wei$max.auc %>% expect_type("double")
  fit.cv.wei$x_incidence %>% expect_type("double")
  fit.cv.wei$x_latency %>% expect_type("double")
  fit.cv.wei$y %>% expect_type("double")
  fit.cv.wei$model %>% expect_type("character")
  fit.cv.wei$scale %>% expect_type("logical")
  fit.cv.wei$method %>% expect_type("character")
  fit.cv.wei$penalty %>% expect_type("character")
  fit.cv.wei$cv %>% expect_type("logical")
  fit.cv.wei$fdr_control %>% expect_type("logical")
  fit.cv.wei$call %>% expect_type("language")
  fit.cv.wei$b %>% expect_length(dim(fit.cv$x_incidence)[2])
  fit.cv.wei$beta %>% expect_length(dim(fit.cv$x_latency)[2])
  fit.cv.wei$b0 %>% expect_length(1)
  fit.cv.wei$rate %>% expect_length(1)
  fit.cv.wei$alpha %>% expect_length(1)
  fit.cv.wei$max.auc %>% expect_length(1)

  fit.cv.wei <- cv_cureem(Surv(Time, Censor) ~ .,
                          data = training, model = "weibull",
                          x_latency = training, fdr_control = FALSE,
                          grid_tuning = FALSE, nlambda_inc = 10, nlambda_lat = 10,
                          n_folds = 2, seed = 23, verbose = FALSE
  )
  fit.cv.wei %>% expect_s3_class("mixturecure")
  expect_setequal(names(fit.cv.wei), c("b0", "b", "beta", "rate", "alpha",
                                       "logLik.inc", "logLik.lat",
                                       "selected_lambda_inc", "selected_lambda_lat",
                                       "max_c", "method", "model", "penalty", "cv",
                                       "y", "x_incidence", "x_latency",
                                       "scale", "call", "fdr_control"))
  fit.cv.wei$b0 %>% expect_type("double")
  fit.cv.wei$b %>% expect_type("double")
  fit.cv.wei$beta %>% expect_type("double")
  fit.cv.wei$rate %>% expect_type("double")
  fit.cv.wei$alpha %>% expect_type("double")
  fit.cv.wei$logLik.inc %>% expect_type("double")
  fit.cv.wei$logLik.lat %>% expect_type("double")
  fit.cv.wei$selected_lambda_inc %>% expect_type("double")
  fit.cv.wei$selected_lambda_lat %>% expect_type("double")
  fit.cv.wei$max_c %>% expect_type("double")
  fit.cv.wei$x_incidence %>% expect_type("double")
  fit.cv.wei$x_latency %>% expect_type("double")
  fit.cv.wei$y %>% expect_type("double")
  fit.cv.wei$model %>% expect_type("character")
  fit.cv.wei$scale %>% expect_type("logical")
  fit.cv.wei$method %>% expect_type("character")
  fit.cv.wei$penalty %>% expect_type("character")
  fit.cv.wei$cv %>% expect_type("logical")
  fit.cv.wei$fdr_control %>% expect_type("logical")
  fit.cv.wei$call %>% expect_type("language")
  fit.cv.wei$b %>% expect_length(dim(fit.cv$x_incidence)[2])
  fit.cv.wei$beta %>% expect_length(dim(fit.cv$x_latency)[2])
  fit.cv.wei$b0 %>% expect_length(1)
  fit.cv.wei$rate %>% expect_length(1)
  fit.cv.wei$alpha %>% expect_length(1)

  fit.cv.fdr <- cv_cureem(Surv(Time, Censor) ~ .,
                          data = training,
                          x_latency = training, model = "weibull", penalty = "lasso",
                          fdr_control = TRUE, grid_tuning = FALSE, nlambda_inc = 10,
                          nlambda_lat = 10, n_folds = 2, seed = 23, verbose = TRUE
  )
  fit.cv.fdr %>% expect_s3_class("mixturecure")
  expect_setequal(names(fit.cv.fdr), c("b0", "b", "beta", "rate", "alpha",
                                   "selected_index_inc", "selected_index_lat",
                                   "method", "model", "penalty", "cv",
                                   "y", "x_incidence", "x_latency",
                                   "scale", "call", "fdr_control"))
  fit.cv.fdr$b0 %>% expect_type("double")
  fit.cv.fdr$b %>% expect_type("double")
  fit.cv.fdr$beta %>% expect_type("double")
  fit.cv.fdr$rate %>% expect_type("double")
  fit.cv.fdr$alpha %>% expect_type("double")
  fit.cv$selected_lambda_inc %>% expect_type("double")
  fit.cv$selected_lambda_lat %>% expect_type("double")
  fit.cv$x_incidence %>% expect_type("double")
  fit.cv$x_latency %>% expect_type("double")
  fit.cv$y %>% expect_type("double")
  fit.cv$model %>% expect_type("character")
  fit.cv$scale %>% expect_type("logical")
  fit.cv$method %>% expect_type("character")
  fit.cv$penalty %>% expect_type("character")
  fit.cv$cv %>% expect_type("logical")
  fit.cv$fdr_control %>% expect_type("logical")
  fit.cv$call %>% expect_type("language")
  fit.cv.fdr$b %>% expect_length(dim(fit.cv.fdr$x_incidence)[2])
  fit.cv.fdr$beta %>% expect_length(dim(fit.cv.fdr$x_latency)[2])
  fit.cv.fdr$b0 %>% expect_length(1)
  fit.cv.fdr$alpha %>% expect_length(1)
  fit.cv.fdr$rate %>% expect_length(1)
  expect_equal(round(fit.cv.fdr$b0, 5), 0.32659)
  expect_equal(round(fit.cv.fdr$rate, 5), 1.71472)
  expect_equal(round(fit.cv.fdr$alpha, 6), 0.752908)

  fit.scad <- cv_cureem(Surv(Time, Censor) ~ .,
                       data = training,
                       x_latency = training, model = "cox", penalty = "SCAD",
                       fdr_control = FALSE, grid_tuning = FALSE, nlambda_inc = 10,
                       nlambda_lat = 10, n_folds = 2, seed = 23, verbose = TRUE
  )
  fit.cv %>% expect_s3_class("mixturecure")
  expect_setequal(names(fit.cv), c("b0", "b", "beta", "logLik.inc", "logLik.lat",
                                   "selected_lambda_inc", "selected_lambda_lat",
                                   "max_c", "method", "model", "penalty", "cv",
                                   "y", "x_incidence", "x_latency",
                                   "scale", "call", "fdr_control"))
  fit.scad$b0 %>% expect_type("double")
  fit.scad$b %>% expect_type("double")
  fit.scad$beta %>% expect_type("double")
  fit.scad$logLik.inc %>% expect_type("double")
  fit.scad$logLik.lat %>% expect_type("double")
  fit.scad$selected_lambda_inc %>% expect_type("double")
  fit.scad$selected_lambda_lat %>% expect_type("double")
  fit.scad$max_c %>% expect_type("double")
  fit.scad$x_incidence %>% expect_type("double")
  fit.scad$x_latency %>% expect_type("double")
  fit.scad$y %>% expect_type("double")
  fit.scad$model %>% expect_type("character")
  fit.scad$scale %>% expect_type("logical")
  fit.scad$method %>% expect_type("character")
  fit.scad$penalty %>% expect_type("character")
  fit.scad$cv %>% expect_type("logical")
  fit.scad$fdr_control %>% expect_type("logical")
  fit.scad$call %>% expect_type("language")
  fit.scad$b %>% expect_length(dim(fit.cv$x_incidence)[2])
  fit.scad$beta %>% expect_length(dim(fit.cv$x_latency)[2])
  fit.scad$b0 %>% expect_length(1)

  expect_error(cv_cureem(Time ~ ., data = training,
                         x_latency = training, model = "cox", penalty = "SCAD"))
  expect_error(cv_cureem(Censor ~ ., data = training,
                         x_latency = training, model = "cox", penalty = "SCAD"))

  expect_error(cv_cureem(training$Time))
  training$subset_group <- gl(2, 75)
  training$penalty <- rnorm(dim(training)[1])
  expect_error(cv_cureem(Surv(Time, Censor) ~ .,
                      data = training, x_latency = training,
                      subset = subset_group))
  expect_error(cv_cureem(Surv(Time, Censor) ~ .,
                      data = training, x_latency = testing))
  expect_error(cv_cureem(Surv(Time, Censor) ~ .,
                         data = training, x_latency = training,
                         subset = subset_group))
  expect_error(cv_cureem(Surv(Time, Censor) ~ .,
                      data = training, x_latency = training,
                      penalty_factor_inc = penalty))
  expect_error(cv_cureem(Surv(Time, Censor) ~ .,
                      data = training, x_latency = training,
                      lambda_inc_list = -1))
  expect_error(cv_cureem(Surv(Time, Censor) ~ .,
                      data = training, x_latency = training,
                      lambda_lat_list = -1))
  expect_error(cv_cureem(Surv(Time, Censor) ~ ., penalty = "MCP",
                      data = training, x_latency = training,
                      gamma_inc = -1))
  expect_error(cv_cureem(Surv(Time, Censor) ~ ., penalty = "MCP",
                      data = training, x_latency = training,
                      gamma_lat = -1))
  expect_error(cv_cureem(Surv(Time, Censor) ~ .,
                         data = training, x_latency = training,
                         fdr.control = TRUE, fdr = 1.2))
  expect_error(cv_cureem(Surv(Time, Censor) ~ .,
                         data = training, x_latency = training,
                         lambda_min_ratio_inc  = 1.3))
  expect_error(cv_cureem(Surv(Time, Censor) ~ .,
                         data = training, x_latency = training,
                         lambda_min_ratio_lat = -0.2))

  withr::local_seed(26)
  temp <- generate_cure_data(n = 200, j = 25, n_true = 5, a = 1.8)
  training <- temp$training
  fit.cv <- cv_cureem(Surv(Time, Censor) ~ .,
                      data = training,
                      x_latency = training, fdr_control = FALSE,
                      grid_tuning = FALSE, nlambda_inc = 10, nlambda_lat = 10,
                      n_folds = 2, seed = 23, verbose = FALSE
  )
})

#' Print the contents of a mixture cure fitted object
#'
#' @description
#' This function prints the first several incidence and latency coefficients,
#' the rate (when fitting an exponential or Weibull mixture cure model), and
#' alpha (when fitting a Weibull mixture cure model). This function returns
#' the fitted object invisible to the user.
#'
#' @param x a \code{mixturecure} object resulting from \code{curegmifs},
#' \code{cureem}, \code{cv_cureem}, or \code{cv_curegmifs}.
#' @param max maximum number of rows in a matrix or elements in a vector to
#' display
#' @param ... other arguments.
#'
#' @note The contents of a \code{mixturecure} fitted object differ depending
#' upon whether the EM (\code{cureem}) or GMIFS (\code{curegmifs}) algorithm is
#' used for model fitting or if cross-validation is used. Also, the output
#' differs depending upon whether \code{x_latency} is specified in the model
#' (i.e., variables are included in the latency portion of the model fit) or
#' only \code{terms} on the right hand side of the equation are included (i.e.,
#' variables are included in the incidence portion of the model).
#'
#' @seealso \code{\link[hdcuremodels]{curegmifs}}, \code{\link[hdcuremodels]{cureem}},
#' \code{\link[hdcuremodels]{coef.mixturecure}}, \code{\link[hdcuremodels]{summary.mixturecure}},
#' \code{\link[hdcuremodels]{plot.mixturecure}}, \code{\link[hdcuremodels]{predict.mixturecure}}
#'
#' @return prints coefficient estimates for the incidence portion of the model
#' and if included, prints the coefficient estimates for the latency portion of
#' the model. Also prints rate for exponential and Weibull models and scale
#' (alpha) for the Weibull mixture cure model. Returns all objects fit using
#' \code{cureem}, \code{curegmifs}, \code{cv_cureem}, or \code{cv_curegmifs}.
#' @export
#'
#' @importFrom utils head
#'
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {RE4.17} *Model objects returned by Regression Software should implement or appropriately extend a default `print` method which provides an on-screen summary of model (input) parameters and (output) coefficients.*
#' @keywords methods
#' @method print mixturecure
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' fit <- curegmifs(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "weibull", thresh = 1e-4, maxit = 2000,
#'   epsilon = 0.01, verbose = FALSE
#' )
#' print(fit)
print.mixturecure <- function(x, max = 6, ...)  {
  model <- paste0(toupper(substr(x$model, 1, 1)), substr(x$model, 2, nchar(x$model)))
  cat("mixturecure object fit using", model, x$method, "algorithm\n")
  cat("\n")
  if (x$cv) {
    cat("$b\n", head(x$b, n = max), "\n")
    cat(length(x$b) - max, "more elements\n\n")
    cat("$beta\n", head(x$beta, n = max), "\n")
    cat(length(x$beta) - max, "more elements\n\n")
    if (x$model %in% c("exponential", "weibull")) {
      cat("$rate\n", head(x$rate, n = max), "\n\n")
    }
    if (x$model %in% c("weibull")) {
      cat("$alpha\n", head(x$alpha, n = max), "\n")
    }
  } else {
    cat("$b_path\n")
    print.default(x$b_path[1 : max, 1 : max])
    cat(dim(x$b_path)[1] - max, "more rows\n")
    if (dim(x$b_path)[2] - max > 0)
      cat(dim(x$b_path)[2] - max, "more columns\n\n")
    cat("$beta_path\n")
    print.default(x$beta_path[1 : max, 1 : max])
    cat(dim(x$beta_path)[1] - max, "more rows\n")
    if (dim(x$beta_path)[2] - max > 0)
      cat(dim(x$beta_path)[2] - max, "more columns\n\n")
    if (x$model %in% c("exponential", "weibull")) {
      cat("$rate\n", head(x$rate, n = max), "\n")
      cat(length(x$rate) - max, "more elements\n\n")
    }
    if (x$model %in% c("weibull")) {
      cat("$alpha\n", head(x$alpha, n = max), "\n")
      cat(length(x$alpha) - max, "more elements\n\n")
    }
  }
  invisible(x)  # Ensure the function returns the object invisibly
}

#' AUC for cure prediction using mean score imputation
#'
#' @description
#' This function calculates the AUC for cure prediction using the mean score
#' imputation (MSI) method proposed by Asano et al (2014).
#'
#' @param object a \code{mixturecure} object resulting from \code{curegmifs},
#' \code{cureem}, \code{cv_curegmifs}, or \code{cv_cureem}.
#' @param newdata an optional data.frame that minimally includes the incidence
#' and/or latency variables to use for predicting the response. If omitted, the
#' training data are used.
#' @param cure_cutoff cutoff value for cure, used to produce a proxy for the
#' unobserved cure status (default is 5 representing 5 years). Users
#' should be careful to note the time scale of their data and adjust this
#' according to the time scale and clinical application.
#' @param model_select either a case-sensitive parameter for models fit using
#' \code{curegmifs} or \code{cureem} or any numeric step along the solution path
#' can be selected. The default is \code{model_select = "AIC"} which calculates
#' the predicted values using the coefficients from the model achieving the
#' minimum AIC. The complete list of options are:
#' \itemize{
#'     \item \code{"AIC"} for the minimum AIC (default).
#'     \item \code{"mAIC"} for the minimum modified AIC.
#'     \item \code{"cAIC"} for the minimum corrected AIC.
#'     \item \code{"BIC"}, for the minimum BIC.
#'     \item \code{"mBIC"} for the minimum modified BIC.
#'     \item \code{"EBIC"} for the minimum extended BIC.
#'     \item \code{"logLik"} for the step that maximizes the
#' log-likelihood.
#'     \item \code{n} where n is any numeric value from the
#'     solution path.
#'   }
#' This option has no effect for objects fit using \code{cv_curegmifs} or
#' \code{cv_cureem}.
#'
#' @return Returns the AUC value for cure prediction using the mean score
#' imputation (MSI) method.
#' @export
#'
#' @references Asano, J., Hirakawa, H., Hamada, C. (2014) Assessing the
#' prediction accuracy of cure in the Cox proportional hazards cure model:
#' an application to breast cancer data. \emph{Pharmaceutical Statistics},
#' \bold{13}:357--363.
#' @srrstats {G1.0} *Statistical Software should list at least one primary reference from published academic literature.*
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G1.3} *All statistical terminology should be clarified and unambiguously defined.*
#' @srrstats {G2.1} *Implement assertions on types of inputs (see the initial point on nomenclature above).*
#' @srrstats {G5.2b} *Explicit tests should demonstrate conditions which trigger every one of those messages, and should compare the result with expected values.*
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*
#' @seealso \code{\link[hdcuremodels]{concordance_mcm}}
#'
#' @keywords univar
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' testing <- temp$testing
#' fit <- curegmifs(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "weibull", thresh = 1e-4, maxit = 2000,
#'   epsilon = 0.01, verbose = FALSE
#' )
#' auc_mcm(fit, model_select = "cAIC")
#' auc_mcm(fit, newdata = testing)
auc_mcm <- function(object, newdata, cure_cutoff = 5, model_select = "AIC") {
  if (!("mixturecure" %in% class(object))) {
    stop("Error: class of object must be mixturecure")
  }
  no_data <- (missing(newdata) || is.null(newdata))
  if (no_data) {
    testing_time <- object[["y"]][, 1]
    testing_delta <- object[["y"]][, 2]
    x_p <- object[["x_incidence"]]
  } else {
    df <- model.frame(parse(text = object$call, keep.source = FALSE)[[2]],
      data = newdata
    )
    testing_time <- df[, 1][, 1]
    testing_delta <- df[, 1][, 2]
    x_p <- as.matrix(model.frame(as.formula(paste(" ~ ", paste(
      colnames(object$x_incidence),
      collapse = "+"
    ))), newdata))
  }
  # Scale x_p if scale is TRUE; match to train if test data used
  if (!is.null(x_p) && identical(x_p, object$x_incidence)) {
    if (object$scale) {
      x_p <- self_scale(x_p, object$scale)
    }
  } else if (!is.null(x_p) && object$scale) {
    newx <- rbind(object$x_incidence, x_p)
    newx <- self_scale(newx, object$scale)
    x_p <- as.matrix(newx[-(seq_len(dim(object$x_incidence)[1])), ,
                          drop = FALSE
    ])
  }
  testing_n <- length(testing_time)
  v <- rep(0, testing_n)
  y <- rep(999, testing_n)
  y[testing_time > cure_cutoff] <- 0
  y[testing_time <= cure_cutoff & testing_delta == 1] <- 1
  v[y < 2] <- 1
  # Extract b_p_hat and intercept at model_select when CV not used
  if (!object$cv) {
    if (!is.numeric(model_select))
      model_select <- select_model(object, model_select)$select
    itct_hat <- object$b0_path[model_select]
    b_p_hat <- object$b_path[model_select, , drop = FALSE]
  } else {
    itct_hat <- object$b0
    b_p_hat <- matrix(object$b, ncol = 1)
  }
  if (all(b_p_hat == 0)) {
    xb <- rep(itct_hat, testing_n)
  } else {
    xb <- itct_hat + x_p[, b_p_hat != 0, drop = FALSE] %*% b_p_hat[b_p_hat != 0]
  }
  p_hat <- 1 / (1 + exp(-xb))
  temp <- v * y + (1 - v) * p_hat
  temp1 <- temp[order(p_hat, decreasing = TRUE)]
  temp_f <- v * (1 - y) + (1 - v) * (1 - p_hat)
  temp1f <- temp_f[order(p_hat, decreasing = TRUE)]
  TPR <- c(0, cumsum(temp1) / cumsum(temp1)[testing_n])
  FPR <- c(0, cumsum(temp1f) / cumsum(temp1f)[testing_n])
  height <- (TPR[-1] + TPR[-length(TPR)]) / 2
  width <- diff(FPR)
  auc_msi <- sum(height * width)
  return(auc_msi)
}

#' C-statistic for mixture cure models
#'
#' @description
#' This function calculates the C-statistic using the cure status weighting
#' (CSW) method proposed by Asano and Hirakawa (2017).
#'
#' @param object a \code{mixturecure} object resulting from \code{curegmifs},
#'  \code{cureem}, \code{cv_curegmifs}, or \code{cv_cureem}.
#' @param newdata an optional data.frame that minimally includes the incidence
#' and/or latency variables to use for predicting the response. If omitted, the
#' training data are used.
#' @param cure_cutoff cutoff value for cure, used to produce a proxy for the
#' unobserved cure status (default is 5 representing 5 years). Users
#' should be careful to note the time scale of their data and adjust this
#' according to the time scale and clinical application.
#' @param model_select either a case-sensitive parameter for models fit using
#' \code{curegmifs} or \code{cureem} or any numeric step along the solution path
#' can be selected. The default is \code{model_select = "AIC"} which calculates
#' the predicted values using the coefficients from the model achieving the
#' minimum AIC. The complete list of options are:
#' \itemize{
#'     \item \code{"AIC"} for the minimum AIC (default).
#'     \item \code{"mAIC"} for the minimum modified AIC.
#'     \item \code{"cAIC"} for the minimum corrected AIC.
#'     \item \code{"BIC"}, for the minimum BIC.
#'     \item \code{"mBIC"} for the minimum modified BIC.
#'     \item \code{"EBIC"} for the minimum extended BIC.
#'     \item \code{"logLik"} for the step that maximizes the
#' log-likelihood.
#'     \item \code{model_select = n} where n is any numeric value from the
#'     solution path.
#'   }
#' This option has no effect for objects fit using \code{cv_curegmifs} or
#' \code{cv_cureem}.
#'
#' @return value of C-statistic for the cure models.
#' @export
#'
#' @references Asano, J. and Hirakawa, H. (2017) Assessing the prediction
#' accuracy of a cure model for censored survival data with long-term survivors:
#' Application to breast cancer data. \emph{Journal of Biopharmaceutical
#' Statistics}, \bold{27}:6, 918--932.
#'
#' @srrstats {G1.0} *Statistical Software should list at least one primary reference from published academic literature.*
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G1.3} *All statistical terminology should be clarified and unambiguously defined.*
#' @srrstats {G2.1} *Implement assertions on types of inputs (see the initial point on nomenclature above).*
#' @srrstats {G5.2b} *Explicit tests should demonstrate conditions which trigger every one of those messages, and should compare the result with expected values.*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*
#' @seealso \code{\link[hdcuremodels]{auc_mcm}}
#'
#' @keywords univar
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' testing <- temp$testing
#' fit <- curegmifs(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "weibull", thresh = 1e-4, maxit = 2000,
#'   epsilon = 0.01, verbose = FALSE
#' )
#' concordance_mcm(fit, model_select = "cAIC")
#' concordance_mcm(fit, newdata = testing, model_select = "cAIC")
concordance_mcm <- function(object, newdata, cure_cutoff = 5,
  model_select = "AIC") {
  if (!("mixturecure" %in% class(object))) {
    stop("Error: class of object must be mixturecure")
  }
  no_data <- (missing(newdata) || is.null(newdata))
  if (no_data) {
    testing_time <- object[["y"]][, 1]
    testing_delta <- object[["y"]][, 2]
    x_p <- object[["x_incidence"]]
    w_p <- object[["x_latency"]]
  } else {
    df <- model.frame(parse(text = object$call, keep.source = FALSE)[[2]],
      data = newdata
    )
    testing_time <- df[, 1][, 1]
    testing_delta <- df[, 1][, 2]
    x_p <- as.matrix(model.frame(as.formula(paste(
      " ~ ",
      paste(colnames(object$x_incidence), collapse = "+")
    )), newdata))
    w_p <- as.matrix(model.frame(as.formula(paste(
      " ~ ",
      paste(colnames(object$x_latency), collapse = "+")
    )), newdata))
  }
  # Scale x_p if scale is TRUE; match to train if test data used
  if (!is.null(x_p) && identical(x_p, object$x_incidence)) {
    if (object$scale) {
      x_p <- self_scale(x_p, object$scale)
    }
  } else if (!is.null(x_p) && object$scale) {
    newx <- rbind(object$x_incidence, x_p)
    newx <- self_scale(newx, object$scale)
    x_p <- as.matrix(newx[-(seq_len(dim(object$x_incidence)[1])), ,
                          drop = FALSE
    ])
  }
  # Scale w_p if scale is TRUE; match to train if test data used
  if (!is.null(w_p) && identical(w_p, object$x_latency)) {
    if (object$scale) {
      w_p <- self_scale(w_p, object$scale)
    }
  } else if (!is.null(w_p) && object$scale) {
    newx <- rbind(object$x_latency, w_p)
    newx <- self_scale(newx, object$scale)
    w_p <- as.matrix(newx[-(seq_len(dim(object$x_latency)[1])), , drop = FALSE])
  }
  c_csw_num <- 0
  c_csw_denom <- 0
  testing_n <- length(testing_time)
  v <- rep(0, testing_n)
  y <- rep(999, testing_n)
  y[testing_time > cure_cutoff] <- 0
  y[testing_time <= cure_cutoff & testing_delta == 1] <- 1
  v[y < 2] <- 1
  # Extract b_p_hat and intercept at model_select when CV not used
  if (!object$cv) {
    if (!is.numeric(model_select))
      model_select <- select_model(object, model_select)$select
    itct_hat <- object$b0_path[model_select]
    b_p_hat <- object$b_path[model_select, , drop = FALSE]
    beta_p_hat <- object$beta_path[model_select, , drop = FALSE]
  } else {
    itct_hat <- object$b0
    b_p_hat <- matrix(object$b, ncol = 1)
    beta_p_hat <- matrix(object$beta, ncol = 1)
  }
  if (all(b_p_hat == 0)) {
    p_hat <- 1 / (1 + exp(-itct_hat))
  } else {
    p_hat <- 1 / (1 + exp(-itct_hat - x_p[, b_p_hat != 0, drop = FALSE] %*%
      b_p_hat[b_p_hat != 0]))
  }
  temp <- v * y + (1 - v) * as.vector(p_hat)
  if (all(beta_p_hat == 0)) {
    w_beta <- rep(0, testing_n)
  } else {
    w_beta <- w_p[, beta_p_hat != 0, drop = FALSE] %*%
      beta_p_hat[beta_p_hat != 0]
  }
  for (i in 1:testing_n) {
    for (j in 1:testing_n) {
      if (j == i || !testing_delta[i] || testing_time[i] > testing_time[j]) next
      I_ij <- testing_time[i] < testing_time[j] |
        (testing_time[i] == testing_time[j] & !testing_delta[j])
      if (!I_ij) next
      if (w_beta[i] > w_beta[j]) c_csw_num <- c_csw_num + temp[j]
      c_csw_denom <- c_csw_denom + temp[j]
    }
  }
  return(c_csw_num / c_csw_denom)
}

#' Simulate data under a mixture cure model
#'
#' @description
#' Simulate data under a mixture cure model.
#'
#' @param n an integer denoting the total sample size.
#' @param j an integer denoting the number of penalized predictors which is the
#' same for both the incidence and latency portions of the model.
#' @param nonp an integer denoting the number of unpenalized
#' predictors (which is the same for both the incidence and latency portions of
#' the model).
#' @param train_prop a numeric value in [0, 1) representing the fraction of `n`
#' to be used in forming the training dataset.
#' @param n_true an integer less than `j` denoting the number of variables truly
#' associated with the outcome (i.e., the number of covariates with nonzero
#' parameter values) among the penalized predictors.
#' @param a a numeric value denoting the effect size (signal amplitude) which is
#' the same for both the incidence and latency portions of the model.
#' @param rho a numeric value in [0, 1) representing the correlation between
#' adjacent covariates in the same block.
#' @param itct_mean a numeric value representing the expectation of the
#' incidence intercept which controls the cure rate.
#' @param cens_ub a numeric value representing the upper bound on the censoring
#' time distribution which follows a uniform distribution on (0, \code{cens_ub}].
#' @param alpha a numeric value representing the shape parameter in the Weibull
#' density.
#' @param lambda a numeric value representing the rate parameter in the Weibull
#' density.
#' @param same_signs logical, if TRUE the incidence and latency coefficients
#' have the same signs.
#' @param model type of regression model to use for the latency portion of
#' mixture cure model. Can be one of the following:
#' \itemize{
#' \item \code{"weibull"} to generate times from a Weibull distribution.
#' \item \code{"GG"} to generate times from a generalized gamma distribution.
#' \item \code{"Gompertz"} to generate times from a Gomertz distribution.
#' \item \code{"nonparametric"} to generate times non-parametrically.
#' \item  \code{"GG_baseline"} to generate times from a generalized gamma
#' baseline distribution.
#' }
#' @return \item{training}{training data.frame which includes Time, Censor, and
#' covariates. Variables prefixed with \code{"U"} indicates unpenalized
#' covariates and is equal to the value
#' passed to \code{nonp} (default is 2). Variables prefixed with \code{"X"}
#' indicates penalized covariates and is equal to the value passed to
#' \code{j}.
#' }
#' @return \item{training_y}{the true status for the training set: uncured = 1;
#' cured = 0}
#' @return \item{testing}{testing data.frame which includes Time, Censor,  Y
#' (the true uncured = 1; cured = 0 status), and
#' covariates. Variables prefixed with \code{"U"} indicates unpenalized
#' covariates and is equal to the value
#' passed to \code{nonp} (default is 2). Variables prefixed with \code{"X"}
#' indicates penalized covariates and is equal to the value passed to
#' \code{j}.
#' }
#' @return \item{testing_y}{the true status for the testing set: uncured = 1;
#' cured = 0}
#' @return \item{parameters}{a list including: the indices of true incidence
#' signals (\code{nonzero_b}), indices of true latency signals
#' (\code{nonzero_beta}), unpenalized incidence parameter values (\code{b_u}),
#' unpenalized latency parameter values (\code{beta_u}), parameter values for
#' the true incidence signals among penalized covariates (\code{b_p_nz}),
#' parameter values for the true latency signals among penalized covariates
#' (\code{beta_p_nz}), parameter value for the incidence intercept
#' (\code{itct})}
#'
#' @export
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G5.9} **Noise susceptibility tests** *Packages should test for expected stochastic behaviour, such as through the following conditions:*
#' @srrstats {G5.9b} *Running under different random seeds or initial conditions does not meaningfully change results*
#' @import mvnfast
#' @import flexsurv
#' @import stats
#'
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' # This dataset has 2 penalized features associated with the outcome,
#' # 3 penalized features not associated with the outcome (noise features), and 1
#' # unpenalized noise feature.
#' data <- generate_cure_data(n = 1000, j = 5, n_true = 2, nonp = 1, a = 2)
#' # Extract the training data
#' training <- data$training
#' # Extract the testing data
#' testing <- data$testing
#' # To identify the features truly associated with incidence
#' names(training)[grep("^X", names(training))][data$parameters$nonzero_b]
#' # To identify the features truly associated with latency
#' names(training)[grep("^X", names(training))][data$parameters$nonzero_beta]
#' # Fit the model to the training data
#' fitem <- cureem(Surv(Time, Censor) ~ ., data = training,
#'   x_latency = training)
#' # Examine the estimated coefficients at the (default) minimum AIC
#' coef(fitem)
#' # As the penalty increases, the coefficients for the noise variables shrink
#' # to or remain at zero, while the truly associated features have coefficient
#' # paths that depart from zero. This shows the model's ability to distinguish
#' # signal from noise.
#' plot(fitem, label = TRUE)
generate_cure_data <- function(n = 400, j = 500, nonp = 2, train_prop = 0.75,
                               n_true = 10, a = 1, rho = 0.5, itct_mean = 0.5,
                               cens_ub = 20, alpha = 1, lambda = 2,
                               same_signs = FALSE, model = "weibull") {
  # j: number of penalized covariates
  # nonp: number of non-penalized covariates
  if (n_true > j) {
    stop("Error: \"n_true\" must be less than or equal to j")
  }
  tr_i <- 1:round(train_prop * n) # training index
  te_i <- (round(train_prop * n) + 1):n # testing index  ##corrected Mar30

  ## covariance matrices
  sd <- 0.5
  block_sz <- round(j / n_true)

  corr_x_p <- matrix(0, j, j)
  for (i in 1:min(ceiling(n_true/block_sz+1), n_true)) {
    if (j %% n_true == 0) {
      corr_x_p[(block_sz * (i - 1) + 1):(block_sz * i), (block_sz * (i - 1) + 1):
                 (block_sz * i)] <- rho^abs(outer(1:block_sz, 1:block_sz, "-"))
    } else {
        if (j - block_sz*i > 0) {
          corr_x_p[(block_sz * (i - 1) + 1):(block_sz * i), (block_sz * (i - 1) + 1):
                 (block_sz * i)] <- rho^abs(outer(1:block_sz, 1:block_sz, "-"))
        }
    }
    diag(corr_x_p) <- 1
    }
  sigma_x_p <- sd^2 * corr_x_p
  x_p <- mvnfast::rmvn(n, mu = rep(0, j), sigma = sigma_x_p)
  ## unpenalized
  if (nonp > 0) {
    x_u <- matrix(rnorm(n * nonp, sd = sd), ncol = nonp)
    colnames(x_u) <- paste0("U", 1:dim(x_u)[2])
    w_u <- x_u
    if (!same_signs) { # true signals from two parts are randomly generated
      b_u <- rnorm(nonp, mean = 0.3, sd = 0.1)
      b_u <- b_u * sample(c(1, -1), length(b_u), replace = TRUE)
      beta_u <- rnorm(nonp, mean = 0.3, sd = 0.1)
      beta_u <- beta_u * sample(c(1, -1), length(beta_u), replace = TRUE)
    } else { # true signals from two parts have same signs
      sign_u <- sample(c(1, -1), nonp, replace = TRUE)
      b_u <- abs(rnorm(nonp, mean = 0.3, sd = 0.1)) * sign_u
      beta_u <- abs(rnorm(nonp, mean = 0.3, sd = 0.1)) * sign_u
    }
  }
  w_p <- x_p
  ## penalized
  if (!same_signs) { # true signals from two parts are randomly generated
    nonzero_b <- nonzero_beta <- rep(NA, n_true)
    for (i in 1:n_true) {
      if (i == n_true) {
        if (length(((n_true-1)*block_sz+1):j) == 1) {
          nonzero_b[i] <- j
          nonzero_beta[i] <- j
        } else {
          nonzero_b[i] <- sample(x=((n_true-1)*block_sz+1):j, size=1)
          nonzero_beta[i] <- sample(x=((n_true-1)*block_sz+1):j, size=1)
        }
      } else {
        nonzero_b[i] <- sample((block_sz * (i - 1) + 1):(block_sz * i), 1)
        nonzero_beta[i] <- sample((block_sz * (i - 1) + 1):(block_sz * i), 1)
      }
    }
    b_p <- beta_p <- rep(0, j)
    b_p[nonzero_b] <- a * sample(c(1, -1), n_true, replace = TRUE)
    beta_p[nonzero_beta] <- a * sample(c(1, -1), n_true, replace = TRUE)
  } else { # true signals from two parts have same signs
    nonzero_b <- rep(NA, n_true)
    for (i in 1:n_true) {
      nonzero_b[i] <- sample((block_sz * (i - 1) + 1):
      (block_sz * i), 1)
    }
    b_p <- rep(0, j)
    b_p[nonzero_b] <- a * sample(c(1, -1), n_true, replace = TRUE)
    beta_p <- b_p
    nonzero_beta <- nonzero_b
  }
  ## cure or not
  itct <- rnorm(1, mean = itct_mean, sd = 0.1)
  if (nonp > 0) {
    bx <- itct + x_u %*% b_u + x_p %*% b_p
  } else {
    bx <- itct + x_p %*% b_p
  }
  p <- 1 / (1 + exp(-bx))
  y <- rbinom(n, 1, p)

  ## survival
  if (nonp > 0) {
    beta_w <- w_u %*% beta_u + w_p %*% beta_p
  } else {
    beta_w <- w_p %*% beta_p
  }
  if (model == "weibull") {
    t <- rweibull(n, shape = alpha, scale = 1 / lambda * exp(-beta_w / alpha))
  } else if (model == "GG") {
    t <- flexsurv::rgengamma(n,
      mu = -log(lambda) - beta_w / alpha,
      sigma = 1 / alpha, Q = 2
    )
  } else if (model == "Gompertz") {
    t <- flexsurv::rgompertz(n, shape = 0.2, rate = exp(beta_w))
  } else if (model == "nonparametric") {
    t <- nonparametric_time_generator(n, beta_w, maxT = cens_ub, knots = 8)
  } else if (model == "GG_baseline") {
    t <- rep(NA, n)
    for (i in 1:n) {
      u <- runif(1)
      t[i] <- uniroot(function(a) flexsurv::pgengamma(a, mu = -log(2), sigma = 1, Q = 0.5, lower.tail = FALSE)^exp(beta_w[i]) - u,
        c(0, 20),
        extendInt = "yes"
      )$root
    }
  }
  u <- runif(n, 0, cens_ub)
  delta <- ifelse(t > u | y == 0, 0, 1)
  time <- pmin(t, u)
  time[y == 0] <- u[y == 0]
  ## training and test
  if (nonp > 0) {
    colnames(x_u) <- paste0("U",1:dim(x_u)[2])
    colnames(w_u) <- paste0("U",1:dim(w_u)[2])
    tr_data <- list(
      x_u = x_u[tr_i, , drop=FALSE], x_p = x_p[tr_i, , drop = FALSE],
      w_u = w_u[tr_i, , drop = FALSE],
      w_p = w_p[tr_i, , drop = FALSE], time = time[tr_i], y = y[tr_i],
      delta = delta[tr_i],
      y = y[tr_i]
    )
    te_data <- list(
     x_u = x_u[te_i, , drop = FALSE], x_p = x_p[te_i, , drop = FALSE],
     w_u = w_u[te_i, , drop = FALSE],
     w_p = w_p[te_i, , drop = FALSE], time = time[te_i], y = y[te_i],
     delta = delta[te_i],
     y = y[tr_i]
    )
   training <- data.frame(
      Time = tr_data$time, Censor = tr_data$delta,
      tr_data$x_u, tr_data$x_p
    )
   training_y <- tr_data$y
   testing <- data.frame(
      Time = te_data$time, Censor = te_data$delta,
      te_data$x_u, te_data$x_p
    )
   testing_y <- te_data$y
    parameters <- list(
      nonzero_b = nonzero_b, nonzero_beta = nonzero_beta,
      b_u = b_u, beta_u = beta_u, b_p_nz = b_p[nonzero_b],
      beta_p_nz = beta_p[nonzero_beta], itct = itct
    )
  } else {
    tr_data <- list(
      x_p = x_p[tr_i, , drop = FALSE],
      w_p = w_p[tr_i, , drop = FALSE ], time = time[tr_i], y = y[tr_i],
      delta = delta[tr_i], y = y[tr_i]
    )
    te_data <- list(
      x_p = x_p[te_i, , drop = FALSE],
      w_p = w_p[te_i, , drop = FALSE], time = time[te_i], y = y[te_i],
      delta = delta[te_i], y = y[te_i]
    )
    training <- data.frame(
      Time = tr_data$time, Censor = tr_data$delta,
      tr_data$x_p
    )
    training_y <- tr_data$y
    testing <- data.frame(
      Time = te_data$time, Censor = te_data$delta,
      te_data$x_p
    )
    testing_y <- te_data$y
    parameters <- list(
      nonzero_b = nonzero_b, nonzero_beta = nonzero_beta,
      b_p_nz = b_p[nonzero_b],
      beta_p_nz = beta_p[nonzero_beta], itct = itct
    )
  }
  return(list(training = training, training_y = training_y, testing = testing,
              testing_y = testing_y, parameters = parameters))
}

#' @srrstats {G5.2b} *Explicit tests should demonstrate conditions which trigger every one of those messages, and should compare the result with expected values.*
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {G5.6} **Parameter recovery tests** *to test that the implementation produce expected results given data with known properties. For instance, a linear regression algorithm should return expected coefficient values for a simulated data set generated from a linear model.*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*
#' @srrstats {G5.9} **Noise susceptibility tests** *Packages should test for expected stochastic behaviour, such as through the following conditions:*
#' @srrstats {G5.9a} *Adding trivial noise (for example, at the scale of `.Machine$double.eps`) to data does not meaningfully change results*

test_that("cv_curegmifs function works correctly", {
  library(survival)
  withr::local_seed(123)
  temp <- generate_cure_data(n = 100, j = 15, n_true = 3, a = 1.8, rho = 0.2)
  training <- temp$training

  fit.cv <- cv_curegmifs(Surv(Time, Censor) ~ .,
                         data = training,
                         x_latency = training, fdr_control = FALSE,
                         maxit = 450, epsilon = 0.01, n_folds = 2,
                         seed = 23, verbose = TRUE
  )
  expect_error(cv_curegmifs(Time ~ ., data = training, x_latency = training))
  expect_error(cv_curegmifs(Censor ~ ., data = training, x_latency = training))

  fit.cv %>% expect_s3_class("mixturecure")
  expect_setequal(names(fit.cv), c("selected_step_inc", "selected_step_lat",
                                   "b0", "b", "beta", "rate", "logLik", "max_c",
                                   "x_incidence", "x_latency", "y", "model",
                                   "scale", "method", "alpha", "cv", "call",
                                   "fdr_control"))
  fit.cv$selected_step_inc %>% expect_type("integer")
  fit.cv$selected_step_lat %>% expect_type("integer")
  fit.cv$b0 %>% expect_type("double")
  fit.cv$b %>% expect_type("double")
  fit.cv$beta %>% expect_type("double")
  fit.cv$rate %>% expect_type("double")
  fit.cv$logLik %>% expect_type("double")
  fit.cv$max_c %>% expect_type("double")
  fit.cv$x_incidence %>% expect_type("double")
  fit.cv$x_latency %>% expect_type("double")
  fit.cv$y %>% expect_type("double")
  fit.cv$model %>% expect_type("character")
  fit.cv$scale %>% expect_type("logical")
  fit.cv$method %>% expect_type("character")
  fit.cv$alpha %>% expect_type("double")
  fit.cv$cv %>% expect_type("logical")
  fit.cv$fdr_control %>% expect_type("logical")
  fit.cv$call %>% expect_type("language")

  fit.cv$b %>% expect_length(dim(fit.cv$x_incidence)[2])
  fit.cv$beta %>% expect_length(dim(fit.cv$x_latency)[2])
  fit.cv$b0 %>% expect_length(1)
  fit.cv$alpha %>% expect_length(1)
  fit.cv$rate %>% expect_length(1)
  fit.cv$selected_step_inc %>% expect_length(1)
  fit.cv$selected_step_lat %>% expect_length(1)
  fit.cv$logLik %>% expect_length(1)
  fit.cv$max_c %>% expect_length(1)
  expect_equal(round(fit.cv$rate, 6), 2.067206)
  expect_equal(round(fit.cv$alpha, 6), 0.648263)

  fit.cv <- cv_curegmifs(Surv(Time, Censor) ~ ., data = training,
                         penalty_factor_inc = rep(c(0, 1), c(1, 11)),
                         measure_inc = "auc",
                         x_latency = training, fdr_control = FALSE,
                         maxit = 450, epsilon = 0.01, n_folds = 2,
                         seed = 23, verbose = FALSE, parallel = FALSE
  )
  fit.cv %>% expect_s3_class("mixturecure")
  expect_setequal(names(fit.cv), c("selected_step_inc", "selected_step_lat",
                                   "b0", "b", "beta", "rate", "logLik", "max_c",
                                   "x_incidence", "x_latency", "y", "model",
                                   "scale", "method", "alpha", "max.auc", "cv",
                                   "call", "fdr_control"))
  fit.cv$selected_step_inc %>% expect_type("integer")
  fit.cv$selected_step_lat %>% expect_type("integer")
  fit.cv$b0 %>% expect_type("double")
  fit.cv$b %>% expect_type("double")
  fit.cv$beta %>% expect_type("double")
  fit.cv$rate %>% expect_type("double")
  fit.cv$logLik %>% expect_type("double")
  fit.cv$max_c %>% expect_type("double")
  fit.cv$max.auc %>% expect_type("double")
  fit.cv$x_incidence %>% expect_type("double")
  fit.cv$x_latency %>% expect_type("double")
  fit.cv$y %>% expect_type("double")
  fit.cv$model %>% expect_type("character")
  fit.cv$scale %>% expect_type("logical")
  fit.cv$method %>% expect_type("character")
  fit.cv$alpha %>% expect_type("double")
  fit.cv$cv %>% expect_type("logical")
  fit.cv$fdr_control %>% expect_type("logical")
  fit.cv$call %>% expect_type("language")

  fit.cv$b %>% expect_length(dim(fit.cv$x_incidence)[2])
  fit.cv$beta %>% expect_length(dim(fit.cv$x_latency)[2])
  fit.cv$b0 %>% expect_length(1)
  fit.cv$alpha %>% expect_length(1)
  fit.cv$rate %>% expect_length(1)
  fit.cv$selected_step_inc %>% expect_length(1)
  fit.cv$selected_step_lat %>% expect_length(1)
  fit.cv$logLik %>% expect_length(1)
  fit.cv$max_c %>% expect_length(1)
  fit.cv$max.auc %>% expect_length(1)


  withr::local_seed(4)
  temp <- generate_cure_data(n = 200, j = 10, n_true = 10, a = 1.8)
  training <- temp$training
  expect_error(cv_curegmifs(training$Time))
  training$group <- gl(2, 75)
  expect_error(cv_curegmifs(Surv(Time, Censor) ~ .,
                         data = training, x_latency = training, thresh = -1))
  expect_error(cv_curegmifs(Surv(Time, Censor) ~ .,
                         data = training, x_latency = training, epsilon = -1))
  expect_error(cv_curegmifs(Surv(Time, Censor) ~ .,
                         data = training, x_latency = training,
                         subset = group))
  expect_error(cv_curegmifs(Surv(Time, Censor) ~ .,
                         data = training, x_latency = testing))
  expect_error(cv_curegmifs(Surv(Time, Censor) ~ .,
                         data = training, x_latency = training,
                         model = "cox"))
  expect_error(cv_curegmifs(Surv(Time, Censor) ~ .,
                         data = training, x_latency = training,
                         penalty_factor_inc = penalty))
  expect_error(cv_curegmifs(Surv(Time, Censor) ~ .,
                            data = training, x_latency = training,
                            fdr_control = TRUE, fdr = 1.2))

  withr::local_seed(16)
  temp <- generate_cure_data(n = 100, j = 15, n_true = 3, a = 1.8, rho = 0.2)
  training <- temp$training
  fit.cv <- cv_curegmifs(Surv(Time, Censor) ~ .,
                         data = training,
                         x_latency = training, fdr_control = FALSE,
                         maxit = 450, epsilon = 0.01, n_folds = 2,
                         seed = 23, verbose = TRUE
  )
  expect_equal(round(fit.cv$rate, 6), 2.042942)
  expect_equal(round(fit.cv$alpha, 6), 0.55764)
})

#' @srrstats {G2.0} *Implement assertions on lengths of inputs, particularly through asserting that inputs expected to be single- or multi-valued are indeed so.*
#' @srrstats {G5.2b} *Explicit tests should demonstrate conditions which trigger every one of those messages, and should compare the result with expected values.*
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*

test_that("multiplication works", {
  library(survival)
  withr::local_seed(1234)
  temp <- generate_cure_data(n = 200, j = 10, n_true = 10, a = 1.8)
  training <- temp$training
  training$group <- gl(2, 75)
  km_fit <- survfit(Surv(Time, Censor) ~ 1, data = training)
  output <- cure_estimate(km_fit)
  output %>% expect_length(1)
  expect_equal(round(output, 7), 0.4866733)
  output %>% expect_type("double")
  expect_error(cure_estimate(survdiff(Surv(Time, Censor) ~ 1, data = training)))
  expect_visible(cure_estimate(km_fit))

  km_fit <- survfit(Surv(Time, Censor) ~ group, data = training)
  output <- cure_estimate(km_fit)
  output %>% expect_length(3)
  expect_equal(round(output[1, 2], 7), 0.5357901)
  output %>% expect_type("list")
  expect_error(cure_estimate(survdiff(Surv(Time, Censor) ~ group, data = training)))
  expect_visible(cure_estimate(km_fit))

  fit.lm <- lm(Time ~ Censor, data = training)
  expect_error(cure_estimate(fit.lm), "Error: object must be a survfit object")

})

#' @srrstats {G5.9} **Noise susceptibility tests** *Packages should test for expected stochastic behaviour, such as through the following conditions:*
#' @srrstats {G5.9b} *Running under different random seeds or initial conditions does not meaningfully change results*

test_that("generate_cure_data works", {
  withr::local_seed(1234)
  temp <- generate_cure_data(n = 200, j = 25, n_true = 5, a = 1.8, rho = 0.3,
                             itct_mean = 0.4, same_signs = TRUE)
  temp %>% expect_type("list")
  expect_setequal(names(temp), c("training", "training_y", "testing",
                                 "testing_y", "parameters"))
  temp %>% expect_length(5)
  temp$training %>% expect_type("list")
  temp$testing %>% expect_type("list")
  temp$parameters %>% expect_type("list")
})

#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*
#' @srrstats {G5.9} **Noise susceptibility tests** *Packages should test for expected stochastic behaviour, such as through the following conditions:*
#' @srrstats {G5.9a} *Adding trivial noise (for example, at the scale of `.Machine$double.eps`) to data does not meaningfully change results*
#' @srrstats {G5.9b} *Running under different random seeds or initial conditions does not meaningfully change results*

test_that("nonzerocure_test works", {
  library(survival)
  withr::local_seed(1234)
  temp <- generate_cure_data(n = 200, j = 10, n_true = 10, a = 1.8)
  training <- temp$training
  training$group <- gl(2, 75)
  km_fit <- survfit(Surv(Time, Censor) ~ 1, data = training)
  output <- nonzerocure_test(km_fit)
  output %>% expect_type("list")
  output %>% expect_length(4)
  expect_equal(round(output$p_value, 3), 0.032)

  fit.lm <- lm(Time ~ Censor, data = training)
  expect_error(nonzerocure_test(fit.lm), "Error: object must be a survfit object")
})

#' @srrstats {G5.2b} *Explicit tests should demonstrate conditions which trigger every one of those messages, and should compare the result with expected values.*
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {G5.6} **Parameter recovery tests** *to test that the implementation produce expected results given data with known properties. For instance, a linear regression algorithm should return expected coefficient values for a simulated data set generated from a linear model.*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*
#' @srrstats {G5.9} **Noise susceptibility tests** *Packages should test for expected stochastic behaviour, such as through the following conditions:*
#' @srrstats {G5.9a} *Adding trivial noise (for example, at the scale of `.Machine$double.eps`) to data does not meaningfully change results*

test_that("cureem works correctly", {
# validate function output
  library(survival)
  withr::local_seed(1234)
  temp <- generate_cure_data(n = 80, j = 100, n_true = 10, a = 1.8)
  training <- temp$training
  fit <- cureem(Surv(Time, Censor) ~ .,
                data = training, x_latency = training,
                model = "cox", penalty = "lasso", lambda_inc = 0.1,
                lambda_lat = 0.1, gamma_inc = 6, gamma_lat = 10
  )
  expect_equal(names(fit), c("b_path", "beta_path", "b0_path", "logLik",
                             "x_incidence", "x_latency", "y",
                             "model", "scale", "method", "call", "cv" ))
  expect_true(class(fit) == "mixturecure")
  expect_true(class(fit$b_path)[1] == "matrix")
  expect_true(class(fit$beta_path)[1] == "matrix")
  expect_true(class(fit$b0_path) == "numeric")
  expect_true(class(fit$logLik) == "numeric")
  expect_equal(round(fit$b0_path[1], 7), 0.5035067)

  fit <- cureem(Surv(Time, Censor) ~ .,
                data = training, x_latency = training,
                model = "cox", penalty = "MCP", lambda_inc = 0.1,
                lambda_lat = 0.1, gamma_inc = 6, gamma_lat = 10
  )
  expect_equal(names(fit), c("b_path", "beta_path", "b0_path", "logLik",
                             "x_incidence", "x_latency", "y",
                             "model", "scale", "method", "call", "cv" ))
  expect_true(class(fit) == "mixturecure")
  expect_true(class(fit$b_path)[1] == "matrix")
  expect_true(class(fit$beta_path)[1] == "matrix")
  expect_true(class(fit$b0_path) == "numeric")
  expect_true(class(fit$logLik) == "numeric")

  fit <- cureem(Surv(Time, Censor) ~ .,
                data = training, x_latency = training,
                model = "cox", penalty = "SCAD", lambda_inc = 0.1,
                lambda_lat = 0.1, gamma_inc = 6, gamma_lat = 10
  )
  expect_equal(names(fit), c("b_path", "beta_path", "b0_path", "logLik",
                             "x_incidence", "x_latency", "y",
                             "model", "scale", "method", "call", "cv" ))
  expect_true(class(fit) == "mixturecure")
  expect_true(class(fit$b_path)[1] == "matrix")
  expect_true(class(fit$beta_path)[1] == "matrix")
  expect_true(class(fit$b0_path) == "numeric")
  expect_true(class(fit$logLik) == "numeric")


  fit <- cureem(Surv(Time, Censor) ~ .,
                data = training, x_latency = training,
                model = "weibull", penalty = "lasso", lambda_inc = 0.1,
                lambda_lat = 0.1, gamma_inc = 6, gamma_lat = 10
  )
  expect_equal(names(fit), c("b_path", "beta_path", "b0_path", "logLik",
                             "x_incidence", "x_latency", "y",
                             "model", "scale", "method", "call", "rate_path",
                             "alpha_path", "cv" ))
  expect_true(class(fit) == "mixturecure")
  expect_true(class(fit$b_path)[1] == "matrix")
  expect_true(class(fit$beta_path)[1] == "matrix")
  expect_true(class(fit$b0_path) == "numeric")
  expect_true(class(fit$logLik) == "numeric")
  expect_equal(round(fit$b0_path[1], 7), 0.2008767)

  fit <- cureem(Surv(Time, Censor) ~ .,
                data = training, x_latency = training,
                model = "exponential", penalty = "lasso", lambda_inc = 0.1,
                lambda_lat = 0.1, gamma_inc = 6, gamma_lat = 10
  )
  expect_equal(names(fit), c("b_path", "beta_path", "b0_path", "logLik",
                             "x_incidence", "x_latency", "y",
                             "model", "scale", "method", "call", "rate_path",
                              "cv" ))
  expect_true(class(fit) == "mixturecure")
  expect_true(class(fit$b_path)[1] == "matrix")
  expect_true(class(fit$beta_path)[1] == "matrix")
  expect_true(class(fit$b0_path) == "numeric")
  expect_true(class(fit$logLik) == "numeric")

  expect_error(cureem(training$Time))
  training$group <- gl(2, 30)
  training$penalty <- rnorm(dim(training)[1])
  expect_error(cureem(Surv(Time, Censor) ~ .,
                      data = training, x_latency = training,
                      subset = group))
  expect_error(cureem(Surv(Time, Censor) ~ .,
                      data = training, x_latency = testing))
  expect_error(cureem(Surv(Time, Censor) ~ .,
                      data = training, x_latency = training,
                      subset = group))
  expect_error(cureem(Surv(Time, Censor) ~ .,
                      data = training, x_latency = training,
                      penalty_factor_inc = penalty))
  expect_error(cureem(Surv(Time, Censor) ~ .,
                      data = training, x_latency = training, lambda_inc = -1))
  expect_error(cureem(Surv(Time, Censor) ~ .,
                      data = training, x_latency = training, lambda_lat = -1))
  expect_error(cureem(Surv(Time, Censor) ~ ., penalty = "MCP",
                      data = training, x_latency = training, gamma_inc = -1))
  expect_error(cureem(Surv(Time, Censor) ~ ., penalty = "MCP",
                      data = training, x_latency = training, gamma_lat = -1))
})

test_that("formula function works correctly", {
  library(survival)
  withr::local_seed(1234)
  temp <- generate_cure_data(n = 80, j = 100, n_true = 10, a = 1.8)
  training <- temp$training
  fit <- cureem(Surv(Time, Censor) ~ .,
                data = training, x_latency = training,
                model = "cox", penalty = "lasso", lambda_inc = 0.1,
                lambda_lat = 0.1, gamma_inc = 6, gamma_lat = 10
  )
  expect_true(is.call(formula(fit)))
})

#' Return model family and fitting algorithm for mixturecure model fits
#'
#' @description Return model family and fitting algorithm for\code{mixturecure}
#' model fits.
#' @param object an object of class \code{mixturecure}
#' @param  ... other arguments.
#'
#' @returns the parametric or semi-parametric model fit and the fitting
#' algorithm.
#' @export
#' @method family mixturecure
#' @keywords methods
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' fit <- curegmifs(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "weibull", thresh = 1e-4, maxit = 2000,
#'   epsilon = 0.01, verbose = FALSE
#' )
#' family(fit)
family.mixturecure <- function(object, ...) {
  if (!("mixturecure" %in% class(object))) {
    stop("Error: class of object must be mixturecure")
  }
  model <- paste0(toupper(substr(object$model, 1, 1)), substr(object$model, 2, nchar(object$model)))
  cat("\n")
  cat("Family:", model, "\n")
  cat("Algorithm:", object$method, "\n\n")
}

#' AML training data
#'
#' Duration of complete response for 306 cytogenetically normal AML patients and
#' a subset of 320 transcript expression from RNA-sequencing.
#'
#'
#' @format
#' A data frame with 306 rows (subjects) and 322 columns:
#' \describe{
#'   \item{cryr}{duration of complete response in years}
#'   \item{relapse.death}{censoring indicator: 1 = relapsed or died; 0 = alive
#'   at last follow=up}
#'   \item{ENSG00000001561}{normalized expression for indicated transcript}
#'   \item{ENSG00000005249}{normalized expression for indicated transcript}
#'   \item{ENSG00000006757}{normalized expression for indicated transcript}
#'   \item{ENSG00000007062}{normalized expression for indicated transcript}
#'   \item{ENSG00000007968}{normalized expression for indicated transcript}
#'   \item{ENSG00000008283}{normalized expression for indicated transcript}
#'   \item{ENSG00000008405}{normalized expression for indicated transcript}
#'   \item{ENSG00000008441}{normalized expression for indicated transcript}
#'   \item{ENSG00000010295}{normalized expression for indicated transcript}
#'   \item{ENSG00000011028}{normalized expression for indicated transcript}
#'   \item{ENSG00000011198}{normalized expression for indicated transcript}
#'   \item{ENSG00000012779}{normalized expression for indicated transcript}
#'   \item{ENSG00000012817}{normalized expression for indicated transcript}
#'   \item{ENSG00000013306}{normalized expression for indicated transcript}
#'   \item{ENSG00000013725}{normalized expression for indicated transcript}
#'   \item{ENSG00000018189}{normalized expression for indicated transcript}
#'   \item{ENSG00000022267}{normalized expression for indicated transcript}
#'   \item{ENSG00000023171}{normalized expression for indicated transcript}
#'   \item{ENSG00000023909}{normalized expression for indicated transcript}
#'   \item{ENSG00000029639}{normalized expression for indicated transcript}
#'   \item{ENSG00000047634}{normalized expression for indicated transcript}
#'   \item{ENSG00000049192}{normalized expression for indicated transcript}
#'   \item{ENSG00000053524}{normalized expression for indicated transcript}
#'   \item{ENSG00000058056}{normalized expression for indicated transcript}
#'   \item{ENSG00000060138}{normalized expression for indicated transcript}
#'   \item{ENSG00000061918}{normalized expression for indicated transcript}
#'   \item{ENSG00000065809}{normalized expression for indicated transcript}
#'   \item{ENSG00000065923}{normalized expression for indicated transcript}
#'   \item{ENSG00000068489}{normalized expression for indicated transcript}
#'   \item{ENSG00000069020}{normalized expression for indicated transcript}
#'   \item{ENSG00000070404}{normalized expression for indicated transcript}
#'   \item{ENSG00000071894}{normalized expression for indicated transcript}
#'   \item{ENSG00000072422}{normalized expression for indicated transcript}
#'   \item{ENSG00000073605}{normalized expression for indicated transcript}
#'   \item{ENSG00000076555}{normalized expression for indicated transcript}
#'   \item{ENSG00000080823}{normalized expression for indicated transcript}
#'   \item{ENSG00000089723}{normalized expression for indicated transcript}
#'   \item{ENSG00000090382}{normalized expression for indicated transcript}
#'   \item{ENSG00000090975}{normalized expression for indicated transcript}
#'   \item{ENSG00000100068}{normalized expression for indicated transcript}
#'   \item{ENSG00000100077}{normalized expression for indicated transcript}
#'   \item{ENSG00000100299}{normalized expression for indicated transcript}
#'   \item{ENSG00000100376}{normalized expression for indicated transcript}
#'   \item{ENSG00000100418}{normalized expression for indicated transcript}
#'   \item{ENSG00000100448}{normalized expression for indicated transcript}
#'   \item{ENSG00000100596}{normalized expression for indicated transcript}
#'   \item{ENSG00000100916}{normalized expression for indicated transcript}
#'   \item{ENSG00000102409}{normalized expression for indicated transcript}
#'   \item{ENSG00000102760}{normalized expression for indicated transcript}
#'   \item{ENSG00000104689}{normalized expression for indicated transcript}
#'   \item{ENSG00000104946}{normalized expression for indicated transcript}
#'   \item{ENSG00000105518}{normalized expression for indicated transcript}
#'   \item{ENSG00000105808}{normalized expression for indicated transcript}
#'   \item{ENSG00000106367}{normalized expression for indicated transcript}
#'   \item{ENSG00000106526}{normalized expression for indicated transcript}
#'   \item{ENSG00000106546}{normalized expression for indicated transcript}
#'   \item{ENSG00000106780}{normalized expression for indicated transcript}
#'   \item{ENSG00000107104}{normalized expression for indicated transcript}
#'   \item{ENSG00000107742}{normalized expression for indicated transcript}
#'   \item{ENSG00000107798}{normalized expression for indicated transcript}
#'   \item{ENSG00000107816}{normalized expression for indicated transcript}
#'   \item{ENSG00000107957}{normalized expression for indicated transcript}
#'   \item{ENSG00000109674}{normalized expression for indicated transcript}
#'   \item{ENSG00000110076}{normalized expression for indicated transcript}
#'   \item{ENSG00000110237}{normalized expression for indicated transcript}
#'   \item{ENSG00000110492}{normalized expression for indicated transcript}
#'   \item{ENSG00000110799}{normalized expression for indicated transcript}
#'   \item{ENSG00000111275}{normalized expression for indicated transcript}
#'   \item{ENSG00000112773}{normalized expression for indicated transcript}
#'   \item{ENSG00000113504}{normalized expression for indicated transcript}
#'   \item{ENSG00000114268}{normalized expression for indicated transcript}
#'   \item{ENSG00000114737}{normalized expression for indicated transcript}
#'   \item{ENSG00000115183}{normalized expression for indicated transcript}
#'   \item{ENSG00000115414}{normalized expression for indicated transcript}
#'   \item{ENSG00000115457}{normalized expression for indicated transcript}
#'   \item{ENSG00000115525}{normalized expression for indicated transcript}
#'   \item{ENSG00000116574}{normalized expression for indicated transcript}
#'   \item{ENSG00000117480}{normalized expression for indicated transcript}
#'   \item{ENSG00000119280}{normalized expression for indicated transcript}
#'   \item{ENSG00000120594}{normalized expression for indicated transcript}
#'   \item{ENSG00000120675}{normalized expression for indicated transcript}
#'   \item{ENSG00000120832}{normalized expression for indicated transcript}
#'   \item{ENSG00000120913}{normalized expression for indicated transcript}
#'   \item{ENSG00000121005}{normalized expression for indicated transcript}
#'   \item{ENSG00000121039}{normalized expression for indicated transcript}
#'   \item{ENSG00000121274}{normalized expression for indicated transcript}
#'   \item{ENSG00000123080}{normalized expression for indicated transcript}
#'   \item{ENSG00000123836}{normalized expression for indicated transcript}
#'   \item{ENSG00000124019}{normalized expression for indicated transcript}
#'   \item{ENSG00000124882}{normalized expression for indicated transcript}
#'   \item{ENSG00000126822}{normalized expression for indicated transcript}
#'   \item{ENSG00000127152}{normalized expression for indicated transcript}
#'   \item{ENSG00000129824}{normalized expression for indicated transcript}
#'   \item{ENSG00000130702}{normalized expression for indicated transcript}
#'   \item{ENSG00000131188}{normalized expression for indicated transcript}
#'   \item{ENSG00000131370}{normalized expression for indicated transcript}
#'   \item{ENSG00000132122}{normalized expression for indicated transcript}
#'   \item{ENSG00000132530}{normalized expression for indicated transcript}
#'   \item{ENSG00000132819}{normalized expression for indicated transcript}
#'   \item{ENSG00000132849}{normalized expression for indicated transcript}
#'   \item{ENSG00000133401}{normalized expression for indicated transcript}
#'   \item{ENSG00000133619}{normalized expression for indicated transcript}
#'   \item{ENSG00000134531}{normalized expression for indicated transcript}
#'   \item{ENSG00000134897}{normalized expression for indicated transcript}
#'   \item{ENSG00000135074}{normalized expression for indicated transcript}
#'   \item{ENSG00000135245}{normalized expression for indicated transcript}
#'   \item{ENSG00000135272}{normalized expression for indicated transcript}
#'   \item{ENSG00000135362}{normalized expression for indicated transcript}
#'   \item{ENSG00000135363}{normalized expression for indicated transcript}
#'   \item{ENSG00000135916}{normalized expression for indicated transcript}
#'   \item{ENSG00000136026}{normalized expression for indicated transcript}
#'   \item{ENSG00000136193}{normalized expression for indicated transcript}
#'   \item{ENSG00000136231}{normalized expression for indicated transcript}
#'   \item{ENSG00000136997}{normalized expression for indicated transcript}
#'   \item{ENSG00000137193}{normalized expression for indicated transcript}
#'   \item{ENSG00000137198}{normalized expression for indicated transcript}
#'   \item{ENSG00000138722}{normalized expression for indicated transcript}
#'   \item{ENSG00000139318}{normalized expression for indicated transcript}
#'   \item{ENSG00000140287}{normalized expression for indicated transcript}
#'   \item{ENSG00000144036}{normalized expression for indicated transcript}
#'   \item{ENSG00000144647}{normalized expression for indicated transcript}
#'   \item{ENSG00000144677}{normalized expression for indicated transcript}
#'   \item{ENSG00000145476}{normalized expression for indicated transcript}
#'   \item{ENSG00000145545}{normalized expression for indicated transcript}
#'   \item{ENSG00000146243}{normalized expression for indicated transcript}
#'   \item{ENSG00000146373}{normalized expression for indicated transcript}
#'   \item{ENSG00000147044}{normalized expression for indicated transcript}
#'   \item{ENSG00000147180}{normalized expression for indicated transcript}
#'   \item{ENSG00000148444}{normalized expression for indicated transcript}
#'   \item{ENSG00000148484}{normalized expression for indicated transcript}
#'   \item{ENSG00000149131}{normalized expression for indicated transcript}
#'   \item{ENSG00000150760}{normalized expression for indicated transcript}
#'   \item{ENSG00000150782}{normalized expression for indicated transcript}
#'   \item{ENSG00000151135}{normalized expression for indicated transcript}
#'   \item{ENSG00000151208}{normalized expression for indicated transcript}
#'   \item{ENSG00000151458}{normalized expression for indicated transcript}
#'   \item{ENSG00000152409}{normalized expression for indicated transcript}
#'   \item{ENSG00000152580}{normalized expression for indicated transcript}
#'   \item{ENSG00000152767}{normalized expression for indicated transcript}
#'   \item{ENSG00000152778}{normalized expression for indicated transcript}
#'   \item{ENSG00000153563}{normalized expression for indicated transcript}
#'   \item{ENSG00000154217}{normalized expression for indicated transcript}
#'   \item{ENSG00000154743}{normalized expression for indicated transcript}
#'   \item{ENSG00000154760}{normalized expression for indicated transcript}
#'   \item{ENSG00000154874}{normalized expression for indicated transcript}
#'   \item{ENSG00000156381}{normalized expression for indicated transcript}
#'   \item{ENSG00000157107}{normalized expression for indicated transcript}
#'   \item{ENSG00000157240}{normalized expression for indicated transcript}
#'   \item{ENSG00000157873}{normalized expression for indicated transcript}
#'   \item{ENSG00000157978}{normalized expression for indicated transcript}
#'   \item{ENSG00000158691}{normalized expression for indicated transcript}
#'   \item{ENSG00000159339}{normalized expression for indicated transcript}
#'   \item{ENSG00000159403}{normalized expression for indicated transcript}
#'   \item{ENSG00000159788}{normalized expression for indicated transcript}
#'   \item{ENSG00000160685}{normalized expression for indicated transcript}
#'   \item{ENSG00000160781}{normalized expression for indicated transcript}
#'   \item{ENSG00000161509}{normalized expression for indicated transcript}
#'   \item{ENSG00000162433}{normalized expression for indicated transcript}
#'   \item{ENSG00000162614}{normalized expression for indicated transcript}
#'   \item{ENSG00000162676}{normalized expression for indicated transcript}
#'   \item{ENSG00000163412}{normalized expression for indicated transcript}
#'   \item{ENSG00000163702}{normalized expression for indicated transcript}
#'   \item{ENSG00000163814}{normalized expression for indicated transcript}
#'   \item{ENSG00000164086}{normalized expression for indicated transcript}
#'   \item{ENSG00000164172}{normalized expression for indicated transcript}
#'   \item{ENSG00000164442}{normalized expression for indicated transcript}
#'   \item{ENSG00000165272}{normalized expression for indicated transcript}
#'   \item{ENSG00000166165}{normalized expression for indicated transcript}
#'   \item{ENSG00000166435}{normalized expression for indicated transcript}
#'   \item{ENSG00000166987}{normalized expression for indicated transcript}
#'   \item{ENSG00000167291}{normalized expression for indicated transcript}
#'   \item{ENSG00000167565}{normalized expression for indicated transcript}
#'   \item{ENSG00000167851}{normalized expression for indicated transcript}
#'   \item{ENSG00000168026}{normalized expression for indicated transcript}
#'   \item{ENSG00000168209}{normalized expression for indicated transcript}
#'   \item{ENSG00000168502}{normalized expression for indicated transcript}
#'   \item{ENSG00000168939}{normalized expression for indicated transcript}
#'   \item{ENSG00000169203}{normalized expression for indicated transcript}
#'   \item{ENSG00000169247}{normalized expression for indicated transcript}
#'   \item{ENSG00000169504}{normalized expression for indicated transcript}
#'   \item{ENSG00000169860}{normalized expression for indicated transcript}
#'   \item{ENSG00000169991}{normalized expression for indicated transcript}
#'   \item{ENSG00000170035}{normalized expression for indicated transcript}
#'   \item{ENSG00000170180}{normalized expression for indicated transcript}
#'   \item{ENSG00000170456}{normalized expression for indicated transcript}
#'   \item{ENSG00000170522}{normalized expression for indicated transcript}
#'   \item{ENSG00000170909}{normalized expression for indicated transcript}
#'   \item{ENSG00000171121}{normalized expression for indicated transcript}
#'   \item{ENSG00000171222}{normalized expression for indicated transcript}
#'   \item{ENSG00000171476}{normalized expression for indicated transcript}
#'   \item{ENSG00000171813}{normalized expression for indicated transcript}
#'   \item{ENSG00000171962}{normalized expression for indicated transcript}
#'   \item{ENSG00000172197}{normalized expression for indicated transcript}
#'   \item{ENSG00000172236}{normalized expression for indicated transcript}
#'   \item{ENSG00000173083}{normalized expression for indicated transcript}
#'   \item{ENSG00000173530}{normalized expression for indicated transcript}
#'   \item{ENSG00000173926}{normalized expression for indicated transcript}
#'   \item{ENSG00000174059}{normalized expression for indicated transcript}
#'   \item{ENSG00000174080}{normalized expression for indicated transcript}
#'   \item{ENSG00000174130}{normalized expression for indicated transcript}
#'   \item{ENSG00000174738}{normalized expression for indicated transcript}
#'   \item{ENSG00000175265}{normalized expression for indicated transcript}
#'   \item{ENSG00000175352}{normalized expression for indicated transcript}
#'   \item{ENSG00000176597}{normalized expression for indicated transcript}
#'   \item{ENSG00000179222}{normalized expression for indicated transcript}
#'   \item{ENSG00000179630}{normalized expression for indicated transcript}
#'   \item{ENSG00000179639}{normalized expression for indicated transcript}
#'   \item{ENSG00000179820}{normalized expression for indicated transcript}
#'   \item{ENSG00000180096}{normalized expression for indicated transcript}
#'   \item{ENSG00000180596}{normalized expression for indicated transcript}
#'   \item{ENSG00000180902}{normalized expression for indicated transcript}
#'   \item{ENSG00000181104}{normalized expression for indicated transcript}
#'   \item{ENSG00000182866}{normalized expression for indicated transcript}
#'   \item{ENSG00000182871}{normalized expression for indicated transcript}
#'   \item{ENSG00000183087}{normalized expression for indicated transcript}
#'   \item{ENSG00000183091}{normalized expression for indicated transcript}
#'   \item{ENSG00000184371}{normalized expression for indicated transcript}
#'   \item{ENSG00000185129}{normalized expression for indicated transcript}
#'   \item{ENSG00000185201}{normalized expression for indicated transcript}
#'   \item{ENSG00000185245}{normalized expression for indicated transcript}
#'   \item{ENSG00000185291}{normalized expression for indicated transcript}
#'   \item{ENSG00000185304}{normalized expression for indicated transcript}
#'   \item{ENSG00000185710}{normalized expression for indicated transcript}
#'   \item{ENSG00000185883}{normalized expression for indicated transcript}
#'   \item{ENSG00000185986}{normalized expression for indicated transcript}
#'   \item{ENSG00000186130}{normalized expression for indicated transcript}
#'   \item{ENSG00000186854}{normalized expression for indicated transcript}
#'   \item{ENSG00000187010}{normalized expression for indicated transcript}
#'   \item{ENSG00000187627}{normalized expression for indicated transcript}
#'   \item{ENSG00000187653}{normalized expression for indicated transcript}
#'   \item{ENSG00000187837}{normalized expression for indicated transcript}
#'   \item{ENSG00000188002}{normalized expression for indicated transcript}
#'   \item{ENSG00000188107}{normalized expression for indicated transcript}
#'   \item{ENSG00000188211}{normalized expression for indicated transcript}
#'   \item{ENSG00000188636}{normalized expression for indicated transcript}
#'   \item{ENSG00000188738}{normalized expression for indicated transcript}
#'   \item{ENSG00000188856}{normalized expression for indicated transcript}
#'   \item{ENSG00000189164}{normalized expression for indicated transcript}
#'   \item{ENSG00000189223}{normalized expression for indicated transcript}
#'   \item{ENSG00000196155}{normalized expression for indicated transcript}
#'   \item{ENSG00000196189}{normalized expression for indicated transcript}
#'   \item{ENSG00000196415}{normalized expression for indicated transcript}
#'   \item{ENSG00000196565}{normalized expression for indicated transcript}
#'   \item{ENSG00000197081}{normalized expression for indicated transcript}
#'   \item{ENSG00000197121}{normalized expression for indicated transcript}
#'   \item{ENSG00000197253}{normalized expression for indicated transcript}
#'   \item{ENSG00000197256}{normalized expression for indicated transcript}
#'   \item{ENSG00000197321}{normalized expression for indicated transcript}
#'   \item{ENSG00000197561}{normalized expression for indicated transcript}
#'   \item{ENSG00000197728}{normalized expression for indicated transcript}
#'   \item{ENSG00000197860}{normalized expression for indicated transcript}
#'   \item{ENSG00000197937}{normalized expression for indicated transcript}
#'   \item{ENSG00000197951}{normalized expression for indicated transcript}
#'   \item{ENSG00000198743}{normalized expression for indicated transcript}
#'   \item{ENSG00000198838}{normalized expression for indicated transcript}
#'   \item{ENSG00000199347}{normalized expression for indicated transcript}
#'   \item{ENSG00000200243}{normalized expression for indicated transcript}
#'   \item{ENSG00000201801}{normalized expression for indicated transcript}
#'   \item{ENSG00000203872}{normalized expression for indicated transcript}
#'   \item{ENSG00000204172}{normalized expression for indicated transcript}
#'   \item{ENSG00000205571}{normalized expression for indicated transcript}
#'   \item{ENSG00000205593}{normalized expression for indicated transcript}
#'   \item{ENSG00000208772}{normalized expression for indicated transcript}
#'   \item{ENSG00000213085}{normalized expression for indicated transcript}
#'   \item{ENSG00000213261}{normalized expression for indicated transcript}
#'   \item{ENSG00000213626}{normalized expression for indicated transcript}
#'   \item{ENSG00000213722}{normalized expression for indicated transcript}
#'   \item{ENSG00000213906}{normalized expression for indicated transcript}
#'   \item{ENSG00000213967}{normalized expression for indicated transcript}
#'   \item{ENSG00000214016}{normalized expression for indicated transcript}
#'   \item{ENSG00000214425}{normalized expression for indicated transcript}
#'   \item{ENSG00000216316}{normalized expression for indicated transcript}
#'   \item{ENSG00000220008}{normalized expression for indicated transcript}
#'   \item{ENSG00000223345}{normalized expression for indicated transcript}
#'   \item{ENSG00000224080}{normalized expression for indicated transcript}
#'   \item{ENSG00000225138}{normalized expression for indicated transcript}
#'   \item{ENSG00000226471}{normalized expression for indicated transcript}
#'   \item{ENSG00000227097}{normalized expression for indicated transcript}
#'   \item{ENSG00000227191}{normalized expression for indicated transcript}
#'   \item{ENSG00000227615}{normalized expression for indicated transcript}
#'   \item{ENSG00000228049}{normalized expression for indicated transcript}
#'   \item{ENSG00000229153}{normalized expression for indicated transcript}
#'   \item{ENSG00000230076}{normalized expression for indicated transcript}
#'   \item{ENSG00000231160}{normalized expression for indicated transcript}
#'   \item{ENSG00000231721}{normalized expression for indicated transcript}
#'   \item{ENSG00000233927}{normalized expression for indicated transcript}
#'   \item{ENSG00000233974}{normalized expression for indicated transcript}
#'   \item{ENSG00000234883}{normalized expression for indicated transcript}
#'   \item{ENSG00000236876}{normalized expression for indicated transcript}
#'   \item{ENSG00000237298}{normalized expression for indicated transcript}
#'   \item{ENSG00000237892}{normalized expression for indicated transcript}
#'   \item{ENSG00000238160}{normalized expression for indicated transcript}
#'   \item{ENSG00000239437}{normalized expression for indicated transcript}
#'   \item{ENSG00000241399}{normalized expression for indicated transcript}
#'   \item{ENSG00000241489}{normalized expression for indicated transcript}
#'   \item{ENSG00000241529}{normalized expression for indicated transcript}
#'   \item{ENSG00000244405}{normalized expression for indicated transcript}
#'   \item{ENSG00000247627}{normalized expression for indicated transcript}
#'   \item{ENSG00000249592}{normalized expression for indicated transcript}
#'   \item{ENSG00000250116}{normalized expression for indicated transcript}
#'   \item{ENSG00000250251}{normalized expression for indicated transcript}
#'   \item{ENSG00000251079}{normalized expression for indicated transcript}
#'   \item{ENSG00000253210}{normalized expression for indicated transcript}
#'   \item{ENSG00000253276}{normalized expression for indicated transcript}
#'   \item{ENSG00000254415}{normalized expression for indicated transcript}
#'   \item{ENSG00000259276}{normalized expression for indicated transcript}
#'   \item{ENSG00000260727}{normalized expression for indicated transcript}
#'   \item{ENSG00000261377}{normalized expression for indicated transcript}
#'   \item{ENSG00000264885}{normalized expression for indicated transcript}
#'   \item{ENSG00000264895}{normalized expression for indicated transcript}
#'   \item{ENSG00000267136}{normalized expression for indicated transcript}
#'   \item{ENSG00000267551}{normalized expression for indicated transcript}
#'   \item{ENSG00000267702}{normalized expression for indicated transcript}
#'   \item{ENSG00000268001}{normalized expression for indicated transcript}
#'   \item{ENSG00000268573}{normalized expression for indicated transcript}
#'   \item{ENSG00000270554}{normalized expression for indicated transcript}
#'   \item{ENSG00000270562}{normalized expression for indicated transcript}
#'   \item{ENSG00000271646}{normalized expression for indicated transcript}
#'   \item{ENSG00000273018}{normalized expression for indicated transcript}
#'   \item{ENSG00000273033}{normalized expression for indicated transcript}
#'}
#' @source <doi:10.1186/s13045-024-01553-6>
"amltrain"
#' AML test data
#'
#' Duration of complete response for 40 cytogenetically normal AML patients
#' and a subset of 320 transcript expression from RNA-sequencing.
#'
#'
#' @format
#' A data frame with 40 rows (subjects) and 322 columns:
#' \describe{
#'   \item{cryr}{duration of complete response in years}
#'   \item{relapse.death}{censoring indicator: 1 = relapsed or died;
#'   0 = alive at last follow=up}
#'   \item{ENSG00000001561}{normalized expression for indicated transcript}
#'   \item{ENSG00000005249}{normalized expression for indicated transcript}
#'   \item{ENSG00000006757}{normalized expression for indicated transcript}
#'   \item{ENSG00000007062}{normalized expression for indicated transcript}
#'   \item{ENSG00000007968}{normalized expression for indicated transcript}
#'   \item{ENSG00000008283}{normalized expression for indicated transcript}
#'   \item{ENSG00000008405}{normalized expression for indicated transcript}
#'   \item{ENSG00000008441}{normalized expression for indicated transcript}
#'   \item{ENSG00000010295}{normalized expression for indicated transcript}
#'   \item{ENSG00000011028}{normalized expression for indicated transcript}
#'   \item{ENSG00000011198}{normalized expression for indicated transcript}
#'   \item{ENSG00000012779}{normalized expression for indicated transcript}
#'   \item{ENSG00000012817}{normalized expression for indicated transcript}
#'   \item{ENSG00000013306}{normalized expression for indicated transcript}
#'   \item{ENSG00000013725}{normalized expression for indicated transcript}
#'   \item{ENSG00000018189}{normalized expression for indicated transcript}
#'   \item{ENSG00000022267}{normalized expression for indicated transcript}
#'   \item{ENSG00000023171}{normalized expression for indicated transcript}
#'   \item{ENSG00000023909}{normalized expression for indicated transcript}
#'   \item{ENSG00000029639}{normalized expression for indicated transcript}
#'   \item{ENSG00000047634}{normalized expression for indicated transcript}
#'   \item{ENSG00000049192}{normalized expression for indicated transcript}
#'   \item{ENSG00000053524}{normalized expression for indicated transcript}
#'   \item{ENSG00000058056}{normalized expression for indicated transcript}
#'   \item{ENSG00000060138}{normalized expression for indicated transcript}
#'   \item{ENSG00000061918}{normalized expression for indicated transcript}
#'   \item{ENSG00000065809}{normalized expression for indicated transcript}
#'   \item{ENSG00000065923}{normalized expression for indicated transcript}
#'   \item{ENSG00000068489}{normalized expression for indicated transcript}
#'   \item{ENSG00000069020}{normalized expression for indicated transcript}
#'   \item{ENSG00000070404}{normalized expression for indicated transcript}
#'   \item{ENSG00000071894}{normalized expression for indicated transcript}
#'   \item{ENSG00000072422}{normalized expression for indicated transcript}
#'   \item{ENSG00000073605}{normalized expression for indicated transcript}
#'   \item{ENSG00000076555}{normalized expression for indicated transcript}
#'   \item{ENSG00000080823}{normalized expression for indicated transcript}
#'   \item{ENSG00000089723}{normalized expression for indicated transcript}
#'   \item{ENSG00000090382}{normalized expression for indicated transcript}
#'   \item{ENSG00000090975}{normalized expression for indicated transcript}
#'   \item{ENSG00000100068}{normalized expression for indicated transcript}
#'   \item{ENSG00000100077}{normalized expression for indicated transcript}
#'   \item{ENSG00000100299}{normalized expression for indicated transcript}
#'   \item{ENSG00000100376}{normalized expression for indicated transcript}
#'   \item{ENSG00000100418}{normalized expression for indicated transcript}
#'   \item{ENSG00000100448}{normalized expression for indicated transcript}
#'   \item{ENSG00000100596}{normalized expression for indicated transcript}
#'   \item{ENSG00000100916}{normalized expression for indicated transcript}
#'   \item{ENSG00000102409}{normalized expression for indicated transcript}
#'   \item{ENSG00000102760}{normalized expression for indicated transcript}
#'   \item{ENSG00000104689}{normalized expression for indicated transcript}
#'   \item{ENSG00000104946}{normalized expression for indicated transcript}
#'   \item{ENSG00000105518}{normalized expression for indicated transcript}
#'   \item{ENSG00000105808}{normalized expression for indicated transcript}
#'   \item{ENSG00000106367}{normalized expression for indicated transcript}
#'   \item{ENSG00000106526}{normalized expression for indicated transcript}
#'   \item{ENSG00000106546}{normalized expression for indicated transcript}
#'   \item{ENSG00000106780}{normalized expression for indicated transcript}
#'   \item{ENSG00000107104}{normalized expression for indicated transcript}
#'   \item{ENSG00000107742}{normalized expression for indicated transcript}
#'   \item{ENSG00000107798}{normalized expression for indicated transcript}
#'   \item{ENSG00000107816}{normalized expression for indicated transcript}
#'   \item{ENSG00000107957}{normalized expression for indicated transcript}
#'   \item{ENSG00000109674}{normalized expression for indicated transcript}
#'   \item{ENSG00000110076}{normalized expression for indicated transcript}
#'   \item{ENSG00000110237}{normalized expression for indicated transcript}
#'   \item{ENSG00000110492}{normalized expression for indicated transcript}
#'   \item{ENSG00000110799}{normalized expression for indicated transcript}
#'   \item{ENSG00000111275}{normalized expression for indicated transcript}
#'   \item{ENSG00000112773}{normalized expression for indicated transcript}
#'   \item{ENSG00000113504}{normalized expression for indicated transcript}
#'   \item{ENSG00000114268}{normalized expression for indicated transcript}
#'   \item{ENSG00000114737}{normalized expression for indicated transcript}
#'   \item{ENSG00000115183}{normalized expression for indicated transcript}
#'   \item{ENSG00000115414}{normalized expression for indicated transcript}
#'   \item{ENSG00000115457}{normalized expression for indicated transcript}
#'   \item{ENSG00000115525}{normalized expression for indicated transcript}
#'   \item{ENSG00000116574}{normalized expression for indicated transcript}
#'   \item{ENSG00000117480}{normalized expression for indicated transcript}
#'   \item{ENSG00000119280}{normalized expression for indicated transcript}
#'   \item{ENSG00000120594}{normalized expression for indicated transcript}
#'   \item{ENSG00000120675}{normalized expression for indicated transcript}
#'   \item{ENSG00000120832}{normalized expression for indicated transcript}
#'   \item{ENSG00000120913}{normalized expression for indicated transcript}
#'   \item{ENSG00000121005}{normalized expression for indicated transcript}
#'   \item{ENSG00000121039}{normalized expression for indicated transcript}
#'   \item{ENSG00000121274}{normalized expression for indicated transcript}
#'   \item{ENSG00000123080}{normalized expression for indicated transcript}
#'   \item{ENSG00000123836}{normalized expression for indicated transcript}
#'   \item{ENSG00000124019}{normalized expression for indicated transcript}
#'   \item{ENSG00000124882}{normalized expression for indicated transcript}
#'   \item{ENSG00000126822}{normalized expression for indicated transcript}
#'   \item{ENSG00000127152}{normalized expression for indicated transcript}
#'   \item{ENSG00000129824}{normalized expression for indicated transcript}
#'   \item{ENSG00000130702}{normalized expression for indicated transcript}
#'   \item{ENSG00000131188}{normalized expression for indicated transcript}
#'   \item{ENSG00000131370}{normalized expression for indicated transcript}
#'   \item{ENSG00000132122}{normalized expression for indicated transcript}
#'   \item{ENSG00000132530}{normalized expression for indicated transcript}
#'   \item{ENSG00000132819}{normalized expression for indicated transcript}
#'   \item{ENSG00000132849}{normalized expression for indicated transcript}
#'   \item{ENSG00000133401}{normalized expression for indicated transcript}
#'   \item{ENSG00000133619}{normalized expression for indicated transcript}
#'   \item{ENSG00000134531}{normalized expression for indicated transcript}
#'   \item{ENSG00000134897}{normalized expression for indicated transcript}
#'   \item{ENSG00000135074}{normalized expression for indicated transcript}
#'   \item{ENSG00000135245}{normalized expression for indicated transcript}
#'   \item{ENSG00000135272}{normalized expression for indicated transcript}
#'   \item{ENSG00000135362}{normalized expression for indicated transcript}
#'   \item{ENSG00000135363}{normalized expression for indicated transcript}
#'   \item{ENSG00000135916}{normalized expression for indicated transcript}
#'   \item{ENSG00000136026}{normalized expression for indicated transcript}
#'   \item{ENSG00000136193}{normalized expression for indicated transcript}
#'   \item{ENSG00000136231}{normalized expression for indicated transcript}
#'   \item{ENSG00000136997}{normalized expression for indicated transcript}
#'   \item{ENSG00000137193}{normalized expression for indicated transcript}
#'   \item{ENSG00000137198}{normalized expression for indicated transcript}
#'   \item{ENSG00000138722}{normalized expression for indicated transcript}
#'   \item{ENSG00000139318}{normalized expression for indicated transcript}
#'   \item{ENSG00000140287}{normalized expression for indicated transcript}
#'   \item{ENSG00000144036}{normalized expression for indicated transcript}
#'   \item{ENSG00000144647}{normalized expression for indicated transcript}
#'   \item{ENSG00000144677}{normalized expression for indicated transcript}
#'   \item{ENSG00000145476}{normalized expression for indicated transcript}
#'   \item{ENSG00000145545}{normalized expression for indicated transcript}
#'   \item{ENSG00000146243}{normalized expression for indicated transcript}
#'   \item{ENSG00000146373}{normalized expression for indicated transcript}
#'   \item{ENSG00000147044}{normalized expression for indicated transcript}
#'   \item{ENSG00000147180}{normalized expression for indicated transcript}
#'   \item{ENSG00000148444}{normalized expression for indicated transcript}
#'   \item{ENSG00000148484}{normalized expression for indicated transcript}
#'   \item{ENSG00000149131}{normalized expression for indicated transcript}
#'   \item{ENSG00000150760}{normalized expression for indicated transcript}
#'   \item{ENSG00000150782}{normalized expression for indicated transcript}
#'   \item{ENSG00000151135}{normalized expression for indicated transcript}
#'   \item{ENSG00000151208}{normalized expression for indicated transcript}
#'   \item{ENSG00000151458}{normalized expression for indicated transcript}
#'   \item{ENSG00000152409}{normalized expression for indicated transcript}
#'   \item{ENSG00000152580}{normalized expression for indicated transcript}
#'   \item{ENSG00000152767}{normalized expression for indicated transcript}
#'   \item{ENSG00000152778}{normalized expression for indicated transcript}
#'   \item{ENSG00000153563}{normalized expression for indicated transcript}
#'   \item{ENSG00000154217}{normalized expression for indicated transcript}
#'   \item{ENSG00000154743}{normalized expression for indicated transcript}
#'   \item{ENSG00000154760}{normalized expression for indicated transcript}
#'   \item{ENSG00000154874}{normalized expression for indicated transcript}
#'   \item{ENSG00000156381}{normalized expression for indicated transcript}
#'   \item{ENSG00000157107}{normalized expression for indicated transcript}
#'   \item{ENSG00000157240}{normalized expression for indicated transcript}
#'   \item{ENSG00000157873}{normalized expression for indicated transcript}
#'   \item{ENSG00000157978}{normalized expression for indicated transcript}
#'   \item{ENSG00000158691}{normalized expression for indicated transcript}
#'   \item{ENSG00000159339}{normalized expression for indicated transcript}
#'   \item{ENSG00000159403}{normalized expression for indicated transcript}
#'   \item{ENSG00000159788}{normalized expression for indicated transcript}
#'   \item{ENSG00000160685}{normalized expression for indicated transcript}
#'   \item{ENSG00000160781}{normalized expression for indicated transcript}
#'   \item{ENSG00000161509}{normalized expression for indicated transcript}
#'   \item{ENSG00000162433}{normalized expression for indicated transcript}
#'   \item{ENSG00000162614}{normalized expression for indicated transcript}
#'   \item{ENSG00000162676}{normalized expression for indicated transcript}
#'   \item{ENSG00000163412}{normalized expression for indicated transcript}
#'   \item{ENSG00000163702}{normalized expression for indicated transcript}
#'   \item{ENSG00000163814}{normalized expression for indicated transcript}
#'   \item{ENSG00000164086}{normalized expression for indicated transcript}
#'   \item{ENSG00000164172}{normalized expression for indicated transcript}
#'   \item{ENSG00000164442}{normalized expression for indicated transcript}
#'   \item{ENSG00000165272}{normalized expression for indicated transcript}
#'   \item{ENSG00000166165}{normalized expression for indicated transcript}
#'   \item{ENSG00000166435}{normalized expression for indicated transcript}
#'   \item{ENSG00000166987}{normalized expression for indicated transcript}
#'   \item{ENSG00000167291}{normalized expression for indicated transcript}
#'   \item{ENSG00000167565}{normalized expression for indicated transcript}
#'   \item{ENSG00000167851}{normalized expression for indicated transcript}
#'   \item{ENSG00000168026}{normalized expression for indicated transcript}
#'   \item{ENSG00000168209}{normalized expression for indicated transcript}
#'   \item{ENSG00000168502}{normalized expression for indicated transcript}
#'   \item{ENSG00000168939}{normalized expression for indicated transcript}
#'   \item{ENSG00000169203}{normalized expression for indicated transcript}
#'   \item{ENSG00000169247}{normalized expression for indicated transcript}
#'   \item{ENSG00000169504}{normalized expression for indicated transcript}
#'   \item{ENSG00000169860}{normalized expression for indicated transcript}
#'   \item{ENSG00000169991}{normalized expression for indicated transcript}
#'   \item{ENSG00000170035}{normalized expression for indicated transcript}
#'   \item{ENSG00000170180}{normalized expression for indicated transcript}
#'   \item{ENSG00000170456}{normalized expression for indicated transcript}
#'   \item{ENSG00000170522}{normalized expression for indicated transcript}
#'   \item{ENSG00000170909}{normalized expression for indicated transcript}
#'   \item{ENSG00000171121}{normalized expression for indicated transcript}
#'   \item{ENSG00000171222}{normalized expression for indicated transcript}
#'   \item{ENSG00000171476}{normalized expression for indicated transcript}
#'   \item{ENSG00000171813}{normalized expression for indicated transcript}
#'   \item{ENSG00000171962}{normalized expression for indicated transcript}
#'   \item{ENSG00000172197}{normalized expression for indicated transcript}
#'   \item{ENSG00000172236}{normalized expression for indicated transcript}
#'   \item{ENSG00000173083}{normalized expression for indicated transcript}
#'   \item{ENSG00000173530}{normalized expression for indicated transcript}
#'   \item{ENSG00000173926}{normalized expression for indicated transcript}
#'   \item{ENSG00000174059}{normalized expression for indicated transcript}
#'   \item{ENSG00000174080}{normalized expression for indicated transcript}
#'   \item{ENSG00000174130}{normalized expression for indicated transcript}
#'   \item{ENSG00000174738}{normalized expression for indicated transcript}
#'   \item{ENSG00000175265}{normalized expression for indicated transcript}
#'   \item{ENSG00000175352}{normalized expression for indicated transcript}
#'   \item{ENSG00000176597}{normalized expression for indicated transcript}
#'   \item{ENSG00000179222}{normalized expression for indicated transcript}
#'   \item{ENSG00000179630}{normalized expression for indicated transcript}
#'   \item{ENSG00000179639}{normalized expression for indicated transcript}
#'   \item{ENSG00000179820}{normalized expression for indicated transcript}
#'   \item{ENSG00000180096}{normalized expression for indicated transcript}
#'   \item{ENSG00000180596}{normalized expression for indicated transcript}
#'   \item{ENSG00000180902}{normalized expression for indicated transcript}
#'   \item{ENSG00000181104}{normalized expression for indicated transcript}
#'   \item{ENSG00000182866}{normalized expression for indicated transcript}
#'   \item{ENSG00000182871}{normalized expression for indicated transcript}
#'   \item{ENSG00000183087}{normalized expression for indicated transcript}
#'   \item{ENSG00000183091}{normalized expression for indicated transcript}
#'   \item{ENSG00000184371}{normalized expression for indicated transcript}
#'   \item{ENSG00000185129}{normalized expression for indicated transcript}
#'   \item{ENSG00000185201}{normalized expression for indicated transcript}
#'   \item{ENSG00000185245}{normalized expression for indicated transcript}
#'   \item{ENSG00000185291}{normalized expression for indicated transcript}
#'   \item{ENSG00000185304}{normalized expression for indicated transcript}
#'   \item{ENSG00000185710}{normalized expression for indicated transcript}
#'   \item{ENSG00000185883}{normalized expression for indicated transcript}
#'   \item{ENSG00000185986}{normalized expression for indicated transcript}
#'   \item{ENSG00000186130}{normalized expression for indicated transcript}
#'   \item{ENSG00000186854}{normalized expression for indicated transcript}
#'   \item{ENSG00000187010}{normalized expression for indicated transcript}
#'   \item{ENSG00000187627}{normalized expression for indicated transcript}
#'   \item{ENSG00000187653}{normalized expression for indicated transcript}
#'   \item{ENSG00000187837}{normalized expression for indicated transcript}
#'   \item{ENSG00000188002}{normalized expression for indicated transcript}
#'   \item{ENSG00000188107}{normalized expression for indicated transcript}
#'   \item{ENSG00000188211}{normalized expression for indicated transcript}
#'   \item{ENSG00000188636}{normalized expression for indicated transcript}
#'   \item{ENSG00000188738}{normalized expression for indicated transcript}
#'   \item{ENSG00000188856}{normalized expression for indicated transcript}
#'   \item{ENSG00000189164}{normalized expression for indicated transcript}
#'   \item{ENSG00000189223}{normalized expression for indicated transcript}
#'   \item{ENSG00000196155}{normalized expression for indicated transcript}
#'   \item{ENSG00000196189}{normalized expression for indicated transcript}
#'   \item{ENSG00000196415}{normalized expression for indicated transcript}
#'   \item{ENSG00000196565}{normalized expression for indicated transcript}
#'   \item{ENSG00000197081}{normalized expression for indicated transcript}
#'   \item{ENSG00000197121}{normalized expression for indicated transcript}
#'   \item{ENSG00000197253}{normalized expression for indicated transcript}
#'   \item{ENSG00000197256}{normalized expression for indicated transcript}
#'   \item{ENSG00000197321}{normalized expression for indicated transcript}
#'   \item{ENSG00000197561}{normalized expression for indicated transcript}
#'   \item{ENSG00000197728}{normalized expression for indicated transcript}
#'   \item{ENSG00000197860}{normalized expression for indicated transcript}
#'   \item{ENSG00000197937}{normalized expression for indicated transcript}
#'   \item{ENSG00000197951}{normalized expression for indicated transcript}
#'   \item{ENSG00000198743}{normalized expression for indicated transcript}
#'   \item{ENSG00000198838}{normalized expression for indicated transcript}
#'   \item{ENSG00000199347}{normalized expression for indicated transcript}
#'   \item{ENSG00000200243}{normalized expression for indicated transcript}
#'   \item{ENSG00000201801}{normalized expression for indicated transcript}
#'   \item{ENSG00000203872}{normalized expression for indicated transcript}
#'   \item{ENSG00000204172}{normalized expression for indicated transcript}
#'   \item{ENSG00000205571}{normalized expression for indicated transcript}
#'   \item{ENSG00000205593}{normalized expression for indicated transcript}
#'   \item{ENSG00000208772}{normalized expression for indicated transcript}
#'   \item{ENSG00000213085}{normalized expression for indicated transcript}
#'   \item{ENSG00000213261}{normalized expression for indicated transcript}
#'   \item{ENSG00000213626}{normalized expression for indicated transcript}
#'   \item{ENSG00000213722}{normalized expression for indicated transcript}
#'   \item{ENSG00000213906}{normalized expression for indicated transcript}
#'   \item{ENSG00000213967}{normalized expression for indicated transcript}
#'   \item{ENSG00000214016}{normalized expression for indicated transcript}
#'   \item{ENSG00000214425}{normalized expression for indicated transcript}
#'   \item{ENSG00000216316}{normalized expression for indicated transcript}
#'   \item{ENSG00000220008}{normalized expression for indicated transcript}
#'   \item{ENSG00000223345}{normalized expression for indicated transcript}
#'   \item{ENSG00000224080}{normalized expression for indicated transcript}
#'   \item{ENSG00000225138}{normalized expression for indicated transcript}
#'   \item{ENSG00000226471}{normalized expression for indicated transcript}
#'   \item{ENSG00000227097}{normalized expression for indicated transcript}
#'   \item{ENSG00000227191}{normalized expression for indicated transcript}
#'   \item{ENSG00000227615}{normalized expression for indicated transcript}
#'   \item{ENSG00000228049}{normalized expression for indicated transcript}
#'   \item{ENSG00000229153}{normalized expression for indicated transcript}
#'   \item{ENSG00000230076}{normalized expression for indicated transcript}
#'   \item{ENSG00000231160}{normalized expression for indicated transcript}
#'   \item{ENSG00000231721}{normalized expression for indicated transcript}
#'   \item{ENSG00000233927}{normalized expression for indicated transcript}
#'   \item{ENSG00000233974}{normalized expression for indicated transcript}
#'   \item{ENSG00000234883}{normalized expression for indicated transcript}
#'   \item{ENSG00000236876}{normalized expression for indicated transcript}
#'   \item{ENSG00000237298}{normalized expression for indicated transcript}
#'   \item{ENSG00000237892}{normalized expression for indicated transcript}
#'   \item{ENSG00000238160}{normalized expression for indicated transcript}
#'   \item{ENSG00000239437}{normalized expression for indicated transcript}
#'   \item{ENSG00000241399}{normalized expression for indicated transcript}
#'   \item{ENSG00000241489}{normalized expression for indicated transcript}
#'   \item{ENSG00000241529}{normalized expression for indicated transcript}
#'   \item{ENSG00000244405}{normalized expression for indicated transcript}
#'   \item{ENSG00000247627}{normalized expression for indicated transcript}
#'   \item{ENSG00000249592}{normalized expression for indicated transcript}
#'   \item{ENSG00000250116}{normalized expression for indicated transcript}
#'   \item{ENSG00000250251}{normalized expression for indicated transcript}
#'   \item{ENSG00000251079}{normalized expression for indicated transcript}
#'   \item{ENSG00000253210}{normalized expression for indicated transcript}
#'   \item{ENSG00000253276}{normalized expression for indicated transcript}
#'   \item{ENSG00000254415}{normalized expression for indicated transcript}
#'   \item{ENSG00000259276}{normalized expression for indicated transcript}
#'   \item{ENSG00000260727}{normalized expression for indicated transcript}
#'   \item{ENSG00000261377}{normalized expression for indicated transcript}
#'   \item{ENSG00000264885}{normalized expression for indicated transcript}
#'   \item{ENSG00000264895}{normalized expression for indicated transcript}
#'   \item{ENSG00000267136}{normalized expression for indicated transcript}
#'   \item{ENSG00000267551}{normalized expression for indicated transcript}
#'   \item{ENSG00000267702}{normalized expression for indicated transcript}
#'   \item{ENSG00000268001}{normalized expression for indicated transcript}
#'   \item{ENSG00000268573}{normalized expression for indicated transcript}
#'   \item{ENSG00000270554}{normalized expression for indicated transcript}
#'   \item{ENSG00000270562}{normalized expression for indicated transcript}
#'   \item{ENSG00000271646}{normalized expression for indicated transcript}
#'   \item{ENSG00000273018}{normalized expression for indicated transcript}
#'   \item{ENSG00000273033}{normalized expression for indicated transcript}
#'}
#' @source <doi:10.1186/s13045-024-01553-6>
#' @srrstats {G1.0} *Statistical Software should list at least one primary reference from published academic literature.*
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G5.1} *Data sets created within, and used to test, a package should be exported (or otherwise made generally available) so that users can confirm tests and run examples.*

"amltest"

#' Pediatric acute myeloid leukemia patients with FLT3-ITD rearrangement data
#'
#' Event-free survival for 246 pediatric acute myeloid leukemia patients having
#' a FLT3-ITD rearrangement along with expression of 200 transcripts from
#' RNA-sequencing.
#'
#'
#' @format
#' A data frame with 246 rows (subjects) and 202 columns:
#' \describe{
#'   \item{efs}{event-free survival}
#'   \item{status}{censoring indicator: 1 = event; 0 = censored}
#'   \item{ENSG00000229807.12}{normalized expression for indicated transcript}
#'   \item{ENSG00000211459.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000264063.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000264462.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000227195.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000198795.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000177133.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000257883.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000189223.15}{normalized expression for indicated transcript}
#'   \item{ENSG00000210082.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000142611.17}{normalized expression for indicated transcript}
#'   \item{ENSG00000054598.9}{normalized expression for indicated transcript}
#'   \item{ENSG00000147180.17}{normalized expression for indicated transcript}
#'   \item{ENSG00000275708.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000227063.5}{normalized expression for indicated transcript}
#'   \item{ENSG00000179455.10}{normalized expression for indicated transcript}
#'   \item{ENSG00000243478.9}{normalized expression for indicated transcript}
#'   \item{ENSG00000239839.7}{normalized expression for indicated transcript}
#'   \item{ENSG00000274253.5}{normalized expression for indicated transcript}
#'   \item{ENSG00000198840.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000186407.7}{normalized expression for indicated transcript}
#'   \item{ENSG00000198763.3}{normalized expression for indicated transcript}
#'   \item{ENSG00000107562.16}{normalized expression for indicated transcript}
#'   \item{ENSG00000188536.13}{normalized expression for indicated transcript}
#'   \item{ENSG00000164821.5}{normalized expression for indicated transcript}
#'   \item{ENSG00000255833.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000107447.8}{normalized expression for indicated transcript}
#'   \item{ENSG00000237541.4}{normalized expression for indicated transcript}
#'   \item{ENSG00000274618.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000206172.8}{normalized expression for indicated transcript}
#'   \item{ENSG00000198886.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000122592.8}{normalized expression for indicated transcript}
#'   \item{ENSG00000038427.16}{normalized expression for indicated transcript}
#'   \item{ENSG00000134755.17}{normalized expression for indicated transcript}
#'   \item{ENSG00000248527.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000273018.7}{normalized expression for indicated transcript}
#'   \item{ENSG00000261377.5}{normalized expression for indicated transcript}
#'   \item{ENSG00000074964.17}{normalized expression for indicated transcript}
#'   \item{ENSG00000118113.12}{normalized expression for indicated transcript}
#'   \item{ENSG00000086548.9}{normalized expression for indicated transcript}
#'   \item{ENSG00000172037.14}{normalized expression for indicated transcript}
#'   \item{ENSG00000198744.5}{normalized expression for indicated transcript}
#'   \item{ENSG00000198899.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000186818.12}{normalized expression for indicated transcript}
#'   \item{ENSG00000154864.13}{normalized expression for indicated transcript}
#'   \item{ENSG00000227081.5}{normalized expression for indicated transcript}
#'   \item{ENSG00000101425.14}{normalized expression for indicated transcript}
#'   \item{ENSG00000158473.8}{normalized expression for indicated transcript}
#'   \item{ENSG00000102362.15}{normalized expression for indicated transcript}
#'   \item{ENSG00000212907.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000121807.6}{normalized expression for indicated transcript}
#'   \item{ENSG00000138829.12}{normalized expression for indicated transcript}
#'   \item{ENSG00000185052.12}{normalized expression for indicated transcript}
#'   \item{ENSG00000111057.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000228253.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000188153.14}{normalized expression for indicated transcript}
#'   \item{ENSG00000166681.14}{normalized expression for indicated transcript}
#'   \item{ENSG00000198888.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000276141.4}{normalized expression for indicated transcript}
#'   \item{ENSG00000230076.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000260314.3}{normalized expression for indicated transcript}
#'   \item{ENSG00000187164.20}{normalized expression for indicated transcript}
#'   \item{ENSG00000212443.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000251692.8}{normalized expression for indicated transcript}
#'   \item{ENSG00000198727.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000131401.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000225345.3}{normalized expression for indicated transcript}
#'   \item{ENSG00000197253.13}{normalized expression for indicated transcript}
#'   \item{ENSG00000163737.4}{normalized expression for indicated transcript}
#'   \item{ENSG00000204010.3}{normalized expression for indicated transcript}
#'   \item{ENSG00000174403.16}{normalized expression for indicated transcript}
#'   \item{ENSG00000249790.3}{normalized expression for indicated transcript}
#'   \item{ENSG00000239559.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000142192.21}{normalized expression for indicated transcript}
#'   \item{ENSG00000151650.8}{normalized expression for indicated transcript}
#'   \item{ENSG00000198712.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000124469.12}{normalized expression for indicated transcript}
#'   \item{ENSG00000105854.13}{normalized expression for indicated transcript}
#'   \item{ENSG00000197629.6}{normalized expression for indicated transcript}
#'   \item{ENSG00000278588.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000196565.15}{normalized expression for indicated transcript}
#'   \item{ENSG00000241743.4}{normalized expression for indicated transcript}
#'   \item{ENSG00000174059.17}{normalized expression for indicated transcript}
#'   \item{ENSG00000150760.13}{normalized expression for indicated transcript}
#'   \item{ENSG00000133742.14}{normalized expression for indicated transcript}
#'   \item{ENSG00000210196.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000204860.5}{normalized expression for indicated transcript}
#'   \item{ENSG00000131398.15}{normalized expression for indicated transcript}
#'   \item{ENSG00000106080.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000198786.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000248334.6}{normalized expression for indicated transcript}
#'   \item{ENSG00000169908.12}{normalized expression for indicated transcript}
#'   \item{ENSG00000171509.16}{normalized expression for indicated transcript}
#'   \item{ENSG00000008438.5}{normalized expression for indicated transcript}
#'   \item{ENSG00000130147.16}{normalized expression for indicated transcript}
#'   \item{ENSG00000180071.20}{normalized expression for indicated transcript}
#'   \item{ENSG00000168329.14}{normalized expression for indicated transcript}
#'   \item{ENSG00000147689.16}{normalized expression for indicated transcript}
#'   \item{ENSG00000136193.17}{normalized expression for indicated transcript}
#'   \item{ENSG00000170180.22}{normalized expression for indicated transcript}
#'   \item{ENSG00000146373.16}{normalized expression for indicated transcript}
#'   \item{ENSG00000172236.18}{normalized expression for indicated transcript}
#'   \item{ENSG00000244734.4}{normalized expression for indicated transcript}
#'   \item{ENSG00000239552.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000125618.17}{normalized expression for indicated transcript}
#'   \item{ENSG00000273983.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000250361.9}{normalized expression for indicated transcript}
#'   \item{ENSG00000277775.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000138119.17}{normalized expression for indicated transcript}
#'   \item{ENSG00000128422.17}{normalized expression for indicated transcript}
#'   \item{ENSG00000160883.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000275895.7}{normalized expression for indicated transcript}
#'   \item{ENSG00000169877.10}{normalized expression for indicated transcript}
#'   \item{ENSG00000163736.4}{normalized expression for indicated transcript}
#'   \item{ENSG00000161944.16}{normalized expression for indicated transcript}
#'   \item{ENSG00000095917.14}{normalized expression for indicated transcript}
#'   \item{ENSG00000022556.16}{normalized expression for indicated transcript}
#'   \item{ENSG00000152078.10}{normalized expression for indicated transcript}
#'   \item{ENSG00000165092.13}{normalized expression for indicated transcript}
#'   \item{ENSG00000170458.14}{normalized expression for indicated transcript}
#'   \item{ENSG00000070182.21}{normalized expression for indicated transcript}
#'   \item{ENSG00000157445.15}{normalized expression for indicated transcript}
#'   \item{ENSG00000206737.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000225217.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000198336.9}{normalized expression for indicated transcript}
#'   \item{ENSG00000177469.13}{normalized expression for indicated transcript}
#'   \item{ENSG00000198695.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000113763.12}{normalized expression for indicated transcript}
#'   \item{ENSG00000223609.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000206177.7}{normalized expression for indicated transcript}
#'   \item{ENSG00000164684.13}{normalized expression for indicated transcript}
#'   \item{ENSG00000197632.9}{normalized expression for indicated transcript}
#'   \item{ENSG00000177575.13}{normalized expression for indicated transcript}
#'   \item{ENSG00000004939.15}{normalized expression for indicated transcript}
#'   \item{ENSG00000121053.6}{normalized expression for indicated transcript}
#'   \item{ENSG00000259207.9}{normalized expression for indicated transcript}
#'   \item{ENSG00000158352.16}{normalized expression for indicated transcript}
#'   \item{ENSG00000198046.12}{normalized expression for indicated transcript}
#'   \item{ENSG00000225630.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000070808.16}{normalized expression for indicated transcript}
#'   \item{ENSG00000012223.13}{normalized expression for indicated transcript}
#'   \item{ENSG00000085265.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000158578.21}{normalized expression for indicated transcript}
#'   \item{ENSG00000112077.17}{normalized expression for indicated transcript}
#'   \item{ENSG00000272398.6}{normalized expression for indicated transcript}
#'   \item{ENSG00000116667.15}{normalized expression for indicated transcript}
#'   \item{ENSG00000101916.12}{normalized expression for indicated transcript}
#'   \item{ENSG00000184293.7}{normalized expression for indicated transcript}
#'   \item{ENSG00000244040.7}{normalized expression for indicated transcript}
#'   \item{ENSG00000229344.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000142405.22}{normalized expression for indicated transcript}
#'   \item{ENSG00000171051.9}{normalized expression for indicated transcript}
#'   \item{ENSG00000183023.18}{normalized expression for indicated transcript}
#'   \item{ENSG00000257178.5}{normalized expression for indicated transcript}
#'   \item{ENSG00000244682.7}{normalized expression for indicated transcript}
#'   \item{ENSG00000166025.18}{normalized expression for indicated transcript}
#'   \item{ENSG00000204613.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000214188.9}{normalized expression for indicated transcript}
#'   \item{ENSG00000179869.15}{normalized expression for indicated transcript}
#'   \item{ENSG00000136315.4}{normalized expression for indicated transcript}
#'   \item{ENSG00000119919.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000123384.14}{normalized expression for indicated transcript}
#'  \item{ENSG00000104918.8}{normalized expression for indicated transcript}
#'   \item{ENSG00000287431.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000227827.3}{normalized expression for indicated transcript}
#'   \item{ENSG00000164047.6}{normalized expression for indicated transcript}
#'   \item{ENSG00000137571.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000143416.21}{normalized expression for indicated transcript}
#'   \item{ENSG00000253578.1}{normalized expression for indicated transcript}
#'   \item{ENSG00000120708.17}{normalized expression for indicated transcript}
#'   \item{ENSG00000220842.6}{normalized expression for indicated transcript}
#'   \item{ENSG00000110077.14}{normalized expression for indicated transcript}
#'   \item{ENSG00000198722.14}{normalized expression for indicated transcript}
#'   \item{ENSG00000121316.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000123689.6}{normalized expression for indicated transcript}
#'   \item{ENSG00000198938.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000163430.12}{normalized expression for indicated transcript}
#'   \item{ENSG00000258227.7}{normalized expression for indicated transcript}
#'   \item{ENSG00000233101.10}{normalized expression for indicated transcript}
#'   \item{ENSG00000163220.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000100985.7}{normalized expression for indicated transcript}
#'   \item{ENSG00000166947.15}{normalized expression for indicated transcript}
#'   \item{ENSG00000275713.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000143546.10}{normalized expression for indicated transcript}
#'   \item{ENSG00000136929.13}{normalized expression for indicated transcript}
#'   \item{ENSG00000103313.13}{normalized expression for indicated transcript}
#'   \item{ENSG00000186529.16}{normalized expression for indicated transcript}
#'   \item{ENSG00000109321.11}{normalized expression for indicated transcript}
#'   \item{ENSG00000104177.18}{normalized expression for indicated transcript}
#'   \item{ENSG00000257335.8}{normalized expression for indicated transcript}
#'   \item{ENSG00000197993.9}{normalized expression for indicated transcript}
#'   \item{ENSG00000134460.18}{normalized expression for indicated transcript}
#'   \item{ENSG00000211892.4}{normalized expression for indicated transcript}
#'   \item{ENSG00000180044.5}{normalized expression for indicated transcript}
#'   \item{ENSG00000268555.2}{normalized expression for indicated transcript}
#'   \item{ENSG00000266402.3}{normalized expression for indicated transcript}
#'   \item{ENSG00000163221.9}{normalized expression for indicated transcript}
#'   \item{ENSG00000225101.6}{normalized expression for indicated transcript}
#'   \item{ENSG00000197249.14}{normalized expression for indicated transcript}
#'   \item{ENSG00000093134.15}{normalized expression for indicated transcript}
#'   }
#' @source <https://www.cancer.gov/ccg/research/genome-sequencing/target/studied-cancers/acute-myeloid-leukemia#targets-aml-project>

"pediatric_flt3"

#' Estimate cured fraction
#'
#' @description
#' Estimates the cured fraction using a Kaplan-Meier fitted object.
#'
#' @param object a \code{survfit} object.
#'
#' @return estimated proportion of cured observations
#' @export
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G1.3} *All statistical terminology should be clarified and unambiguously defined.*
#' @srrstats {G2.0} *Implement assertions on lengths of inputs, particularly through asserting that inputs expected to be single- or multi-valued are indeed so.*
#' @srrstats {G5.2b} *Explicit tests should demonstrate conditions which trigger every one of those messages, and should compare the result with expected values.*
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*
#' @seealso \code{\link[survival]{survfit}}, \code{\link[hdcuremodels]{sufficient_fu_test}},
#' \code{\link{nonzerocure_test}}
#' @import survival
#' @import stats
#' @keywords univar
#'
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' km_fit <- survfit(Surv(Time, Censor) ~ 1, data = training)
#' cure_estimate(km_fit)
cure_estimate <- function(object) {
  if (!(c("survfit") %in% class(object))) {
    stop("Error: object must be a survfit object")
  }
  summary_kme <- summary(object)
  is_strata <- grep("strata", names(summary_kme))
  if (length(is_strata) != 0) {
    min_surv <- aggregate(summary_kme$surv,
      by = list(summary_kme$strata),
      min
    )
    colnames(min_surv)[2] <- "Cured"
    min_surv$"Susceptible" <- 1 - min_surv$Cured
  } else {
    min_surv <- min(summary_kme[["surv"]])
  }
  min_surv
}

#' Dimension method for mixturecure objects
#'
#' @description Dimension method for \code{mixturecure} objects.
#' @param x An object of class `mixturecure`.
#'
#' @return \item{nobs}{ number of subjects in the dataset.}
#' @return \item{p_incidence}{ number of variables in the incidence portion of the model.}
#' @return \item{p_latency}{ number of variables in the latency portion of the model.}
#' @export
#' @keywords methods
#' @method dim mixturecure
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' fit <- curegmifs(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "weibull", thresh = 1e-4, maxit = 2000,
#'   epsilon = 0.01, verbose = FALSE
#' )
#' dim(fit)
dim.mixturecure <- function(x) {
  if (!("mixturecure" %in% class(x))) {
    stop("Error: class of object must be mixturecure")
  }
  d1name <- "nobs"
  d2name <- "p_incidence"
  d3name <- "p_latency"
  if (is.null(x$y)) {
    d1 <- d1name <- NULL
  }
  else d1 <- length(x$y)
  if (is.null(x$x_incidence)) {
    d2 <- d2name <- NULL
  } else {
    d2 <- ncol(x$x_incidence)
  }
  if (is.null(x$x_latency)) {
    d3 <- d3name <- NULL
  } else {
    d3 <- ncol(x$x_latency)
  }
  dd <- c(d1, d2, d3)
  names(dd) <- c(d1name, d2name, d3name)
  dd
}

#' Non-parametric test for a non-zero cured fraction
#'
#' @description
#' Tests the null hypothesis that the proportion of observations susceptible to
#' the event = 1 against the alternative that the proportion of observations
#' susceptible to the event is < 1. If the null hypothesis is rejected, there
#' is a significant cured fraction.
#'
#' @param object  a \code{survfit} object.
#' @param reps number of simulations on which to base the p-value
#' (default = 1000).
#' @param seed optional random seed.
#' @param plot logical. If TRUE a histogram of the estimated susceptible
#' proportions over all simulations is produced.
#' @param b optional. If specified the maximum observed time for the uniform
#' distribution for generating the censoring times. If not specified, an
#' exponential model is used for generating the censoring times (default).
#'
#' @return \item{proportion_susceptible}{estimated proportion of susceptibles}
#' @return \item{proportion_cured}{estimated proportion of those cured}
#' @return \item{p_value}{p-value testing the null hypothesis that the
#' proportion of susceptibles = 1 (cured fraction = 0) against the alternative
#' that the proportion of susceptibles < 1 (non-zero cured fraction)}
#' @return \item{time_95_percent_of_events}{estimated time at which 95% of
#' events should have occurred}
#' @export
#'
#' @references Maller, R. A. and Zhou, X. (1996) \emph{Survival Analysis with
#' Long-Term Survivors}. John Wiley & Sons.
#'
#' @srrstats {G1.0} *Statistical Software should list at least one primary reference from published academic literature.*
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G2.1} *Implement assertions on types of inputs (see the initial point on nomenclature above).*
#' @srrstats {G5.2} *Appropriate error and warning behaviour of all functions should be explicitly demonstrated through tests. In particular,*
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*
#' @srrstats {G5.9} **Noise susceptibility tests** *Packages should test for expected stochastic behaviour, such as through the following conditions:*
#' @srrstats {G5.9a} *Adding trivial noise (for example, at the scale of `.Machine$double.eps`) to data does not meaningfully change results*
#' @srrstats {G5.9b} *Running under different random seeds or initial conditions does not meaningfully change results*
#' @srrstats {RE4.5} *Numbers of observations submitted to model (via `nobs()`)*
#' @seealso \code{\link[survival]{survfit}}, \code{\link[hdcuremodels]{cure_estimate}},
#' \code{\link[hdcuremodels]{sufficient_fu_test}}
#'
#' @import survival
#' @import stats
#' @import flexsurv
#' @import flexsurvcure
#' @keywords htest
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' km_fit <- survfit(Surv(Time, Censor) ~ 1, data = training)
#' nonzerocure_test(km_fit)
nonzerocure_test <- function(object, reps = 1000, seed = NULL, plot = FALSE,
                             b = NULL) {
  if (!(c("survfit") %in% class(object))) {
    stop("Error: object must be a survfit object")
  }
  # Surv(Time, Censor)
  Y <- Surv(object$time, object$n.event)
  n <- length(Y)
  if (is.null(object$strata)) {
    exp_model <- survival::survreg(Y ~ 1,
      data = Y,
      dist = "exponential"
    )
    mcm <- flexsurvcure::flexsurvcure(Y ~ 1,
      data = Y,
      dist = "exp", mixture = TRUE
    )
    susceptible <- 1 - cure_estimate(object)
    mean <- unname(exp(exp_model$coefficients["(Intercept)"]))
    rate <- unname(mcm$coefficients["rate"])
    cured <- cure_estimate(object)
    if (cured > 0 && cured < 1) {
      time_95 <- 1 / exp(rate) - log((0.05 * cured) / (0.95 * (1 - cured)))
    } else {
      time_95 <- 1 / exp(rate)
    }
    p <- sim_cure(n,
      mu = mean, censor_mu = time_95, reps = reps, seed = seed,
      b = b
    )
    p_value <- ifelse(sum(p < susceptible) / reps == 0, paste("<", 1 / reps),
      sum(p < susceptible) / reps
    )
    if (plot) {
      hist(p,
        xlab = "Proportion Susceptible",
        main = "Non-parametric Reference Distribution"
      )
    }
  } else {
    strata <- gsub(".*=", "", names(object$strata))
    mean <- rep(NA, length(strata))
    rate <- rep(NA, length(strata))
    for (i in seq_along(strata)) {
      group <- extract_rhs_values(object)
      frame <- data.frame(surv = Y, group = group)
      exp_model <- survival::survreg(surv ~ 1,
        dist = "exponential", data = frame,
        subset = (group == strata[i]))
      mean[i] <- exp(exp_model$coefficients["(Intercept)"])
      mcm <- flexsurvcure::flexsurvcure(surv ~ 1,
        data = frame, subset = group == strata[i],
        dist = "exp", mixture = TRUE
      )
      rate[i] <- mcm$coefficients["rate"]
      # }
    }
    mean <- unname(mean)
    rate <- unname(rate)
    cured <- cure_estimate(object)[, "Cured"]
    susceptible <- cure_estimate(object)
    n <- unname(object$strata)
    time_95 <- numeric()
    for (i in seq_along(strata)) {
      if (cured[i] > 0 && cured[i] < 1) {
        time_95[i] <- 1 / exp(rate[i]) - log((0.05 * cured[i]) /
          (0.95 * (1 - cured[i])))
      } else {
        time_95[i] <- 1 / exp(rate[i])
      }
    }
    p <- list()
    for (i in seq_along(object$strata)) {
      p[[i]] <- sim_cure(n[i],
        mu = mean[i], censor_mu = time_95[i],
        reps = reps, seed = seed, b = b
      )
    }
    p_value <- numeric()
    for (i in seq_along(object$strata)) {
      p_value[i] <- ifelse(sum(p[[i]] <
        susceptible[i, "Susceptible"]) / reps == 0,
      paste("<", 1 / reps),
      sum(p[[i]] < susceptible[i, "Susceptible"]) / reps
      )
    }
    names(p_value) <- names(object$strata)
    if (plot) {
      for (i in seq_along(p)) {
        hist(p[[i]],
          xlab = "Proportion Susceptible",
          main = paste(names(object$strata[i]),
            "Non-parametric Reference Distribution",
            sep = " "
          )
        )
      }
    }
    susceptible <- susceptible[, "Susceptible"]
    names(time_95) <- names(susceptible) <- names(cured) <- names(object$strata)
  }
  list(
    proportion_susceptible = susceptible, proportion_cured = cured,
    p_value = p_value, time_95_percent_of_events = time_95
  )
}

#' Plot fitted mixture cure model
#'
#' @description
#' This function plots either the coefficient path, the AIC, the cAIC, the BIC,
#' or the log-likelihood for a fitted \code{curegmifs} or \code{cureem} object.
#' This function produces a lollipop plot of the coefficient estimates for a
#' fitted \code{cv_curegmifs} or \code{cv_cureem} object.
#'
#' @param x a \code{mixturecure} object resulting from \code{curegmifs} or
#' \code{cureem}, \code{cv_curegmifs} or \code{cv_cureem}.
#' @param type a case-sensitive parameter indicating what to plot on the y-axis.
#' The complete list of options are:
#' \itemize{
#'     \item \code{"trace"} plots the coefficient path for the fitted object
#'     (default).
#'     \item \code{"AIC"} plots the AIC against step of model fit.
#'     \item \code{"mAIC"} plots the modified AIC against step of model fit.
#'     \item \code{"cAIC"} plots the corrected AIC against step of model fit.
#'     \item \code{"BIC"}, plots the BIC against step of model fit.
#'     \item \code{"mBIC"} plots the modified BIC against step of model fit.
#'     \item \code{"EBIC"} plots the extended BIC against step of model fit.
#'     \item \code{"logLik"} plots the log-likelihood against step of model fit.
#'   }
#' This option has no effect for objects fit using
#' \code{cv_curegmifs} or \code{cv_cureem}.
#' @param xlab  a default x-axis label will be used which can be changed by
#' specifying a user-defined x-axis label.
#' @param ylab a default y-axis label will be used which can be changed by
#' specifying a user-defined y-axis label.
#' @param label logical. If TRUE the variable names will appear in a legend.
#' Applicable only when \code{type = "trace"}. Be reminded that this works well
#' only for small to moderate numbers of variables. For many predictors, the
#' plot will be cluttered. The variables may be more easily identified using
#' the \code{coef} function indicating the step of interest.
#' @param main a default main title will be used which can be changed by
#' specifying a user-defined main title. This option is not used for
#' \code{cv_curegmifs} or \code{cv_cureem} fitted objects.
#' @param ... other arguments.
#'
#' @return this function has no returned value but is called for its side
#' effects
#'
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {RE6.0} *Model objects returned by Regression Software (see* **RE4***) should have default `plot` methods, either through explicit implementation, extension of methods for existing model objects, or through ensuring default methods work appropriately.*
#' @srrstats {RE6.1} *Where the default `plot` method is **NOT** a generic `plot` method dispatched on the class of return objects (that is, through an S3-type `plot.<myclass>` function or equivalent), that method dispatch (or equivalent) should nevertheless exist in order to explicitly direct users to the appropriate function.*
#' @seealso \code{\link[hdcuremodels]{curegmifs}}, \code{\link[hdcuremodels]{cureem}},
#' \code{\link[hdcuremodels]{coef.mixturecure}}, \code{\link[hdcuremodels]{summary.mixturecure}},
#' \code{\link[hdcuremodels]{predict.mixturecure}}
#' @keywords methods
#' @method plot mixturecure
#' @import ggplot2
#' @import graphics
#' @importFrom ggpubr ggarrange
#' @export
#'
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' fit <- curegmifs(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "weibull", thresh = 1e-4, maxit = 2000,
#'   epsilon = 0.01, verbose = FALSE
#' )
#' plot(fit)
plot.mixturecure <-
  function(x, type = c(
             "trace", "AIC", "BIC", "logLik", "cAIC", "mAIC",
             "mBIC", "EBIC"), xlab = NULL, ylab = NULL,
              label = FALSE, main = NULL, ...) {
    type <- match.arg(type)
    if (!x$cv) {
      if (is.null(xlab)) {
        xlab <- "Step"
      }
      if (type == "trace") {
        if (is.null(ylab)) {
          ylab <- expression(hat(beta))
        }
        if (!is.null(x$x_latency) && !is.null(x$x_incidence)) {
          colnames(x$b_path) <- paste0("I_", colnames(x$b_path))
          colnames(x$beta_path) <- paste0("L_", colnames(x$beta_path))
          coef <- cbind(b = x$b_path, beta = x$beta_path)
        } else if (is.null(x$x_latency) && !is.null(x$x_incidence)) {
          colnames(x$b_path) <- paste0("I_", colnames(x$b_path))
          coef <- x$b_path
        } else if (!is.null(x$x_latency) && is.null(x$x_incidence)) {
          colnames(x$beta_path) <- paste0("L_", colnames(x$beta_path))
          coef <- x$beta_path
        }
        graphics::matplot(coef, ylab = ylab, xlab = xlab, type = "l",
                    lty = rep(c(1,2), c(dim(x$b_pat)[2], dim(x$beta_path)[2])),
                    col = rep(1:(dim(x$b_path)[2]+dim(x$beta_path)[2])))
        if (label == TRUE)
          legend("topright", legend = colnames(coef),
                 lty = rep(c(1,2), c(dim(x$b_pat)[2], dim(x$beta_path)[2])),
                 cex = 0.6, col = rep(1:(dim(x$b_path)[2]+dim(x$beta_path)[2])))
      } else {
        select <- select_model(x, type)
        if (type == "AIC") {
        if (is.null(ylab)) {
          ylab <- "AIC"
        }
        plot(select$AIC, xlab = xlab, ylab = ylab)
      } else if (type == "cAIC") {
        if (is.null(ylab)) {
          ylab <- "cAIC"
        }
        plot(select$cAIC, xlab = xlab, ylab = ylab)
      } else if (type == "mAIC") {
        if (is.null(ylab)) {
          ylab <- "mAIC"
        }
        plot(select$mAIC, xlab = xlab, ylab = ylab)
      } else if (type == "BIC") {
        if (is.null(ylab)) {
          ylab <- "BIC"
        }
        plot(select$BIC, xlab = xlab, ylab = ylab)
      } else if (type == "mBIC") {
        if (is.null(ylab)) {
          ylab <- "mBIC"
        }
        plot(select$mBIC, xlab = xlab, ylab = ylab)
      } else if (type == "EBIC") {
        if (is.null(ylab)) {
          ylab <- "EBIC"
        }
        plot(select$EBIC, xlab = xlab, ylab = ylab)
      } else if (type == "logLik") {
        if (is.null(ylab)) {
          ylab <- "logLikelihood"
        }
        plot(select$logLik, xlab = xlab, ylab = ylab)
      }
      if (is.null(main)) {
        graphics::title(paste(type, "from MCM ", x$method, " fit",
          sep = " "
        ))
      } else {
        graphics::title(main)
      }
    }
    } else {
      if (is.null(xlab)) xlab <- "Predictors"
      if (is.null(ylab)) ylab <- "Coefficient Estimates"
      names(x$b) <- colnames(x$x_incidence)
      names(x$beta) <- colnames(x$x_latency)
      if (!is.null(x$x_incidence)) {
        if (sum(x$b > 0) > 0) {
          b <- x$b[x$b != 0]
        } else {
          b <- x$b
        }
      }
      if (!is.null(x$x_latency)) {
        if (sum(x$beta > 0) > 0) {
          beta <- x$beta[x$beta != 0]
        } else {
          beta <- x$beta
        }
      }
      y <- NULL
      if (!is.null(x$x_incidence)) {
        data_b <- data.frame(
          x = seq_along(b), y = b, vars = names(b),
          model_component = "Incidence"
        )
      }
      if (!is.null(x$x_latency)) {
        data_beta <- data.frame(
          x = seq_along(beta), y = beta,
          vars = names(beta), model_component = "Latency"
        )
      }
      if (!is.null(x$x_incidence) && !is.null(x$x_latency)) {
        bplot <- ggplot(data_b, aes(x = vars, y = y)) +
          geom_point() +
          geom_segment(aes(x = vars, xend = vars, y = 0, yend = y)) +
          xlab(xlab) +
          ylab(ylab) +
          ggtitle("Incidence") +
          theme(axis.text.x = element_text(angle = 45))
        betaplot <- ggplot(data_beta, aes(x = vars, y = y)) +
          geom_point() +
          geom_segment(aes(x = vars, xend = vars, y = 0, yend = y)) +
          xlab(xlab) +
          ylab(ylab) +
          ggtitle("Latency") +
          theme(axis.text.x = element_text(angle = 45))
        ggpubr::ggarrange(bplot, betaplot, nrow = 2)
      } else if (is.null(x$x_incidence) && !is.null(x$x_latency)) {
        ggplot(data_beta, aes(x = vars, y = y)) +
          geom_point() +
          geom_segment(aes(x = vars, xend = vars, y = 0, yend = y)) +
          xlab(xlab) +
          ylab(ylab) +
          ggtitle("Latency") +
          theme(axis.text.x = element_text(angle = 45))
      } else if (!is.null(x$x_incidence) && is.null(x$x_latency)) {
        ggplot(data_b, aes(x = vars, y = y)) +
          geom_point() +
          geom_segment(aes(x = vars, xend = vars, y = 0, yend = y)) +
          xlab(xlab) +
          ylab(ylab) +
          ggtitle("Incidence") +
          theme(axis.text.x = element_text(angle = 45))
      }
    }
  }

#' Fit penalized mixture cure model using the E-M algorithm
#'
#' @description
#' Fits penalized parametric and semi-parametric mixture cure models (MCM)
#' using the E-M algorithm with user-specified penalty parameters. The lasso
#' (L1), MCP, and SCAD penalty are supported for the Cox MCM while only lasso is
#' currently supported for parametric MCMs.
#'
#' @param formula an object of class "\code{formula}" (or one that can be
#' coerced to that class): a symbolic description of the model to be fitted.
#' The response must be a survival object as returned by the \code{Surv}
#' function while the variables on the right side of the formula are the
#' covariates that are included in the incidence portion of the model.
#' @param data a data.frame in which to interpret the variables named in the
#' \code{formula} or in the \code{subset} argument. Rows with missing data are
#' omitted (only \code{na.action = na.omit} is operational) therefore users may
#' want to impute missing data prior to calling this function.
#' @param subset an optional expression indicating which subset of observations
#' to be used in the fitting process, either a numeric or factor variable
#' should be used in subset, not a character variable. All observations are
#' included by default.
#' @param x_latency specifies the variables to be included in the latency
#' portion of the model and can be either a matrix of predictors, a model
#' formula with the right hand side specifying the latency variables, or the
#' same data.frame passed to the \code{data} parameter. Note that when using
#' the model formula syntax for \code{x_latency} it cannot handle
#' \code{x_latency = ~ .}.
#' @param model type of regression model to use for the latency portion of
#' mixture cure model. Can be "cox", "weibull", or "exponential" (default is
#' "cox").
#' @param penalty type of penalty function. Can be "lasso", "MCP", or "SCAD"
#' (default is "lasso").
#' @param penalty_factor_inc vector of binary indicators representing the
#' penalty to apply to each incidence coefficient: 0 implies no shrinkage
#' and 1 implies shrinkage. If not supplied, 1 is applied to all incidence
#' variables.
#' @param penalty_factor_lat vector of binary indicators representing the
#' penalty to apply to each latency coefficient: 0 implies no shrinkage and 1
#' implies shrinkage. If not supplied, 1 is applied to all latency variables.
#' @param thresh small numeric value. The iterative process stops when the
#' differences between successive expected penalized complete-data
#' log-likelihoods for both incidence and latency components are less than this
#' specified level of tolerance (default is 10^-3).
#' @param scale logical, if TRUE the predictors are centered and scaled.
#' @param maxit integer specifying the maximum number of passes over the data
#' for each lambda. If not specified, 100 is applied when
#' \code{penalty = "lasso"} and 1000 is applied when \code{penalty = "MCP"} or
#' \code{penalty = "SCAD"}.
#' @param inits an optional list specifiying the initial values. This includes:
#' \itemize{
#' \item \code{itct} the incidence intercept.
#' \item \code{b_u} a numeric vector for the unpenalized
#' incidence coefficients for the incidence portion of the model.
#' \item \code{beta_u} a numeric vector for unpenalized
#' latency coefficients in the incidence portion of the model.
#' \item \code{lambda} a numeric value for the rate parameter when fitting
#' either a Weibull or exponential MCM using \code{model = "weibull"} or
#' \code{model = "exponential"}.
#' \item \code{alpha} a numeric value for the shape parameter when fitting a
#' Weibull MCM using \code{model = "weibull"}.
#' \item \code{survprob} a numeric vector for the
#' latency survival probabilities \eqn{S_u(t_i|w_i)} for i=1,...,N when fitting
#' a Cox MCM \code{model = "cox"}.
#' }
#' Penalized coefficients are initialized to zero. If \code{inits} is not specified or improperly specified, initialization is
#' automatically provided by the function.
#' @param lambda_inc numeric value for the penalization parameter \eqn{\lambda}
#' for variables in the incidence portion of the model.
#' @param lambda_lat numeric value for the penalization parameter \eqn{\lambda}
#' for variables in the latency portion of the model.
#' @param gamma_inc numeric value for the penalization parameter \eqn{\gamma}
#' for variables in the incidence portion of the model when
#' \code{penalty = "MCP"} or \code{penalty = "SCAD"} (default is 3).
#' @param gamma_lat numeric value for the penalization parameter \eqn{\gamma}
#' for variables in the latency portion of the model when \code{penalty = "MCP"}
#' or \code{penalty = "SCAD"} (default is 3).
#' @param na.action this function requires complete data so \code{"na.omit"} is
#' invoked. Users can impute missing data as an alternative prior to model fitting.
#' @param ... additional arguments.
#'
#' @return \item{b_path}{Matrix representing the solution path of the
#' coefficients in the incidence portion of the model. Row is step and column
#' is variable.}
#' @return \item{beta_path}{Matrix representing the solution path of the
#' coefficients in the latency portion of the model. Row is step and column
#' is variable.}
#' @return \item{b0_path }{Vector representing the solution path of the
#' intercept in the incidence portion of the model.}
#' @return \item{logLik_inc }{Vector representing the expected penalized
#' complete-data log-likelihood for the incidence portion of the model for
#' each step in the solution path.}
#' @return \item{logLik_lat }{Vector representing the expected penalized
#' complete-data log-likelihood for the latency portion of the model for each
#' step in the solution path.}
#' @return \item{x_incidence}{Matrix representing the design matrix of the
#' incidence predictors.}
#' @return \item{x_latency}{Matrix representing the design matrix of the
#' latency predictors.}
#' @return \item{y}{Vector representing the survival object response as
#' returned by the \code{Surv} function }
#' @return \item{model}{Character string indicating the type of regression
#' model used for the latency portion of mixture cure model ("weibull" or
#' "exponential").}
#' @return \item{scale}{Logical value indicating whether the predictors were
#' centered and scaled.}
#' @return \item{method}{Character string indicating the EM algorithm was used
#' in fitting the mixture cure model.}
#' @return \item{rate_path}{Vector representing the solution path of the rate
#' parameter for the Weibull or exponential density in the latency portion of
#' the model.}
#' @return \item{alpha_path}{Vector representing the solution path of the shape
#' parameter for the Weibull density in the latency portion of the model.}
#' @return \item{call}{the matched call.}
#' @export
#'
#' @import stats
#' @import survival
#' @import glmnet
#' @import foreach
#' @import plyr
#' @importFrom methods is
#'
#' @references Archer, K. J., Fu, H., Mrozek, K., Nicolet, D., Mims, A. S.,
#' Uy, G. L., Stock, W., Byrd, J. C., Hiddemann, W., Braess, J.,
#' Spiekermann, K., Metzeler, K. H., Herold, T., Eisfeld, A.-K. (2024)
#' Identifying long-term survivors and those at higher or lower risk of relapse
#' among patients with cytogenetically normal acute myeloid leukemia using a
#' high-dimensional mixture cure model. \emph{Journal of Hematology & Oncology},
#' \bold{17}:28.
#'
#' @srrstats {G1.0} *Statistical Software should list at least one primary reference from published academic literature.*
#' @srrstats {G1.1} *Statistical Software should document whether the algorithm(s) it implements are: An improvement on other implementations of similar algorithms in **R**.
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G1.3} *All statistical terminology should be clarified and unambiguously defined.*
#' @srrstats {G1.5} *Software should include all code necessary to reproduce results which form the basis of performance claims made in associated publications.*
#' @srrstats {G2.0a} *Provide explicit secondary documentation of any expectations on lengths of inputs*
#' @srrstats {G2.1} *Implement assertions on types of inputs (see the initial point on nomenclature above).*
#' @srrstats {G2.3a} *Use `match.arg()` or equivalent where applicable to only permit expected values.*
#' @srrstats {G2.1a} *Provide explicit secondary documentation of expectations on data types of all vector inputs.*
#' @srrstats {G2.2} *Appropriately prohibit or restrict submission of multivariate input to parameters expected to be univariate.*
#' @srrstats {G2.3} *For univariate character input:*
#' @srrstats {G2.3b} *Either: use `tolower()` or equivalent to ensure input of character parameters is not case dependent; or explicitly document that parameters are strictly case-sensitive.*
#' @srrstats {G2.4} *Provide appropriate mechanisms to convert between different data types, potentially including:*
#' @srrstats {G2.4e} *explicit conversion from factor via `as...()` functions*
#' @srrstats {G2.10} *Software should ensure that extraction or filtering of single columns from tabular inputs should not presume any particular default behaviour, and should ensure all column-extraction operations behave consistently regardless of the class of tabular data used as input.*
#' @srrstats {G2.13} *Statistical Software should implement appropriate checks for missing data as part of initial pre-processing prior to passing data to analytic algorithms.*
#' @srrstats {G5.2} *Appropriate error and warning behaviour of all functions should be explicitly demonstrated through tests. In particular,*
#' @srrstats {G5.2a} *Every message produced within R code by `stop()`, `warning()`, `message()`, or equivalent should be unique*
#' @srrstats {G5.2b} *Explicit tests should demonstrate conditions which trigger every one of those messages, and should compare the result with expected values.*
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {G5.6} **Parameter recovery tests** *to test that the implementation produce expected results given data with known properties. For instance, a linear regression algorithm should return expected coefficient values for a simulated data set generated from a linear model.*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*
#' @srrstats {G5.9} **Noise susceptibility tests** *Packages should test for expected stochastic behaviour, such as through the following conditions:*
#' @srrstats {G5.9a} *Adding trivial noise (for example, at the scale of `.Machine$double.eps`) to data does not meaningfully change results*
#' @srrstats {RE1.0} *Regression Software should enable models to be specified via a formula interface, unless reasons for not doing so are explicitly documented.*
#' @srrstats {RE1.1} *Regression Software should document how formula interfaces are converted to matrix representations of input data.*
#' @srrstats {RE1.2} *Regression Software should document expected format (types or classes) for inputting predictor variables, including descriptions of types or classes which are not accepted.*
#' @srrstats {RE1.3} *Regression Software which passes or otherwise transforms aspects of input data onto output structures should ensure that those output structures retain all relevant aspects of input data, notably including row and column names, and potentially information from other `attributes()`.*
#' @srrstats {RE1.3a} *Where otherwise relevant information is not transferred, this should be explicitly documented.*
#' @srrstats {RE1.4} *Regression Software should document any assumptions made with regard to input data; for example distributional assumptions, or assumptions that predictor data have mean values of zero. Implications of violations of these assumptions should be both documented and tested.*
#' @srrstats {RE2.3} *Where applicable, Regression Software should enable data to be centred (for example, through converting to zero-mean equivalent values; or to z-scores) or offset (for example, to zero-intercept equivalent values) via additional parameters, with the effects of any such parameters clearly documented and tested.*
#' @srrstats {RE4.0} *Regression Software should return some form of "model" object, generally through using or modifying existing class structures for model objects (such as `lm`, `glm`, or model objects from other packages), or creating a new class of model objects.*
#' @srrstats {RE4.4} *The specification of the model, generally as a formula (via `formula()`)*
#' @srrstats {RE4.7} *Where appropriate, convergence statistics*
#' @srrstats {RE4.8} *Response variables, and associated "metadata" where applicable.*
#' @seealso \code{\link[hdcuremodels]{cv_cureem}}
#'
#' @keywords models
#' @keywords regression
#'
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 80, j = 100, n_true = 10, a = 1.8)
#' training <- temp$training
#' fit <- cureem(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "cox", penalty = "lasso", lambda_inc = 0.1,
#'   lambda_lat = 0.1, gamma_inc = 6, gamma_lat = 10
#' )
cureem <- function(formula, data, subset, x_latency = NULL,
                   model = c("cox", "weibull", "exponential"),
                   penalty = c("lasso", "MCP", "SCAD"),
                   penalty_factor_inc = NULL, penalty_factor_lat = NULL,
                   thresh = 1e-03, scale = TRUE, maxit = NULL, inits = NULL,
                   lambda_inc = 0.1, lambda_lat = 0.1, gamma_inc = 3,
                   gamma_lat = 3, na.action = na.omit, ...) {
  mf <- match.call(expand.dots = FALSE)
  cl <- match.call()
  m <- match(c("formula", "data", "subset", "na.action"), names(mf), 0L) #KJA 06-23
  if (m[1] == 0) stop("Error: A \"formula\" argument is required")
  mf <- mf[c(1L, m)]
  mf[[1L]] <- as.name("model.frame")
  if (missing(data)) {
    mf[["data"]] <- environment(formula)
  }
  if (missing(data)) {
    data <- environment(formula)
  }
  mf <- eval(mf, parent.frame())
  mt <- attr(mf, "terms")
  omitted <- attr(mf, "na.action")
  model <- tolower(model)
  model <- match.arg(model)
  penalty <- match.arg(penalty)
  y <- model.response(mf)
  event <- y[, 2]
  time <- y[, 1]
  x <- model.matrix(mt, mf)
  if (!is.null(x_latency)) {
    if (missing(subset)) {
      r <- rep(TRUE, dim(data)[1])
      r[omitted] <- FALSE
    } else {
      e <- substitute(subset)
      r <- eval(e, data)
      if (!is.logical(r)) {
        stop("Error: 'subset' must evaluate to logical")
      }
      r <- r & !is.na(r)
      r[omitted] <- FALSE
    }
    if ("character" %in% is(x_latency) || "numeric" %in% is(x_latency)) {
      nl <- as.list(seq_len(ncol(data)))
      names(nl) <- names(data)
      vars <- eval(substitute(x_latency), nl, parent.frame())
      x_latency <- data[r, vars, drop = FALSE]
      x_latency <- as.matrix(x_latency)
    } else if ("matrix" %in% is(x_latency) || "data.frame" %in% is(x_latency)) {
      text <- parse(text = cl)[[2]]
      survnames <- strsplit(as.character(text), ",")
      time_name <- substr(survnames[[2]][1], 6, nchar(survnames[[2]][1]))
      censor_name <- trimws(strsplit(survnames[[2]][2], ")")[[1]][1])
      x_latency <- x_latency[r, !(colnames(x_latency) %in% c(
        time_name,
        censor_name
      )),
      drop = FALSE
      ]
      x_latency <- as.matrix(x_latency)
    } else if ("formula" %in% is(x_latency)) {
      x_latency <- model.matrix(update.formula(x_latency, new = ~ . - 1), data)
    }
  }
  x_inc <- x
  is_intercept <- grep("Intercept", dimnames(x_inc)[[2]])
  if (length(is_intercept) == 1) {
    x_inc <- x_inc[, -is_intercept, drop = FALSE]
  }
  x_lat <- x_latency
  if (is.null(maxit)) maxit <- ifelse(penalty == "lasso", 100, 1000)
  if (nrow(x_inc) != nrow(x_lat) || nrow(x_lat) != length(time) || length(time) != length(event)) {
    stop("Error: Input dimension mismatch")
  }
  if (is.data.frame(x_inc) || is.data.frame(x_lat)) { #
    x_inc <- as.matrix(x_inc)
    x_lat <- as.matrix(x_lat)
  }
  if (is.null(penalty_factor_inc)) {
    penalty_factor_inc <- rep(1, ncol(x_inc))
  }
  if (is.null(penalty_factor_lat)) {
    penalty_factor_lat <- rep(1, ncol(x_lat))
  }
  if (any(!c(penalty_factor_inc, penalty_factor_lat) %in% c(0, 1))) {
    stop("Error: Penalty factors specified in penalty_factor_inc and
         penalty_factor_lat can only include 0 or 1")
  }
  if (any(c(lambda_inc, lambda_lat, gamma_inc, gamma_lat) <= 0)) {
    stop("Error: Penalty pamameters lambda and gamma should be positive")
  }
  if (!is.null(inits)) {
    inits <- inits_check(model,
                         N = length(time), penalty_factor_inc,
                         penalty_factor_lat, inits
    )
  }
  if (model != "cox" && penalty != "lasso") {
    warning("Warning: MCP/SCAD penalized parametric models are not currently supported.
            An L1 penalized model was fitted instead.")
    penalty <- "lasso"
  }
  x_u <- self_scale(x_inc[, penalty_factor_inc == 0, drop = FALSE], scale)
  x_p <- self_scale(x_inc[, penalty_factor_inc == 1, drop = FALSE], scale)
  w_u <- self_scale(x_lat[, penalty_factor_lat == 0, drop = FALSE], scale)
  w_p <- self_scale(x_lat[, penalty_factor_lat == 1, drop = FALSE], scale)
  if (model == "cox" && penalty == "lasso") {
    fit <- cox_l1(
      x_u, x_p, w_u, w_p, time, event, lambda_inc, lambda_lat,
      inits, maxit, thresh
    )
  } else if (model == "cox" && penalty %in% c("MCP", "SCAD")) {
    fit <- cox_mcp_scad(
      x_u, x_p, w_u, w_p, time, event, penalty, lambda_inc,
      lambda_lat, gamma_inc, gamma_lat, inits, maxit, thresh
    )
  } else if (model == "weibull") {
    fit <- weib_EM(
      x_u, x_p, w_u, w_p, time, event, lambda_inc, lambda_lat,
      inits, maxit, thresh
    )
  } else if (model == "exponential") {
    fit <- exp_EM(
      x_u, x_p, w_u, w_p, time, event, lambda_inc, lambda_lat,
      inits, maxit, thresh
    )
  }
  b_path <- matrix(NA, nrow = dim(fit$b_p_path)[1], ncol = dim(x_inc)[2])
  beta_path <- matrix(NA, nrow = dim(fit$beta_p_path)[1], ncol = dim(x_lat)[2])
  b_path[, penalty_factor_inc == 0] <- fit$b_u_path
  b_path[, penalty_factor_inc == 1] <- fit$b_p_path
  beta_path[, penalty_factor_lat == 0] <- fit$beta_u_path
  beta_path[, penalty_factor_lat == 1] <- fit$beta_p_path
  colnames(b_path) <- colnames(x_inc)
  colnames(beta_path) <- colnames(x_lat)
  b0_path <- fit$itct_path
  logLik_inc <- fit$lik_inc
  logLik_lat <- fit$lik_lat
  logLik <- logLik_inc + logLik_lat
  output <- list(
    b_path = b_path, beta_path = beta_path,
    b0_path = b0_path, logLik = logLik,
    x_incidence = x_inc,
    x_latency = x_lat, y = y, model = model, scale = scale,
    method = "EM", call = cl
  )
  if (model %in% c("exponential", "weibull")) {
    output$rate_path <- fit$lambda_path
  }
  if (model == "weibull") {
    output$alpha_path <- fit$alpha_path
  }
  output$cv <- FALSE
  class(output) <- "mixturecure"
  return(output)
}

#' Number of observations in mixturecure object
#'
#' @description Number of observations in fitted `mixturecure` object.
#' @param object An object of class `mixturecure`.
#' @param ... other arguments.
#'
#' @return number of subjects in the dataset.
#' @export
#' @keywords methods
#' @method nobs mixturecure
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' fit <- curegmifs(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "weibull", thresh = 1e-4, maxit = 2000,
#'   epsilon = 0.01, verbose = FALSE
#' )
#' nobs(fit)
nobs.mixturecure <- function(object, ...) {
  if (!("mixturecure" %in% class(object))) {
    stop("Error: class of object must be mixturecure")
  }
  if (is.null(object$y)) {
    d1 <- NULL
  }
  else d1 <- length(object$y)
  d1
}

#' Log-likelihood for fitted mixture cure model
#'
#' @description
#' This function returns the log-likelihood for a user-specified model criterion
#' or step for a \code{curegmifs}, \code{cureem},
#' \code{cv_curegmifs} or \code{cv_cureem} fitted object.
#'
#' @param object a \code{mixturecure} object resulting from \code{curegmifs},
#' \code{cureem}, \code{cv_curegmifs}, \code{cv_cureem}.
#' @param model_select either a case-sensitive parameter for models fit using
#' \code{curegmifs} or \code{cureem} or any numeric step along the solution path
#' can be selected. The default is \code{model_select = "AIC"} which calculates
#' the predicted values using the coefficients from the model achieving the
#' minimum AIC. The complete list of options are:
#' \itemize{
#'     \item \code{"AIC"} for the minimum AIC (default).
#'     \item \code{"mAIC"} for the minimum modified AIC.
#'     \item \code{"cAIC"} for the minimum corrected AIC.
#'     \item \code{"BIC"}, for the minimum BIC.
#'     \item \code{"mBIC"} for the minimum modified BIC.
#'     \item \code{"EBIC"} for the minimum extended BIC.
#'     \item \code{"logLik"} for the step that maximizes the
#' log-likelihood.
#'     \item \code{n} where n is any numeric value from the
#'     solution path.
#'   }
#' This option has no effect for objects fit using \code{cv_curegmifs} or
#' \code{cv_cureem}.
#' @param ... other arguments.
#'
#' @return log-likelihood of the fitted mixture cure model using the specified
#' criteria.
#'
#' @keywords methods
#' @method logLik mixturecure
#'
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @export
#'
#' @examples
#' library(survival)
#' withr::local_seed(1234)
#' temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
#' training <- temp$training
#' fit <- curegmifs(Surv(Time, Censor) ~ .,
#'   data = training, x_latency = training,
#'   model = "weibull", thresh = 1e-4, maxit = 2000,
#'   epsilon = 0.01, verbose = FALSE
#' )
#' logLik(fit, model_select = "AIC")
logLik.mixturecure <- function(object, model_select = "AIC", ...) {
  if (object$cv) {
    if (object$method == "GMIFS") {
      logLik <- object$logLik
    } else {
      logLik <- object$logLik.inc + object$logLik.lat
    }
  } else {
    if (is.numeric(model_select) && model_select > length(object$logLik))
      stop("Error: model_select step must be less than or equal to ", length(object$logLik))
    if (!is.numeric(model_select))
      model_select <- select_model(object, model_select)$select
    logLik <- object$logLik[model_select]
  }
  structure(logLik, df = npar_mixturecure(object, model_select),
            class = "logLik")
}

#' srr_stats
#'
#' All of the following standards initially have `@srrstatsTODO` tags.
#' These may be moved at any time to any other locations in your code.
#' Once addressed, please modify the tag from `@srrstatsTODO` to `@srrstats`,
#' or `@srrstatsNA`, ensuring that references to every one of the following
#' standards remain somewhere within your code.
#' (These comments may be deleted at any time.)
#'
#' @srrstatsVerbose TRUE
#'
#' @srrstats {G1.3} *All statistical terminology should be clarified and unambiguously defined.*
#' @srrstats {G1.4} *Software should use [`roxygen2`](https://roxygen2.r-lib.org/) to document all functions.*
#' @srrstats {G1.4a} *All internal (non-exported) functions should also be documented in standard [`roxygen2`](https://roxygen2.r-lib.org/) format, along with a final `@noRd` tag to suppress automatic generation of `.Rd` files.*
#' @srrstats {G1.5} *Software should include all code necessary to reproduce results which form the basis of performance claims made in associated publications.*
#' @srrstats {G2.0} *Implement assertions on lengths of inputs, particularly through asserting that inputs expected to be single- or multi-valued are indeed so.*
#' @srrstats {G2.0a} *Provide explicit secondary documentation of any expectations on lengths of inputs*
#' @srrstats {G2.1} *Implement assertions on types of inputs (see the initial point on nomenclature above).*
#' @srrstats {G2.1a} *Provide explicit secondary documentation of expectations on data types of all vector inputs.*
#' @srrstats {G2.2} *Appropriately prohibit or restrict submission of multivariate input to parameters expected to be univariate.*
#' @srrstats {G2.3} *For univariate character input:*
#' @srrstats {G2.3a} *Use `match.arg()` or equivalent where applicable to only permit expected values.*
#' @srrstats {G2.3b} *Either: use `tolower()` or equivalent to ensure input of character parameters is not case dependent; or explicitly document that parameters are strictly case-sensitive.*
#' @srrstats {G2.4} *Provide appropriate mechanisms to convert between different data types, potentially including:*
#' @srrstats {G2.4b} *explicit conversion to continuous via `as.numeric()`*
#' @srrstats {G2.4e} *explicit conversion from factor via `as...()` functions*
#' @srrstats {G2.8} *Software should provide appropriate conversion or dispatch routines as part of initial pre-processing to ensure that all other sub-functions of a package receive inputs of a single defined class or type.*
#' @srrstats {G2.10} *Software should ensure that extraction or filtering of single columns from tabular inputs should not presume any particular default behaviour, and should ensure all column-extraction operations behave consistently regardless of the class of tabular data used as input.*
#' @srrstats {G5.1} *Data sets created within, and used to test, a package should be exported (or otherwise made generally available) so that users can confirm tests and run examples.*
#' @srrstats {G5.2} *Appropriate error and warning behaviour of all functions should be explicitly demonstrated through tests. In particular,*
#' @srrstats {G5.2a} *Every message produced within R code by `stop()`, `warning()`, `message()`, or equivalent should be unique*
#' @srrstats {G5.2b} *Explicit tests should demonstrate conditions which trigger every one of those messages, and should compare the result with expected values.*
#' @srrstats {G5.5} *Correctness tests should be run with a fixed random seed*
#' @srrstats {G5.6} **Parameter recovery tests** *to test that the implementation produce expected results given data with known properties. For instance, a linear regression algorithm should return expected coefficient values for a simulated data set generated from a linear model.*
#' @srrstats {G5.6a} *Parameter recovery tests should generally be expected to succeed within a defined tolerance rather than recovering exact values.*
#' @srrstats {G5.6b} *Parameter recovery tests should be run with multiple random seeds when either data simulation or the algorithm contains a random component. (When long-running, such tests may be part of an extended, rather than regular, test suite; see G5.10-4.12, below).*
#' @srrstats {G5.7} **Algorithm performance tests** *to test that implementation performs as expected as properties of data change. For instance, a test may show that parameters approach correct estimates within tolerance as data size increases, or that convergence times decrease for higher convergence thresholds.*
#' @srrstats {G5.9} **Noise susceptibility tests** *Packages should test for expected stochastic behaviour, such as through the following conditions:*
#' @srrstats {G5.9a} *Adding trivial noise (for example, at the scale of `.Machine$double.eps`) to data does not meaningfully change results*
#' @srrstats {G5.9b} *Running under different random seeds or initial conditions does not meaningfully change results*
#' @srrstats {RE1.0} *Regression Software should enable models to be specified via a formula interface, unless reasons for not doing so are explicitly documented.*
#' @srrstats {RE1.1} *Regression Software should document how formula interfaces are converted to matrix representations of input data.*
#' @srrstats {RE1.2} *Regression Software should document expected format (types or classes) for inputting predictor variables, including descriptions of types or classes which are not accepted.*
#' @srrstats {RE1.3} *Regression Software which passes or otherwise transforms aspects of input data onto output structures should ensure that those output structures retain all relevant aspects of input data, notably including row and column names, and potentially information from other `attributes()`.*
#' @srrstats {RE1.3a} *Where otherwise relevant information is not transferred, this should be explicitly documented.*
#' @srrstats {RE1.4} *Regression Software should document any assumptions made with regard to input data; for example distributional assumptions, or assumptions that predictor data have mean values of zero. Implications of violations of these assumptions should be both documented and tested.*
#' @srrstats {RE2.3} *Where applicable, Regression Software should enable data to be centred (for example, through converting to zero-mean equivalent values; or to z-scores) or offset (for example, to zero-intercept equivalent values) via additional parameters, with the effects of any such parameters clearly documented and tested.*
#' @srrstats {RE3.0} *Issue appropriate warnings or other diagnostic messages for models which fail to converge.*
#' @srrstats {RE3.2} *Ensure that convergence thresholds have sensible default values, demonstrated through explicit documentation.*
#' @srrstats {RE3.3} *Allow explicit setting of convergence thresholds, unless reasons against doing so are explicitly documented.*
#' @srrstats {RE4.0} *Regression Software should return some form of "model" object, generally through using or modifying existing class structures for model objects (such as `lm`, `glm`, or model objects from other packages), or creating a new class of model objects.*
#' @srrstats {RE4.2} *Model coefficients (via `coef()` / `coefficients()`)*
#' @srrstats {RE4.4} *The specification of the model, generally as a formula (via `formula()`)*
#' @srrstats {RE4.5} *Numbers of observations submitted to model (via `nobs()`)*
#' @srrstats {RE4.7} *Where appropriate, convergence statistics*
#' @srrstats {RE4.8} *Response variables, and associated "metadata" where applicable.*
#' @srrstats {RE4.9} *Modelled values of response variables.*
#' @srrstats {RE4.13} *Predictor variables, and associated "metadata" where applicable.*
#' @srrstats {RE4.14} *Where possible, values should also be provided for extrapolation or forecast *errors*.*
#' @srrstats {RE4.17} *Model objects returned by Regression Software should implement or appropriately extend a default `print` method which provides an on-screen summary of model (input) parameters and (output) coefficients.*
#' @srrstats {RE4.18} *Regression Software may also implement `summary` methods for model objects, and in particular should implement distinct `summary` methods for any cases in which calculation of summary statistics is computationally non-trivial (for example, for bootstrapped estimates of confidence intervals).*
#' @srrstats {RE6.0} *Model objects returned by Regression Software (see* **RE4***) should have default `plot` methods, either through explicit implementation, extension of methods for existing model objects, or through ensuring default methods work appropriately.*
#' @srrstats {RE6.1} *Where the default `plot` method is **NOT** a generic `plot` method dispatched on the class of return objects (that is, through an S3-type `plot.<myclass>` function or equivalent), that method dispatch (or equivalent) should nevertheless exist in order to explicitly direct users to the appropriate function.*
#' @srrstats {RE6.3} *Where a model object is used to generate a forecast (for example, through a `predict()` method), the default `plot` method should provide clear visual distinction between modelled (interpolated) and forecast (extrapolated) values.*
#' @noRd
NULL

#' NA_standards
#' @srrstatsNA {G1.2} *Statistical Software should include a* Life Cycle Statement *describing current and anticipated future states of development.*
#' @srrstatsNA {G1.6} *Software should include code necessary to compare performance claims with alternative implementations in other R packages. No other R package fits high dimensional mixture cure models*
#' @srrstatsNA {G2.4a} *explicit conversion to `integer` via `as.integer()`*
#' @srrstatsNA {G2.4c} *explicit conversion to character via `as.character()` (and not `paste` or `paste0`)*
#' @srrstatsNA {G2.4d} *explicit conversion to factor via `as.factor()`*
#' @srrstatsNA {G2.5} *Where inputs are expected to be of `factor` type, secondary documentation should explicitly state whether these should be `ordered` or not, and those inputs should provide appropriate error or other routines to ensure inputs follow these expectations. No inputs are expected to be factor type.*
#' @srrstatsNA {G2.6} *Software which accepts one-dimensional input should ensure values are appropriately pre-processed regardless of class structures.*
#' @srrstatsNA {G2.7} *Software should accept as input as many of the above standard tabular forms as possible, including extension to domain-specific forms.*
#' @srrstatsNA {G2.9} *Software should issue diagnostic messages for type conversion in which information is lost (such as conversion of variables from factor to character; standardisation of variable names; or removal of meta-data such as those associated with [`sf`-format](https://r-spatial.github.io/sf/) data) or added (such as insertion of variable or column names where none were provided). No conversions are performed.*
#' @srrstatsNA {G2.11} *Software should ensure that `data.frame`-like tabular objects which have columns which do not themselves have standard class attributes (typically, `vector`) are appropriately processed, and do not error without reason. This behaviour should be tested. Again, columns created by the [`units` package](https://github.com/r-quantities/units/) provide a good test case.*
#' @srrstatsNA {G2.12} *Software should ensure that `data.frame`-like tabular objects which have list columns should ensure that those columns are appropriately pre-processed either through being removed, converted to equivalent vector columns where appropriate, or some other appropriate treatment such as an informative error. This behaviour should be tested.*
#' @srrstatsNA {G2.14} *Where possible, all functions should provide options for users to specify how to handle missing (`NA`) data, with options minimally including:*
#' @srrstatsNA {G2.14a} *error on missing data*
#' @srrstatsNA {G2.14b} *ignore missing data with default warnings or messages issued*
#' @srrstatsNA {G2.14c} *replace missing data with appropriately imputed values*
#' @srrstatsNA {G2.15} *Functions should never assume non-missingness, and should never pass data with potential missing values to any base routines with default `na.rm = FALSE`-type parameters (such as [`mean()`](https://stat.ethz.ch/R-manual/R-devel/library/base/html/mean.html), [`sd()`](https://stat.ethz.ch/R-manual/R-devel/library/stats/html/sd.html) or [`cor()`](https://stat.ethz.ch/R-manual/R-devel/library/stats/html/cor.html)).*
#' @srrstatsNA {G2.16} *All functions should also provide options to handle undefined values (e.g., `NaN`, `Inf` and `-Inf`), including potentially ignoring or removing such values.*
#' @srrstatsNA {G3.0} *Statistical software should never compare floating point numbers for equality. All numeric equality comparisons should either ensure that they are made between integers, or use appropriate tolerances for approximate equality.*
#' @srrstatsNA {G3.1} *Statistical software which relies on covariance calculations should enable users to choose between different algorithms for calculating covariances, and should not rely solely on covariances from the `stats::cov` function. Covariances are not returned for our fitted penalized mixture cure models.*
#' @srrstatsNA {G3.1a} *The ability to use arbitrarily specified covariance methods should be documented (typically in examples or vignettes). Covariances are not returned for our fitted penalized models.*
#' @srrstatsNA {G4.0} *Statistical Software which enables outputs to be written to local files should parse parameters specifying file names to ensure appropriate file suffices are automatically generated where not provided. Output is not written to local files.*
#' @srrstatsNA {G5.0} *Where applicable or practicable, tests should use standard data sets with known properties (for example, the [NIST Standard Reference Datasets](https://www.itl.nist.gov/div898/strd/), or data sets provided by other widely-used R packages). No standard datasets with a cured fraction exist so we provide amldata as an example dataset.*
#' @srrstatsNA {G5.3} *For functions which are expected to return objects containing no missing (`NA`) or undefined (`NaN`, `Inf`) values, the absence of any such values in return objects should be explicitly tested.*
#' @srrstatsNA {G5.4} **Correctness tests** *to test that statistical algorithms produce expected results to some fixed test data sets (potentially through comparisons using binding frameworks such as [RStata](https://github.com/lbraglia/RStata)).*
#' @srrstatsNA {G5.4a} *For new methods, it can be difficult to separate out correctness of the method from the correctness of the implementation, as there may not be reference for comparison. In this case, testing may be implemented against simple, trivial cases or against multiple implementations such as an initial R implementation compared with results from a C/C++ implementation.*
#' @srrstatsNA {G5.4b} *For new implementations of existing methods, correctness tests should include tests against previous implementations. Such testing may explicitly call those implementations in testing, preferably from fixed-versions of other software, or use stored outputs from those where that is not possible.*
#' @srrstatsNA {G5.4c} *Where applicable, stored values may be drawn from published paper outputs when applicable and where code from original implementations is not available*
#' @srrstatsNA {G5.10} *Extended tests should included and run under a common framework with other tests but be switched on by flags such as as a `<MYPKG>_EXTENDED_TESTS="true"` environment variable.* - The extended tests can be then run automatically by GitHub Actions for example by adding the following to the `env` section of the workflow:
#' @srrstatsNA {G5.8} **Edge condition tests** *to test that these conditions produce expected behaviour such as clear warnings or errors when confronted with data with extreme properties including but not limited to:*
#' @srrstatsNA {G5.8a} *Zero-length data*
#' @srrstatsNA {G5.8b} *Data of unsupported types (e.g., character or complex numbers in for functions designed only for numeric data)*
#' @srrstatsNA {G5.8c} *Data with all-`NA` fields or columns or all identical fields or columns*
#' @srrstatsNA {G5.8d} *Data outside the scope of the algorithm (for example, data with more fields (columns) than observations (rows) for some regression algorithms)*
#' @srrstatsNA {G5.11} *Where extended tests require large data sets or other assets, these should be provided for downloading and fetched as part of the testing workflow.*
#' @srrstatsNA {G5.11a} *When any downloads of additional data necessary for extended tests fail, the tests themselves should not fail, rather be skipped and implicitly succeed with an appropriate diagnostic message.*
#' @srrstatsNA {G5.12} *Any conditions necessary to run extended tests such as platform requirements, memory, expected runtime, and artefacts produced that may need manual inspection, should be described in developer documentation such as a `CONTRIBUTING.md` or `tests/README.md` file.*
#' @srrstatsNA {RE2.0} *Regression Software should document any transformations applied to input data, for example conversion of label-values to `factor`, and should provide ways to explicitly avoid any default transformations (with error or warning conditions where appropriate).*
#' @srrstatsNA {RE2.1} *Regression Software should implement explicit parameters controlling the processing of missing values, ideally distinguishing `NA` or `NaN` values from `Inf` values (for example, through use of `na.omit()` and related functions from the `stats` package).*
#' @srrstatsNA {RE2.2} *Regression Software should provide different options for processing missing values in predictor and response data. For example, it should be possible to fit a model with no missing predictor data in order to generate values for all associated response points, even where submitted response values may be missing.*
#' @srrstatsNA {RE2.4} *Regression Software should implement pre-processing routines to identify whether aspects of input data are perfectly collinear, notably including:*
#' @srrstatsNA {RE2.4a} *Perfect collinearity among predictor variables*
#' @srrstatsNA {RE2.4b} *Perfect collinearity between independent and dependent variables*
#' @srrstatsNA {RE3.1} *Enable such messages to be optionally suppressed, yet should ensure that the resultant model object nevertheless includes sufficient data to identify lack of convergence.*
#' @srrstatsNA {RE4.1} *Regression Software may enable an ability to generate a model object without actually fitting values. This may be useful for controlling batch processing of computationally intensive fitting algorithms.*
#' @srrstatsNA {RE4.3} *Confidence intervals on those coefficients (via `confint()`). Confidence intervals are not returned for our fitted penalized models.*
#' @srrstatsNA {RE4.6} *The variance-covariance matrix of the model parameters (via `vcov()`). Not returned for our fitted penalized mixture cure models.*
#' @srrstatsNA {RE4.10} *Model Residuals, including sufficient documentation to enable interpretation of residuals, and to enable users to submit residuals to their own tests. Residuals are not provided for our penalized mixture cure models.*
#' @srrstatsNA {RE4.11} *Goodness-of-fit and other statistics associated such as effect sizes with model coefficients.*
#' @srrstatsNA {RE4.12} *Where appropriate, functions used to transform input data, and associated inverse transform functions.*
#' @srrstatsNA {RE4.15} *Sufficient documentation and/or testing should be provided to demonstrate that forecast errors, confidence intervals, or equivalent values increase with forecast horizons.*
#' @srrstatsNA {RE4.16} *Regression Software which models distinct responses for different categorical groups should include the ability to submit new groups to `predict()` methods.*
#' @srrstatsNA {RE5.0} *Scaling relationships between sizes of input data (numbers of observations, with potential extension to numbers of variables/columns) and speed of algorithm.*
#' @srrstatsNA {RE6.2} *The default `plot` method should produce a plot of the `fitted` values of the model, with optional visualisation of confidence intervals or equivalent.*
#' @srrstatsNA {RE7.0} *Tests with noiseless, exact relationships between predictor (independent) data.*
#' @srrstatsNA {RE7.0a} *In particular, these tests should confirm ability to reject perfectly noiseless input data.*
#' @srrstatsNA {RE7.1} *Tests with noiseless, exact relationships between predictor (independent) and response (dependent) data.*
#' @srrstatsNA {RE7.1a} *In particular, these tests should confirm that model fitting is at least as fast or (preferably) faster than testing with equivalent noisy data (see RE2.4b).*
#' @srrstatsNA {RE7.2} *Demonstrate that output objects retain aspects of input data such as row or case names (see **RE1.3**).*
#' @srrstatsNA {RE7.3} Demonstrate and test expected behaviour when objects returned from regression software are submitted to the accessor methods of **RE4.2**--**RE4.7**.
#' @srrstatsNA {RE7.4} Extending directly from **RE4.15**, where appropriate, tests should demonstrate and confirm that forecast errors, confidence intervals, or equivalent values increase with forecast horizons.

#' Any non-applicable standards can have their tags changed from `@srrstatsTODO`
#' to `@srrstatsNA`, and placed together in this block, along with explanations
#' for why each of these standards have been deemed not applicable.
#' (These comments may also be deleted at any time.)
#' @noRd
NULL

---
title: "Estimating Cure Models in High Dimensions: A Guide with hdcuremodels"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{hdcuremodels-vignette}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

Recent medical advances have led to long-term overall or disease-free survival for at least a subset of treated subjects for various diseases, such that some patients' overall survival is consistent with their population expected survival. Conventionally, we call the subset of subjects who are immune to the event of interest **cured** while all other subjects are **susceptible** to the event. When performing a time-to-event analysis to data that includes a subset of subjects who are cured, mixture cure models (MCMs) are a useful alternative to the Cox proportional hazards (PH) model. This is because the Cox PH model assumes a constant hazard applies to all subjects throughout the observed follow-up time which is violated when some subjects in the dataset are cured (Goldman 1991). Additionally, when cured subjects comprise a portion of the dataset, the survival function is improper. 

To introduce notation for the MCM, consider that $f(t)$ represents the probability density function for the overall population which is comprised of $K$ independent subgroups. Considering finite mixture models generally, the overall probability density can be decomposed $f(t)=\sum_{k=1}^Kp_k f_k (t)$. For MCMs, $K = 2$ because one subgroup includes those immune to the event of interest (i.e., cured) and the other subgroup consists of those susceptible to the event of interest (i.e., uncured). For subjects $i=1,\ldots, N$, let $Y_i=0$ represent those cured (c) and $Y_i=1$ represent those who are susceptible or uncured (u), the finite mixture model for survival is $S(t)=p_c S_c (t)+p_u S_u (t)$. Letting $p$ represent the $p_u$ such that $1 – p = p_c$ and recognizing $S_c (t)=1$, this MCM model simplifies to $S(t)=(1-p)+pS_u (t)$. Therefore, this model consists of two components: the incidence component which models susceptible versus cured and the latency component which models time-to-event for those susceptible. The latency or time-to-event function for susceptibles, $S_u (t)$, can be modeled in different ways, using for example a parametric, non-parametric, or semi-parametric survival model, whereas the incidence component is typically modelled using logistic regression. 

Often, we want to estimate the effect of covariates on overall outcome and because different covariates may affect incidence versus latency portions of the MCM, we let $\mathbf{x}_i$ represent the $P$ incidence covariates and  $\mathbf{w}_i$ represent the $Q$ latency covariates, where $\mathbf{x}_i$ and $\mathbf{w}_i$ may or may not differ.  The mixture cure survival function is given by 
$$S(t| \mathbf{x, w}) = (1-p(\mathbf{x})) + p(\mathbf{x})S_u(t,\mathbf{w}|Y=1).$$
Here, $p(\mathbf{x})$ represents the probability of being susceptible, $1 - p(\mathbf{x})$ represents the probability of being cured, and $S_u(t, \mathbf(w)|Y=1)$ represents the survival function (or latency) for those susceptible. Thus, the mixture cure model structure allows one to investigate the effect of covariates on two components of the model: incidence (susceptible versus cured) and latency (time-to-event for susceptibles). The incidence component is ordinarily modeled using logistic regression and we have parameters $\beta_0$ and vector of coefficients  $\boldsymbol{\beta}_{inc}$. The latency or time-to-event function for susceptibles can be modeled in different ways, using for example a parametric, a non-parametric, or semi-parametric survival model. Therefore, the vector of coefficients $\boldsymbol{\beta}_{lat}$ in the latency portion of the model may be accompanied by a shape and scale parameters if a Weibull or exponential model are fit.

The `hdcuremodels` R package was developed for fitting penalized mixture cure models when there is a high-dimensional covariate space, such as when high-throughput genomic data are used in modeling time-to-event data when some subjects will not experience the event of interest. The package includes the following functions for model fitting: `curegmifs`, `cureem`, `cv_curegmifs`, and `cv_cureem`. The functions `curegmifs` and `cureem` are used for fitting a penalized mixture cure model. The distinction between `curegmifs` and `cureem` is the algorithm used and the types of time-to-event models that can be fit.  The `curegmifs` function  can be used to fit penalized Weibull and penalized exponential models where the solution is obtained using the generalized monotone incremental forward stagewise (GMIFS) method (Fu et al, 2022). The `cureem` function can be used to fit penalized Cox proportional hazards, Weibull, and exponential models where the solution is obtained using the Expectation-Maximization (E-M) algorithm (Archer et al, 2024). Both `cv_curegmifs` and `cv_cureem` can be used for performing cross-validation for model selection and for performing variable selection using the model-X knockoff procedure with false discovery rate control (Candes et al, 2018). Aside from these model fitting functions, other functions have been included for testing assumptions required for fitting a mixture cure model.  This vignette describes the syntax required for each of our penalized mixture cure models.

# Package description

The `hdcuremodels` and `survival` packages should be loaded.

```{r setup}
library(hdcuremodels)
library(survival)
```

## Data examples

The package includes two datasets: `amltrain` (Archer et al, 2024) and `amltest` (Archer et al, 2024; Bamopoulos et al 2020). Both datasets include patients diagnosed with acute myeloid leukemia (AML) who were cytogenetically normal at diagnosis along with the same variables: `cryr` is the duration of complete response (in years), `relapse.death` is a censoring variable where 1 indicates the patient relapsed or died and 0 indicates the patient was alive at last follow-up, and expression for 320 transcripts measured using RNA-sequencing. The restriction to 320 transcripts was to reduce run time. Therefore, results obtained with these data will not precisely recapitulate those in the original publication (Archer et al, 2024). `amltrain` includes the 306 subjects that were used for training the penalized MCM while `amltest` includes the 40 subjects that were used to test the penalized MCM.   

We also included a function, `generate_cure_data`, that allows the user to generate time-to-event data that includes a cured fraction. Various parameters in this function will allow the user to explore the impact of sample size (`n`), number of penalized variables in the model (`j`), number of penalized variables truly associated with the outcome (`n_true`), effect size (referred to as the signal amplitude) of the penalized variables (`a`), correlation among penalized variables (`rho`), `same_signs` which if `TRUE` indicates that the penalized incidence and latency coefficients should have the same signs, number of unpenalized variables to coerce into the model (`nonp`), `itct_mean` the incidence intercept which controls the cure fraction, `cens_ub` the upper bound for the Uniform[0, `cens_ub`] distribution for generating censoring times, `model` which specifies the type of regression model to use for generating event times, `alpha` and `lambda` the numeric shape and rate parameter values to specify when `model = "weibull"`. When `a = 0` there is no relationship between the `j` variables and the outcome. When `rho = 0` the `j` variables generated are uncorrelated while as `rho` approaches 1 the variables generated are highly correlated. Note that `rho` cannot be set equal to 1.

```{r}
withr::local_seed(23)
data <- generate_cure_data(n = 200, j = 10, n_true = 5, a = 1.8, rho = 0.2)
training <- data$training
testing <- data$testing
head(training)
head(data$training_y)
```

The training and testing data.frames have the same structure which includes Time, Censor, and covariates. Covariates prefixed with "U" are the unpenalized covariates and is equal to the value passed to `nonp` (default is 2). Variables prefixed with "X" are the penalized covariates and is equal to the value passed to `j`. The `training_y` and `testing_y` vectors return represent the true cure status of the subject (1 = uncured; 0 = cured). To identify which penalized covariates are truly related to the incidence and latency portions of the model, the user can inspect the indices returned by `nonzero_b` and `nonzero_beta` in the `parameters` component of the list.

```{r}
data$parameters
```

In this example, the 2nd, 3rd, 6th, 8th, and 10th penalized covariates, corresponding to

```{r}
names(training)[grep("^X", names(training))][data$parameters$nonzero_b]
```

are truly associated with incidence while the 1st, 3rd, 6th, 8th, and 10th penalized covariates are truly associated with latency.

```{r}
names(training)[grep("^X", names(training))][data$parameters$nonzero_beta]
```

## Assessing model assumptions for fitting a mixture cure model

The workflow for fitting a mixture cure model should include the assessment of two assumptions: first, that a non-zero cure fraction is present; second, that there is sufficient follow-up (Maller and Zhou, 1996). Inferential tests for assessing these two assumptions are included in the `hdcuremodels` package. The functions `nonzerocure_test` and `sufficient_fu_test` both take a `survfit` object as their argument. 

```{r}
km_train <- survfit(Surv(cryr, relapse.death) ~ 1, data = amltrain)
```

```{r, echo=FALSE, fig=TRUE}
plot(km_train, mark.time = TRUE, xlab = "Time (years)", ylab = "Relapse-free survival")
```

As can be seen from the Kaplan-Meier plot, there is a long-plateau that does not drop down to zero. This may indicate the presence of a cured fraction. We can test the null hypothesis that the cured fraction is zero against the alternative hypothesis that the cured fraction is not zero using `nonzerocure_test` (Maller and Zhou, 1996).

```{r, echo=TRUE}
nonzerocure_test(km_train)
```

Given the small p-value we reject the null hypothesis and conclude there is a non-zero cure fraction present. We can also extract the cured fraction as the Kaplan-Meier estimate beyond the last observed event (Goldman, 1991) using the `cure_estimate` function.

```{r}
cure_estimate(km_train)
```

This estimate requires sufficiently long follow-up which can be tested using the `sufficient_fu_test` function (Maller and Zhou, 1996).

```{r, echo=TRUE}
sufficient_fu_test(km_train)
```

This function tests the null hypothesis of insufficient follow-up against the alternative that there is sufficient follow-up. Based on these results, we reject the null hypothesis and conclude there is sufficient follow-up. Having verified these two assumptions, we can now fit a mixture cure model.

## Penalized mixture cure models 

### Fitting penalized mixture cure models using GMIFS

The primary function for fitting parametric models using the GMIFS algorithm in the `hdcuremodels` package is `curegmifs`. The function arguments are

```{r args}
args(curegmifs)
```

The `curegmifs` function accepts a model formula that specifies the time-to-event outcome on the left-hand side of the equation as a `Surv` object and any incidence predictor variable(s) on the right-hand side of the equation. Note that at least some incidence predictor variables must be included in order to fit a penalized mixture cure model, otherwise, the `survival` package functions should be used to fit time-to-event models that lack an incidence component. The `data` parameter specifies the name of the data.frame. Users should note that rows with missing data are omitted (that is, only `na.action = na.omit` is operational) therefore users may want to impute missing data prior to calling this function. The optional `subset` parameter can be used to limit model fitting to a subset of observations in the data. The `x_latency` parameter specifies the variables to be included in the latency portion of the model and can be either a matrix of predictors, a model formula with the right hand side specifying the latency variables, or the same data.frame passed to the `data` parameter. Note that when using the model formula syntax for `x_latency` it cannot handle `x_latency = ~ .`. The `curegmifs` function can fit either a either `"weibull"` or `"exponential"` model, which is specified using the `model` parameter. Other  parameters include `penalty_factor_inc` which is an optional numeric vector with length equal to the number of incidence variables, where 1 indicates that variable should be penalized and 0 indicates that variable is unpenalized (default is that all variables are penalized). Likewise `penalty_factor_lat` is an optional numeric vector with length equal to the number of latency variables, where 1 indicates that variable should be penalized and 0 indicates that variable is unpenalized (default is that all variables are penalized). Unpenalized predictors are those that we want to coerce into the model (e.g., age) so that no penalty is applied. By default the variables are centered and scaled (`scale = TRUE`). The parameter `epsilon` is the size of the coefficient update at each step (default = 0.001). The GMIFS algorithm stops when either the difference between successive log-likelihoods  is less than `thresh` (default 1e-05) or the algorithm has exceeded the maximum number of iterations (`maxit`). Initialization is automatically provided by the function though `inits` can be used to provide initial values for the incidence intercept, unpenalized incidence and latency coefficients, rate parameter, and shape parameter if fitting a Weibull mixture cure model. By default `verbose = TRUE` so that running information is echoed to the R console.

```{r, eval=FALSE}
fitgmifs <- curegmifs(Surv(cryr, relapse.death) ~ .,
  data = amltrain,
  x_latency = amltrain, model = "weibull"
)
```

Details of the GMIFS mixture cure model have been described in Fu et al, 2022.

### Fitting penalized mixture cure models using E-M algorithm

The primary function for fitting penalized MCMs using the E-M algorithm in the `hdcuremodels` package is `cureem`. The function arguments are

```{r args2}
args(cureem)
```

The `cureem` function accepts a model formula that specifies the time-to-event outcome on the left-hand side of the equation as a `Surv` object and any incidence predictor variable(s) on the right-hand side of the equation. Note that at least some incidence predictor variables must be included in order to fit a penalized mixture cure model, otherwise, the `survival` package functions should be used to fit time-to-event models that lack an incidence component. The `data` parameter specifies the name of the data.frame and the optional `subset` parameter can be used to limit model fitting to a subset of observations in the data. The `x_latency` parameter specifies the variables to be included in the latency portion of the model and can be either a matrix of predictors, a model formula with the right hand side specifying the latency variables, or the same data.frame passed to the `data` parameter. Note that when using the model formula syntax for `x_latency` it cannot handle `x_latency = ~ .`. The `cureem` function can fit one of three models which is specified using the `model` parameter, which can be either `"cox"` (default), `"weibull"`, or `"exponential"`. Other  parameters include `penalty` which can be `"lasso"`, `"MCP"`, or `"SCAD"` when fitting a `"cox"` model but must be `"lasso"` when fitting a `"weibull"` or `"exponential"` model. `penalty_factor_inc` is an optional numeric vector with length equal to the number of incidence variables, where 1 indicates that variable should be penalized and 0 indicates that variable is unpenalized (default is that all variables are penalized). Likewise `penalty_factor_lat` is an optional numeric vector with length equal to the number of latency variables, where 1 indicates that variable should be penalized and 0 indicates that variable is unpenalized (default is that all variables are penalized). Unpenalized predictors are those that we want to coerce into the model (e.g., age) so that no penalty is applied. The iterative process stops when the differences between successive expected penalized complete-data log-likelihoods for both incidence and latency components are less than `thresh` (default = 0.001). By default the variables are centered and scaled (`scale = TRUE`). The user can specify the maximum number of passes over the data for each lambda using `maxit`, which defaults to 100 when `penalty = "lasso"` and 1000 when either `penalty = "MCP"` or `penalty = "SCAD"`. Initialization is automatically provided by the function though `inits` can be used to provide initial values for the incidence intercept, unpenalized indicidence and latency coefficients, rate parameter (for Weibull and exponential MCM), and shape parameter (for Weibull MCM). By default `verbose = TRUE` so that running information is echoed to the R console. The user can also specify the penalty parameter for the incidence (`lambda_inc`) and latency (`lambda_lat`) portions of the model and the $\gamma$ penalty when MCP or SCAD is used (`gamma_inc` and `gamma_lat`).

Details of the E-M MCM have been described in the Supplementary Material of Archer et al, 2024.

```{r}
fitem <- cureem(Surv(cryr, relapse.death) ~ .,
  data = amltrain,
  x_latency = amltrain, model = "cox",
  lambda_inc = 0.009993, lambda_lat = 0.02655
)
```

### Cross-validation

There is a function for performing cross-validation (CV) corresponding to each of the two optimization methods. The primary function for fitting cross-validated penalized MCMs using the E-M algorithm in the `hdcuremodels` package is `cv_cureem`. The function arguments are

```{r args3}
args(cv_cureem)
```

The `cv_cureem` function accepts a model formula that specifies the time-to-event outcome on the left-hand side of the equation as a `Surv` object and any incidence predictor variable(s) on the right-hand side of the equation. Note that at least some incidence predictor variables must be included in order to fit a penalized mixture cure model, otherwise, the `survival` package functions should be used to fit time-to-event models that lack an incidence component. The `data` parameter specifies the name of the data.frame and the optional `subset` parameter can be used to limit model fitting to a subset of observations in the data. The `x_latency` parameter specifies the variables to be included in the latency portion of the model and can be either a matrix of predictors, a model formula with the right hand side specifying the latency variables, or the same data.frame passed to the `data` parameter. Note that when using the model formula syntax for `x_latency` it cannot handle `x_latency = ~ .`. The `cv_cureem` function can fit one of three models which is specified using the `model` parameter, which can be either `"cox"` (default), `"weibull"`, or `"exponential"`. Other  parameters include `penalty` which can be `"lasso"`, `"MCP"`, or `"SCAD"` when fitting a `"cox"` model but must be `"lasso"` when fitting a `"weibull"` or `"exponential"` model. `penalty_factor-inc` is an optional numeric vector with length equal to the number of incidence variables, where 1 indicates that variable should be penalized and 0 indicates that variable is unpenalized (default is that all variables are penalized). Likewise `penalty_factor_lat` is an optional numeric vector with length equal to the number of latency variables, where 1 indicates that variable should be penalized and 0 indicates that variable is unpenalized (default is that all variables are penalized). Unpenalized predictors are those that we want to coerce into the model (e.g., age) so that no penalty is applied. The user can choose to use the model-X knock-off procedure to control the false discovery rate (FDR) by specifying `fdr_control = TRUE` and optionally changing the FDR threshold (default `fdr = 0.20`) (Candes et al, 2018). To identify the optimal $\lambda$ for the incidence and latency portions of the model, the user can set `grid_tuning = TRUE` (default is that one value for $\lambda$ is used in both portions of the model). Other useful parameters for the cross-validation function include `n_folds`, an integer specifying the number of folds for the k-fold cross-validation procedure (default is 5); `measure_inc` which specifies the evaluation criterion used in selecting the optimal penalty which can be `"c"` for the C-statistic using cure status weighting (Asano and Hirakawa, 2017) or `"auc"` for cure prediction using mean score imputation (Asano et al, 2014) (default is `measure_inc = "c"`); `one_se` is a logical variable that if TRUE then the one standard error rule is used which selects the most parsimonious model having evaluation criterion no more than one standard error worse than that of the best evaluation criterion (default is FALSE); and `cure_cutoff` which is a numeric value representing the cutoff time used to represent subjects not experiencing the event by this time are cured which is used to produce a proxy for the unobserved cure status when calculating the C-statistic and AUC (default is 5). If the logical parameter `parallel` is TRUE, cross-validation will be performed using parallel processing which requires the `foreach` and `doMC` R packages. To foster reproducibility of cross-validation results, `seed` can be set to an integer. 

As with `cureem`, the iterative process stops when the differences between successive expected penalized complete-data log-likelihoods for both incidence and latency components are less than `thresh` (default = 0.001). By default the variables are centered and scaled (`scale = TRUE`). The user can specify the maximum number of passes over the data for each lambda using `maxit`, which defaults to 100 when `penalty = "lasso"` and 1000 when either `penalty = "MCP"` or `penalty = "SCAD"`. Initialization is automatically provided by the function though `inits` can be used to provide initial values for the incidence intercept, unpenalized indicidence and latency coefficients, rate parameter (for Weibull and exponential MCM), and shape parameter (for Weibull MCM). When  `model = "cox"`, `inits` should also include a numeric vector for the latency survival probabilities. Optionally, the user can supply a numeric vector to search for the optimal penalty for the incidence portion (`lambda_inc_list`) and a numeric vector to search for the optimal penalty for the latency portion (`lambda_lat_list`) of the model. By default the number of values to search for the optimal incidence penalty is 10 which can be changed by specifying an integer for `nlambda_int` and similarly for latency by specifying an integer for `nlambda_lat`. If `penalty` is either `"MCP"` or `"SCAD"`, the user can optionally specify the penalization parameter $\gamma$ for the incidence (`gamma_inc`) and latency (`gamma_lat`) portions of the model. By default `verbose = TRUE` so that running information is echoed to the R console. The user can also specify the penalty parameter for the incidence (`lambda_inc`) and latency (`lambda_lat`) portions of the model and the $\gamma$ penalty when MCP or SCAD is used (`gamma_inc` and `gamma_lat`).

```{r}
fit_cv <- cv_cureem(Surv(Time, Censor) ~ .,
  data = training,
  x_latency = training, fdr_control = FALSE,
  grid_tuning = FALSE, nlambda_inc = 10,
  nlambda_lat = 10, n_folds = 2, seed = 23,
  verbose = TRUE
)
```

Notice in the previous section describing `cureem` that values were supplied for the $\lambda$ penalty parameters for both the incidence and latency portions of the model using `lambda_inc` and `lambda_lat`. Those values were determined from the following repeated 10-fold cross-validation where the optimal $\lambda$ for the incidence portion was identified by fitting the models to maximize the AUC while the optimal $\lambda$ for the latency portion was identified by fitting the models to maximize the C-statistic. After the CV procedure the mode for each was taken. Because the run time for the repeated 10-fold CV procedure was 5.65 hours, this code chunk is not evaluated herein.

```{r, eval = FALSE}
lambda_inc <- lambda_lat <- rep(0, 100)
for (k in 1:100) {
  print(k)
  coxem_auc_k <- cv_cureem(Surv(cryr, relapse.death) ~ .,
    data = amltrain, x_latency = amltrain,
    model = "cox", penalty = "lasso",
    scale = TRUE, grid_tuning = TRUE,
    nfolds = 10, nlambda_inc = 20,
    nlambda_lat = 20, verbose = FALSE,
    parallel = TRUE, measure_inc = "auc"
  )
  lambda_inc[k] <- coxem_auc_k$selected_lambda_inc
  coxem_c_k <- cv_cureem(Surv(cryr, relapse.death) ~ .,
    data = amltrain,
    x_latency = amltrain, model = "cox",
    penalty = "lasso", scale = TRUE,
    grid_tuning = TRUE, nfolds = 10,
    nlambda_inc = 20, nlambda_lat = 20,
    verbose = FALSE, parallel = TRUE,
    measure_inc = "c"
  )
  lambda_lat[k] <- coxem_c_k$selected_lambda_lat
}
table(lambda_inc)
table(lambda_lat)
```

The primary function for fitting cross-validated penalized MCMs using the GMIFS algorithm in the `hdcuremodels` package is `cv_curegmifs`. The function arguments are

```{r args4}
args(cv_curegmifs)
```

The `cv_curegmifs` function accepts a model formula that specifies the time-to-event outcome on the left-hand side of the equation as a `Surv` object and any incidence predictor variable(s) on the right-hand side of the equation. Note that at least some incidence predictor variables must be included in order to fit a penalized mixture cure model, otherwise, the `survival` package functions should be used to fit time-to-event models that lack an incidence component. The `data` parameter specifies the name of the data.frame and the optional `subset` parameter can be used to limit model fitting to a subset of observations in the data. The `x_latency` parameter specifies the variables to be included in the latency portion of the model and can be either a matrix of predictors, a model formula with the right hand side specifying the latency variables, or the same data.frame passed to the `data` parameter. Note that when using the model formula syntax for `x_latency` it cannot handle `x_latency = ~ .`. The `cv_curegmifs` function can fit either a either `"weibull"` or `"exponential"` model, which is specified using the `model` parameter. Other  parameters include `penalty_factor_inc` which is an optional numeric vector with length equal to the number of incidence variables, where 1 indicates that variable should be penalized and 0 indicates that variable is unpenalized (default is that all variables are penalized). Likewise `penalty_factor_lat` is an optional numeric vector with length equal to the number of latency variables, where 1 indicates that variable should be penalized and 0 indicates that variable is unpenalized (default is that all variables are penalized). Unpenalized predictors are those that we want to coerce into the model (e.g., age) so that no penalty is applied. The user can choose to use the model-X knock-off procedure to control the false discovery rate (FDR) by specifying `fdr_control = TRUE` and optionally changing the FDR threshold (default `fdr = 0.20`) (Candes et al, 2018). By default the variables are centered and scaled (`scale = TRUE`). The parameter `epsilon` is the size of the coefficient update at each step (default = 0.001). The GMIFS algorithm stops when either the difference between successive log-likelihoods  is less than `thresh` (default 1e-05) or the algorithm has exceeded the maximum number of iterations (`maxit`). Initialization is automatically provided by the function though `inits` can be used to provide initial values for the incidence intercept, unpenalized indicidence and latency coefficients, rate parameter, and shape parameter if fitting a Weibull mixture cure model. Other useful parameters for the cross-validation function include `n_folds`, an integer specifying the number of folds for the k-fold cross-validation procedure (default is 5); `measure_inc` which specifies the evaluation criterion used in selecting the optimal penalty which can be `"c"` for the C-statistic using cure status weighting (Asano and Hirakawa, 2017) or `"auc"` for cure prediction using mean score imputation (Asano et al, 2014) (default is `measure_inc = "c"`); `one_se` is a logical variable that if TRUE then the one standard error rule is used which selects the most parsimonious model having evaluation criterion no more than one standard error worse than that of the best evaluation criterion (default is FALSE); and `cure_cutoff` which is a numeric value representing the cutoff time used to represent subjects not experiencing the event by this time are cured which is used to produce a proxy for the unobserved cure status when calculating the C-statistic and AUC (default is 5). If the logical parameter `parallel` is TRUE, cross-validation will be performed using parallel processing which requires the `foreach` and `doMC` R packages. To foster reproducibility of cross-validation results, `seed` can be set to an integer. By default `verbose = TRUE` so that running information is echoed to the R console.

## Other Package Functions
The four modeling functions `cureem`, `curegmifs`, `cv_cureem`, and `cv_curegmifs` all result in an object of class `mixturecure`. Generic functions for resulting `mixturecure` objects are available for extracting meaningful results. The `print` function prints the first several incidence and latency coefficients and the rate (exponential and Weibull) and alpha (Weibull) when fitting a parametric MCM and returns the fitted object invisible to the user.

```{r print, eval = FALSE}
print(fitem)
```

The `summary` function prints the following output for a model fit using either `cureem` or `curegmifs`:

* the number of non-zero coefficients in the incidence portion of the fitted mixture cure model when using the minimum AIC
* the number of non-zero coefficients in the latency portions of the fitted mixture cure model when using the minimum AIC
* the step and value that maximizes the log-likelihood;
* the step and value that minimizes the AIC;  
* the step and value that minimizes the modified AIC (mAIC); 
* the step and value that minimizes the corrected AIC (cAIC);  
* the step and value that minimizes the BIC; 
* the step and value that minimizes the modified BIC (mBIC); 
* the step and value that minimizes the extended BIC (EBIC). 

```{r summary}
summary(fitem)
```

The `summary` function prints the following output for a model fit using either `cv_cureem` or `cv_curegmifs` when `fdr_control = FALSE`:

* the number of non-zero coefficients in the incidence portion of the fitted mixture cure model 
* the number of non-zero coefficients in the latency portions of the fitted mixture cure model 


```{r cv}
summary(fit_cv)
```

The `summary` function prints the following output for a model fit using either `cv_cureem` or `cv_curegmifs` when `fdr_control = TRUE`:

* Number of non-zero incidence covariates  
* Number of non-zero latency covariates 

For a `cureem` or `curegmifs` fitted `mixturecure` object, the `plot` function provides a trace of the coefficients' paths by default. Because it is common for the user to include at least some of the same variables in both the incidence and latency portions of the MCM, when `label = TRUE` a legend is provided with variables included in the incidence portion prefixed with "I" and variables included in the latency portion prefixed with "L." Alternatively, the `type` parameter can be used to specify any of the information criterion ("logLik", "AIC", "cAIC", "mAIC", "BIC", "mBIC", "EBIC"). For a `cv_cureem` or `cv_curegmifs` fitted `mixturecure` object, a lollipop plot of the estimated incidence and latency coefficients is produced. The height of each lollipop represents the estimated coefficient for the optimal cross-validated model.

```{r plot}
plot(fitem)
```

```{r plotAIC}
plot(fitem, type = "cAIC")
```

```{r cv2}
plot(fit_cv)
```

Coefficient estimates can be extracted from the fitted model using the `coef` for any of these model criteria ("logLik", "AIC", "cAIC", "mAIC", "BIC", "mBIC", "EBIC") or by specifying the step at which the model is desired by specifying the `model.select` parameter. For example,

```{r cAIC}
coef_cAIC <- coef(fitem, model_select = "cAIC")
```

is equivalent to

```{r m12}
coef_12 <- coef(fitem, model_select = 12)
```

as demonstrated by comparing the results in each object:

```{r compareAIC}
names(coef_cAIC)
all.equal(coef_cAIC$rate, coef_12$rate)
all.equal(coef_cAIC$alpha, coef_12$alpha)
all.equal(coef_cAIC$b0, coef_12$b0)
all.equal(coef_cAIC$beta_inc, coef_12$beta_inc)
all.equal(coef_cAIC$beta_lat, coef_12$beta_lat)
```

Again, there are two sets of coefficients: those in the incidence portion of the model (`beta_inc`) and those in the latency portion of the model (`beta_lat`). Additionally, `b0` is the intercept in the incidence portion of the model. Depending on the model fit, `coef` will return `rate` (exponential and Weibull) and `alpha` (Weibull).

Predictions can be extracted at a given step or information criterion ("logLik", "AIC", "cAIC", "mAIC", "BIC", "mBIC", "EBIC") using the `predict` function with `model_select` specified.

```{r pred}
train_predict <- predict(fitem, model_select = "cAIC")
```

This returns three objects: `p_uncured` is the estimated probability of being susceptible ($\hat{p}(\mathbf{x})$), `linear_latency` is $\hat{\boldsymbol{\beta}}\mathbf{w}$, while `latency_risk` applies high risk and low risk labels using zero as the cutpoint from the `linear_latency` vector. Perhaps we want to apply the 0.5 threshold to `p_uncured` to create Cured and Susceptible labels.

```{r group}
p_group <- ifelse(train_predict$p_uncured < 0.50, "Cured", "Susceptible")
```

Then we can assess how well our MCM identified patients likely to be cured from those likely to be susceptible visually by examining the Kaplan-Meier curves.

```{r km}
km_cured <- survfit(Surv(cryr, relapse.death) ~ p_group, data = amltrain)
```

```{r plotkm, echo = FALSE, fig = TRUE}
plot(km_cured, mark.time = TRUE, lty = c(1, 2), xlab = "Time (years)", ylab = "Relapse-free survival")
legend(c(.9, .1), legend = c("Cured", "Susceptible"), lty = c(1, 2), bty = "n")
```

We can assess how well our MCM identified higher versus lower risk patients among those predicted to be susceptible visually by examining the Kaplan-Meier curves.

```{r suscept}
km_suscept <- survfit(Surv(cryr, relapse.death) ~ train_predict$latency_risk, data = amltrain, 
                      subset = (p_group == "Susceptible"))
```

```{r plotsusc, echo = FALSE, fig = TRUE}
plot(km_suscept, mark.time = TRUE, lty = c(1, 2), xlab = "Time (years)", ylab = "Relapse-free survival")
legend(c(.9, .1), legend = c("Higher risk", "Lower risk"), lty = c(1, 2), bty = "n")
```

Of course, we expect our model to perform well on our training data. We can also assess how well our fitted MCM performs using the independent test set `amltest`. In this case we use the `predict` function with `newdata` specified.

```{r testpred}
test_predict <- predict(fitem, newdata = amltest, model_select = "cAIC")
```

Again we will apply the 0.5 threshold to `p_uncured` to create Cured and Susceptible labels.

```{r pgroup}
test_p_group <- ifelse(test_predict$p_uncured < 0.50, "Cured", "Susceptible")
```

Then we can assess how well our MCM identified patients likely to be cured from those likely to be susceptible visually by examining the Kaplan-Meier curves.

```{r kmtest}
km_cured_test <- survfit(Surv(cryr, relapse.death) ~ test_p_group, data = amltest)
```

```{r plottest, echo = FALSE, fig = TRUE}
plot(km_cured_test, mark.time = TRUE, lty = c(1, 2), xlab = "Time (years)", ylab = "Relapse-free survival")
legend(c(.4, .1), legend = c("Cured", "Susceptible"), lty = c(1, 2), bty = "n")
```

```{r}
km_suscept_test <- survfit(Surv(cryr, relapse.death) ~ test_predict$latency_risk, 
                          data = amltest, subset = (test_p_group == "Susceptible"))
```

```{r, echo = FALSE, fig = TRUE}
plot(km_suscept_test, mark.time = TRUE, lty = c(1, 2), xlab = "Time (years)", ylab = "Relapse-free survival")
legend(c(.4, .1), legend = c("Higher risk", "Lower risk"), lty = c(1, 2), bty = "n")
```

The `hdcuremodels` package also includes two functions for assessing the performance of MCMs. The ability of the MCM to discriminate between those cured ($Y_i=0$) versus those susceptible ($Y_i=1$) can be assessed by calculating the mean score imputation area under the curve using the `auc_mcm` function (Asano et al, 2014). In a MCM, when $\delta_i=1$ we know that the subject experienced the event. However, when $\delta_i=0$ either the subject was cured or the subject would have experienced the event if followed longer than their censoring time. Therefore, for a `cure_cutoff` $\tau$ (default is 5) the outcome $Y_i$ is defined as
$$
Y_i = \begin{cases}
0 \text{ if } T_i >\tau\\
1 \text{ if } T_i \le\tau \text{ and } \delta_i=1\\
\text{missing} \text{ if } T_i \le\tau \text{ and } \delta_i=0\\
\end{cases}.
$$
The mean score imputation AUC lets $Y_i = 1-\hat{p}(\mathbf{x}_i)$ for those subjects with a missing outcome. The C-statistic for MCMs was adapted to weight patients by their outcome (cured, susceptible, censored) and is available in the `concordance_mcm` function (Asano & Hirakawa, 2017). In both functions, if `newdata` is not specified, the training data are used.

```{r}
auc_mcm(fitem, model_select = "cAIC")
auc_mcm(fitem, newdata = amltest, model_select = "cAIC")
concordance_mcm(fitem, model_select = "cAIC")
concordance_mcm(fitem, newdata = amltest, model_select = "cAIC")
```

Other generic functions return the number of observations for the training data `nobs`, the log-likehood at the selected step `logLik`, the family of the mixture cure model `family`, and the model formula and data used for training the incidence and latency components `formula`.

# Comparison to other mixture cure modeling packages

Other R packages that can be used for fitting MCMs include: 

* `cuRe` (Jakobsen, 2023) can be used to fit parametric MCMs on a relative survival scale;
* `CureDepCens` (Schneider and Grandemagne dos Santos, 2023) can be used to fit piecewise exponential or Weibull model with dependent censoring;
* `curephEM` (Hou and Ren, 2024) can be used to fit a MCM where the latency is modeled using a Cox PH model;
* `flexsurvcure` (Amdahl, 2022) can be used to fit parametric mixture and non-mixture cure models;
* `geecure` (Niu and Peng, 2018) can be used to fit marginal MCM for clustered survival data;
* `GORCure` (Zhou et al, 2017) can be used to fit generalized odds rate MCM with interval censored data;
* `mixcure` (Peng, 2020) can be used to fit non-parametric, parametric, and semiparametric MCMs;
* `npcure` (López-de-Ullibarri and López-Cheda, 2020) can be used to non-parametrically estimate incidence and latency;
* `npcurePK` (Safari et al, 2023) can be used to non-parametrically estimate incidence and latency when cure is partially observed;
* `penPHcure`(Beretta and Heuchenne, 2019) can be used to fit semi-parametric PH MCMs with time-varying covariates; and
* `smcure` (Cai et al 2022) can be used to fit semi-parametric (PH and AFT) MCMs.

None of these packages are capable of handling high-dimensional datasets. Only `penPHcure` includes LASSO penalty to perform variable selection for scenarios when the sample size exceeds the number of predictors.

# Conclusions

Our `hdcuremodels` R package can be used to model a censored time-to-event outcome when a cured fraction is present, and because penalized models are fit, our `hdcuremodels` package can accommodate datasets where the number of predictors exceeds the sample size. The user can fit a model using one of two different optimization methods (E-M or GMIFS) and can choose to perform cross-valiation with or without FDR control. The modeling functions are flexible in that there is no requirement for the predictors to be the same in the incidence and latency components of the model. The package also includes functions for testing mixture cure modeling assumptions. Generic functions for resulting `mixturecure` objects include `print`, `summary`, `coef`, `plot`, and `predict` can be used to extract meaningful results from the fitted model. Additionally, `auc_mcm` and `concordance_mcm` were specifically tailored to provide model performance statistics of the fitted MCM. Finally, our previous paper demonstrated that our GMIFS and E-M algorithms outperformed existing methods with respect to both variable selection and prediction (Fu et al, 2022).

# References

1. Amdahl, J. (2022) _flexsurvcure: Flexible Parametric Cure Models_. R package version 1.3.1, <https://CRAN.R-project.org/package=flexsurvcure>.
2. Archer, K.J.; Fu, H.; Mrozek, K.; Nicolet, D.; Mims, A.S.; Uy, G.L.; Stock, W.; Byrd, J.C.; Hiddenmann, W.; Braess, J.; Spiekermann, K.; Metzeler, K.H.; Herold, T.; Eisfeld, A.K. Identifying long-term survivors and those at higher or lower risk of relapse among patients with cytogenetically normal acute myeloid leukemia using a high-dimensional mixture cure model. *Journal of Hematology and Oncology* 2024, 17(1), 28.
3. Asano, J.; Hirakawa, A.; Hamada, C. Assessing the prediction accuracy of cure in the Cox proportional hazards cure model: An application to breast cancer data. *Pharmaceutical Statistics* 2014, 13, 357–363.
4. Asano, J.; Hirakawa, A. Assessing the prediction accuracy of a cure model for censored survival daa with long-term survivors: Application to breast cancer data. *Journal of Biopharmaceutical Statistics* 2017, 27(6), 918--932.
5. Bamopoulos, S.A.; Batcha, A.M.N.; Jurinovic, V.; Rothenberg-Thurley, M.; Janke, H.; Ksienzyk, B.; et al. Clinical presentation and differential splicing of SRSF2, U2AF1, and SF3B1 mutations in patients with acute myeloid leukemia. *Leukemia* 2020, 34, 2621--34.
6. Beretta, A.; Heuchenne, C. (2019) _penPHcure: Variable Selection in PH Cure Model with Time-Varying Covariates_. R package version 1.0.2, <https://CRAN.R-project.org/package=penPHcure>.
7. Candes, E.; Fan, Y.; Janson, L.; Lv, J. Panning for gold: ‘model-X’ knockoffs for high dimensional controlled variable selection.
*Journal of the Royal Statistical Society Series B Stat Methodology* 2018, 80(3), 551--577.
8. Cai, C.; Zou, Y.; Peng, Y.; Zhang, J. (2022). _smcure: Fit Semiparametric Mixture Cure Models_. R package version 2.1, <https://CRAN.R-project.org/package=smcure>.
9. Fu, H.; Nicolet, D.; Mrozek, K.; Stone, R.M.; Eisfeld, A.K.; Byrd, J.C.; Archer, K.J. Contolled variable selection in Weibull mixture cure models for high-dimensional data. *Statistics in Medicine* 2022, 41(22), 4340--4366.
10. Goldman, A. The cure model and time confounded risk in the analysis of survival and other timed events. *Journal of Clinical Epidemiology* 1991, 44(12), 1327--1340.
11. Hou, J.; Ren, E. (2024) _curephEM: NPMLE for Logistic-Cox Cure-Rate Model_. R package version 0.3.0, <https://CRAN.R-project.org/package=curephEM>.
10. Jakobsen, L.H. (2023) _cuRe: Parametric Cure Model Estimation_. R package version 1.1.1, <https://CRAN.R-project.org/package=cuRe>.
12. López-de-Ullibarri I, López-Cheda A, Jácome M (2020). _npcure: Nonparametric Estimation in Mixture Cure Models_. R package version 0.1-5, <https://CRAN.R-project.org/package=npcure>.
13. Maller, R.A.; Zhou, X. *Survival Analysis with Long-Term Survivors* John Wiley & Sons, 1996.
14. Niu, Y.; Peng, Y. (2018) _geecure: Marginal Proportional Hazards Mixture Cure Models with Generalized Estimating Equations_. R package version 1.0-6, <https://CRAN.R-project.org/package=geecure>.
15. Peng, Y. (2020) _mixcure: Mixture Cure Models_. R package version 2.0,
  <https://CRAN.R-project.org/package=mixcure>.
16. Safari, W.; López-de-Ullibarri, I.; Jácome, M. (2023) _npcurePK: Mixture Cure Model Estimation with Cure Status Partially Known_. R package version 1.0-2, <https://CRAN.R-project.org/package=npcurePK>.
17. Schneider, S.; Grandemagne dos Santos, G. (2023) CureDepCens: _Dependent Censoring Regression Models with Cure Fraction_. R package version 0.1.0, <https://CRAN.R-project.org/package=CureDepCens>.
18. Zhou, J.; Zhang, J.; Lu, W. (2017) _GORCure: Fit Generalized Odds Rate Mixture Cure Model with Interval Censored Data_. R package version 2.0, <https://CRAN.R-project.org/package=GORCure>.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{pediatric_flt3}
\alias{pediatric_flt3}
\title{Pediatric acute myeloid leukemia patients with FLT3-ITD rearrangement data}
\format{
A data frame with 246 rows (subjects) and 202 columns:
\describe{
  \item{efs}{event-free survival}
  \item{status}{censoring indicator: 1 = event; 0 = censored}
  \item{ENSG00000229807.12}{normalized expression for indicated transcript}
  \item{ENSG00000211459.2}{normalized expression for indicated transcript}
  \item{ENSG00000264063.1}{normalized expression for indicated transcript}
  \item{ENSG00000264462.1}{normalized expression for indicated transcript}
  \item{ENSG00000227195.11}{normalized expression for indicated transcript}
  \item{ENSG00000198795.11}{normalized expression for indicated transcript}
  \item{ENSG00000177133.11}{normalized expression for indicated transcript}
  \item{ENSG00000257883.1}{normalized expression for indicated transcript}
  \item{ENSG00000189223.15}{normalized expression for indicated transcript}
  \item{ENSG00000210082.2}{normalized expression for indicated transcript}
  \item{ENSG00000142611.17}{normalized expression for indicated transcript}
  \item{ENSG00000054598.9}{normalized expression for indicated transcript}
  \item{ENSG00000147180.17}{normalized expression for indicated transcript}
  \item{ENSG00000275708.1}{normalized expression for indicated transcript}
  \item{ENSG00000227063.5}{normalized expression for indicated transcript}
  \item{ENSG00000179455.10}{normalized expression for indicated transcript}
  \item{ENSG00000243478.9}{normalized expression for indicated transcript}
  \item{ENSG00000239839.7}{normalized expression for indicated transcript}
  \item{ENSG00000274253.5}{normalized expression for indicated transcript}
  \item{ENSG00000198840.2}{normalized expression for indicated transcript}
  \item{ENSG00000186407.7}{normalized expression for indicated transcript}
  \item{ENSG00000198763.3}{normalized expression for indicated transcript}
  \item{ENSG00000107562.16}{normalized expression for indicated transcript}
  \item{ENSG00000188536.13}{normalized expression for indicated transcript}
  \item{ENSG00000164821.5}{normalized expression for indicated transcript}
  \item{ENSG00000255833.2}{normalized expression for indicated transcript}
  \item{ENSG00000107447.8}{normalized expression for indicated transcript}
  \item{ENSG00000237541.4}{normalized expression for indicated transcript}
  \item{ENSG00000274618.2}{normalized expression for indicated transcript}
  \item{ENSG00000206172.8}{normalized expression for indicated transcript}
  \item{ENSG00000198886.2}{normalized expression for indicated transcript}
  \item{ENSG00000122592.8}{normalized expression for indicated transcript}
  \item{ENSG00000038427.16}{normalized expression for indicated transcript}
  \item{ENSG00000134755.17}{normalized expression for indicated transcript}
  \item{ENSG00000248527.1}{normalized expression for indicated transcript}
  \item{ENSG00000273018.7}{normalized expression for indicated transcript}
  \item{ENSG00000261377.5}{normalized expression for indicated transcript}
  \item{ENSG00000074964.17}{normalized expression for indicated transcript}
  \item{ENSG00000118113.12}{normalized expression for indicated transcript}
  \item{ENSG00000086548.9}{normalized expression for indicated transcript}
  \item{ENSG00000172037.14}{normalized expression for indicated transcript}
  \item{ENSG00000198744.5}{normalized expression for indicated transcript}
  \item{ENSG00000198899.2}{normalized expression for indicated transcript}
  \item{ENSG00000186818.12}{normalized expression for indicated transcript}
  \item{ENSG00000154864.13}{normalized expression for indicated transcript}
  \item{ENSG00000227081.5}{normalized expression for indicated transcript}
  \item{ENSG00000101425.14}{normalized expression for indicated transcript}
  \item{ENSG00000158473.8}{normalized expression for indicated transcript}
  \item{ENSG00000102362.15}{normalized expression for indicated transcript}
  \item{ENSG00000212907.2}{normalized expression for indicated transcript}
  \item{ENSG00000121807.6}{normalized expression for indicated transcript}
  \item{ENSG00000138829.12}{normalized expression for indicated transcript}
  \item{ENSG00000185052.12}{normalized expression for indicated transcript}
  \item{ENSG00000111057.11}{normalized expression for indicated transcript}
  \item{ENSG00000228253.1}{normalized expression for indicated transcript}
  \item{ENSG00000188153.14}{normalized expression for indicated transcript}
  \item{ENSG00000166681.14}{normalized expression for indicated transcript}
  \item{ENSG00000198888.2}{normalized expression for indicated transcript}
  \item{ENSG00000276141.4}{normalized expression for indicated transcript}
  \item{ENSG00000230076.1}{normalized expression for indicated transcript}
  \item{ENSG00000260314.3}{normalized expression for indicated transcript}
  \item{ENSG00000187164.20}{normalized expression for indicated transcript}
  \item{ENSG00000212443.1}{normalized expression for indicated transcript}
  \item{ENSG00000251692.8}{normalized expression for indicated transcript}
  \item{ENSG00000198727.2}{normalized expression for indicated transcript}
  \item{ENSG00000131401.11}{normalized expression for indicated transcript}
  \item{ENSG00000225345.3}{normalized expression for indicated transcript}
  \item{ENSG00000197253.13}{normalized expression for indicated transcript}
  \item{ENSG00000163737.4}{normalized expression for indicated transcript}
  \item{ENSG00000204010.3}{normalized expression for indicated transcript}
  \item{ENSG00000174403.16}{normalized expression for indicated transcript}
  \item{ENSG00000249790.3}{normalized expression for indicated transcript}
  \item{ENSG00000239559.2}{normalized expression for indicated transcript}
  \item{ENSG00000142192.21}{normalized expression for indicated transcript}
  \item{ENSG00000151650.8}{normalized expression for indicated transcript}
  \item{ENSG00000198712.1}{normalized expression for indicated transcript}
  \item{ENSG00000124469.12}{normalized expression for indicated transcript}
  \item{ENSG00000105854.13}{normalized expression for indicated transcript}
  \item{ENSG00000197629.6}{normalized expression for indicated transcript}
  \item{ENSG00000278588.2}{normalized expression for indicated transcript}
  \item{ENSG00000196565.15}{normalized expression for indicated transcript}
  \item{ENSG00000241743.4}{normalized expression for indicated transcript}
  \item{ENSG00000174059.17}{normalized expression for indicated transcript}
  \item{ENSG00000150760.13}{normalized expression for indicated transcript}
  \item{ENSG00000133742.14}{normalized expression for indicated transcript}
  \item{ENSG00000210196.2}{normalized expression for indicated transcript}
  \item{ENSG00000204860.5}{normalized expression for indicated transcript}
  \item{ENSG00000131398.15}{normalized expression for indicated transcript}
  \item{ENSG00000106080.11}{normalized expression for indicated transcript}
  \item{ENSG00000198786.2}{normalized expression for indicated transcript}
  \item{ENSG00000248334.6}{normalized expression for indicated transcript}
  \item{ENSG00000169908.12}{normalized expression for indicated transcript}
  \item{ENSG00000171509.16}{normalized expression for indicated transcript}
  \item{ENSG00000008438.5}{normalized expression for indicated transcript}
  \item{ENSG00000130147.16}{normalized expression for indicated transcript}
  \item{ENSG00000180071.20}{normalized expression for indicated transcript}
  \item{ENSG00000168329.14}{normalized expression for indicated transcript}
  \item{ENSG00000147689.16}{normalized expression for indicated transcript}
  \item{ENSG00000136193.17}{normalized expression for indicated transcript}
  \item{ENSG00000170180.22}{normalized expression for indicated transcript}
  \item{ENSG00000146373.16}{normalized expression for indicated transcript}
  \item{ENSG00000172236.18}{normalized expression for indicated transcript}
  \item{ENSG00000244734.4}{normalized expression for indicated transcript}
  \item{ENSG00000239552.2}{normalized expression for indicated transcript}
  \item{ENSG00000125618.17}{normalized expression for indicated transcript}
  \item{ENSG00000273983.1}{normalized expression for indicated transcript}
  \item{ENSG00000250361.9}{normalized expression for indicated transcript}
  \item{ENSG00000277775.2}{normalized expression for indicated transcript}
  \item{ENSG00000138119.17}{normalized expression for indicated transcript}
  \item{ENSG00000128422.17}{normalized expression for indicated transcript}
  \item{ENSG00000160883.11}{normalized expression for indicated transcript}
  \item{ENSG00000275895.7}{normalized expression for indicated transcript}
  \item{ENSG00000169877.10}{normalized expression for indicated transcript}
  \item{ENSG00000163736.4}{normalized expression for indicated transcript}
  \item{ENSG00000161944.16}{normalized expression for indicated transcript}
  \item{ENSG00000095917.14}{normalized expression for indicated transcript}
  \item{ENSG00000022556.16}{normalized expression for indicated transcript}
  \item{ENSG00000152078.10}{normalized expression for indicated transcript}
  \item{ENSG00000165092.13}{normalized expression for indicated transcript}
  \item{ENSG00000170458.14}{normalized expression for indicated transcript}
  \item{ENSG00000070182.21}{normalized expression for indicated transcript}
  \item{ENSG00000157445.15}{normalized expression for indicated transcript}
  \item{ENSG00000206737.1}{normalized expression for indicated transcript}
  \item{ENSG00000225217.1}{normalized expression for indicated transcript}
  \item{ENSG00000198336.9}{normalized expression for indicated transcript}
  \item{ENSG00000177469.13}{normalized expression for indicated transcript}
  \item{ENSG00000198695.2}{normalized expression for indicated transcript}
  \item{ENSG00000113763.12}{normalized expression for indicated transcript}
  \item{ENSG00000223609.11}{normalized expression for indicated transcript}
  \item{ENSG00000206177.7}{normalized expression for indicated transcript}
  \item{ENSG00000164684.13}{normalized expression for indicated transcript}
  \item{ENSG00000197632.9}{normalized expression for indicated transcript}
  \item{ENSG00000177575.13}{normalized expression for indicated transcript}
  \item{ENSG00000004939.15}{normalized expression for indicated transcript}
  \item{ENSG00000121053.6}{normalized expression for indicated transcript}
  \item{ENSG00000259207.9}{normalized expression for indicated transcript}
  \item{ENSG00000158352.16}{normalized expression for indicated transcript}
  \item{ENSG00000198046.12}{normalized expression for indicated transcript}
  \item{ENSG00000225630.1}{normalized expression for indicated transcript}
  \item{ENSG00000070808.16}{normalized expression for indicated transcript}
  \item{ENSG00000012223.13}{normalized expression for indicated transcript}
  \item{ENSG00000085265.11}{normalized expression for indicated transcript}
  \item{ENSG00000158578.21}{normalized expression for indicated transcript}
  \item{ENSG00000112077.17}{normalized expression for indicated transcript}
  \item{ENSG00000272398.6}{normalized expression for indicated transcript}
  \item{ENSG00000116667.15}{normalized expression for indicated transcript}
  \item{ENSG00000101916.12}{normalized expression for indicated transcript}
  \item{ENSG00000184293.7}{normalized expression for indicated transcript}
  \item{ENSG00000244040.7}{normalized expression for indicated transcript}
  \item{ENSG00000229344.1}{normalized expression for indicated transcript}
  \item{ENSG00000142405.22}{normalized expression for indicated transcript}
  \item{ENSG00000171051.9}{normalized expression for indicated transcript}
  \item{ENSG00000183023.18}{normalized expression for indicated transcript}
  \item{ENSG00000257178.5}{normalized expression for indicated transcript}
  \item{ENSG00000244682.7}{normalized expression for indicated transcript}
  \item{ENSG00000166025.18}{normalized expression for indicated transcript}
  \item{ENSG00000204613.11}{normalized expression for indicated transcript}
  \item{ENSG00000214188.9}{normalized expression for indicated transcript}
  \item{ENSG00000179869.15}{normalized expression for indicated transcript}
  \item{ENSG00000136315.4}{normalized expression for indicated transcript}
  \item{ENSG00000119919.11}{normalized expression for indicated transcript}
  \item{ENSG00000123384.14}{normalized expression for indicated transcript}
 \item{ENSG00000104918.8}{normalized expression for indicated transcript}
  \item{ENSG00000287431.1}{normalized expression for indicated transcript}
  \item{ENSG00000227827.3}{normalized expression for indicated transcript}
  \item{ENSG00000164047.6}{normalized expression for indicated transcript}
  \item{ENSG00000137571.11}{normalized expression for indicated transcript}
  \item{ENSG00000143416.21}{normalized expression for indicated transcript}
  \item{ENSG00000253578.1}{normalized expression for indicated transcript}
  \item{ENSG00000120708.17}{normalized expression for indicated transcript}
  \item{ENSG00000220842.6}{normalized expression for indicated transcript}
  \item{ENSG00000110077.14}{normalized expression for indicated transcript}
  \item{ENSG00000198722.14}{normalized expression for indicated transcript}
  \item{ENSG00000121316.11}{normalized expression for indicated transcript}
  \item{ENSG00000123689.6}{normalized expression for indicated transcript}
  \item{ENSG00000198938.2}{normalized expression for indicated transcript}
  \item{ENSG00000163430.12}{normalized expression for indicated transcript}
  \item{ENSG00000258227.7}{normalized expression for indicated transcript}
  \item{ENSG00000233101.10}{normalized expression for indicated transcript}
  \item{ENSG00000163220.11}{normalized expression for indicated transcript}
  \item{ENSG00000100985.7}{normalized expression for indicated transcript}
  \item{ENSG00000166947.15}{normalized expression for indicated transcript}
  \item{ENSG00000275713.2}{normalized expression for indicated transcript}
  \item{ENSG00000143546.10}{normalized expression for indicated transcript}
  \item{ENSG00000136929.13}{normalized expression for indicated transcript}
  \item{ENSG00000103313.13}{normalized expression for indicated transcript}
  \item{ENSG00000186529.16}{normalized expression for indicated transcript}
  \item{ENSG00000109321.11}{normalized expression for indicated transcript}
  \item{ENSG00000104177.18}{normalized expression for indicated transcript}
  \item{ENSG00000257335.8}{normalized expression for indicated transcript}
  \item{ENSG00000197993.9}{normalized expression for indicated transcript}
  \item{ENSG00000134460.18}{normalized expression for indicated transcript}
  \item{ENSG00000211892.4}{normalized expression for indicated transcript}
  \item{ENSG00000180044.5}{normalized expression for indicated transcript}
  \item{ENSG00000268555.2}{normalized expression for indicated transcript}
  \item{ENSG00000266402.3}{normalized expression for indicated transcript}
  \item{ENSG00000163221.9}{normalized expression for indicated transcript}
  \item{ENSG00000225101.6}{normalized expression for indicated transcript}
  \item{ENSG00000197249.14}{normalized expression for indicated transcript}
  \item{ENSG00000093134.15}{normalized expression for indicated transcript}
  }
}
\source{
<https://www.cancer.gov/ccg/research/genome-sequencing/target/studied-cancers/acute-myeloid-leukemia#targets-aml-project>
}
\usage{
pediatric_flt3
}
\description{
Event-free survival for 246 pediatric acute myeloid leukemia patients having
a FLT3-ITD rearrangement along with expression of 200 transcripts from
RNA-sequencing.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{amltest}
\alias{amltest}
\title{AML test data}
\format{
A data frame with 40 rows (subjects) and 322 columns:
\describe{
  \item{cryr}{duration of complete response in years}
  \item{relapse.death}{censoring indicator: 1 = relapsed or died;
  0 = alive at last follow=up}
  \item{ENSG00000001561}{normalized expression for indicated transcript}
  \item{ENSG00000005249}{normalized expression for indicated transcript}
  \item{ENSG00000006757}{normalized expression for indicated transcript}
  \item{ENSG00000007062}{normalized expression for indicated transcript}
  \item{ENSG00000007968}{normalized expression for indicated transcript}
  \item{ENSG00000008283}{normalized expression for indicated transcript}
  \item{ENSG00000008405}{normalized expression for indicated transcript}
  \item{ENSG00000008441}{normalized expression for indicated transcript}
  \item{ENSG00000010295}{normalized expression for indicated transcript}
  \item{ENSG00000011028}{normalized expression for indicated transcript}
  \item{ENSG00000011198}{normalized expression for indicated transcript}
  \item{ENSG00000012779}{normalized expression for indicated transcript}
  \item{ENSG00000012817}{normalized expression for indicated transcript}
  \item{ENSG00000013306}{normalized expression for indicated transcript}
  \item{ENSG00000013725}{normalized expression for indicated transcript}
  \item{ENSG00000018189}{normalized expression for indicated transcript}
  \item{ENSG00000022267}{normalized expression for indicated transcript}
  \item{ENSG00000023171}{normalized expression for indicated transcript}
  \item{ENSG00000023909}{normalized expression for indicated transcript}
  \item{ENSG00000029639}{normalized expression for indicated transcript}
  \item{ENSG00000047634}{normalized expression for indicated transcript}
  \item{ENSG00000049192}{normalized expression for indicated transcript}
  \item{ENSG00000053524}{normalized expression for indicated transcript}
  \item{ENSG00000058056}{normalized expression for indicated transcript}
  \item{ENSG00000060138}{normalized expression for indicated transcript}
  \item{ENSG00000061918}{normalized expression for indicated transcript}
  \item{ENSG00000065809}{normalized expression for indicated transcript}
  \item{ENSG00000065923}{normalized expression for indicated transcript}
  \item{ENSG00000068489}{normalized expression for indicated transcript}
  \item{ENSG00000069020}{normalized expression for indicated transcript}
  \item{ENSG00000070404}{normalized expression for indicated transcript}
  \item{ENSG00000071894}{normalized expression for indicated transcript}
  \item{ENSG00000072422}{normalized expression for indicated transcript}
  \item{ENSG00000073605}{normalized expression for indicated transcript}
  \item{ENSG00000076555}{normalized expression for indicated transcript}
  \item{ENSG00000080823}{normalized expression for indicated transcript}
  \item{ENSG00000089723}{normalized expression for indicated transcript}
  \item{ENSG00000090382}{normalized expression for indicated transcript}
  \item{ENSG00000090975}{normalized expression for indicated transcript}
  \item{ENSG00000100068}{normalized expression for indicated transcript}
  \item{ENSG00000100077}{normalized expression for indicated transcript}
  \item{ENSG00000100299}{normalized expression for indicated transcript}
  \item{ENSG00000100376}{normalized expression for indicated transcript}
  \item{ENSG00000100418}{normalized expression for indicated transcript}
  \item{ENSG00000100448}{normalized expression for indicated transcript}
  \item{ENSG00000100596}{normalized expression for indicated transcript}
  \item{ENSG00000100916}{normalized expression for indicated transcript}
  \item{ENSG00000102409}{normalized expression for indicated transcript}
  \item{ENSG00000102760}{normalized expression for indicated transcript}
  \item{ENSG00000104689}{normalized expression for indicated transcript}
  \item{ENSG00000104946}{normalized expression for indicated transcript}
  \item{ENSG00000105518}{normalized expression for indicated transcript}
  \item{ENSG00000105808}{normalized expression for indicated transcript}
  \item{ENSG00000106367}{normalized expression for indicated transcript}
  \item{ENSG00000106526}{normalized expression for indicated transcript}
  \item{ENSG00000106546}{normalized expression for indicated transcript}
  \item{ENSG00000106780}{normalized expression for indicated transcript}
  \item{ENSG00000107104}{normalized expression for indicated transcript}
  \item{ENSG00000107742}{normalized expression for indicated transcript}
  \item{ENSG00000107798}{normalized expression for indicated transcript}
  \item{ENSG00000107816}{normalized expression for indicated transcript}
  \item{ENSG00000107957}{normalized expression for indicated transcript}
  \item{ENSG00000109674}{normalized expression for indicated transcript}
  \item{ENSG00000110076}{normalized expression for indicated transcript}
  \item{ENSG00000110237}{normalized expression for indicated transcript}
  \item{ENSG00000110492}{normalized expression for indicated transcript}
  \item{ENSG00000110799}{normalized expression for indicated transcript}
  \item{ENSG00000111275}{normalized expression for indicated transcript}
  \item{ENSG00000112773}{normalized expression for indicated transcript}
  \item{ENSG00000113504}{normalized expression for indicated transcript}
  \item{ENSG00000114268}{normalized expression for indicated transcript}
  \item{ENSG00000114737}{normalized expression for indicated transcript}
  \item{ENSG00000115183}{normalized expression for indicated transcript}
  \item{ENSG00000115414}{normalized expression for indicated transcript}
  \item{ENSG00000115457}{normalized expression for indicated transcript}
  \item{ENSG00000115525}{normalized expression for indicated transcript}
  \item{ENSG00000116574}{normalized expression for indicated transcript}
  \item{ENSG00000117480}{normalized expression for indicated transcript}
  \item{ENSG00000119280}{normalized expression for indicated transcript}
  \item{ENSG00000120594}{normalized expression for indicated transcript}
  \item{ENSG00000120675}{normalized expression for indicated transcript}
  \item{ENSG00000120832}{normalized expression for indicated transcript}
  \item{ENSG00000120913}{normalized expression for indicated transcript}
  \item{ENSG00000121005}{normalized expression for indicated transcript}
  \item{ENSG00000121039}{normalized expression for indicated transcript}
  \item{ENSG00000121274}{normalized expression for indicated transcript}
  \item{ENSG00000123080}{normalized expression for indicated transcript}
  \item{ENSG00000123836}{normalized expression for indicated transcript}
  \item{ENSG00000124019}{normalized expression for indicated transcript}
  \item{ENSG00000124882}{normalized expression for indicated transcript}
  \item{ENSG00000126822}{normalized expression for indicated transcript}
  \item{ENSG00000127152}{normalized expression for indicated transcript}
  \item{ENSG00000129824}{normalized expression for indicated transcript}
  \item{ENSG00000130702}{normalized expression for indicated transcript}
  \item{ENSG00000131188}{normalized expression for indicated transcript}
  \item{ENSG00000131370}{normalized expression for indicated transcript}
  \item{ENSG00000132122}{normalized expression for indicated transcript}
  \item{ENSG00000132530}{normalized expression for indicated transcript}
  \item{ENSG00000132819}{normalized expression for indicated transcript}
  \item{ENSG00000132849}{normalized expression for indicated transcript}
  \item{ENSG00000133401}{normalized expression for indicated transcript}
  \item{ENSG00000133619}{normalized expression for indicated transcript}
  \item{ENSG00000134531}{normalized expression for indicated transcript}
  \item{ENSG00000134897}{normalized expression for indicated transcript}
  \item{ENSG00000135074}{normalized expression for indicated transcript}
  \item{ENSG00000135245}{normalized expression for indicated transcript}
  \item{ENSG00000135272}{normalized expression for indicated transcript}
  \item{ENSG00000135362}{normalized expression for indicated transcript}
  \item{ENSG00000135363}{normalized expression for indicated transcript}
  \item{ENSG00000135916}{normalized expression for indicated transcript}
  \item{ENSG00000136026}{normalized expression for indicated transcript}
  \item{ENSG00000136193}{normalized expression for indicated transcript}
  \item{ENSG00000136231}{normalized expression for indicated transcript}
  \item{ENSG00000136997}{normalized expression for indicated transcript}
  \item{ENSG00000137193}{normalized expression for indicated transcript}
  \item{ENSG00000137198}{normalized expression for indicated transcript}
  \item{ENSG00000138722}{normalized expression for indicated transcript}
  \item{ENSG00000139318}{normalized expression for indicated transcript}
  \item{ENSG00000140287}{normalized expression for indicated transcript}
  \item{ENSG00000144036}{normalized expression for indicated transcript}
  \item{ENSG00000144647}{normalized expression for indicated transcript}
  \item{ENSG00000144677}{normalized expression for indicated transcript}
  \item{ENSG00000145476}{normalized expression for indicated transcript}
  \item{ENSG00000145545}{normalized expression for indicated transcript}
  \item{ENSG00000146243}{normalized expression for indicated transcript}
  \item{ENSG00000146373}{normalized expression for indicated transcript}
  \item{ENSG00000147044}{normalized expression for indicated transcript}
  \item{ENSG00000147180}{normalized expression for indicated transcript}
  \item{ENSG00000148444}{normalized expression for indicated transcript}
  \item{ENSG00000148484}{normalized expression for indicated transcript}
  \item{ENSG00000149131}{normalized expression for indicated transcript}
  \item{ENSG00000150760}{normalized expression for indicated transcript}
  \item{ENSG00000150782}{normalized expression for indicated transcript}
  \item{ENSG00000151135}{normalized expression for indicated transcript}
  \item{ENSG00000151208}{normalized expression for indicated transcript}
  \item{ENSG00000151458}{normalized expression for indicated transcript}
  \item{ENSG00000152409}{normalized expression for indicated transcript}
  \item{ENSG00000152580}{normalized expression for indicated transcript}
  \item{ENSG00000152767}{normalized expression for indicated transcript}
  \item{ENSG00000152778}{normalized expression for indicated transcript}
  \item{ENSG00000153563}{normalized expression for indicated transcript}
  \item{ENSG00000154217}{normalized expression for indicated transcript}
  \item{ENSG00000154743}{normalized expression for indicated transcript}
  \item{ENSG00000154760}{normalized expression for indicated transcript}
  \item{ENSG00000154874}{normalized expression for indicated transcript}
  \item{ENSG00000156381}{normalized expression for indicated transcript}
  \item{ENSG00000157107}{normalized expression for indicated transcript}
  \item{ENSG00000157240}{normalized expression for indicated transcript}
  \item{ENSG00000157873}{normalized expression for indicated transcript}
  \item{ENSG00000157978}{normalized expression for indicated transcript}
  \item{ENSG00000158691}{normalized expression for indicated transcript}
  \item{ENSG00000159339}{normalized expression for indicated transcript}
  \item{ENSG00000159403}{normalized expression for indicated transcript}
  \item{ENSG00000159788}{normalized expression for indicated transcript}
  \item{ENSG00000160685}{normalized expression for indicated transcript}
  \item{ENSG00000160781}{normalized expression for indicated transcript}
  \item{ENSG00000161509}{normalized expression for indicated transcript}
  \item{ENSG00000162433}{normalized expression for indicated transcript}
  \item{ENSG00000162614}{normalized expression for indicated transcript}
  \item{ENSG00000162676}{normalized expression for indicated transcript}
  \item{ENSG00000163412}{normalized expression for indicated transcript}
  \item{ENSG00000163702}{normalized expression for indicated transcript}
  \item{ENSG00000163814}{normalized expression for indicated transcript}
  \item{ENSG00000164086}{normalized expression for indicated transcript}
  \item{ENSG00000164172}{normalized expression for indicated transcript}
  \item{ENSG00000164442}{normalized expression for indicated transcript}
  \item{ENSG00000165272}{normalized expression for indicated transcript}
  \item{ENSG00000166165}{normalized expression for indicated transcript}
  \item{ENSG00000166435}{normalized expression for indicated transcript}
  \item{ENSG00000166987}{normalized expression for indicated transcript}
  \item{ENSG00000167291}{normalized expression for indicated transcript}
  \item{ENSG00000167565}{normalized expression for indicated transcript}
  \item{ENSG00000167851}{normalized expression for indicated transcript}
  \item{ENSG00000168026}{normalized expression for indicated transcript}
  \item{ENSG00000168209}{normalized expression for indicated transcript}
  \item{ENSG00000168502}{normalized expression for indicated transcript}
  \item{ENSG00000168939}{normalized expression for indicated transcript}
  \item{ENSG00000169203}{normalized expression for indicated transcript}
  \item{ENSG00000169247}{normalized expression for indicated transcript}
  \item{ENSG00000169504}{normalized expression for indicated transcript}
  \item{ENSG00000169860}{normalized expression for indicated transcript}
  \item{ENSG00000169991}{normalized expression for indicated transcript}
  \item{ENSG00000170035}{normalized expression for indicated transcript}
  \item{ENSG00000170180}{normalized expression for indicated transcript}
  \item{ENSG00000170456}{normalized expression for indicated transcript}
  \item{ENSG00000170522}{normalized expression for indicated transcript}
  \item{ENSG00000170909}{normalized expression for indicated transcript}
  \item{ENSG00000171121}{normalized expression for indicated transcript}
  \item{ENSG00000171222}{normalized expression for indicated transcript}
  \item{ENSG00000171476}{normalized expression for indicated transcript}
  \item{ENSG00000171813}{normalized expression for indicated transcript}
  \item{ENSG00000171962}{normalized expression for indicated transcript}
  \item{ENSG00000172197}{normalized expression for indicated transcript}
  \item{ENSG00000172236}{normalized expression for indicated transcript}
  \item{ENSG00000173083}{normalized expression for indicated transcript}
  \item{ENSG00000173530}{normalized expression for indicated transcript}
  \item{ENSG00000173926}{normalized expression for indicated transcript}
  \item{ENSG00000174059}{normalized expression for indicated transcript}
  \item{ENSG00000174080}{normalized expression for indicated transcript}
  \item{ENSG00000174130}{normalized expression for indicated transcript}
  \item{ENSG00000174738}{normalized expression for indicated transcript}
  \item{ENSG00000175265}{normalized expression for indicated transcript}
  \item{ENSG00000175352}{normalized expression for indicated transcript}
  \item{ENSG00000176597}{normalized expression for indicated transcript}
  \item{ENSG00000179222}{normalized expression for indicated transcript}
  \item{ENSG00000179630}{normalized expression for indicated transcript}
  \item{ENSG00000179639}{normalized expression for indicated transcript}
  \item{ENSG00000179820}{normalized expression for indicated transcript}
  \item{ENSG00000180096}{normalized expression for indicated transcript}
  \item{ENSG00000180596}{normalized expression for indicated transcript}
  \item{ENSG00000180902}{normalized expression for indicated transcript}
  \item{ENSG00000181104}{normalized expression for indicated transcript}
  \item{ENSG00000182866}{normalized expression for indicated transcript}
  \item{ENSG00000182871}{normalized expression for indicated transcript}
  \item{ENSG00000183087}{normalized expression for indicated transcript}
  \item{ENSG00000183091}{normalized expression for indicated transcript}
  \item{ENSG00000184371}{normalized expression for indicated transcript}
  \item{ENSG00000185129}{normalized expression for indicated transcript}
  \item{ENSG00000185201}{normalized expression for indicated transcript}
  \item{ENSG00000185245}{normalized expression for indicated transcript}
  \item{ENSG00000185291}{normalized expression for indicated transcript}
  \item{ENSG00000185304}{normalized expression for indicated transcript}
  \item{ENSG00000185710}{normalized expression for indicated transcript}
  \item{ENSG00000185883}{normalized expression for indicated transcript}
  \item{ENSG00000185986}{normalized expression for indicated transcript}
  \item{ENSG00000186130}{normalized expression for indicated transcript}
  \item{ENSG00000186854}{normalized expression for indicated transcript}
  \item{ENSG00000187010}{normalized expression for indicated transcript}
  \item{ENSG00000187627}{normalized expression for indicated transcript}
  \item{ENSG00000187653}{normalized expression for indicated transcript}
  \item{ENSG00000187837}{normalized expression for indicated transcript}
  \item{ENSG00000188002}{normalized expression for indicated transcript}
  \item{ENSG00000188107}{normalized expression for indicated transcript}
  \item{ENSG00000188211}{normalized expression for indicated transcript}
  \item{ENSG00000188636}{normalized expression for indicated transcript}
  \item{ENSG00000188738}{normalized expression for indicated transcript}
  \item{ENSG00000188856}{normalized expression for indicated transcript}
  \item{ENSG00000189164}{normalized expression for indicated transcript}
  \item{ENSG00000189223}{normalized expression for indicated transcript}
  \item{ENSG00000196155}{normalized expression for indicated transcript}
  \item{ENSG00000196189}{normalized expression for indicated transcript}
  \item{ENSG00000196415}{normalized expression for indicated transcript}
  \item{ENSG00000196565}{normalized expression for indicated transcript}
  \item{ENSG00000197081}{normalized expression for indicated transcript}
  \item{ENSG00000197121}{normalized expression for indicated transcript}
  \item{ENSG00000197253}{normalized expression for indicated transcript}
  \item{ENSG00000197256}{normalized expression for indicated transcript}
  \item{ENSG00000197321}{normalized expression for indicated transcript}
  \item{ENSG00000197561}{normalized expression for indicated transcript}
  \item{ENSG00000197728}{normalized expression for indicated transcript}
  \item{ENSG00000197860}{normalized expression for indicated transcript}
  \item{ENSG00000197937}{normalized expression for indicated transcript}
  \item{ENSG00000197951}{normalized expression for indicated transcript}
  \item{ENSG00000198743}{normalized expression for indicated transcript}
  \item{ENSG00000198838}{normalized expression for indicated transcript}
  \item{ENSG00000199347}{normalized expression for indicated transcript}
  \item{ENSG00000200243}{normalized expression for indicated transcript}
  \item{ENSG00000201801}{normalized expression for indicated transcript}
  \item{ENSG00000203872}{normalized expression for indicated transcript}
  \item{ENSG00000204172}{normalized expression for indicated transcript}
  \item{ENSG00000205571}{normalized expression for indicated transcript}
  \item{ENSG00000205593}{normalized expression for indicated transcript}
  \item{ENSG00000208772}{normalized expression for indicated transcript}
  \item{ENSG00000213085}{normalized expression for indicated transcript}
  \item{ENSG00000213261}{normalized expression for indicated transcript}
  \item{ENSG00000213626}{normalized expression for indicated transcript}
  \item{ENSG00000213722}{normalized expression for indicated transcript}
  \item{ENSG00000213906}{normalized expression for indicated transcript}
  \item{ENSG00000213967}{normalized expression for indicated transcript}
  \item{ENSG00000214016}{normalized expression for indicated transcript}
  \item{ENSG00000214425}{normalized expression for indicated transcript}
  \item{ENSG00000216316}{normalized expression for indicated transcript}
  \item{ENSG00000220008}{normalized expression for indicated transcript}
  \item{ENSG00000223345}{normalized expression for indicated transcript}
  \item{ENSG00000224080}{normalized expression for indicated transcript}
  \item{ENSG00000225138}{normalized expression for indicated transcript}
  \item{ENSG00000226471}{normalized expression for indicated transcript}
  \item{ENSG00000227097}{normalized expression for indicated transcript}
  \item{ENSG00000227191}{normalized expression for indicated transcript}
  \item{ENSG00000227615}{normalized expression for indicated transcript}
  \item{ENSG00000228049}{normalized expression for indicated transcript}
  \item{ENSG00000229153}{normalized expression for indicated transcript}
  \item{ENSG00000230076}{normalized expression for indicated transcript}
  \item{ENSG00000231160}{normalized expression for indicated transcript}
  \item{ENSG00000231721}{normalized expression for indicated transcript}
  \item{ENSG00000233927}{normalized expression for indicated transcript}
  \item{ENSG00000233974}{normalized expression for indicated transcript}
  \item{ENSG00000234883}{normalized expression for indicated transcript}
  \item{ENSG00000236876}{normalized expression for indicated transcript}
  \item{ENSG00000237298}{normalized expression for indicated transcript}
  \item{ENSG00000237892}{normalized expression for indicated transcript}
  \item{ENSG00000238160}{normalized expression for indicated transcript}
  \item{ENSG00000239437}{normalized expression for indicated transcript}
  \item{ENSG00000241399}{normalized expression for indicated transcript}
  \item{ENSG00000241489}{normalized expression for indicated transcript}
  \item{ENSG00000241529}{normalized expression for indicated transcript}
  \item{ENSG00000244405}{normalized expression for indicated transcript}
  \item{ENSG00000247627}{normalized expression for indicated transcript}
  \item{ENSG00000249592}{normalized expression for indicated transcript}
  \item{ENSG00000250116}{normalized expression for indicated transcript}
  \item{ENSG00000250251}{normalized expression for indicated transcript}
  \item{ENSG00000251079}{normalized expression for indicated transcript}
  \item{ENSG00000253210}{normalized expression for indicated transcript}
  \item{ENSG00000253276}{normalized expression for indicated transcript}
  \item{ENSG00000254415}{normalized expression for indicated transcript}
  \item{ENSG00000259276}{normalized expression for indicated transcript}
  \item{ENSG00000260727}{normalized expression for indicated transcript}
  \item{ENSG00000261377}{normalized expression for indicated transcript}
  \item{ENSG00000264885}{normalized expression for indicated transcript}
  \item{ENSG00000264895}{normalized expression for indicated transcript}
  \item{ENSG00000267136}{normalized expression for indicated transcript}
  \item{ENSG00000267551}{normalized expression for indicated transcript}
  \item{ENSG00000267702}{normalized expression for indicated transcript}
  \item{ENSG00000268001}{normalized expression for indicated transcript}
  \item{ENSG00000268573}{normalized expression for indicated transcript}
  \item{ENSG00000270554}{normalized expression for indicated transcript}
  \item{ENSG00000270562}{normalized expression for indicated transcript}
  \item{ENSG00000271646}{normalized expression for indicated transcript}
  \item{ENSG00000273018}{normalized expression for indicated transcript}
  \item{ENSG00000273033}{normalized expression for indicated transcript}
}
}
\source{
<doi:10.1186/s13045-024-01553-6>
}
\usage{
amltest
}
\description{
Duration of complete response for 40 cytogenetically normal AML patients
and a subset of 320 transcript expression from RNA-sequencing.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nonzerocure_test.R
\name{nonzerocure_test}
\alias{nonzerocure_test}
\title{Non-parametric test for a non-zero cured fraction}
\usage{
nonzerocure_test(object, reps = 1000, seed = NULL, plot = FALSE, b = NULL)
}
\arguments{
\item{object}{a \code{survfit} object.}

\item{reps}{number of simulations on which to base the p-value
(default = 1000).}

\item{seed}{optional random seed.}

\item{plot}{logical. If TRUE a histogram of the estimated susceptible
proportions over all simulations is produced.}

\item{b}{optional. If specified the maximum observed time for the uniform
distribution for generating the censoring times. If not specified, an
exponential model is used for generating the censoring times (default).}
}
\value{
\item{proportion_susceptible}{estimated proportion of susceptibles}

\item{proportion_cured}{estimated proportion of those cured}

\item{p_value}{p-value testing the null hypothesis that the
proportion of susceptibles = 1 (cured fraction = 0) against the alternative
that the proportion of susceptibles < 1 (non-zero cured fraction)}

\item{time_95_percent_of_events}{estimated time at which 95% of
events should have occurred}
}
\description{
Tests the null hypothesis that the proportion of observations susceptible to
the event = 1 against the alternative that the proportion of observations
susceptible to the event is < 1. If the null hypothesis is rejected, there
is a significant cured fraction.
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
training <- temp$training
km_fit <- survfit(Surv(Time, Censor) ~ 1, data = training)
nonzerocure_test(km_fit)
}
\references{
Maller, R. A. and Zhou, X. (1996) \emph{Survival Analysis with
Long-Term Survivors}. John Wiley & Sons.
}
\seealso{
\code{\link[survival]{survfit}}, \code{\link[hdcuremodels]{cure_estimate}},
\code{\link[hdcuremodels]{sufficient_fu_test}}
}
\keyword{htest}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dim.R
\name{dim.mixturecure}
\alias{dim.mixturecure}
\title{Dimension method for mixturecure objects}
\usage{
\method{dim}{mixturecure}(x)
}
\arguments{
\item{x}{An object of class `mixturecure`.}
}
\value{
\item{nobs}{ number of subjects in the dataset.}

\item{p_incidence}{ number of variables in the incidence portion of the model.}

\item{p_latency}{ number of variables in the latency portion of the model.}
}
\description{
Dimension method for \code{mixturecure} objects.
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
training <- temp$training
fit <- curegmifs(Surv(Time, Censor) ~ .,
  data = training, x_latency = training,
  model = "weibull", thresh = 1e-4, maxit = 2000,
  epsilon = 0.01, verbose = FALSE
)
dim(fit)
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sufficient_fu_test.R
\name{sufficient_fu_test}
\alias{sufficient_fu_test}
\title{Test for sufficient follow-up}
\usage{
sufficient_fu_test(object)
}
\arguments{
\item{object}{a \code{survfit} object.}
}
\value{
\item{p_value}{p-value from testing the null hypothesis that there
was not sufficient follow-up against the alternative that there was
sufficient follow-up}

\item{n_n}{total number of events that occurred at time >
pmax(0, 2*(last observed event time)-(last observed time)) and < the last
observed event time}

\item{N}{number of observations in the dataset}
}
\description{
Tests for sufficient follow-up using a Kaplan-Meier fitted object.
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
training <- temp$training
km_fit <- survfit(Surv(Time, Censor) ~ 1, data = training)
sufficient_fu_test(km_fit)
}
\references{
Maller, R. A. and Zhou, X. (1996) \emph{Survival Analysis with
Long-Term Survivors}. John Wiley & Sons.
}
\seealso{
\code{\link[survival]{survfit}}, \code{\link{cure_estimate}},
\code{\link{nonzerocure_test}}
}
\keyword{htest}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv_curegmifs.R
\name{cv_curegmifs}
\alias{cv_curegmifs}
\title{Fit a penalized parametric mixture cure model using the generalized monotone
incremental forward stagewise (GMIFS) algorithm (Hastie et al 2007) with
cross-validation for model selection}
\usage{
cv_curegmifs(
  formula,
  data,
  subset,
  x_latency = NULL,
  model = c("weibull", "exponential"),
  penalty_factor_inc = NULL,
  penalty_factor_lat = NULL,
  fdr_control = FALSE,
  fdr = 0.2,
  epsilon = 0.001,
  thresh = 1e-05,
  scale = TRUE,
  maxit = 10000,
  inits = NULL,
  n_folds = 5,
  measure_inc = c("c", "auc"),
  one_se = FALSE,
  cure_cutoff = 5,
  parallel = FALSE,
  seed = NULL,
  verbose = TRUE,
  na.action = na.omit,
  ...
)
}
\arguments{
\item{formula}{an object of class "\code{formula}" (or one that can be
coerced to that class): a symbolic description of the model to be fitted.
The response must be a survival object as returned by the \code{Surv}
function while the variables on the right side of the formula are the
covariates that are included in the incidence portion of the model.}

\item{data}{a data.frame in which to interpret the variables named in the
\code{formula} or in the \code{subset} argument. Rows with missing data are
omitted (only \code{na.action = na.omit} is operational) therefore users may
want to impute missing data prior to calling this function.}

\item{subset}{an optional expression indicating which subset of observations
to be used in the fitting process, either a numeric or factor variable should
be used in subset, not a character variable. All observations are included by
default.}

\item{x_latency}{specifies the variables to be included in the latency
portion of the model and can be either a matrix of predictors, a model
formula with the right hand side specifying the latency variables, or the
same data.frame passed to the \code{data} parameter. Note that when using the
model formula syntax for \code{x_latency} it cannot handle
\code{x_latency = ~ .}.}

\item{model}{type of regression model to use for the latency portion of
mixture cure model. Can be "weibull" or "exponential"; default is "weibull".}

\item{penalty_factor_inc}{vector of binary indicators representing the
penalty to apply to each incidence coefficient: 0 implies no shrinkage and 1
implies shrinkage. If not supplied, 1 is applied to all incidence variables.}

\item{penalty_factor_lat}{vector of binary indicators representing the
penalty to apply to each latency coefficient: 0 implies no shrinkage and 1
implies shrinkage. If not supplied, 1 is applied to all latency variables.}

\item{fdr_control}{logical, if TRUE, model-X knockoffs are used for
FDR-controlled variable selection and indices of selected variables are
returned (default is FALSE).}

\item{fdr}{numeric value in (0, 1) range specifying the target FDR level to
use for variable selection when \code{fdr_control = TRUE} (default is 0.2).}

\item{epsilon}{small numeric value reflecting incremental value used to
update a coefficient at a given step (default is 0.001).}

\item{thresh}{small numeric value. The iterative process stops when the
differences between successive expected penalized complete-data
log-likelihoods for both incidence and latency components are less than this
specified level of tolerance (default is 10^-5).}

\item{scale}{logical, if TRUE the predictors are centered and scaled.}

\item{maxit}{integer specifying the maximum number of steps to run in the
iterative algorithm (default is 10^4).}

\item{inits}{an optional list specifying the initial values as follows:
\itemize{
\item \code{itct} a numeric value for the unpenalized incidence
intercept.
\item \code{b_u} a numeric vector for the unpenalized incidence coefficients.
\item \code{beta_u} a numeric vector for unpenalized latency
coefficients.
\item \code{lambda} a numeric value for the rate parameter.
\item \code{alpha} a numeric value for the shape parameter
when \code{model = "weibull"}.
}
If \code{inits} is not specified or improperly supplied, initialization is
automatically provided by the function.}

\item{n_folds}{an integer specifying the number of folds for the k-fold
cross-validation procedure (default is 5).}

\item{measure_inc}{character string specifying the evaluation criterion used
in selecting the optimal \eqn{\lambda_b} which can be either
\itemize{
\item \code{"c"} specifying to use the C-statistic for cure status
weighting (CSW) method proposed by Asano and Hirakawa (2017) to
select both \eqn{\lambda_b} and \eqn{\lambda_{\beta}}
\item \code{"auc"} specifying to use the AUC for cure prediction using the
mean score imputation (MSI) method proposed by Asano et al. (2014) to select
\eqn{\lambda_b} while the C-statistic with CSW is used for
\eqn{\lambda_{\beta}}.
}}

\item{one_se}{logical, if TRUE then the one standard error rule is applied
for selecting the optimal parameters. The one standard error rule selects the
most parsimonious model having evaluation criterion no more than one standard
error worse than that of the best evaluation criterion (default is FALSE).}

\item{cure_cutoff}{numeric value representing the cutoff time value that
represents subjects not experiencing the event by this time are cured. This
value is used to produce a proxy for the unobserved cure status when
calculating C-statistic and AUC (default is 5 representing 5 years). Users
should be careful to note the time scale of their data and adjust this
according to the time scale and clinical application.}

\item{parallel}{logical. If TRUE, parallel processing is performed for K-fold
CV using \code{foreach} and the \pkg{doParallel} package is required.}

\item{seed}{optional integer representing the random seed. Setting the random
seed fosters reproducibility of the results.}

\item{verbose}{logical, if TRUE running information is printed to the console
(default is FALSE).}

\item{na.action}{this function requires complete data so \code{"na.omit"} is
invoked. Users can impute missing data as an alternative prior to model fitting.}

\item{...}{additional arguments.}
}
\value{
\item{b0 }{Estimated intercept for the incidence portion of the
model.}

\item{b }{Estimated coefficients for the incidence portion of the
model.}

\item{beta }{Estimated coefficients for the latency portion of the
model.}

\item{alpha }{Estimated shape parameter if the Weibull model is fit.}

\item{rate }{Estimated rate parameter if the Weibull or exponential
model is fit.}

\item{logLik }{Log-likelihood value.}

\item{selected_step_inc }{Iteration step selected for the incidence
portion of the model using cross-validation. NULL when fdr_control is TRUE.}

\item{selected_step_lat }{Iteration step selected for the latency
portion of the model using cross-validation. NULL when fdr_control is TRUE.}

\item{max_c }{Maximum C-statistic achieved}

\item{max_auc }{Maximum AUC for cure prediction achieved; only output
when \code{measure_inc = "auc"}.}

\item{selected_index_inc }{Indices of selected variables for the
incidence portion of the model when \code{fdr_control = TRUE}. If none
selected, \code{int(0)} will be returned.}

\item{selected_index_lat }{Indices of selected variables for the
latency portion of the model when \code{fdr_control = TRUE}. If none selected,
\code{int(0)} will be returned.}

\item{call}{the matched call.}
}
\description{
Fits a penalized Weibull or exponential mixture cure model using the
generalized monotone incremental forward stagewise (GMIFS) algorithm with
k-fold cross-validation to select the optimal iteration step along the
solution path. When FDR controlled variable selection is used, the model-X
knockoffs method is applied and indices of selected variables are returned.
}
\examples{
\dontrun{
library(survival)
withr::local_seed(123)
temp <- generate_cure_data(n = 100, j = 15, n_true = 3, a = 1.8, rho = 0.1)
training <- temp$training

fit.cv <- cv_curegmifs(Surv(Time, Censor) ~ .,
  data = training,
  x_latency = training, fdr_control = FALSE,
  maxit = 450, epsilon = 0.01, n_folds = 2, thresh = 1e-4,
  seed = 23, verbose = TRUE
)
}
}
\references{
Fu, H., Nicolet, D., Mrozek, K., Stone, R. M., Eisfeld, A. K.,
Byrd, J. C., Archer, K. J. (2022) Controlled variable selection in Weibull
mixture cure models for high-dimensional data. \emph{Statistics in Medicine},
\bold{41}(22), 4340--4366.

Hastie, T., Taylor J., Tibshirani R., Walther G. (2007) Forward
stagewise regression and the monotone lasso. \emph{Electron J Stat},
\bold{1}:1--29.
}
\seealso{
\code{\link{curegmifs}}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.mixturecure}
\alias{summary.mixturecure}
\title{Summarize a fitted mixture cure object}
\usage{
\method{summary}{mixturecure}(object, ...)
}
\arguments{
\item{object}{a \code{mixturecure} object resulting from \code{curegmifs},
\code{cureem}, \code{cv_curegmifs}, or \code{cv_cureem}.}

\item{...}{other arguments.}
}
\value{
prints the number of non-zero coefficients from the incidence and
latency portions of the fitted mixture cure model when using the minimum AIC
to select the final model. When fitting a model using \code{curegmifs} or
\code{cureem} the summary function additionally prints results associated
with the following model selection methods: the step and value that maximizes
the log-likelihood; the step and value that minimizes the AIC, modified AIC
(mAIC), corrected AIC (cAIC), BIC, modified BIC (mBIC), and extended BIC
(EBIC). This information can be used to guide the user in the selection of
a final model from the solution path.
}
\description{
\code{summary} method for a mixturecure object fit using \code{curegmifs},
\code{cureem}, \code{cv_curegmifs}, or \code{cv_cureem}.
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
training <- temp$training
fit <- curegmifs(Surv(Time, Censor) ~ .,
  data = training, x_latency = training,
  model = "weibull", thresh = 1e-4, maxit = 2000,
  epsilon = 0.01, verbose = FALSE
)
summary(fit)
}
\seealso{
\code{\link[hdcuremodels]{curegmifs}}, \code{\link[hdcuremodels]{cureem}},
\code{\link[hdcuremodels]{coef.mixturecure}}, \code{\link[hdcuremodels]{plot.mixturecure}},
\code{\link[hdcuremodels]{predict.mixturecure}}
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{predict.mixturecure}
\alias{predict.mixturecure}
\title{Predicted probabilities for susceptibles, linear predictor for latency, and
risk class for latency for mixture cure fit}
\usage{
\method{predict}{mixturecure}(object, newdata, model_select = "AIC", ...)
}
\arguments{
\item{object}{a \code{mixturecure} object resulting from \code{curegmifs},
\code{cureem}, \code{cv_curegmifs}, or \code{cv_cureem}.}

\item{newdata}{an optional data.frame that minimally includes the incidence
and/or latency variables to use for predicting the response. If omitted, the
training data are used.}

\item{model_select}{either a case-sensitive parameter for models fit using
\code{curegmifs} or \code{cureem} or any numeric step along the solution path
can be selected. The default is \code{model_select = "AIC"} which calculates
the predicted values using the coefficients from the model achieving the
minimum AIC. The complete list of options are:
\itemize{
    \item \code{"AIC"} for the minimum AIC (default).
    \item \code{"mAIC"} for the minimum modified AIC.
    \item \code{"cAIC"} for the minimum corrected AIC.
    \item \code{"BIC"}, for the minimum BIC.
    \item \code{"mBIC"} for the minimum modified BIC.
    \item \code{"EBIC"} for the minimum extended BIC.
    \item \code{"logLik"} for the step that maximizes the
log-likelihood.
    \item \code{n} where n is any numeric value from the
    solution path.
  }
This option has no effect for objects fit using \code{cv_curegmifs} or
\code{cv_cureem}.}

\item{...}{other arguments}
}
\value{
\item{p_uncured}{ a vector of probabilities from the incidence
portion of the fitted model representing the P(uncured).}

\item{linear_latency}{ a vector for the linear predictor from the
latency portion of the model.}

\item{latency_risk}{ a dichotomous class representing low (below the
median) versus high risk for the latency portion of the model.}
}
\description{
This function returns a list that includes the predicted probabilities for
susceptibles as well as the linear predictor for the latency distribution
and a dichotomous risk for latency for a \code{curegmifs}, \code{cureem},
\code{cv_curegmifs} or \code{cv_cureem} fitted object.
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
training <- temp$training
fit <- curegmifs(Surv(Time, Censor) ~ .,
  data = training, x_latency = training,
  model = "weibull", thresh = 1e-4, maxit = 2000,
  epsilon = 0.01, verbose = FALSE
)
predict_train <- predict(fit)
names(predict_train)
testing <- temp$testing
predict_test <- predict(fit, newdata = testing)
}
\seealso{
\code{\link[hdcuremodels]{curegmifs}}, \code{\link[hdcuremodels]{cureem}},
\code{\link[hdcuremodels]{coef.mixturecure}}, \code{\link[hdcuremodels]{summary.mixturecure}},
\code{\link[hdcuremodels]{plot.mixturecure}}
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/family.R
\name{family.mixturecure}
\alias{family.mixturecure}
\title{Return model family and fitting algorithm for mixturecure model fits}
\usage{
\method{family}{mixturecure}(object, ...)
}
\arguments{
\item{object}{an object of class \code{mixturecure}}

\item{...}{other arguments.}
}
\value{
the parametric or semi-parametric model fit and the fitting
algorithm.
}
\description{
Return model family and fitting algorithm for\code{mixturecure}
model fits.
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
training <- temp$training
fit <- curegmifs(Surv(Time, Censor) ~ .,
  data = training, x_latency = training,
  model = "weibull", thresh = 1e-4, maxit = 2000,
  epsilon = 0.01, verbose = FALSE
)
family(fit)
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\docType{package}
\name{hdcuremodels-package}
\alias{hdcuremodels}
\alias{hdcuremodels-package}
\title{hdcuremodels: High-Dimensional Cure Models}
\description{
Provides functions for fitting various penalized parametric and semi-parametric mixture cure models with different penalty functions, testing for a significant cure fraction, and testing for sufficient follow-up as described in Fu et al (2022)\doi{10.1002/sim.9513} and Archer et al (2024)\doi{10.1186/s13045-024-01553-6}. False discovery rate controlled variable selection is provided using model-X knock-offs.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ropensci/hdcuremodels}
  \item Report bugs at \url{https://github.com/ropensci/hdcuremodels/issues}
}

}
\author{
\strong{Maintainer}: Kellie J. Archer \email{archer.43@osu.edu} (\href{https://orcid.org/0000-0003-1555-5781}{ORCID})

Authors:
\itemize{
  \item Kellie J. Archer \email{archer.43@osu.edu} (\href{https://orcid.org/0000-0003-1555-5781}{ORCID})
  \item Han Fu
}

Other contributors:
\itemize{
  \item Tung Lam Nguyen \email{nguyenlamtung10@gmail.com} (Reviewed the package for ROpenSci) [reviewer]
  \item Panagiotis Papastamoulis \email{papapast@yahoo.gr} (Reviewed the package for ROpenSci) [reviewer]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.mixturecure}
\alias{print.mixturecure}
\title{Print the contents of a mixture cure fitted object}
\usage{
\method{print}{mixturecure}(x, max = 6, ...)
}
\arguments{
\item{x}{a \code{mixturecure} object resulting from \code{curegmifs},
\code{cureem}, \code{cv_cureem}, or \code{cv_curegmifs}.}

\item{max}{maximum number of rows in a matrix or elements in a vector to
display}

\item{...}{other arguments.}
}
\value{
prints coefficient estimates for the incidence portion of the model
and if included, prints the coefficient estimates for the latency portion of
the model. Also prints rate for exponential and Weibull models and scale
(alpha) for the Weibull mixture cure model. Returns all objects fit using
\code{cureem}, \code{curegmifs}, \code{cv_cureem}, or \code{cv_curegmifs}.
}
\description{
This function prints the first several incidence and latency coefficients,
the rate (when fitting an exponential or Weibull mixture cure model), and
alpha (when fitting a Weibull mixture cure model). This function returns
the fitted object invisible to the user.
}
\note{
The contents of a \code{mixturecure} fitted object differ depending
upon whether the EM (\code{cureem}) or GMIFS (\code{curegmifs}) algorithm is
used for model fitting or if cross-validation is used. Also, the output
differs depending upon whether \code{x_latency} is specified in the model
(i.e., variables are included in the latency portion of the model fit) or
only \code{terms} on the right hand side of the equation are included (i.e.,
variables are included in the incidence portion of the model).
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
training <- temp$training
fit <- curegmifs(Surv(Time, Censor) ~ .,
  data = training, x_latency = training,
  model = "weibull", thresh = 1e-4, maxit = 2000,
  epsilon = 0.01, verbose = FALSE
)
print(fit)
}
\seealso{
\code{\link[hdcuremodels]{curegmifs}}, \code{\link[hdcuremodels]{cureem}},
\code{\link[hdcuremodels]{coef.mixturecure}}, \code{\link[hdcuremodels]{summary.mixturecure}},
\code{\link[hdcuremodels]{plot.mixturecure}}, \code{\link[hdcuremodels]{predict.mixturecure}}
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/concordance_mcm.R
\name{concordance_mcm}
\alias{concordance_mcm}
\title{C-statistic for mixture cure models}
\usage{
concordance_mcm(object, newdata, cure_cutoff = 5, model_select = "AIC")
}
\arguments{
\item{object}{a \code{mixturecure} object resulting from \code{curegmifs},
\code{cureem}, \code{cv_curegmifs}, or \code{cv_cureem}.}

\item{newdata}{an optional data.frame that minimally includes the incidence
and/or latency variables to use for predicting the response. If omitted, the
training data are used.}

\item{cure_cutoff}{cutoff value for cure, used to produce a proxy for the
unobserved cure status (default is 5 representing 5 years). Users
should be careful to note the time scale of their data and adjust this
according to the time scale and clinical application.}

\item{model_select}{either a case-sensitive parameter for models fit using
\code{curegmifs} or \code{cureem} or any numeric step along the solution path
can be selected. The default is \code{model_select = "AIC"} which calculates
the predicted values using the coefficients from the model achieving the
minimum AIC. The complete list of options are:
\itemize{
    \item \code{"AIC"} for the minimum AIC (default).
    \item \code{"mAIC"} for the minimum modified AIC.
    \item \code{"cAIC"} for the minimum corrected AIC.
    \item \code{"BIC"}, for the minimum BIC.
    \item \code{"mBIC"} for the minimum modified BIC.
    \item \code{"EBIC"} for the minimum extended BIC.
    \item \code{"logLik"} for the step that maximizes the
log-likelihood.
    \item \code{model_select = n} where n is any numeric value from the
    solution path.
  }
This option has no effect for objects fit using \code{cv_curegmifs} or
\code{cv_cureem}.}
}
\value{
value of C-statistic for the cure models.
}
\description{
This function calculates the C-statistic using the cure status weighting
(CSW) method proposed by Asano and Hirakawa (2017).
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
training <- temp$training
testing <- temp$testing
fit <- curegmifs(Surv(Time, Censor) ~ .,
  data = training, x_latency = training,
  model = "weibull", thresh = 1e-4, maxit = 2000,
  epsilon = 0.01, verbose = FALSE
)
concordance_mcm(fit, model_select = "cAIC")
concordance_mcm(fit, newdata = testing, model_select = "cAIC")
}
\references{
Asano, J. and Hirakawa, H. (2017) Assessing the prediction
accuracy of a cure model for censored survival data with long-term survivors:
Application to breast cancer data. \emph{Journal of Biopharmaceutical
Statistics}, \bold{27}:6, 918--932.
}
\seealso{
\code{\link[hdcuremodels]{auc_mcm}}
}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef.R
\name{coef.mixturecure}
\alias{coef.mixturecure}
\alias{coefficients}
\title{Extract model coefficients from a fitted mixturecure object}
\usage{
\method{coef}{mixturecure}(object, model_select = "AIC", ...)
}
\arguments{
\item{object}{a \code{mixturecure} object resulting from \code{curegmifs},
\code{cureem}, \code{cv_curegmifs}, or \code{cv_cureem}.}

\item{model_select}{either a case-sensitive parameter for models fit using
\code{curegmifs} or \code{cureem} or any numeric step along the solution path
can be selected. The default is \code{model_select = "AIC"} which calculates
the predicted values using the coefficients from the model achieving the
minimum AIC. The complete list of options are:
\itemize{
    \item \code{"AIC"} for the minimum AIC (default).
    \item \code{"mAIC"} for the minimum modified AIC.
    \item \code{"cAIC"} for the minimum corrected AIC.
    \item \code{"BIC"}, for the minimum BIC.
    \item \code{"mBIC"} for the minimum modified BIC.
    \item \code{"EBIC"} for the minimum extended BIC.
    \item \code{"logLik"} for the step that maximizes the
log-likelihood.
    \item \code{n} where n is any numeric value from the
    solution path.
  }
This option has no effect for objects fit using \code{cv_curegmifs} or
\code{cv_cureem}.}

\item{...}{other arguments.}
}
\value{
\item{rate}{ estimated rate parameter when fitting a Weibull or
exponential mixture cure model.}

\item{shape}{ estimated shape parameter when fitting a Weibull
mixture cure model.}

\item{b0}{ estimated intercept for the incidence portion of the
mixture cure model.}

\item{beta_inc}{ the vector of coefficient estimates for the
incidence portion of the mixture cure model.}

\item{beta_lat}{ the vector of coefficient estimates for the
latency portion of the mixture cure model.}

\item{p_uncured}{ a vector of probabilities from the incidence
portion of the fitted model representing the P(uncured).}
}
\description{
\code{coef.mixturecure} is a generic function which extracts the model
coefficients from a fitted \code{mixturecure} model object fit using
\code{curegmifs}, \code{cureem}, \code{cv_curegmifs}, or \code{cv_cureem}.
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
training <- temp$training
fit <- curegmifs(Surv(Time, Censor) ~ .,
  data = training, x_latency = training,
  model = "weibull", thresh = 1e-4, maxit = 2000,
  epsilon = 0.01, verbose = FALSE
)
coef(fit)
}
\seealso{
\code{\link[hdcuremodels]{curegmifs}}, \code{\link[hdcuremodels]{cureem}},
\code{\link[hdcuremodels]{summary.mixturecure}}, \code{\link[hdcuremodels]{plot.mixturecure}},
\code{\link[hdcuremodels]{predict.mixturecure}}
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_cure_data.R
\name{generate_cure_data}
\alias{generate_cure_data}
\title{Simulate data under a mixture cure model}
\usage{
generate_cure_data(
  n = 400,
  j = 500,
  nonp = 2,
  train_prop = 0.75,
  n_true = 10,
  a = 1,
  rho = 0.5,
  itct_mean = 0.5,
  cens_ub = 20,
  alpha = 1,
  lambda = 2,
  same_signs = FALSE,
  model = "weibull"
)
}
\arguments{
\item{n}{an integer denoting the total sample size.}

\item{j}{an integer denoting the number of penalized predictors which is the
same for both the incidence and latency portions of the model.}

\item{nonp}{an integer denoting the number of unpenalized
predictors (which is the same for both the incidence and latency portions of
the model).}

\item{train_prop}{a numeric value in [0, 1) representing the fraction of `n`
to be used in forming the training dataset.}

\item{n_true}{an integer less than `j` denoting the number of variables truly
associated with the outcome (i.e., the number of covariates with nonzero
parameter values) among the penalized predictors.}

\item{a}{a numeric value denoting the effect size (signal amplitude) which is
the same for both the incidence and latency portions of the model.}

\item{rho}{a numeric value in [0, 1) representing the correlation between
adjacent covariates in the same block.}

\item{itct_mean}{a numeric value representing the expectation of the
incidence intercept which controls the cure rate.}

\item{cens_ub}{a numeric value representing the upper bound on the censoring
time distribution which follows a uniform distribution on (0, \code{cens_ub}].}

\item{alpha}{a numeric value representing the shape parameter in the Weibull
density.}

\item{lambda}{a numeric value representing the rate parameter in the Weibull
density.}

\item{same_signs}{logical, if TRUE the incidence and latency coefficients
have the same signs.}

\item{model}{type of regression model to use for the latency portion of
mixture cure model. Can be one of the following:
\itemize{
\item \code{"weibull"} to generate times from a Weibull distribution.
\item \code{"GG"} to generate times from a generalized gamma distribution.
\item \code{"Gompertz"} to generate times from a Gomertz distribution.
\item \code{"nonparametric"} to generate times non-parametrically.
\item  \code{"GG_baseline"} to generate times from a generalized gamma
baseline distribution.
}}
}
\value{
\item{training}{training data.frame which includes Time, Censor, and
covariates. Variables prefixed with \code{"U"} indicates unpenalized
covariates and is equal to the value
passed to \code{nonp} (default is 2). Variables prefixed with \code{"X"}
indicates penalized covariates and is equal to the value passed to
\code{j}.
}

\item{training_y}{the true status for the training set: uncured = 1;
cured = 0}

\item{testing}{testing data.frame which includes Time, Censor,  Y
(the true uncured = 1; cured = 0 status), and
covariates. Variables prefixed with \code{"U"} indicates unpenalized
covariates and is equal to the value
passed to \code{nonp} (default is 2). Variables prefixed with \code{"X"}
indicates penalized covariates and is equal to the value passed to
\code{j}.
}

\item{testing_y}{the true status for the testing set: uncured = 1;
cured = 0}

\item{parameters}{a list including: the indices of true incidence
signals (\code{nonzero_b}), indices of true latency signals
(\code{nonzero_beta}), unpenalized incidence parameter values (\code{b_u}),
unpenalized latency parameter values (\code{beta_u}), parameter values for
the true incidence signals among penalized covariates (\code{b_p_nz}),
parameter values for the true latency signals among penalized covariates
(\code{beta_p_nz}), parameter value for the incidence intercept
(\code{itct})}
}
\description{
Simulate data under a mixture cure model.
}
\examples{
library(survival)
withr::local_seed(1234)
# This dataset has 2 penalized features associated with the outcome,
# 3 penalized features not associated with the outcome (noise features), and 1
# unpenalized noise feature.
data <- generate_cure_data(n = 1000, j = 5, n_true = 2, nonp = 1, a = 2)
# Extract the training data
training <- data$training
# Extract the testing data
testing <- data$testing
# To identify the features truly associated with incidence
names(training)[grep("^X", names(training))][data$parameters$nonzero_b]
# To identify the features truly associated with latency
names(training)[grep("^X", names(training))][data$parameters$nonzero_beta]
# Fit the model to the training data
fitem <- cureem(Surv(Time, Censor) ~ ., data = training,
  x_latency = training)
# Examine the estimated coefficients at the (default) minimum AIC
coef(fitem)
# As the penalty increases, the coefficients for the noise variables shrink
# to or remain at zero, while the truly associated features have coefficient
# paths that depart from zero. This shows the model's ability to distinguish
# signal from noise.
plot(fitem, label = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.mixturecure}
\alias{plot.mixturecure}
\title{Plot fitted mixture cure model}
\usage{
\method{plot}{mixturecure}(
  x,
  type = c("trace", "AIC", "BIC", "logLik", "cAIC", "mAIC", "mBIC", "EBIC"),
  xlab = NULL,
  ylab = NULL,
  label = FALSE,
  main = NULL,
  ...
)
}
\arguments{
\item{x}{a \code{mixturecure} object resulting from \code{curegmifs} or
\code{cureem}, \code{cv_curegmifs} or \code{cv_cureem}.}

\item{type}{a case-sensitive parameter indicating what to plot on the y-axis.
The complete list of options are:
\itemize{
    \item \code{"trace"} plots the coefficient path for the fitted object
    (default).
    \item \code{"AIC"} plots the AIC against step of model fit.
    \item \code{"mAIC"} plots the modified AIC against step of model fit.
    \item \code{"cAIC"} plots the corrected AIC against step of model fit.
    \item \code{"BIC"}, plots the BIC against step of model fit.
    \item \code{"mBIC"} plots the modified BIC against step of model fit.
    \item \code{"EBIC"} plots the extended BIC against step of model fit.
    \item \code{"logLik"} plots the log-likelihood against step of model fit.
  }
This option has no effect for objects fit using
\code{cv_curegmifs} or \code{cv_cureem}.}

\item{xlab}{a default x-axis label will be used which can be changed by
specifying a user-defined x-axis label.}

\item{ylab}{a default y-axis label will be used which can be changed by
specifying a user-defined y-axis label.}

\item{label}{logical. If TRUE the variable names will appear in a legend.
Applicable only when \code{type = "trace"}. Be reminded that this works well
only for small to moderate numbers of variables. For many predictors, the
plot will be cluttered. The variables may be more easily identified using
the \code{coef} function indicating the step of interest.}

\item{main}{a default main title will be used which can be changed by
specifying a user-defined main title. This option is not used for
\code{cv_curegmifs} or \code{cv_cureem} fitted objects.}

\item{...}{other arguments.}
}
\value{
this function has no returned value but is called for its side
effects
}
\description{
This function plots either the coefficient path, the AIC, the cAIC, the BIC,
or the log-likelihood for a fitted \code{curegmifs} or \code{cureem} object.
This function produces a lollipop plot of the coefficient estimates for a
fitted \code{cv_curegmifs} or \code{cv_cureem} object.
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
training <- temp$training
fit <- curegmifs(Surv(Time, Censor) ~ .,
  data = training, x_latency = training,
  model = "weibull", thresh = 1e-4, maxit = 2000,
  epsilon = 0.01, verbose = FALSE
)
plot(fit)
}
\seealso{
\code{\link[hdcuremodels]{curegmifs}}, \code{\link[hdcuremodels]{cureem}},
\code{\link[hdcuremodels]{coef.mixturecure}}, \code{\link[hdcuremodels]{summary.mixturecure}},
\code{\link[hdcuremodels]{predict.mixturecure}}
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cureem.R
\name{cureem}
\alias{cureem}
\title{Fit penalized mixture cure model using the E-M algorithm}
\usage{
cureem(
  formula,
  data,
  subset,
  x_latency = NULL,
  model = c("cox", "weibull", "exponential"),
  penalty = c("lasso", "MCP", "SCAD"),
  penalty_factor_inc = NULL,
  penalty_factor_lat = NULL,
  thresh = 0.001,
  scale = TRUE,
  maxit = NULL,
  inits = NULL,
  lambda_inc = 0.1,
  lambda_lat = 0.1,
  gamma_inc = 3,
  gamma_lat = 3,
  na.action = na.omit,
  ...
)
}
\arguments{
\item{formula}{an object of class "\code{formula}" (or one that can be
coerced to that class): a symbolic description of the model to be fitted.
The response must be a survival object as returned by the \code{Surv}
function while the variables on the right side of the formula are the
covariates that are included in the incidence portion of the model.}

\item{data}{a data.frame in which to interpret the variables named in the
\code{formula} or in the \code{subset} argument. Rows with missing data are
omitted (only \code{na.action = na.omit} is operational) therefore users may
want to impute missing data prior to calling this function.}

\item{subset}{an optional expression indicating which subset of observations
to be used in the fitting process, either a numeric or factor variable
should be used in subset, not a character variable. All observations are
included by default.}

\item{x_latency}{specifies the variables to be included in the latency
portion of the model and can be either a matrix of predictors, a model
formula with the right hand side specifying the latency variables, or the
same data.frame passed to the \code{data} parameter. Note that when using
the model formula syntax for \code{x_latency} it cannot handle
\code{x_latency = ~ .}.}

\item{model}{type of regression model to use for the latency portion of
mixture cure model. Can be "cox", "weibull", or "exponential" (default is
"cox").}

\item{penalty}{type of penalty function. Can be "lasso", "MCP", or "SCAD"
(default is "lasso").}

\item{penalty_factor_inc}{vector of binary indicators representing the
penalty to apply to each incidence coefficient: 0 implies no shrinkage
and 1 implies shrinkage. If not supplied, 1 is applied to all incidence
variables.}

\item{penalty_factor_lat}{vector of binary indicators representing the
penalty to apply to each latency coefficient: 0 implies no shrinkage and 1
implies shrinkage. If not supplied, 1 is applied to all latency variables.}

\item{thresh}{small numeric value. The iterative process stops when the
differences between successive expected penalized complete-data
log-likelihoods for both incidence and latency components are less than this
specified level of tolerance (default is 10^-3).}

\item{scale}{logical, if TRUE the predictors are centered and scaled.}

\item{maxit}{integer specifying the maximum number of passes over the data
for each lambda. If not specified, 100 is applied when
\code{penalty = "lasso"} and 1000 is applied when \code{penalty = "MCP"} or
\code{penalty = "SCAD"}.}

\item{inits}{an optional list specifiying the initial values. This includes:
\itemize{
\item \code{itct} the incidence intercept.
\item \code{b_u} a numeric vector for the unpenalized
incidence coefficients for the incidence portion of the model.
\item \code{beta_u} a numeric vector for unpenalized
latency coefficients in the incidence portion of the model.
\item \code{lambda} a numeric value for the rate parameter when fitting
either a Weibull or exponential MCM using \code{model = "weibull"} or
\code{model = "exponential"}.
\item \code{alpha} a numeric value for the shape parameter when fitting a
Weibull MCM using \code{model = "weibull"}.
\item \code{survprob} a numeric vector for the
latency survival probabilities \eqn{S_u(t_i|w_i)} for i=1,...,N when fitting
a Cox MCM \code{model = "cox"}.
}
Penalized coefficients are initialized to zero. If \code{inits} is not specified or improperly specified, initialization is
automatically provided by the function.}

\item{lambda_inc}{numeric value for the penalization parameter \eqn{\lambda}
for variables in the incidence portion of the model.}

\item{lambda_lat}{numeric value for the penalization parameter \eqn{\lambda}
for variables in the latency portion of the model.}

\item{gamma_inc}{numeric value for the penalization parameter \eqn{\gamma}
for variables in the incidence portion of the model when
\code{penalty = "MCP"} or \code{penalty = "SCAD"} (default is 3).}

\item{gamma_lat}{numeric value for the penalization parameter \eqn{\gamma}
for variables in the latency portion of the model when \code{penalty = "MCP"}
or \code{penalty = "SCAD"} (default is 3).}

\item{na.action}{this function requires complete data so \code{"na.omit"} is
invoked. Users can impute missing data as an alternative prior to model fitting.}

\item{...}{additional arguments.}
}
\value{
\item{b_path}{Matrix representing the solution path of the
coefficients in the incidence portion of the model. Row is step and column
is variable.}

\item{beta_path}{Matrix representing the solution path of the
coefficients in the latency portion of the model. Row is step and column
is variable.}

\item{b0_path }{Vector representing the solution path of the
intercept in the incidence portion of the model.}

\item{logLik_inc }{Vector representing the expected penalized
complete-data log-likelihood for the incidence portion of the model for
each step in the solution path.}

\item{logLik_lat }{Vector representing the expected penalized
complete-data log-likelihood for the latency portion of the model for each
step in the solution path.}

\item{x_incidence}{Matrix representing the design matrix of the
incidence predictors.}

\item{x_latency}{Matrix representing the design matrix of the
latency predictors.}

\item{y}{Vector representing the survival object response as
returned by the \code{Surv} function }

\item{model}{Character string indicating the type of regression
model used for the latency portion of mixture cure model ("weibull" or
"exponential").}

\item{scale}{Logical value indicating whether the predictors were
centered and scaled.}

\item{method}{Character string indicating the EM algorithm was used
in fitting the mixture cure model.}

\item{rate_path}{Vector representing the solution path of the rate
parameter for the Weibull or exponential density in the latency portion of
the model.}

\item{alpha_path}{Vector representing the solution path of the shape
parameter for the Weibull density in the latency portion of the model.}

\item{call}{the matched call.}
}
\description{
Fits penalized parametric and semi-parametric mixture cure models (MCM)
using the E-M algorithm with user-specified penalty parameters. The lasso
(L1), MCP, and SCAD penalty are supported for the Cox MCM while only lasso is
currently supported for parametric MCMs.
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 80, j = 100, n_true = 10, a = 1.8)
training <- temp$training
fit <- cureem(Surv(Time, Censor) ~ .,
  data = training, x_latency = training,
  model = "cox", penalty = "lasso", lambda_inc = 0.1,
  lambda_lat = 0.1, gamma_inc = 6, gamma_lat = 10
)
}
\references{
Archer, K. J., Fu, H., Mrozek, K., Nicolet, D., Mims, A. S.,
Uy, G. L., Stock, W., Byrd, J. C., Hiddemann, W., Braess, J.,
Spiekermann, K., Metzeler, K. H., Herold, T., Eisfeld, A.-K. (2024)
Identifying long-term survivors and those at higher or lower risk of relapse
among patients with cytogenetically normal acute myeloid leukemia using a
high-dimensional mixture cure model. \emph{Journal of Hematology & Oncology},
\bold{17}:28.
}
\seealso{
\code{\link[hdcuremodels]{cv_cureem}}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cure_estimate.R
\name{cure_estimate}
\alias{cure_estimate}
\title{Estimate cured fraction}
\usage{
cure_estimate(object)
}
\arguments{
\item{object}{a \code{survfit} object.}
}
\value{
estimated proportion of cured observations
}
\description{
Estimates the cured fraction using a Kaplan-Meier fitted object.
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
training <- temp$training
km_fit <- survfit(Surv(Time, Censor) ~ 1, data = training)
cure_estimate(km_fit)
}
\seealso{
\code{\link[survival]{survfit}}, \code{\link[hdcuremodels]{sufficient_fu_test}},
\code{\link{nonzerocure_test}}
}
\keyword{univar}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curegmifs.R
\name{curegmifs}
\alias{curegmifs}
\title{Fit penalized parametric mixture cure model using the GMIFS algorithm}
\usage{
curegmifs(
  formula,
  data,
  subset,
  x_latency = NULL,
  model = c("weibull", "exponential"),
  penalty_factor_inc = NULL,
  penalty_factor_lat = NULL,
  epsilon = 0.001,
  thresh = 1e-05,
  scale = TRUE,
  maxit = 10000,
  inits = NULL,
  verbose = TRUE,
  suppress_warning = FALSE,
  na.action = na.omit,
  ...
)
}
\arguments{
\item{formula}{an object of class "\code{formula}" (or one that can be
coerced to that class): a symbolic description of the model to be fitted. The
response must be a survival object as returned by the \code{Surv} function
while the variables on the right side of the formula are the covariates that
are included in the incidence portion of the model.}

\item{data}{a data.frame in which to interpret the variables named in the
\code{formula} or in the \code{subset} argument. Rows with missing data are
omitted (only \code{na.action = na.omit} is operational) therefore users may
want to impute missing data prior to calling this function.}

\item{subset}{an optional expression indicating which subset of observations
to be used in the fitting process, either a numeric or factor variable should
be used in subset, not a character variable. All observations are included by
default.}

\item{x_latency}{specifies the variables to be included in the latency
portion of the model and can be either a matrix of predictors, a model
formula with the right hand side specifying the latency variables, or the
same data.frame passed to the \code{data} parameter. Note that when using the
model formula syntax for \code{x_latency} it cannot handle
\code{x_latency = ~ .}.}

\item{model}{type of regression model to use for the latency portion of
mixture cure model. Can be "weibull" or "exponential"; default is "weibull".}

\item{penalty_factor_inc}{vector of binary indicators representing the
penalty to apply to each incidence coefficient: 0 implies no shrinkage and 1
implies shrinkage. If not supplied, 1 is applied to all incidence variables.}

\item{penalty_factor_lat}{vector of binary indicators representing the
penalty to apply to each latency coefficient: 0 implies no shrinkage and 1
implies shrinkage. If not supplied, 1 is applied to all latency variables.}

\item{epsilon}{small numeric value reflecting the incremental value used to
update a coefficient at a given step (default is 0.001).}

\item{thresh}{small numeric value. The iterative process stops when the
differences between successive expected penalized
log-likelihoods for both incidence and latency components are less than this
specified level of tolerance (default is 10^-5).}

\item{scale}{logical, if TRUE the predictors are centered and scaled.}

\item{maxit}{integer specifying the maximum number of steps to run in the
iterative algorithm (default is 10^4).}

\item{inits}{an optional list specifying the initial values as follows:
\itemize{
\item \code{itct} a numeric value for the unpenalized incidence
intercept.
\item \code{b_u} a numeric vector for the unpenalized incidence coefficients.
\item \code{beta_u} a numeric vector for unpenalized latency
coefficients.
\item \code{lambda} a numeric value for the rate parameter.
\item \code{alpha} a numeric value for the shape parameter
when \code{model = "weibull"}.
}
If not supplied or improperly supplied,
initialization is automatically provided by the function.}

\item{verbose}{logical, if TRUE running information is printed to the
console (default is FALSE).}

\item{suppress_warning}{logical, if TRUE, suppresses echoing the warning that
the maximum number of iterations was reached so that the algorithm may not
have converged. Instead, warning is returned as part of the output with this
message.}

\item{na.action}{this function requires complete data so \code{"na.omit"} is
invoked. Users can impute missing data as an alternative prior to model fitting.}

\item{...}{additional arguments.}
}
\value{
\item{b_path}{Matrix representing the solution path of the
coefficients in the incidence portion of the model. Row is step and
column is variable.}

\item{beta_path}{Matrix representing the solution path of the
coefficients in the latency portion of the model. Row is step and column is
variable.}

\item{b0_path}{Vector representing the solution path of the intercept
in the incidence portion of the model.}

\item{rate_path}{Vector representing the solution path of the rate
parameter for the Weibull or exponential density in the latency portion of
the model.}

\item{logLik}{Vector representing the log-likelihood for each step
in the solution path.}

\item{x_incidence}{Matrix representing the design matrix of the
incidence predictors.}

\item{x_latency}{Matrix representing the design matrix of the latency
predictors.}

\item{y}{Vector representing the survival object response as returned
by the \code{Surv} function }

\item{model}{Character string indicating the type of regression model
used for the latency portion of mixture cure model ("weibull" or
"exponential").}

\item{scale}{Logical value indicating whether the predictors were
centered and scaled.}

\item{alpha_path}{Vector representing the solution path of the shape
parameter for the Weibull density in the latency portion of the model.}

\item{call}{the matched call.}

\item{warning}{message indicating whether the maximum number of
iterations was achieved which may indicate the model did not converge.}
}
\description{
Fits a penalized Weibull or exponential mixture cure model using the
generalized monotone incremental forward stagewise (GMIFS) algorithm (Hastie
et al 2007) and yields solution paths for parameters in the incidence and
latency portions of the model.
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
training <- temp$training

fit <- curegmifs(Surv(Time, Censor) ~ .,
  data = training, x_latency = training,
  model = "weibull", thresh = 1e-4, maxit = 2000, epsilon = 0.01,
  verbose = FALSE
)
}
\references{
Fu, H., Nicolet, D., Mrozek, K., Stone, R. M., Eisfeld, A. K.,
Byrd, J. C., Archer, K. J. (2022) Controlled variable selection in Weibull
mixture cure models for high-dimensional data. \emph{Statistics in Medicine},
\bold{41}(22), 4340--4366.

Hastie, T., Taylor J., Tibshirani R., Walther G. (2007) Forward
stagewise regression and the monotone lasso. \emph{Electron J Stat},
\bold{1}:1--29.
}
\seealso{
\code{\link[hdcuremodels]{cv_curegmifs}}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logLik.R
\name{logLik.mixturecure}
\alias{logLik.mixturecure}
\title{Log-likelihood for fitted mixture cure model}
\usage{
\method{logLik}{mixturecure}(object, model_select = "AIC", ...)
}
\arguments{
\item{object}{a \code{mixturecure} object resulting from \code{curegmifs},
\code{cureem}, \code{cv_curegmifs}, \code{cv_cureem}.}

\item{model_select}{either a case-sensitive parameter for models fit using
\code{curegmifs} or \code{cureem} or any numeric step along the solution path
can be selected. The default is \code{model_select = "AIC"} which calculates
the predicted values using the coefficients from the model achieving the
minimum AIC. The complete list of options are:
\itemize{
    \item \code{"AIC"} for the minimum AIC (default).
    \item \code{"mAIC"} for the minimum modified AIC.
    \item \code{"cAIC"} for the minimum corrected AIC.
    \item \code{"BIC"}, for the minimum BIC.
    \item \code{"mBIC"} for the minimum modified BIC.
    \item \code{"EBIC"} for the minimum extended BIC.
    \item \code{"logLik"} for the step that maximizes the
log-likelihood.
    \item \code{n} where n is any numeric value from the
    solution path.
  }
This option has no effect for objects fit using \code{cv_curegmifs} or
\code{cv_cureem}.}

\item{...}{other arguments.}
}
\value{
log-likelihood of the fitted mixture cure model using the specified
criteria.
}
\description{
This function returns the log-likelihood for a user-specified model criterion
or step for a \code{curegmifs}, \code{cureem},
\code{cv_curegmifs} or \code{cv_cureem} fitted object.
}
\examples{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 100, j = 10, n_true = 10, a = 1.8)
training <- temp$training
fit <- curegmifs(Surv(Time, Censor) ~ .,
  data = training, x_latency = training,
  model = "weibull", thresh = 1e-4, maxit = 2000,
  epsilon = 0.01, verbose = FALSE
)
logLik(fit, model_select = "AIC")
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv_cureem.R
\name{cv_cureem}
\alias{cv_cureem}
\title{Fit penalized mixture cure model using the E-M algorithm with
cross-validation for parameter tuning}
\usage{
cv_cureem(
  formula,
  data,
  subset,
  x_latency = NULL,
  model = c("cox", "weibull", "exponential"),
  penalty = c("lasso", "MCP", "SCAD"),
  penalty_factor_inc = NULL,
  penalty_factor_lat = NULL,
  fdr_control = FALSE,
  fdr = 0.2,
  grid_tuning = FALSE,
  thresh = 0.001,
  scale = TRUE,
  maxit = NULL,
  inits = NULL,
  lambda_inc_list = NULL,
  lambda_lat_list = NULL,
  nlambda_inc = NULL,
  nlambda_lat = NULL,
  gamma_inc = 3,
  gamma_lat = 3,
  lambda_min_ratio_inc = 0.1,
  lambda_min_ratio_lat = 0.1,
  n_folds = 5,
  measure_inc = c("c", "auc"),
  one_se = FALSE,
  cure_cutoff = 5,
  parallel = FALSE,
  seed = NULL,
  verbose = TRUE,
  na.action = na.omit,
  ...
)
}
\arguments{
\item{formula}{an object of class "\code{formula}" (or one that can be
coerced to that class): a symbolic description of the model to be fitted.
The response must be a survival object as returned by the \code{Surv}
function while the variables on the right side of the formula are the
covariates that are included in the incidence portion of the model.}

\item{data}{a data.frame in which to interpret the variables named in
the \code{formula} or in the \code{subset} argument. Rows with missing data are
omitted (only \code{na.action = na.omit} is operational) therefore users may
want to impute missing data prior to calling this function.}

\item{subset}{an optional expression indicating which subset of observations
to be used in the fitting process, either a numeric or factor variable
should be used in subset, not a character variable. All observations are
included by default.}

\item{x_latency}{specifies the variables to be included in the latency
portion of the model and can be either a matrix of predictors, a model
formula with the right hand side specifying the latency variables, or the
same data.frame passed to the \code{data} parameter. Note that when using
the model formula syntax for \code{x_latency} it cannot handle
\code{x_latency = ~ .}.}

\item{model}{type of regression model to use for the latency portion of
mixture cure model. Can be "cox", "weibull", or "exponential" (default is
"cox").}

\item{penalty}{type of penalty function. Can be "lasso", "MCP", or "SCAD"
(default is "lasso").}

\item{penalty_factor_inc}{vector of binary indicators representing the
penalty to apply to each incidence coefficient: 0 implies no shrinkage and
1 implies shrinkage. If not supplied, 1 is applied to all incidence
variables.}

\item{penalty_factor_lat}{vector of binary indicators representing the
penalty to apply to each latency coefficient: 0 implies no shrinkage and 1
implies shrinkage. If not supplied, 1 is applied to all latency variables.}

\item{fdr_control}{logical, if TRUE, model-X knockoffs are used for
FDR-controlled variable selection and indices of selected variables are
returned (default is FALSE).}

\item{fdr}{numeric value in (0, 1) range specifying the target FDR level to
use for variable selection when \code{fdr_control = TRUE} (default is 0.2).}

\item{grid_tuning}{logical, if TRUE a 2-D grid tuning approach is used to
select the optimal pair of \eqn{\lambda_b} and \eqn{\lambda_{\beta}} penalty
parameters for the incidence and latency portions of the model, respectively.
Otherwise the \eqn{\lambda_b} and \eqn{\lambda_{\beta}} are selected from a
1-D sequence and are equal to one another (default is FALSE).}

\item{thresh}{small numeric value. The iterative process stops when the
differences between successive expected penalized complete-data
log-likelihoods for both incidence and latency components are less than this
specified level of tolerance (default is 10^-3).}

\item{scale}{logical, if TRUE the predictors are centered and scaled.}

\item{maxit}{maximum number of passes over the data for each lambda. If not
specified, 100 is applied when \code{penalty = "lasso"} and 1000 is applied
when \code{penalty = "MCP"} or \code{penalty = "SCAD"}.}

\item{inits}{an optional list specifiying the initial values to be used for
model fitting as follows:
\itemize{
\item \code{itct} the incidence intercept.
\item \code{b_u} a numeric vector for the unpenalized.
incidence coefficients for the incidence portion of the model.
\item \code{beta_u} a numeric vector for unpenalized
latency coefficients in the incidence portion of the model.
\item \code{lambda} a numeric value for the rate parameter when fitting
either a Weibull or exponential MCM using \code{model = "weibull"} or
\code{model = "exponential"}.
\item \code{alpha} a numeric value for the shape parameter when fitting a
Weibull MCM using \code{model = "weibull"}.
\item \code{survprob} a numeric vector for the
latency survival probabilities \eqn{S_u(t_i|w_i)} for i=1,...,N when fitting
a Cox MCM \code{model = "cox"}.
}
Penalized coefficients are initialized to zero. If \code{inits} is not specified or improperly specified, initialization is
automatically provided by the function.}

\item{lambda_inc_list}{a numeric vector used to search for the optimal
\eqn{\lambda_b} tuning parameter. If not supplied, the function computes a
\eqn{\lambda_b} sequence based on \code{nlambda_inc} and
\code{lambda_min_ratio_inc}. If \code{grid_tuning = FALSE}, the same sequence
should be used for both \eqn{\lambda_b} and \eqn{\lambda_{\beta}}.}

\item{lambda_lat_list}{a numeric vector used to search for the optimal
\eqn{\lambda_{\beta}} tuning parameter. If not supplied, the function
computes a \eqn{\lambda_{\beta}} sequence based on \code{nlambda_lat} and
\code{lambda_min_ratio_lat}. If \code{grid_tuning = FALSE}, the same sequence
should be used for both \eqn{\lambda_b} and \eqn{\lambda_{\beta}}.}

\item{nlambda_inc}{an integer specifying the number of values to search for
the optimal \eqn{\lambda_b} tuning parameter; default is 10 if
\code{grid_tuning = TRUE} and 50 otherwise.}

\item{nlambda_lat}{an integer specifying the number of values to search
for the optimal \eqn{\lambda_{\beta}} tuning parameter; default is 10 if
\code{grid_tuning = TRUE} and 50 otherwise.}

\item{gamma_inc}{numeric value for the penalization parameter \eqn{\gamma}
for variables in the incidence portion of the model when
\code{penalty = "MCP"} or \code{penalty = "SCAD"} (default is 3).}

\item{gamma_lat}{numeric value for the penalization parameter \eqn{\gamma}
for variables in the latency portion of the model when \code{penalty = "MCP"}
or \code{penalty = "SCAD"} (default is 3).}

\item{lambda_min_ratio_inc}{numeric value in (0,1) representing the smallest
value for \eqn{\lambda_b} as a fraction of \code{lambda.max_inc}, the
data-derived entry value at which essentially all penalized variables in the
incidence portion of the model have a coefficient estimate of 0 (default is
0.1).}

\item{lambda_min_ratio_lat}{numeric value in (0.1) representing the smallest
value for \eqn{\lambda_{\beta}} as a fraction of \code{lambda.max_lat}, the
data-derived entry value at essentially all penalized variables in the
latency portion of the model have a coefficient estimate of 0 (default is
0.1).}

\item{n_folds}{an integer specifying the number of folds for the k-fold
cross-valiation procedure (default is 5).}

\item{measure_inc}{character string specifying the evaluation criterion used
in selecting the optimal \eqn{\lambda_b} which can be either
\itemize{
\item \code{"c"} specifying to use the C-statistic for cure status
weighting (CSW) method proposed by Asano and Hirakawa (2017) to
select both \eqn{\lambda_b} and \eqn{\lambda_{\beta}}
\item \code{"auc"} specifying to use the AUC for cure prediction using the
mean score imputation (MSI) method proposed by Asano et al. (2014) to select
\eqn{\lambda_b} while the C-statistic with CSW is used for
\eqn{\lambda_{\beta}}.
}}

\item{one_se}{logical, if TRUE then the one standard error rule is applied
for selecting the optimal parameters. The one standard error rule selects the
most parsimonious model having evaluation criterion no more than one standard
error worse than that of the best evaluation criterion (default is FALSE).}

\item{cure_cutoff}{numeric value representing the cutoff time value that
represents subjects not experiencing the event by this time are cured. This
value is used to produce a proxy for the unobserved cure status when
calculating C-statistic and AUC (default is 5 representing 5 years). Users
should be careful to note the time scale of their data and adjust this
according to the time scale and clinical application.}

\item{parallel}{logical. If TRUE, parallel processing is performed for K-fold
CV using \code{foreach} and the \pkg{doParallel} package is required.}

\item{seed}{optional integer representing the random seed. Setting the random
seed fosters reproducibility of the results.}

\item{verbose}{logical, if TRUE running information is printed to the console
(default is FALSE).}

\item{na.action}{this function requires complete data so \code{"na.omit"} is
invoked. Users can impute missing data as an alternative prior to model fitting.}

\item{...}{additional arguments.}
}
\value{
\item{b0}{Estimated intercept for the incidence portion of the
model.}

\item{b}{Estimated coefficients for the incidence portion of the
model.}

\item{beta}{Estimated coefficients for the latency portion of the
model.}

\item{alpha}{Estimated shape parameter if the Weibull model is fit.}

\item{rate}{Estimated rate parameter if the Weibull or exponential
model is fit.}

\item{logLik_inc}{Expected penalized complete-data log-likelihood for
the incidence portion of the model.}

\item{logLik_lat}{Expected penalized complete-data log-likelihood for
the latency portion of the model.}

\item{selected_lambda_inc}{Value of \eqn{\lambda_b} selected using
cross-validation. NULL when fdr_control is TRUE.}

\item{selected_lambda_lat}{Value of \eqn{\lambda_{\beta}} selected
using cross-validation. NULL when fdr_control is TRUE.}

\item{max_c}{Maximum C-statistic achieved.}

\item{max_auc}{Maximum AUC for cure prediction achieved; only output
when \code{measure_inc="auc"}.}

\item{selected_index_inc }{Indices of selected variables for the
incidence portion of the model when \code{fdr_control=TRUE}. If no variables
are selected, \code{int(0)} will be returned.}

\item{selected_index_lat }{Indices of selected variables for the
latency portion of the model when \code{fdr_control=TRUE}. If no variables
are selected, \code{int(0)} will be returned.}

\item{call}{the matched call.}
}
\description{
Fits penalized parametric and semi-parametric mixture cure models (MCM)
using the E-M algorithm with with k-fold cross-validation for parameter
tuning. The lasso (L1), MCP and SCAD penalty are supported for the Cox MCM
while only lasso is currently supported for parametric MCMs. When FDR
controlled variable selection is used, the model-X knockoffs method is
applied and indices of selected variables are returned.
}
\examples{
\dontrun{
library(survival)
withr::local_seed(1234)
temp <- generate_cure_data(n = 200, j = 25, n_true = 5, a = 1.8, rho = 0.2)
training <- temp$training
# Fit a penalized Cox MCM selecting parameters using 2-fold CV
fit.cv <- cv_cureem(Surv(Time, Censor) ~ .,
  data = training,
  x_latency = training, fdr_control = FALSE,
  grid_tuning = FALSE, nlambda_inc = 10, nlambda_lat = 10,
  n_folds = 2, seed = 23, verbose = FALSE
)
fit.cv.fdr <- cv_cureem(Surv(Time, Censor) ~ .,
   data = training,
   x_latency = training, model = "weibull", penalty = "lasso",
   fdr_control = TRUE, grid_tuning = FALSE, nlambda_inc = 10,
   nlambda_lat = 10, n_folds = 2, seed = 23, verbose = TRUE
   )
}
}
\references{
Archer, K. J., Fu, H., Mrozek, K., Nicolet, D., Mims, A. S.,
Uy, G. L., Stock, W., Byrd, J. C., Hiddemann, W., Braess, J.,
Spiekermann, K., Metzeler, K. H., Herold, T., Eisfeld, A.-K. (2024)
Identifying long-term survivors and those at higher or lower risk of relapse
among patients with cytogenetically normal acute myeloid leukemia using a
high-dimensional mixture cure model. \emph{Journal of Hematology & Oncology},
\bold{17}:28.
}
\seealso{
\code{\link{cureem}}
}
\keyword{models}
\keyword{regression}
